Package: R453Plus1Toolbox
Type: Package
Title: A package for importing and analyzing data from Roche's Genome
        Sequencer System
Version: 1.63.2
Date: 2026-07-01
Author: Hans-Ulrich Klein, Christoph Bartenhagen, Christian Ruckert
Maintainer: Hans-Ulrich Klein <h.klein@uni-muenster.de>
Depends: R (>= 2.12.0), methods, VariantAnnotation (>= 1.25.11),
        Biostrings (>= 2.47.6), pwalign, Biobase
Imports: utils, grDevices, graphics, stats, tools, xtable, R2HTML,
        TeachingDemos, BiocGenerics, S4Vectors (>= 0.17.25), IRanges
        (>= 2.13.12), XVector, GenomicRanges (>= 1.31.8),
        SummarizedExperiment, biomaRt, BSgenome (>= 1.47.3), Rsamtools,
        ShortRead (>= 1.37.1)
Suggests: rtracklayer, BSgenome.Hsapiens.UCSC.hg19,
        BSgenome.Scerevisiae.UCSC.sacCer2
Description: The R453Plus1 Toolbox comprises useful functions for the
        analysis of data generated by Roche's 454 sequencing platform.
        It adds functions for quality assurance as well as for
        annotation and visualization of detected variants,
        complementing the software tools shipped by Roche with their
        product. Further, a pipeline for the detection of structural
        variants is provided.
License: LGPL-3
biocViews: Sequencing, Infrastructure, DataImport, DataRepresentation,
        Visualization, QualityControl, ReportWriting
git_url: https://git.bioconductor.org/packages/R453Plus1Toolbox
git_branch: devel
git_last_commit: c022d30
git_last_commit_date: 2026-07-01
Repository: Bioconductor 3.24
Date/Publication: 2026-07-03
NeedsCompilation: yes
Packaged: 2026-07-03 21:23:39 UTC; biocbuild
Built: R 4.6.1; aarch64-apple-darwin23; 2026-07-04 03:48:19 UTC; unix

import(methods)
importFrom(utils, read.csv, read.table, write.csv, Sweave)
importFrom(grDevices, dev.off, pdf, png, rgb, rainbow)
importFrom(graphics, abline, axis, barplot, hist, legend, lines, mtext, par, plot, points,
    polygon, rect, strheight, strwidth, text)
importFrom(stats, as.dist, cutree, filter, hclust)
importFrom(tools, texi2dvi)
importFrom(xtable, xtable)
importFrom(R2HTML, HTML, HTML.title, HTMLEndFile, HTMLInitFile)
importFrom(TeachingDemos, spread.labs)

import(BiocGenerics)
import(S4Vectors)
import(IRanges)
import(XVector)
import(GenomicRanges)
import(SummarizedExperiment)

importClassesFrom(Biobase, AnnotatedDataFrame, AssayData, MIAME, Versions, Versioned, VersionedBiobase, eSet)
importMethodsFrom(Biobase, assayData, "assayData<-", combine, fData, featureData, "featureData<-",
    pData, phenoData, "phenoData<-", sampleNames, varMetadata)
importFrom(Biobase, assayDataElementNames, assayDataNew)
    
importFrom(biomaRt, getBM, getSequence, useMart, useEnsembl)

importClassesFrom(Biostrings, BString, DNAString, XStringSet, DNAStringSet,
    XStringQuality, PhredQuality, SolexaQuality,
    QualityScaledDNAStringSet, QualityScaledXStringSet)
importMethodsFrom(Biostrings, PDict, alphabetFrequency, complement, endIndex,
    matchPDict, matchPattern, quality, reverseComplement, startIndex,
    translate, vmatchPattern)
importFrom(Biostrings, DNAString, BStringSet, DNAStringSet, getSeq,
    dinucleotideFrequency, trinucleotideFrequency, oligonucleotideFrequency,
    PhredQuality, QualityScaledDNAStringSet, writeXStringSet)

importFrom(pwalign, nucleotideSubstitutionMatrix, pairwiseAlignment,
    aligned, insertion, deletion, mismatchTable)

importClassesFrom(BSgenome, BSgenome)

importFrom(Rsamtools, scanBam)

importClassesFrom(ShortRead, AlignedRead)
importMethodsFrom(ShortRead, chromosome, id, name, position)
importFrom(ShortRead, AlignedDataFrame, AlignedRead, alignData, sread, alphabetByCycle)

importFrom(VariantAnnotation, VCF, VCFHeader)
importMethodsFrom(VariantAnnotation, writeVcf)

exportClasses(
    AVASet,
    MapperSet,
    AnnotatedVariants,
    Breakpoints,
    SFFRead,
    SFFContainer)


exportMethods(
    alignShortReads,
    readsOnTarget,
    coverageOnTarget,
    calculateTiTv,
    genomeSequencerMIDs,
    sequenceCaptureLinkers,
    demultiplexReads,
    removeLinker,

    filterChimericReads,
    detectBreakpoints,
    mergeBreakpoints,
    breakpointTable,

    annotateVariants,
    annotatedVariants,
    htmlReport,

    ava2vcf,
    
    plotVariationFrequency,
    plotAmpliconCoverage,
    plotVariants,
    plotChimericReads,
            
    seqsC1, "seqsC1<-",
    seqsC2, "seqsC2<-",
    commonBpsC1, "commonBpsC1<-",
    commonBpsC2, "commonBpsC2<-",
    commonAlignC1, "commonAlignC1<-",
    commonAlignC2, "commonAlignC2<-",
    alignedReadsC1, "alignedReadsC1<-",
    alignedReadsC2, "alignedReadsC2<-",
             
    MapperSet,
    AVASet,
    fDataAmp,
    featureDataAmp, "featureDataAmp<-",
    assayDataAmp, "assayDataAmp<-",
    getAlignedReads,
    referenceSequences, "referenceSequences<-",
    getReadStatus,
    getVariantPercentages,
    setVariantFilter,
    
    name, "name<-",
    flowgramFormat, "flowgramFormat<-",
    flowChars, "flowChars<-",
    keySequence, "keySequence<-",
    clipQualityLeft, "clipQualityLeft<-",
    clipQualityRight, "clipQualityRight<-",
    clipAdapterLeft, "clipAdapterLeft<-",
    clipAdapterRight, "clipAdapterRight<-",
    flowgram, "flowgram<-",   
    flowgrams, "flowgrams<-",     
    flowIndexes, "flowIndexes<-",
    read, "read<-",
    quality, "quality<-",
    reads, "reads<-",
    addRead, getRead,
    sff2fastq,
    
    readLengthStats,
    readLengthHist,
    baseQualityStats,
    baseQualityHist,
    sequenceQualityHist,
    positionQualityBoxplot,
    baseFrequency,
    nucleotideCharts,
    gcContent,
    gcPerPosition,
    gcContentHist,
    complexity.dust,
    complexity.entropy,
    dinucleotideOddsRatio,
    flowgramBarplot,
    homopolymerHist
)

export(
    extendedCIGARToList,
    listToExtendedCIGAR,
    readSFF,
    writeSFF,
    qualityReportSFF
)

useDynLib(R453Plus1Toolbox)

.getAlignedReads = function(object, amplicons, dir){

    ## catch some bad input
    if(!missing(amplicons)){
    	if(!is.vector(amplicons, mode="character"))
	    stop("invalid argument: please specify your amplicon names as a charcter vector (see fDataAmp() for details)")
	if(!all(amplicons %in% rownames(fDataAmp(object))))
	    stop("invalid argument: (some of) your amplicon names were not found in the AVA set (see fDataAmp() for details)")
    }else
	amplicons = rownames(fDataAmp(object))

    ## directory of alignment file is either given directly by a function argument or is read from the AVASet object
    if(!missing(dir)){
    	if(!is.character(dir))
	    stop("invalid argument: directory has to be of type character")
	rootDir = file.path(dir, "Amplicons/Results/Align")
    }else
	rootDir = dirs(object)["align"]
    if(!file.exists(rootDir))
        stop("directory not found: ", rootDir)


    sampleIDs = pData(object)$SampleID
    refSeqs = fDataAmp(object)[amplicons, "referenceSeqID"]
    allReads = vector(mode="character")

    ## read individual file for each sample and every amplicon
    cat(length(amplicons), " amplicons were found\n")
    cat("Reading aligned reads from ", rootDir, " ... ")
    for(sample in sampleIDs){

    	for(ref in refSeqs){
	
	    file = paste(sample, "_vs_", ref, ".readAlign.txt", sep="")
	    file = file.path(rootDir, sample, ref, file)

	    if(file.exists(file)){

              ## read aligned reads; format: two rows per read, one name and one sequence
              df = read.table(file, fill=TRUE, sep="\t", skip=2, stringsAsFactors=FALSE, col.names=c("accessionNumber", "read", "tmp"))
              accessionNumbers = df$accessionNumber[seq(1, nrow(df), 2)] 		## remove entries from rows with sequences
              reads = df$read[df$read != ""]                              	## remove (empty) entries from rows with read names

              ## extract accession numbers from other information in that row
              accessionNumbers = strsplit(accessionNumbers, split = " /")
              accessionNumbers = sapply(accessionNumbers, function(x) substr(x[1], 2, nchar(x[1])))	    
              
              ## remove filling characters ("-") from read sequence (for example: reads have the form "TCTTCTTCAC-AGGTGCTTT-CAA-G-AAC-AGG--G")
              reads = gsub("-","",reads)

              names(reads) = accessionNumbers
              allReads = c(allReads, reads)
            }
            else
              warning("file not found: ", file)
	}
    }
    cat("done\n")
    cat(length(allReads), " aligned reads were found\n")
    
    ## return the reads as a DNAStringSet
    return(DNAStringSet(allReads, use.names=TRUE))

}

setMethod("getAlignedReads",
    signature(object="AVASet"),
    .getAlignedReads)

# Getter and setter methods
setMethod("dataTCAG", signature(object="Roche454BaseCallerMetrics"),
    function(object) {
        return(object@dataTCAG)
    }
)

setReplaceMethod("dataTCAG",
    signature=signature(object="Roche454BaseCallerMetrics",
        value="data.frame"),
    function(object, value) {
        object@dataTCAG = value
        return(object)
    }
)

setMethod("dataCATG", signature(object="Roche454BaseCallerMetrics"),
    function(object) {
        return(object@dataCATG)
    }
)

setReplaceMethod("dataCATG",
    signature=signature(object="Roche454BaseCallerMetrics",
        value="data.frame"),
    function(object, value) {
        object@dataCATG = value
        return(object)
    }
)

setMethod("lengthTCAG", signature(object="Roche454BaseCallerMetrics"),
    function(object) {
        return(object@lengthTCAG)
    }
)

setReplaceMethod("lengthTCAG",
    signature=signature(object="Roche454BaseCallerMetrics",
        value="matrix"),
    function(object, value) {
        object@lengthTCAG = value
        return(object)
    }
)

setMethod("scoreTCAG", signature(object="Roche454BaseCallerMetrics"),
    function(object) {
        return(object@scoreTCAG)
    }
)

setReplaceMethod("scoreTCAG",
    signature=signature(object="Roche454BaseCallerMetrics",
        value="matrix"),
    function(object, value) {
        object@scoreTCAG = value
        return(object)
    }
)

setMethod("lengthCATG", signature(object="Roche454BaseCallerMetrics"),
    function(object) {
        return(object@lengthCATG)
    }
)

setReplaceMethod("lengthCATG",
    signature=signature(object="Roche454BaseCallerMetrics",
        value="matrix"),
    function(object, value) {
        object@lengthCATG = value
        return(object)
    }
)

setMethod("scoreCATG", signature(object="Roche454BaseCallerMetrics"),
    function(object) {
        return(object@scoreCATG)
    }
)

setReplaceMethod("scoreCATG",
    signature=signature(object="Roche454BaseCallerMetrics",
        value="matrix"),
    function(object, value) {
        object@scoreTCAG = value
        return(object)
    }
)


# subsequent methods are removed due to warnings by R CMD check

# Subsetting
#setMethod("[", signature=signature(
#        x="Roche454BaseCallerMetrics",
#        i="ANY",
#        j="missing"),          
#    function(x, i, ..., drop) {
#        if (missing(drop)) {
#            drop = FALSE
#        }
#        dataTCAG(x) = dataTCAG(x)[i, , drop=drop]
#        dataCATG(x) = dataCATG(x)[i, , drop=drop]
#        lengthTCAG(x) = lengthTCAG(x)[, i, drop=drop]
#        scoreTCAG(x) = scoreTCAG(x)[, i, drop=drop]
#        lengthCATG(x) = lengthCATG(x)[, i, drop=drop]
#        scoreCATG(x) =scoreCATG(x)[, i, drop=drop]
#        return(x)
#    }
#)


# Other convenience functions
#setMethod(show, signature(object="Roche454BaseCallerMetrics"),
#    function(object) {
#        show(dataTCAG(object))
#    }
#)

#setMethod("sampleNames", signature(object="Roche454BaseCallerMetrics"),
#    function(object) {
#      return(rownames(object@dataTCAG))
#    }
#)

#setMethod("combine", signature(
#        x="Roche454BaseCallerMetrics",
#        y="Roche454BaseCallerMetrics"),
#    function(x, y) {
#      if (any(is.element(sampleNames(x), sampleNames(y)))) {
#        stop(paste("Cannot combine objects: Objects contain the same regions",
#                   "from the same pico titer plate."))
#      }
#      bcm = new("Roche454BaseCallerMetrics")
#      bcm@dataTCAG = rbind(x@dataTCAG, y@dataTCAG)
#      bcm@dataCATG = rbind(x@dataCATG, y@dataCATG)
#
#      ncolX = ncol(x@lengthTCAG)
#      ncolY = ncol(y@lengthTCAG)
#      z = matrix(0, nrow=max(nrow(x@lengthTCAG), nrow(y@lengthTCAG)),
#          ncol=ncolX+ncolY)
#      z[1:nrow(x@lengthTCAG), 1:ncolX] = x@lengthTCAG
#      z[1:nrow(y@lengthTCAG), (ncolX+1):(ncolX+ncolY)] = y@lengthTCAG
#      rownames(z) = c(rownames(x@lengthTCAG), rownames(y@lengthTCAG))
#      bcm@lengthTCAG = z
#
#      ncolX = ncol(x@scoreTCAG)
#      ncolY = ncol(y@scoreTCAG)
#      z = matrix(0, nrow=max(nrow(x@scoreTCAG), nrow(y@scoreTCAG)),
#          ncol=ncolX+ncolY)
#      z[1:nrow(x@scoreTCAG), 1:ncolX] = x@scoreTCAG
#      z[1:nrow(y@scoreTCAG), (ncolX+1):(ncolX+ncolY)] = y@scoreTCAG
#      rownames(z) = c(rownames(x@scoreTCAG), rownames(y@scoreTCAG))
#      bcm@scoreTCAG = z
#
#      ncolX = ncol(x@lengthCATG)
#      ncolY = ncol(y@lengthCATG)
#      z = matrix(0, nrow=max(nrow(x@lengthCATG), nrow(y@lengthCATG)),
#          ncol=ncolX+ncolY)
#      z[1:nrow(x@lengthCATG), 1:ncolX] = x@lengthCATG
#      z[1:nrow(y@lengthCATG), (ncolX+1):(ncolX+ncolY)] = y@lengthCATG
#      rownames(z) = c(rownames(x@lengthCATG), rownames(y@lengthCATG))
#      bcm@lengthCATG = z
#
#      ncolX = ncol(x@scoreCATG)
#      ncolY = ncol(y@scoreCATG)
#      z = matrix(0, nrow=max(nrow(x@scoreCATG), nrow(y@scoreCATG)),
#          ncol=ncolX+ncolY)
#      z[1:nrow(x@scoreCATG), 1:ncolX] = x@scoreCATG
#      z[1:nrow(y@scoreCATG), (ncolX+1):(ncolX+ncolY)] = y@scoreCATG
#      rownames(z) = c(rownames(x@scoreCATG), rownames(y@scoreCATG))
#      bcm@scoreCATG = z
#
#      return(bcm)
#    }
#)

# private function for computing breakpoint distance
# This function assumes that both reads are from different cases.
# This can be checked for breakpoints with strand changes
# (not for breakpoints with strand chnges).
.computeBPDistance_sc <- function(chr1.1, pos1.1, chr1.2, pos1.2, 
                        chr2.1, pos2.1, chr2.2, pos2.2) {

    # inversion
    if (chr1.1 == chr1.2 & chr1.1 == chr2.1 & chr2.1 == chr2.2) {
        dist = c(abs(pos1.1 - pos2.1) + abs(pos1.2 - pos2.2),
            abs(pos1.1 - pos2.2) + abs(pos1.2 - pos2.1))
        return(min(dist))

    # not inversion
    } else if (chr1.1 == chr2.1 & chr1.2 == chr2.2) {
        return(abs(pos1.1 - pos2.1) + abs(pos1.2 - pos2.2))
    } else if (chr1.1 == chr2.2 & chr1.2 == chr2.1) {
        return(abs(pos1.1 - pos2.2) + abs(pos1.2 - pos2.1))

    # chromosomes do not fit
    } else {
        return(Inf)
    }
}


# private function for computing breakpoint distance
# This function is for breakpoints without strand changes.
.computeBPDistance_nsc <- function(strand1, chr1.5p, pos1.5p, chr1.3p, pos1.3p, 
                        strand2, chr2.5p, pos2.5p, chr2.3p, pos2.3p) {

    # data refers to the same strand
    if (strand1 == strand2) {

        if (chr1.5p == chr2.3p & chr1.3p == chr2.5p) {
            return(abs(pos1.5p - pos2.3p) + abs(pos1.3p - pos2.5p))

        } else {
            return(Inf)
        }

    # data refers to different strands
    } else {

        if (chr1.5p == chr2.5p & chr1.3p == chr2.3p) {
            return(abs(pos1.5p - pos2.5p) + abs(pos1.3p == pos2.3p))
        } else {
            return(Inf)
        }
    }
}


# private function for selecting breakpoints for merging
.markRelatedBreakpoints <- function(breakpoints, maxDist=1000) {

    #bps = seqsC1(breakpoints)
    mergeList = list()

    strandChange = sapply(commonBpsC1(breakpoints), function (x) {
        return(x[1, "strand"] != x[2, "strand"])
    })
    bpsc = names(breakpoints)[strandChange]
    bpnc = names(breakpoints)[!strandChange]


    
    # iterate through all bps with _strand changes_ and note cases
    # using this annotation:
    #
    #      Case 1        Case 2
    #
    #     3'   5'       3'   5'          
    #      |   |         |   | 
    #    - |   | +     + |   | -    
    #      | B |         | A |
    #      |   |         |   |
    #      |---|         |---| 
    #      |   |         |   | 
    #      |   |         |   |
    #    + | A | -     - | B | +
    #      |   |         |   |
    #     5'   3'       5'   3'
    #
    if (length(bpsc) > 1) {
        case = character(length(bpsc))
        names(case) = bpsc
        for (bp in bpsc) {
            ind5p =  commonBpsC1(breakpoints)[[bp]][, "localStart"] ==
                min(commonBpsC1(breakpoints)[[bp]][, "localStart"])
            if (commonBpsC1(breakpoints)[[bp]][ind5p, "strand"] == "+") {
                case[bp] = "case 1"
            } else {
                case[bp] = "case 2"
            }
        }

        # iterate breakpoints again and write merge list
        for (i in 1:(length(bpsc)-1)) {
            for (j in (i+1):length(bpsc)) {
                
                bi = bpsc[i]
                bj = bpsc[j]
                
                #different cases may be merged
                if (case[bi] != case[bj]) {
                    d = .computeBPDistance_sc(
                        chr1.1=commonBpsC1(breakpoints)[[bi]]$chr[1],
                        pos1.1=commonBpsC1(breakpoints)[[bi]]$breakpoint[1],
                        chr1.2=commonBpsC1(breakpoints)[[bi]]$chr[2],
                        pos1.2=commonBpsC1(breakpoints)[[bi]]$breakpoint[2],
                        chr2.1=commonBpsC1(breakpoints)[[bj]]$chr[1],
                        pos2.1=commonBpsC1(breakpoints)[[bj]]$breakpoint[1],
                        chr2.2=commonBpsC1(breakpoints)[[bj]]$chr[2],
                        pos2.2=commonBpsC1(breakpoints)[[bj]]$breakpoint[2])
                    
                    if (d <= maxDist) {
                        # we should merge
                        mergeBps = c(bi, bj)

                        if (any(is.element(mergeBps, unlist(mergeList)))) {
                            # we cannot merge
                            warning(paste("Want to merge ", mergeBps[1],
                                " with ", mergeBps[2], ", but one of these ",
                                "already merges with another breakpoint!",
                                sep=""))
                        } else {
                            mergeList[[length(mergeList)+1]] = mergeBps
                        }
                    }
                }
            }
        }
    }

    # Iterate through all bps with _no strand changes_ and merge bps.note
    # For same strand chimeric reads, cases 1 and 2 cannot be distinguished,
    # but when two reads are given, we can check whether they are from
    # different strands. (This is done in .computeBPDistance_nsc)
    # using this annotation:
    #
    #      Case 1        Case 2
    #
    #     3'   5'       3'   5'          
    #      |   |         |   | 
    #    + |   | -     + |   | -
    #      | B |         | A |
    #      |   |         |   |
    #      |---|         |---|
    #      |   |         |   |
    #      |   |         |   |
    #    + | A | -     + | B | -
    #      |   |         |   |
    #     5'   3'       5'   3'
    #
    if (length(bpnc) > 1) {

        # iterate breakpoints and write merge list
        for (i in 1:(length(bpnc)-1)) {
            for (j in (i+1):length(bpnc)) {

                bi = bpnc[i]
                bj = bpnc[j]
                
                ind5pi =  commonBpsC1(breakpoints)[[bi]][, "localStart"] ==
                    min(commonBpsC1(breakpoints)[[bi]][, "localStart"])
                ind5pj =  commonBpsC1(breakpoints)[[bj]][, "localStart"] ==
                    min(commonBpsC1(breakpoints)[[bj]][, "localStart"])

                d = .computeBPDistance_nsc(
                    strand1=commonBpsC1(breakpoints)[[bi]]$strand[1],
                    chr1.5p=commonBpsC1(breakpoints)[[bi]]$chr[ind5pi],
                    pos1.5p=commonBpsC1(breakpoints)[[bi]]$breakpoint[ind5pi],
                    chr1.3p=commonBpsC1(breakpoints)[[bi]]$chr[!ind5pi],
                    pos1.3p=commonBpsC1(breakpoints)[[bi]]$breakpoint[!ind5pi],
                    strand2=commonBpsC1(breakpoints)[[bj]]$strand[1],
                    chr2.5p=commonBpsC1(breakpoints)[[bj]]$chr[ind5pj],
                    pos2.5p=commonBpsC1(breakpoints)[[bj]]$breakpoint[ind5pj],
                    chr2.3p=commonBpsC1(breakpoints)[[bj]]$chr[!ind5pj],
                    pos2.3p=commonBpsC1(breakpoints)[[bj]]$breakpoint[!ind5pj])

                if (d <= maxDist) {
                    # we should merge
                    mergeBps = c(bi, bj)

                    if (any(is.element(mergeBps, unlist(mergeList)))) {
                        # we cannot merge
                        warning(paste("Want to merge ", mergeBps[1],
                            " with ", mergeBps[2], ", but one of these ",
                            "already merges with another breakpoint!",
                            sep=""))
                    } else {
                        mergeList[[length(mergeList)+1]] = mergeBps
                    }                    
                }
            }
        }
    }

    return(mergeList)
}


.mergeBreakpoints <- function(breakpoints, maxDist, mergeBPs) {

    if(length(breakpoints) == 0)
      stop("The given object contains no breakpoints")
  
    # no merging on a breakpoint object that has already been merged
    if (any(sapply(seqsC2(breakpoints), function(x) {nrow(x) != 0}))) {
        stop("The given object already contains merged breakpoints - merging aborted.")
    }

    # we need distinct breakpoint names
    if (is.null(names(breakpoints)) |
        any(is.na(names(breakpoints))) |
        any(duplicated(names(breakpoints)))) {
        stop("Breakpoints must have proper unique names.")
    }

    # compute breakpoints that should be merged
    if (missing(mergeBPs)) {
        if(missing(maxDist)) {
            maxDist = 1000
          }
        mergeBPs = .markRelatedBreakpoints(breakpoints, maxDist)
    # breakpoints to be merged are given by user
    } else {
        if (any(duplicated(unlist(mergeBPs)))) {
            stop("Breakpoints must not be merged twice!")
        }
        if (is.numeric(unlist(mergeBPs))) {
            lapply(mergeBPs, function (x) {names(breakpoints)[x]})
        }
    }

    
    # start merging
    if (length(mergeBPs) > 0) {
        for (i in 1:length(mergeBPs)) {
            k = mergeBPs[[i]][1]
            l = mergeBPs[[i]][2]

            message(paste("Merging breakpoints ", k, " and ", l, ".", sep=""))

            # seqs
            breakpoints@seqsC2[[k]] = breakpoints@seqsC1[[l]]

            # commonBps
            breakpoints@commonBpsC2[[k]] = breakpoints@commonBpsC1[[l]]

            # commonAlign (PairwiseAlignmentsSingleSubject)
            breakpoints@commonAlignC2[[k]] = breakpoints@commonAlignC1[[l]]
            
            # alignedReads (AlignedRead)
            breakpoints@alignedReadsC2[[k]] = breakpoints@alignedReadsC1[[l]]
            
            # rename  entry k
            n = names(breakpoints)
            n[which(n == k)] = paste(k, l, sep="_")
            names(breakpoints) = n

            # remove entry l
            breakpoints@seqsC1[l] = NULL
            breakpoints@seqsC2[l] = NULL
            breakpoints@commonBpsC1[l] = NULL
            breakpoints@commonBpsC2[l] = NULL
            breakpoints@commonAlignC1[l] = NULL
            breakpoints@commonAlignC2[l] = NULL
            breakpoints@alignedReadsC1[l] = NULL
            breakpoints@alignedReadsC2[l] = NULL
        }
    }

    ind = order(summary(breakpoints)$NoReadsTotal, decreasing=TRUE)
    breakpoints@seqsC1 = breakpoints@seqsC1[ind]
    breakpoints@seqsC2 =  breakpoints@seqsC2[ind]
    breakpoints@commonBpsC1 = breakpoints@commonBpsC1[ind]
    breakpoints@commonBpsC2 = breakpoints@commonBpsC2[ind]
    breakpoints@commonAlignC1 = breakpoints@commonAlignC1[ind]
    breakpoints@commonAlignC2 = breakpoints@commonAlignC2[ind]
    breakpoints@alignedReadsC1 = breakpoints@alignedReadsC1[ind]
    breakpoints@alignedReadsC2 = breakpoints@alignedReadsC2[ind]

    return(breakpoints)
}

setMethod("mergeBreakpoints",
    signature=signature(breakpoints="Breakpoints", maxDist="missing", mergeBPs="missing"),
    .mergeBreakpoints)

setMethod("mergeBreakpoints",
    signature=signature(breakpoints="Breakpoints", maxDist="numeric", mergeBPs="missing"),
    .mergeBreakpoints)

setMethod("mergeBreakpoints",
    signature=signature(breakpoints="Breakpoints", maxDist="missing", mergeBPs="list"),
    .mergeBreakpoints)

.read454QualityFilterMetrics_character <- function(object){

    qfm = read.table(file=object, sep=",",
        colClasses=c("character","character"),
        fill=TRUE, col.names=c("V1", "V2"))

    # Extract the user specified analysis name, i.e. the last part of the
    # analysis directory name.
    ind = qfm[,1] == "analysisName"
    analysisName = unlist(strsplit(qfm[ind, 2], "_"))
    analysisName = analysisName[length(analysisName)]

    # Extract indices
    regInd = qfm[,1] == "region"
    totalRWInd = qfm[,1] == "totalRawWells"
    keySeqInd = qfm[,1] == "keySequence"
    numKeyPassInd = qfm[,1] == "numKeyPass"
    numDotFailedInd = qfm[,1] == "numDotFailed"
    numMixedFailedInd = qfm[,1] == "numMixedFailed"
    numTTSQInd = qfm[,1] == "numTrimmedTooShortQuality"
    numTTSPInd = qfm[,1] == "numTrimmedTooShortPrimer"
    totalPFInd = qfm[,1] == "totalPassedFiltering"

    # Create data frame for aggregated information (both keys)
    dataTotal = data.frame(
        AnalysisName=analysisName,
        Region=qfm[regInd, 2],
        RawWells=as.numeric(qfm[totalRWInd, 2]))
    rownames(dataTotal) = paste(dataTotal$AnalysisName,
        dataTotal$Region, sep="_")

    # Create data frame for TCAG and CATG keys
    dataTmp = data.frame(
        AnalysisName=analysisName,
        Region=as.vector(rbind(qfm[regInd, 2], qfm[regInd, 2])),
        NumKeyPass=as.numeric(qfm[numKeyPassInd, 2]),
        NumDotFailed=as.numeric(qfm[numDotFailedInd, 2]),
        NumMixedFailed=as.numeric(qfm[numMixedFailedInd, 2]),
        NumTrimmedTooShortQuality=as.numeric(qfm[numTTSQInd, 2]),
        NumTrimmedTooShortPrimer=as.numeric(qfm[numTTSPInd, 2]),
        TotalPassedFiltering=as.numeric(qfm[totalPFInd, 2]))
    dataTmp = split(dataTmp, factor(qfm[keySeqInd, 2]))
    dataTCAG = dataTmp$TCAG
    dataCATG = dataTmp$CATG
    rownames(dataTCAG) = paste(dataTCAG$AnalysisName, dataTCAG$Region, sep="_")
    rownames(dataCATG) = paste(dataCATG$AnalysisName, dataCATG$Region, sep="_")

    qfm = new("Roche454QualityFilterMetrics")
    qfm@dataTotal = dataTotal
    qfm@dataTCAG = dataTCAG
    qfm@dataCATG = dataCATG

    return(qfm)
}

setMethod("read454QualityFilterMetrics", signature="character",
    .read454QualityFilterMetrics_character)

.demultiplexReads <- function(reads, mids, numMismatches, trim) {

    if (missing(numMismatches)) {
        numMismatches = 2
    }
    if (missing(trim)) {
        trim = TRUE
    }
    
    w = width(mids)[1]
    if (!all(width(mids) == w)) {
        stop("All MIDs must have the same length.")
    }
    
    readsPrefix = subseq(reads, start=1, end=pmin(width(reads), w+5))

    submat = matrix(-1, nrow=5, ncol=5,
        dimnames=list(c("A","C","G","T","N"), c("A","C","G","T","N")))
    diag(submat)[1:4] = 0

    scores = matrix(NA, nrow=length(readsPrefix), ncol=length(mids))
    patternRanges = list()
    for (i in 1:length(mids)) {
        pA = pairwiseAlignment(pattern=readsPrefix, subject=mids[[i]],
            type="local-global", substitutionMatrix=submat,
            gapOpening=0, gapExtension=1)
        scores[,i] = score(pA)
        patternRanges[[i]] = pA@pattern@range
    }

    # find unique maxima >= numMismatches
    sample = apply(scores, 1,
        function(x) {
            m = max(x)
            if (sum(x == m) == 1 & m >= -numMismatches) {
                return(which(x == m))
            } else {
                return(NA)
            }
        }
    )

    sampleSeqs = list()
    for (i in 1:length(mids)) {
        ind = sample == i
        ind[is.na(ind)] = FALSE
        if (trim & any(ind)) {
           sampleSeqs[[i]] = subseq(reads[ind],
               start=end(patternRanges[[i]][ind])+1)
        } else {
           sampleSeqs[[i]] = reads[ind]
        }    
    }
    names(sampleSeqs) = names(mids)

    return(sampleSeqs)
}

setMethod("demultiplexReads",
    signature=signature(reads="XStringSet", mids="XStringSet", numMismatches="numeric", trim="logical"),
    .demultiplexReads)

setMethod("demultiplexReads",
    signature=signature(reads="XStringSet", mids="XStringSet", numMismatches="numeric", trim="missing"),
    .demultiplexReads)

setMethod("demultiplexReads",
    signature=signature(reads="XStringSet", mids="XStringSet", numMismatches="missing", trim="logical"),
    .demultiplexReads)

setMethod("demultiplexReads",
    signature=signature(reads="XStringSet", mids="XStringSet", numMismatches="missing", trim="missing"),
    .demultiplexReads)

#################################################################################
########## METHODS TO CREATE AN AVASET ##########################################
#################################################################################

## AVASet reads all relevant information (sample-, variant- and amplicon data)
## from the (sub-) directories of the project
## Input:
## Project driectory
## Output:
## AVASet, object from a subclass of the Biobase eSet and extended by
## two slots for amplicon data (assay- and feature data) and reference sequences

setMethod("AVASet",
    #signature(dirname="character"),
          signature(dirname="character", avaBin="missing", file_sample="missing", file_amp="missing", file_reference="missing", file_variant="missing", file_variantHits="missing"),

    function(dirname){


        # check if the given dirname leads to all relevant files and directories
	dir_root = file.path(dirname, "Amplicons")
	dir_results = file.path(dir_root,"Results")
	dir_projectDef = file.path(dir_root,"ProjectDef")
	dir_variants = file.path(dir_results, "Variants")
	dir_align = file.path(dir_results, "Align")
	dirs = c(dir_results, dir_projectDef, dir_variants, dir_align)
	names(dirs) = c("results", "projectDef", "variants", "align")

        if(!file.exists(dirname)
            | !file.exists(dir_root) 
            | !file.exists(dir_results) 
            | !file.exists(dir_projectDef)
            | !file.exists(dir_variants)
            | !file.exists(dir_align)
            | !file.exists(file.path(dir_variants, "currentVariantDefs.txt"))
            | !file.exists(file.path(dir_projectDef, "ampliconsProject.txt"))
        )
            stop("No AVA project found under given directory or files in the directories are missing")

	# if all files were found, read sample, variant and amplicon data
        message("Reading sample data ... ", appendLF=FALSE)
        sampleData = readSampleData(dir_projectDef)

        message("done\nReading reference sequences ... ", appendLF=FALSE)
        refSeqData = readReferenceSequences(dir_projectDef)

        #reading all variant data requires the sample data and the reference
        # sequences
        message("done\nReading variant data ... ", appendLF=FALSE)
        varData = readVariants(dir_variants, sampleData[[1]], sread(refSeqData))

        #reading all amplicon data requires the sample data
        message("done\nReading amplicon data ... ", appendLF=FALSE)
        ampData = readAmplicons(dir_projectDef, dir_align, sampleData[[1]])
        message("done")

        avaSet = new("AVASet",
            variantForwCount=varData[[1]], 
            totalForwCount=varData[[2]], 
            variantRevCount=varData[[3]],
	    totalRevCount=varData[[4]], 
            forwCount=ampData[[1]],
            revCount=ampData[[2]],
            featureDf=varData[[5]], 
            featureMeta=varData[[6]], 
            phenoDf=sampleData[[1]], 
            phenoMeta=sampleData[[2]],
            featureDfAmp=ampData[[3]], 
            featureMetaAmp=ampData[[4]],
            refSeqs=refSeqData,
	    dirs = dirs
        )
        return(avaSet)
 
      }
)

## This AVASet method reads the project data via the AVA Commad Line Interface (AVA-CLI)
## The AVA-CLI is started with the command "doAmplicon" from the AVA-Software installation directory.
## It is started from R and its output (tables) is piped directly to the "read.table" function
## Input:
## Project directory
## AVA-Software installation directory
## Output:
## AVASet, object from a subclass of the Biobase eSet and extended by
## two slots for amplicon data (assay- and feature data) and reference sequences

setMethod("AVASet",
          signature(dirname="character", avaBin="character", file_sample="missing", file_amp="missing", file_reference="missing", file_variant="missing", file_variantHits="missing"),

    function(dirname, avaBin){
      dirs=dirname
      doAmplicon = file.path(avaBin, "doAmplicon")

      ## 1.) SAMPLES
      message("Reading sample data ... ", appendLF=FALSE)
      cmd = "list sample"
      samples =  read.table(pipe(paste("printf \"open ", dirname, " -control readOnly\\n", cmd, "\" | ", doAmplicon, " -", sep="")), sep="\t", header=TRUE, stringsAsFactors=FALSE)
      if(any(duplicated(samples$Name))){
        samples = samples[!duplicated(samples$Name), ]
        warning("Samples with duplicate names found and removed.")
      }
      sampleData = readSampleData_AVACLI(samples)
            
      ## 2.) REFERENCE SEQUENCES
      message("done\nReading reference sequences ... ", appendLF=FALSE)
      cmd = "list reference"
      references = read.table(pipe(paste("printf \"open ", dirname, " -control readOnly\\n", cmd, "\" | ", doAmplicon, " -", sep="")), sep="\t", header=TRUE, stringsAsFactors=FALSE)
      if(any(duplicated(references$Name))){
        references = references[!duplicated(references$Name), ]
        warning("Reference sequences with duplicate names found and removed.")
      }
      refSeqData = readReferenceSequences_AVACLI(references)
            
      ## 3.) VARIANTS
      message("done\nReading variant data ... ", appendLF=FALSE)
      cmd = "list variant"
      variants = data.frame()
      tryCatch({
        variants = read.table(pipe(paste("printf \"open ", dirname, " -control readOnly\\n", cmd, "\" | ", doAmplicon, " -", sep="")), sep="\t", header=TRUE, stringsAsFactors=FALSE)
        v = paste(variants$Name, variants$Reference, sep="_")
        variants = variants[!duplicated(v), ]
        rownames(variants) = v
        variants$CVariant = paste("C", 1:nrow(variants), sep="")  ## Assign a unique ID to every variant
      },
               error = function(e) {variants = data.frame()}
      )
      cmd = "report variantHits"
      variantHits = read.table(pipe(paste("printf \"open ", dirname, " -control readOnly\\n", cmd, "\" | ", doAmplicon, " -", sep="")), sep="\t", header=TRUE, stringsAsFactors=FALSE)
      ## Use the "Consensus" calls instead if "Individual"
      variantHits = variantHits[variantHits$Read.Type=="Consensus", ]
      if(nrow(variantHits) == 0){
         variantHits = data.frame()
       }
      if(any(nrow(variants) == 0, nrow(variantHits) == 0)){
        warning("No variants reported for this experiment: ", dirname)
      }
      else{
        v = paste(variantHits$Variant.Name, variantHits$Reference.Name, sep="_")
        variantHits$CVariant = variants[v, "CVariant"]  ## retrieve variant ID for every hit
      }
      varData = readVariants_AVACLI(variants, variantHits, samples)
      
      ## 4.) AMPLICONS
      message("done\nReading amplicon data ... ", appendLF=FALSE)
      cmd = "list amplicon"      
      amps = read.table(pipe(paste("printf \"open ", dirname, " -control readOnly\\n", cmd, "\" | ", doAmplicon, " -", sep="")), sep="\t", header=TRUE, stringsAsFactors=FALSE)
      if(any(duplicated(amps$Name))){
        amps = amps[!duplicated(amps$Name), ]
        warning("Amplicons with duplicate names found and removed.")
      }
      
      ## read one amplicon alignment file for every combination of sample and reference/amplicon sequence
      amps_align = vector(mode="list", length=nrow(samples) * nrow(amps))
      i = 1
      for(s in samples$Name){
        for(r in references$Name){
          ## Take care:
          ## Annotate spaces as special characters (" " as "\ " or in R "\\\\ ")
          ## Otherwise the call for "doAmplicon will not work!
          s2= gsub(" ", "\\\\ ", s)
          r2= gsub(" ", "\\\\ ", r)
          cmd = paste("report align -sample ", s2, " -reference ", r2, sep="")  
          amps_align[[i]] = NA
          tryCatch({
            amps_align[[i]] = read.table(pipe(paste("printf \"open ", dirname, " -control readOnly\\n", cmd, "\" | ", doAmplicon, " -", sep="")), sep="\t", header=FALSE, stringsAsFactors=FALSE)
            amps_align[[i]] = amps_align[[i]][, 1]  ## convert data.frame to a vector of strings -> combatibility to the alternative import of a fasta file line by line (see below)
          },
                   error = function(e) {amps_align[[i]] = NA}
          )
          names(amps_align)[i] = paste(s, r, sep="_")
          i = i+1
        }
      }
      ampData = readAmplicons_AVACLI(amps, amps_align, samples)
      message("done")
      
      avaSet = new("AVASet",
        variantForwCount=varData[[1]], 
        totalForwCount=varData[[2]], 
        variantRevCount=varData[[3]],
        totalRevCount=varData[[4]], 
        forwCount=ampData[[1]],
        revCount=ampData[[2]],
        featureDf=varData[[5]], 
        featureMeta=varData[[6]], 
        phenoDf=sampleData[[1]], 
        phenoMeta=sampleData[[2]],
        featureDfAmp=ampData[[3]], 
        featureMetaAmp=ampData[[4]],
        refSeqs=refSeqData,
        dirs = dirs
        )
      return(avaSet)
 
    }
)


## This AVASet method reads the project data from tables that are saved on hard disc
## Only exceptions are the aligned amplicons. Here, it expects a directory name which points to
## a directory structure with subdirectories "samplename/referencename" with a fasta file in each of these directories;
## this directory structure can be created using AVACLI command "report alignment -sample * -reference * -outputDir dir"
## Input:
## The filenames/dirnames of all tables: Samples, Amplicons, References, Variants and Varianthits
## Output:
## AVASet, object from a subclass of the Biobase eSet and extended by
## two slots for amplicon data (assay- and feature data) and reference sequences

setMethod("AVASet",
          signature(dirname="character", avaBin="missing", file_sample="character", file_amp="character", file_reference="character", file_variant="character", file_variantHits="character"),
          function(dirname, avaBin, file_sample, file_amp, file_reference, file_variant, file_variantHits){
            
            if(!all(file.exists(file_sample, file_amp, file_reference, file_variant, file_variantHits))){
              file_sample = file.path(dirname, file_sample)
              file_amp = file.path(dirname, file_amp)              
              file_reference = file.path(dirname, file_reference)
              file_variant = file.path(dirname, file_variant)
              file_variantHits = file.path(dirname, file_variantHits)
            }
            if(!all(file.exists(file_sample, file_amp, file_reference, file_variant, file_variantHits))){
              stop("File(s) not found in directory ", dirname)
            }
            dirs = dirname
            
            ## 1.) SAMPLES
            message("Reading sample data ... ", appendLF=FALSE)
            samples = read.table(file_sample, sep=",", header=TRUE, stringsAsFactors=FALSE)
            if(any(duplicated(samples$Name))){
              samples = samples[!duplicated(samples$Name), ]
              warning("Samples with duplicate names found and removed.")
            }
            sampleData = readSampleData_AVACLI(samples)
            
            ## 2.) REFERENCE SEQUENCES
            message("done\nReading reference sequences ... ", appendLF=FALSE)
            references = read.table(file_reference, sep=",", header=TRUE, stringsAsFactors=FALSE)
            if(any(duplicated(references$Name))){
              references = references[!duplicated(references$Name), ]
              warning("Reference sequences with duplicate names found and removed.")
            } 
            refSeqData = readReferenceSequences_AVACLI(references)
            
            ## 3.) VARIANTS
            message("done\nReading variant data ... ", appendLF=FALSE)
            variants = read.table(file_variant, sep=",", header=TRUE, stringsAsFactors=FALSE)
            v = paste(variants$Name, variants$Reference, sep="_")
            variants = variants[!duplicated(v), ]
            rownames(variants) = v
            variants$CVariant = paste("C", 1:nrow(variants), sep="")  ## Assign a unique ID to every variant
            variantHits = read.table(file_variantHits, sep=",", header=TRUE, stringsAsFactors=FALSE)
            v = paste(variantHits$Variant.Name, variantHits$Reference.Name, sep="_")
            variantHits$CVariant = variants[v, "CVariant"]  ## retrieve variant ID for every hit
            varData = readVariants_AVACLI(variants, variantHits, samples)

            ## 4.) AMPLICONS            
            message("done\nReading amplicon data ... ", appendLF=FALSE)
            amps = read.table(file_amp, sep=",", header=TRUE, stringsAsFactors=FALSE)
            if(any(duplicated(amps$Name))){
              amps = amps[!duplicated(amps$Name), ]
              warning("Amplicons with duplicate names found and removed.")
            }                                               
            ## read one amplicon alignment file for every combination of sample and reference/amplicon sequence
            amps_align = vector(mode="list", length=nrow(samples) * nrow(amps))
            i = 1
            for(s in samples$Name){
              for(r in references$Name){
                path = file.path(dirname, s, r)
                files = list.files(path)
                if(length(files) == 0){
                  warning("There are no alignment data for the combination of sample ", s, " and reference sequence ", r)
                  amps_align[[i]] = NA
                }else{
                  ## there should normally be only one file in each folder; select this one
                  file = files[1]
                  if(length(files) > 1){
                    warning("More than one alignment exported for a sample-reference-combination. Reading only the first one.")
                  }
                  amps_align[[i]]= readLines(file.path(path, file))
                }
                names(amps_align)[i] = paste(s, r, sep="_")
                i = i+1
              }
            }
            ampData = readAmplicons_AVACLI(amps, amps_align, samples)
            message("done")
  
            avaSet = new("AVASet",
              variantForwCount=varData[[1]], 
              totalForwCount=varData[[2]], 
              variantRevCount=varData[[3]],
              totalRevCount=varData[[4]], 
              forwCount=ampData[[1]],
              revCount=ampData[[2]],
              featureDf=varData[[5]], 
              featureMeta=varData[[6]], 
              phenoDf=sampleData[[1]], 
              phenoMeta=sampleData[[2]],
              featureDfAmp=ampData[[3]], 
              featureMetaAmp=ampData[[4]],
              refSeqs=refSeqData,
              dirs = dirs
              )
            return(avaSet)
            
          }
          )
## This AVASet method reads the project data from tables that are saved on hard disc
## Only exceptions are the aligned amplicons. Here, it expects a directory name which points to
## a directory structure with subdirectories "samplename/referencename" with a fasta file in each of these directories;
## this directory structure can be created using AVACLI command "report alignment -sample * -reference * -outputDir dir"
## Input:
## The filenames/dirnames of all tables: Samples, Amplicons, References, Variants and Varianthits
## Output:
## AVASet, object from a subclass of the Biobase eSet and extended by
## two slots for amplicon data (assay- and feature data) and reference sequences

setMethod("AVASet",
          signature(dirname="character", avaBin="missing", file_sample="character", file_amp="character", file_reference="character", file_variant="missing", file_variantHits="missing"),
          function(dirname, avaBin, file_sample, file_amp, file_reference){
            
            if(!all(file.exists(file_sample, file_amp, file_reference))){
              file_sample = file.path(dirname, file_sample)
              file_amp = file.path(dirname, file_amp)              
              file_reference = file.path(dirname, file_reference)
            }
            if(!all(file.exists(file_sample, file_amp, file_reference))){
              stop("File(s) not found.")
            }
            dirs = dirname
            
            ## 1.) SAMPLES
            message("Reading sample data ... ", appendLF=FALSE)
            samples = read.table(file_sample, sep=",", header=TRUE, stringsAsFactors=FALSE)
            if(any(duplicated(samples$Name))){
              samples = samples[!duplicated(samples$Name), ]
              warning("Samples with duplicate names found and removed.")
            }
            sampleData = readSampleData_AVACLI(samples)
            
            ## 2.) REFERENCE SEQUENCES
            message("done\nReading reference sequences ... ", appendLF=FALSE)
            references = read.table(file_reference, sep=",", header=TRUE, stringsAsFactors=FALSE)
            if(any(duplicated(references$Name))){
              references = references[!duplicated(references$Name), ]
              warning("Reference sequences with duplicate names found and removed.")
            } 
            refSeqData = readReferenceSequences_AVACLI(references)
            
            ## 3.) VARIANTS
            message("done\nReading variant data ... No variants for this experiment", appendLF=FALSE)
            variants = data.frame()
            variantHits = data.frame()
            varData = readVariants_AVACLI(variants, variantHits, samples)

            ## 4.) AMPLICONS            
            message("\nReading amplicon data ... ", appendLF=FALSE)
            amps = read.table(file_amp, sep=",", header=TRUE, stringsAsFactors=FALSE)
            if(any(duplicated(amps$Name))){
              amps = amps[!duplicated(amps$Name), ]
              warning("Amplicons with duplicate names found and removed.")
            }                                               
            ## read one amplicon alignment file for every combination of sample and reference/amplicon sequence
            amps_align = vector(mode="list", length=nrow(samples) * nrow(amps))
            i = 1
            for(s in samples$Name){
              for(r in references$Name){
                path = file.path(dirname, s, r)
                files = list.files(path)
                if(length(files) == 0){
                  warning("There are no alignment data for the combination of sample ", s, " and reference sequence ", r)
                  amps_align[[i]] = NA
                }else{
                  ## there should normally be only one file in each folder; select this one
                  file = files[1]
                  if(length(files) > 1){
                    warning("More than one alignment exported for a sample-reference-combination. Reading only the first one.")
                  }
                  amps_align[[i]]= readLines(file.path(path, file))
                }
                names(amps_align)[i] = paste(s, r, sep="_")
                i = i+1
              }
            }
            ampData = readAmplicons_AVACLI(amps, amps_align, samples)
            message("done")
  
            avaSet = new("AVASet",
              variantForwCount=varData[[1]], 
              totalForwCount=varData[[2]], 
              variantRevCount=varData[[3]],
              totalRevCount=varData[[4]], 
              forwCount=ampData[[1]],
              revCount=ampData[[2]],
              featureDf=varData[[5]], 
              featureMeta=varData[[6]], 
              phenoDf=sampleData[[1]], 
              phenoMeta=sampleData[[2]],
              featureDfAmp=ampData[[3]], 
              featureMetaAmp=ampData[[4]],
              refSeqs=refSeqData,
              dirs = dirs
              )
            return(avaSet)
            
          }
          )

# extend the eSet initialize method by the two new amplicon slots

setMethod("initialize",
    signature(.Object="AVASet"),
    function(.Object,
        assayData=assayDataNew("list", variantForwCount=variantForwCount,
            totalForwCount=totalForwCount,
            variantRevCount=variantRevCount,
            totalRevCount=totalRevCount),
        assayDataAmp=assayDataNew("list",
            forwCount=forwCount,
            revCount=revCount),
        featureData=new("AnnotatedDataFrame", data=featureDf,
            varMetadata=featureMeta),
        phenoData=new("AnnotatedDataFrame", data=phenoDf, varMetadata=phenoMeta),
        featureDataAmp=new("AnnotatedDataFrame", data=featureDfAmp,
            varMetadata=featureMetaAmp),
        variantForwCount=new("matrix"),
        variantRevCount=new("matrix"),
        totalForwCount=new("matrix"),
        totalRevCount=new("matrix"),
        forwCount=new("matrix"),
        revCount=new("matrix"),
        featureDf=new("data.frame"),
        featureMeta=new("data.frame"),
        phenoDf=new("data.frame"),
        phenoMeta=new("data.frame"),
        featureDfAmp=new("data.frame"),
        featureMetaAmp=new("data.frame"),
        refSeqs=new("AlignedRead"),
	dirs="",
        ...)
    {
        .Object@assayDataAmp = assayDataAmp
        .Object@featureDataAmp= featureDataAmp
        .Object@referenceSequences = refSeqs
	.Object@dirs = dirs
        callNextMethod(.Object, assayData = assayData, featureData=featureData,
            phenoData=phenoData, ...)
    }
)

#################################################################################
###### IMPORT OF FEATURE AND PHENODATA 454 COMMAND LINE EXPORT (AVA-CLI) ########
#################################################################################
## test: setwd("/home/c_bart07/R453Plus1Toolbox_AVASoftware_Update_v2.6/testdata")


setMethod("readSampleData_AVACLI",
    signature(samples="data.frame"),
          function(samples){

            ## Only when sample annotation has a consistent format:
            ## reading MID and Lane from sample annotation which has the format like "07-009440_PTP 421677_MID1_Lane 1"
            ## mid = sapply(strsplit(samples$Annotation, split="_"), function(x) return(grep("MID", x, value=TRUE)))
            ## lane = sapply(strsplit(samples$Annotation, split="_"), function(x) return(grep("Lane", x, value=TRUE)))
            ## Otherwise mid and lane are not accessible:
            mid = NA
            lane = NA
            sampleData = data.frame(row.names=samples$Name, SampleID=samples$Name, MID1=mid, MID2=mid, PTP_AccNum=NA, Lane=lane, ReadGroup=NA, Annotation=samples$Annotation)

            ## no label desription needed
            sampleMeta=data.frame(row.names=colnames(sampleData),
              labelDescription=c("-","-","-","-","-","-","-"))
            
            return(list(sampleData,sampleMeta))
            
          }
          )

# readVariants runs through the subdirectory ".../Amplicons/Results/Variants/"
# and reads feature- and assay data for each variant 
# output: list containing
# variantForwCounts, the number of occurences of the variants in a reference
# sequence (read forwards)
# totalForwCount, the number of forward reads
# variantRevCounts, the number of occurences of the variants in a reference
# sequence (read backwards)
# totalRevCount, the number of reverse reads
# featureData, for each variant the name, canonical pattern and reference
# sequence
# featureMeta, some further information about the featureData

setMethod("readVariants_AVACLI",
          signature(variants="data.frame", variantHits="data.frame", samples="data.frame"),
          function(variants, variantHits, samples){

            ## 1. Read featureData and featureMetaData
            ## If variant information is available:
            if(nrow(variants)>0){
              ## Read positions from variant names
              ## SNPs and indels can are named like "333-335:CAC/---", "333:A/G", "334.5:-/C"
              var = strsplit(variants$Name, split=":")
              start = sapply(var, function(x) return(strsplit(x[1], split="-")[[1]][1]))
              end = sapply(var, function(x) return(strsplit(x[1], split="-")[[1]][2]))
              ## No end position for SNP or insertion
              end[is.na(end)] = start[is.na(end)]
            
              ## Read mutated sequence from variant names
              ## SNPs and indels can are named like "333-335:CAC/---", "333:A/G", "334.5:-/C"
              var = strsplit(variants$Name, split=":")
              referenceSeq = sapply(var, function(x) return(strsplit(x[2], split="/")[[1]][1]))
              variantSeq = sapply(var, function(x) return(strsplit(x[2], split="/")[[1]][2]))
            
              featureData=data.frame(row.names=variants$CVariant, name=variants$Name,
                canonicalPattern=variants$Pattern,
                referenceSeqID=variants$Reference,
                start=as.numeric(start),
                end=as.numeric(end),
                variantBase=variantSeq,
                referenceBases=referenceSeq,
                stringsAsFactors=FALSE)
            ## If no variant information is available, return a dummy data.frame
            }else{
              featureData=data.frame(name=NA,
                canonicalPattern=NA,
                referenceSeqID=NA,
                start=NA,
                end=NA,
                variantBase=NA,
                referenceBases=NA,
                stringsAsFactors=FALSE)
              featureData = featureData[-1, ]
            }
            
            ## construct meta data for the features
            ## no label description needed
            featureMeta=data.frame(row.names=colnames(featureData),
              labelDescription=c("-","-","-","-","-","-","-"))

            ## 2. Read assayData:
            ## If variant information is available:
            sampleNames=as.character(samples$Name)
            num_samples=length(sampleNames)
            if(nrow(variants)>0){
              ## two separate matrices for forward and reverse reads each
              init = matrix(0, nrow(variants), num_samples)
              rownames(init) = variants$CVariant
              colnames(init) = sampleNames
              variantForwCount = init
              totalForwCount = init
              variantRevCount = init
              totalRevCount = init
              for(s in sampleNames){
                v = variantHits[variantHits$Sample.Name == as.character(s), ]
                variantForwCount[v$CVariant, as.character(s)] = v$Forward.Hits
                variantRevCount[v$CVariant, as.character(s)] = v$Reverse.Hits
                totalForwCount[v$CVariant, as.character(s)] = v$Forward.Denom
                totalRevCount[v$CVariant, as.character(s)] = v$Reverse.Denom
              }
            ## If no variant information is available, return dummy data.frames
            }else{
              init = matrix(0, 0, num_samples)
              colnames(init) = sampleNames
              variantForwCount = variantRevCount = totalForwCount = totalRevCount = init
            }
            return(list(variantForwCount, totalForwCount,
                        variantRevCount, totalRevCount, featureData,
                        featureMeta))
            
          }
          )

setMethod("readReferenceSequences_AVACLI",
    signature(references="data.frame"),
          function(references){

            ## build DNAStringSet for the reference sequence slot
            refSeqDNASet=DNAStringSet(references$Sequence)
            names(refSeqDNASet)=references$Name
            refSeqNames=references$Name
            refSeqGenes=sapply(refSeqNames, function(x) strsplit(x, "_")[[1]][1] )
            refSeqData=data.frame(row.names=refSeqNames, name=refSeqNames,
              refSeqID=references$Name, gene=refSeqGenes, stringsAsFactors=FALSE)
            refSeqAligned = AlignedRead(sread=refSeqDNASet, 
              id=BStringSet(references$Name),
              position=as.integer(rep(1, length(references$Name))),
              alignData=AlignedDataFrame(data=refSeqData,
                metadata=data.frame(row.names=colnames(refSeqData))))
            return(refSeqAligned)
            
          }
          )

setMethod("readAmplicons_AVACLI",
    signature(amps="data.frame", amps_align="list", samples="data.frame"),
          function(amps, amps_align, samples){

            ## 1. Read featureData and featureMetaData

            ## read the featureData for the amplicons (its name, reference sequence,
            ## primer and end/start):
            ##amp = read.table(file, sep=",", header=TRUE, stringsAsFactors=FALSE)
            featureData=data.frame(row.names=amps$Name,
              ampID=amps$Name,
              primer1=amps$Primer1,
              primer2=amps$Primer2,
              referenceSeqID=amps$Reference,
              targetStart=as.numeric(amps$Start),
              targetEnd=as.numeric(amps$End),
              stringsAsFactors=FALSE)
            
            ## construct meta data for the features
            featureMeta=data.frame(row.names=colnames(featureData),
              labelDescription=c("-", "-", "-", "-", "-", "-"))
            
            ## 2. Read assayData:
            
            ## initialize assay data consisting of forward and reverse coverage
            sampleNames = as.character(samples$Name)
            ampNames = as.character(amps$Reference)
            init = matrix(0, length(ampNames), length(sampleNames))
            rownames(init) = ampNames
            colnames(init) = sampleNames
            numReadsForward = init
            numReadsReverse = init
            num_amps = length(amps_align)
            for(s in sampleNames){
              for(r in ampNames){
                aln = amps_align[[paste(s, r, sep="_")]]  ## take care: list names must be pasted by sample- and reference name!
                if(!any(is.na(aln))){
                  ## parse entries with "reverseCount=x" and forwardCount=x"
                  counts = grep(">", aln, value=TRUE)[-1]   ## lines with amplicon counts start with ">"; except for first line
                  if(length(counts) > 0){
                    counts = lapply(strsplit(counts, split=" "), function(x) return(grep("forwardCount", x, value=TRUE)))
                    if(length(counts) > 0){
                      counts = sapply(strsplit(unlist(counts), split="="), function(x) return(as.numeric(x[2])))
                      if(length(counts) > 0){
                        numReadsForward[r, s] = sum(counts)
                      }
                    }
                  }
                  counts = grep(">", aln, value=TRUE)[-1]   ## lines with amplicon counts start with ">"; except for first line
                  if(length(counts) > 0){
                    counts = lapply(strsplit(counts, split=" "), function(x) return(grep("reverseCount", x, value=TRUE)))
                    if(length(counts) > 0){
                      counts = sapply(strsplit(unlist(counts), split="="), function(x) return(as.numeric(x[2])))
                      if(length(counts) > 0){
                        numReadsReverse[r, s] = sum(counts)
                      }
                    }
                  }
                }
              }
            }
            return(list(numReadsForward,numReadsReverse,featureData,featureMeta))	
          }
          )

#################################################################################
########## IMPORT OF FEATURE AND PHENODATA (internal methods) ###################
#################################################################################


# readSampleData runs through the subdirectory ".../Amplicons/Results/Variants"
# and reads the sample names and group infos

setMethod("readSampleData",
    signature(dir_projectDef="character"),
    function(dir_projectDef){

    # read the sample data (internal id):
    # file has no data frame structure (inconsistent number of columns), so read
    # it row-wise
    text = readLines(file.path(dir_projectDef, "ampliconsProject.txt"))

    # select lines with sample data
    sample_lines = grep("^Sample", text, value=TRUE)
    sample_con = textConnection(sample_lines)
    if(length(sample_lines) > 0){
    	sample_tab = read.csv(sample_con, sep="\t", col.names=c("Def", "Sample",
            "annotation", "name"), header=FALSE, colClasses=c("character",
            "character", "character", "character"), na.strings="", stringsAsFactors=FALSE)
	samples = sample_tab$Sample
	sampleID = sample_tab$name
    	numSamples = nrow(sample_tab)
    }else{
	samples = NA
    	warning(paste("sample information missing in", file.path(dir_projectDef, "ampliconsProject.txt")))
    }
    close(sample_con)

    if(!any(is.na(samples))){

    	# select lines with ReadData
    	RD_lines = grep("^RD\t", text, value=TRUE)
    	RD_con = textConnection(RD_lines)
    	if(length(RD_lines) > 0)
    	    RD_tab = read.csv(RD_con, sep="\t", col.names=c("Def", "RD", "active",
            	"annotation", "currentPath", "name", "originalPath", "readDataGroup", "sequenceBlueprint"),
            	header=FALSE, colClasses=c("character", "character", "character",
            	"character", "character", "character", "character", "character", "character"), na.strings="", stringsAsFactors=FALSE)
    	close(RD_con)

    	# select lines with ReadDataGroup
    	RDG_lines=grep("^RDG", text, value=TRUE)
    	RDG_con = textConnection(RDG_lines)
    	if(length(RDG_lines) > 0)
    	    RDG_tab = read.csv(RDG_con, sep="\t", col.names=c("Def", "RDG", "annotation",
            	"name"), header=FALSE, colClasses=c("character", "character",
            	"character", "character"), na.strings="", stringsAsFactors=FALSE)
    	close(RDG_con)

    	# select lines with lane ptp and read group information

    	RDS_lines = grep("^RD_Samp_Amp", text, value=TRUE)
    	RDS_con = textConnection(RDS_lines)
    	if(length(RDS_lines) > 0 & length(RD_lines) > 0 & length(RDG_lines) > 0){
    	    RDS_tab = read.csv(RDS_con, sep="\t", col.names=c("Def", "RD_Samp_Amp",
            	"amplicon", "readData", "sample"), header=FALSE, colClasses=c(
            	"character", "character", "character", "character", "character"), na.strings="", stringsAsFactors=FALSE)

    	    # filter those samples, which have several different ReadData-entries		
	    RDS_tab = subset(RDS_tab, duplicated(RDS_tab[,4:5]) == FALSE)

	    RData = merge(merge(RDS_tab, RD_tab, by.x="readData", by.y="RD", all.x=TRUE), RDG_tab, by.x="readDataGroup", by.y="RDG", all.x=TRUE)
	    MUX_MID1_tab = data.frame(name=rep(NA, numSamples), sample=samples)
	    MUX_MID2_tab = data.frame(name=rep(NA, numSamples), sample=samples)

    	}else{
    	    MID_lines = grep("^MIDSeq", text, value=TRUE)
    	    MID_con = textConnection(MID_lines)
    	    MUX_lines = grep("^Mux_Mid12_Samp", text, value=TRUE)
    	    MUX_con = textConnection(MUX_lines)
    	    RDMUX_lines = grep("^RD_Mux\t", text, value=TRUE)
    	    RDMUX_con = textConnection(RDMUX_lines)

    	    if(length(MID_lines) > 0 & length(MUX_lines) > 0 & length(RDMUX_lines) > 0 & length(RD_lines) > 0 & length(RDG_lines) > 0){
    	    	MID_tab = read.csv(MID_con, sep="\t", col.names=c("Def", "MIDSeq", "annotation", "midGroup", "name", "sequence"), 
		    header=FALSE, colClasses=c("character", "character", "character", "character", "character", "character"), na.strings="", stringsAsFactors=FALSE)
    	    	MUX_tab = read.csv(MUX_con, sep="\t", col.names=c("Def", "Mux_Mid12_Samp", "mid1", "mid2", "mux", "sample"), 
		    header=FALSE, colClasses=c("character", "character", "character", "character", "character", "character"), na.strings="", stringsAsFactors=FALSE)
    	    	RDMUX_tab = read.csv(RDMUX_con, sep="\t", col.names=c("Def", "RD_Mux", "mux", "readData"), 
		    header=FALSE, colClasses=c("character", "character", "character", "character"), na.strings="", stringsAsFactors=FALSE)

	    	MUX_MID1_tab = subset(merge(MUX_tab, MID_tab, by.x="mid1", by.y="MIDSeq", all.x=TRUE), !is.na(name))
	    	MUX_MID2_tab = subset(merge(MUX_tab, MID_tab, by.x="mid2", by.y="MIDSeq", all.x=TRUE), !is.na(name))
	    	RData = merge(merge(merge(MUX_tab, RDMUX_tab, by.x="mux", by.y="mux", all.x=TRUE, suffixes=c(".mux", ".rdmux")), RD_tab, by.x="readData", by.y="RD", 
		    all.x=TRUE), RDG_tab, by.x="readDataGroup", by.y="RDG", all.x=TRUE, suffixes=c(".rd", ".rdg"))

                RData = merge(merge(merge(MUX_tab, RDMUX_tab, by.x="mux", by.y="mux", all.x=TRUE, suffixes=c(".mux", ".rdmux")), RD_tab, by.x="readData", by.y="RD", 
		    all.x=TRUE), RDG_tab, by.x="readDataGroup", by.y="RDG", all.x=TRUE)


                
	    }else{
	    	warning(paste("Read data or MID entries missing in", file.path(dir_projectDef, "ampliconsProject.txt")))
	    	MUX_MID1_tab = data.frame(name=rep(NA, numSamples), sample=samples)
	    	MUX_MID2_tab = data.frame(name=rep(NA, numSamples), sample=samples)
	    	RData = data.frame(sample=samples, currentPath=rep(NA, numSamples), 
		     annotation.x=rep(NA, numSamples), name.y=rep(NA, numSamples))
	    }
	    close(MID_con)
    	    close(MUX_con)
    	    close(RDMUX_con)
    	}
    	close(RDS_con)

    	sampleMatrix = matrix(NA, numSamples, 7)
    	i = 1
    	for(s in samples){
	    mid1 = paste(subset(MUX_MID1_tab, sample==s)$name,collapse=",") 
	    mid2 = paste(subset(MUX_MID2_tab, sample==s)$name, collapse=",")
	    path = unique(c(subset(RData, sample==s)$currentPath, subset(RData, sample==s)$currentPath))
	    if(!any(is.na(path))){
	        path = sapply(strsplit(path, split="\\."), function(x)x[1])
	        ptp = paste(substr(path, 1, nchar(path)-2), collapse=",")
	        lane = paste(substr(path, nchar(path)-1, nchar(path)), collapse=",")
	    }else{
	    	ptp = NA
	    	lane = NA
	    }
	    rdAnnot = paste(unique(subset(RData, sample==s)$annotation.rd), collapse=",")
	    rgName = paste(unique(subset(RData, sample==s)$name.rdg), collapse=",")
	    sampleMatrix[i, ] = c(s, mid1, mid2, ptp, lane, rgName, rdAnnot)
	    i = i+1
    	}
    	sampleData = data.frame(sampleMatrix, row.names= sample_tab$name, stringsAsFactors=FALSE)
    	colnames(sampleData) = c("SampleID", "MID1", "MID2", "PTP_AccNum", "Lane", "ReadGroup", "Annotation")

    }else
	sampleData = data.frame(SampleID=NA, MID1=NA, MID2=NA, PTP_AccNum=NA, Lane=NA, ReadGroup=NA, Annotation=NA)

    # no label desription needed
    sampleMeta=data.frame(row.names=colnames(sampleData),
    	labelDescription=c("-","-","-","-","-","-","-"))

    return(list(sampleData,sampleMeta))
})


# readVariants runs through the subdirectory ".../Amplicons/Results/Variants/"
# and reads feature- and assay data for each variant 
# output: list containing
# variantForwCounts, the number of occurences of the variants in a reference
# sequence (read forwards)
# totalForwCount, the number of forward reads
# variantRevCounts, the number of occurences of the variants in a reference
# sequence (read backwards)
# totalRevCount, the number of reverse reads
# featureData, for each variant the name, canonical pattern and reference
# sequence
# featureMeta, some further information about the featureData

setMethod("readVariants",
    signature(dir_variants="character", samples="data.frame",
        refSeqs="DNAStringSet"),
    function(dir_variants, samples, refSeqs){

        # read the featureData for the variants (its name, canonical pattern and
        # reference sequence):
        variantDefs=read.table(file=file.path(dir_variants, "currentVariantDefs.txt"), sep="\t",
            header=TRUE, colClasses=c("character", "character", "character",
            "character"), stringsAsFactors=FALSE)	
        # read position and sequence information for each variant
        num_variants=length(variantDefs$CVariant)
        referenceSeq=matrix(0, num_variants, 1)
        v_splits=strsplit(variantDefs$canonicalPattern, split="[\\)\\(,-]!?")
        variantSeq=sapply(v_splits, function(x)x[3])
        deletions=sapply(v_splits, function(x) x[1] == "d")
        start=as.numeric(sapply(v_splits,function(x)x[2]))
        end=start
        # for deletions, variantSeq contains the end coordinate; take care of deletions of a single base pair (format d(pos) instead of d(start-end))
        end[deletions & !is.na(variantSeq)]=as.numeric(variantSeq[deletions & !is.na(variantSeq)])
        del_length=end[deletions] - start[deletions] + 1
        del_length=sapply(del_length, function(x) paste(rep("-",x), collapse=""))
        
        if(length(del_length) > 0)
           variantSeq[deletions] = del_length
           
        # get the bases from the reference sequences corresponding to the
        # variants
        for(i in 1:num_variants){
            v_split=v_splits[[i]]
            refSeq_start=as.numeric(v_split[2])
            if(v_split[1] == "s" | v_split[1] == "i") {
                refSeq_end=refSeq_start
            } else {
                if(v_split[1] == "d") {
                  # end coordinate is NA if deletion affects only one base pair
                  if(is.na(v_split[3])){
                    refSeq_end=refSeq_start
                  }else{
                    refSeq_end=as.numeric(v_split[3])
                  }
	        } else {
                  stop("illegal canonical pattern")
                }
            }
            # no reference sequence for insertions
            if(v_split[1] == "i"){
              referenceSeq[i]="-"
            }else{
              refSeq=refSeqs[names(refSeqs) == variantDefs$referenceSeq[i]]
              referenceSeq[i]=toString(subseq(refSeq[[1]], refSeq_start, refSeq_end))
            }
        }
        # build name of the variant
        name=paste(start, ":", referenceSeq, "/", variantSeq, sep="")
        name[deletions]=paste(start[deletions], "-", end[deletions], ":DEL(",
            end[deletions] - start[deletions] + 1, ")", sep="")

        featureData=data.frame(row.names=variantDefs$CVariant, name=name,
            canonicalPattern=variantDefs$canonicalPattern,
            referenceSeqID=variantDefs$referenceSeq,
            start=start, 
            end=end,
            variantBase=variantSeq,
            referenceBases=referenceSeq,
            stringsAsFactors=FALSE)

        # construct meta data for the features
	# no label description needed
        featureMeta=data.frame(row.names=colnames(featureData),
            labelDescription=c("-","-","-","-","-","-","-"))

        # two separate matrices for forward and reverse reads each
        sampleNames=as.character(rownames(samples))
        num_samples=length(sampleNames)
        init = matrix(0, length(variantDefs$CVariant), num_samples)
	rownames(init) = variantDefs$CVariant
        colnames(init) = sampleNames
        variantForwCount = init
        totalForwCount = init
        variantRevCount = init
        totalRevCount = init

        # read all detections for all samples
        for(i in 1:num_samples){
            s_id=samples$SampleID[i]
            s_name=sampleNames[i]
            if(file.exists(file.path(dir_variants, s_id))){
                detections = dir(file.path(dir_variants, s_id), pattern=".txt$", ignore.case=FALSE)
                for(d in detections){
                    det = read.table(file.path(dir_variants, s_id, d), sep="\t",
                    	header=TRUE, 
			col.names=c("Def", "Det_Samp_Var", "canonicalVariant", 
			"forwardCount", "forwardDepth", "readType", "reverseCount", 
			"reverseDepth", "sample "),
			colClasses=c("character", "character",
                        "character", "numeric", "numeric", "numeric",
                        "numeric", "numeric", "character"),
			stringsAsFactors=FALSE)
		    if(nrow(det) > 0){
                    	det=subset(det, det$readType == 0)
                    	variantForwCount[det$canonicalVariant, s_name]=
                            det$forwardCount
                    	variantRevCount[det$canonicalVariant, s_name]=
                            det$reverseCount
                    	totalForwCount[det$canonicalVariant, s_name]=
                            det$forwardDepth
                    	totalRevCount[det$canonicalVariant, s_name]=
                            det$reverseDepth
		    }
                }
            } else {
                warning(paste("no variant data for ",s_name))
            }
        }
        return(list(variantForwCount, totalForwCount,
            variantRevCount, totalRevCount, featureData,
            featureMeta))
    }
)

          
# readReferenceSequences runs through the file
# ".../Amplicons/ProjectDef/ampliconsProject.txt" and reads the reference
# sequences
# output: refSeqAligned, an AlignedRead data frame with the reference sequences
# and additional information (name, id, gene)

setMethod("readReferenceSequences",

    signature(dir_projectDef="character"),
    function(dir_projectDef){
        # file has no data frame structure (inconsistent number of columns), so
        # read it row-wise
        pfLines=readLines(file.path(dir_projectDef, "ampliconsProject.txt"))

        # select lines with reference sequences (beginning with "RefSeq")
        refSeq_lines=grep("^RefSeq\t", pfLines, value=TRUE)

        # referenceSeqs above are in data frame format, so convert them to tables
        refSeq_con=textConnection(refSeq_lines)
        refSeq_tab=read.csv(refSeq_con, sep="\t", col.names=c("Def", "RefSeq",
            "annotation", "name", "sequence", "startPos"), header=FALSE,               
            colClasses=c("character", "character", "character", "character",
                "character", "numeric"), stringsAsFactors=FALSE)

        # build DNAStringSet for the reference sequence slot
        refSeqDNASet=DNAStringSet(refSeq_tab$sequence)
        names(refSeqDNASet)=refSeq_tab$RefSeq
        refSeqNames=refSeq_tab$name
        refSeqGenes=sapply(refSeqNames, function(x) strsplit(x, "_")[[1]][1] )
        refSeqData=data.frame(row.names=refSeqNames, name=refSeqNames,
            refSeqID=refSeq_tab$RefSeq, gene=refSeqGenes, stringsAsFactors=FALSE)
        refSeqAligned = AlignedRead(sread=refSeqDNASet, 
            id=BStringSet(refSeq_tab$RefSeq),
            position=as.integer(rep(1, length(refSeq_tab$RefSeq))),
            alignData=AlignedDataFrame(data=refSeqData,
                metadata=data.frame(row.names=colnames(refSeqData))))
        close(refSeq_con)
        return(refSeqAligned)
    }
)


# readAmplicons runs through the file
# ".../Amplicons/ProjectDef/ampliconsProject.txt" and reads feature- and assay
# data for each amplicon
# output: list containing
# numReadsForward, number of forward reads for a given amplicon
# numReadsReverse, number of reverse reads for a given amplicon
# featureData, for each amplicon the name, reference sequence, primer and
# end/start
# featureMeta, some useful information about the featureData

setMethod("readAmplicons",
    signature(dir_projectDef="character", dir_align="character", samples="data.frame"),
    function(dir_projectDef, dir_align, samples){

        # read the featureData for the amplicons (its name, reference sequence,
        # primer and end/start):
        # file has no data frame structure (inconsistent number of columns), so
        # read it row-wise
        pfLines=readLines(file.path(dir_projectDef, "ampliconsProject.txt"))

        # select lines with amplicons/primers (beginning with "Amp") 
        primer_lines=grep("^Amp\t", pfLines, value=TRUE)

        # amplicons above are in data frame format, so convert them to tables
        primer_con=textConnection(primer_lines)
        primer_tab=read.csv(		
            primer_con, sep="\t",
            col.names=c("Def", "Amp", "annotation", "name", "primer1", "primer2",
                "referenceSeq", "targetEnd", "targetStart"),
            header=FALSE,
            colClasses=c("character", "character", "character", "character",
                "character", "character", "character", "numeric", "numeric"),
	    stringsAsFactors=FALSE)
        # filter the desired feature data
        featureData=data.frame(row.names=primer_tab$name,
            ampID=primer_tab$Amp, 
            primer1=primer_tab$primer1,
            primer2=primer_tab$primer2,
            referenceSeqID=primer_tab$referenceSeq,
            targetStart=primer_tab$targetStart,
            targetEnd=primer_tab$targetEnd,
            stringsAsFactors=FALSE)

        # construct meta data for the features
        featureMeta=data.frame(row.names=colnames(featureData),
            labelDescription=c("-", "-", "-", "-", "-", "-"))

        close(primer_con)

        # count the amplicons for each sample
        refSeqsInd=grep("^RefSeq", pfLines)
        refSeqSplit=strsplit(pfLines[refSeqsInd], "\t")
        refSeqs=data.frame(
            RefSeqID=sapply(refSeqSplit, function(x) {x[2]}),
            RefSeqName=sapply(refSeqSplit, function(x) {x[4]}))

	## initialize assay data consisting of forward and reverse coverage
	sampleNames = rownames(samples)
	ampNames = rownames(featureData)
	init = matrix(0, length(sampleNames), length(ampNames))
	init = matrix(0, length(ampNames), length(sampleNames))
	rownames(init) = ampNames
	colnames(init) = sampleNames
        numReadsForward = init
        numReadsReverse = init

	## create named vector for fast connection between an ampid and its reference
	ampid2ref = rownames(featureData)
	names(ampid2ref) = featureData$ampID

	## read assay data
        for (s in 1:nrow(samples)) {
	    sampleName = sampleNames[s]
            thisSampleDir=file.path(dir_align, samples$SampleID[s])
            for (r in refSeqs$RefSeqID) {
		
                thisRefDir=file.path(thisSampleDir, r)
                alignFile=file.path(thisRefDir, paste(samples$SampleID[s],
                    "_vs_", r, ".ampIds.txt", sep=""))

                if (file.exists(alignFile)) {
                    seqCounts = read.table(file=alignFile, sep="\t", col.names=c("ampId", "forwardCount", "reverseCount"), stringsAsFactors=FALSE)
		    numReadsForward[ampid2ref[seqCounts$ampId], sampleName] = seqCounts[,"forwardCount"]
		    numReadsReverse[ampid2ref[seqCounts$ampId], sampleName] = seqCounts[,"reverseCount"]
                } 
            }
	}
        return(list(numReadsForward,numReadsReverse,featureData,featureMeta))	
    }
)


#################################################################################
########## ENSEMBL METHODS ######################################################
#################################################################################

setMethod("readAmpliconPositions",
    signature(object="AVASet", dnaSet="DNAStringSet", genes="character"), 
    function (object, dnaSet, genes = "", dataset = "hsapiens_gene_ensembl") {
        upDownStream = max(width(dnaSet))
        ensembl = useMart("ensembl", dataset = dataset)
        geneInfo = getBM(attributes = c("hgnc_symbol", "ensembl_gene_id", 
            "start_position", "end_position", "strand", "chromosome_name"), 
            filters = "hgnc_symbol", values = unique(genes), mart = ensembl)

        geneSeqs = getSequence(id=geneInfo$ensembl_gene_id, type="ensembl_gene_id", 
            seqType="gene_exon_intron", mart=ensembl)
    
        upSeqs = getSequence(id=geneInfo$ensembl_gene_id, type="ensembl_gene_id", 
            seqType="gene_exon_intron", upstream=upDownStream, mart=ensembl)
        upSeqs = upSeqs[match(geneSeqs$ensembl_gene_id, upSeqs$ensembl_gene_id), ]
        upSeqs$gene_exon_intron = gsub(geneSeqs$gene_exon_intron, "", upSeqs$gene_exon_intron, fixed=TRUE)
    
        downSeqs = getSequence(id=geneInfo$ensembl_gene_id, type="ensembl_gene_id", 
            seqType="gene_exon_intron", downstream=upDownStream, mart=ensembl)    
        downSeqs = downSeqs[match(geneSeqs$ensembl_gene_id, downSeqs$ensembl_gene_id), ]
        downSeqs$gene_exon_intron = gsub(geneSeqs$gene_exon_intron, "", downSeqs$gene_exon_intron, fixed=TRUE)
        
        gSeqs = DNAStringSet(paste(upSeqs$gene_exon_intron,
            geneSeqs$gene_exon_intron, downSeqs$gene_exon_intron, sep=""))
        names(gSeqs) = geneSeqs$ensembl_gene_id

        numSeqs = length(dnaSet)

	numAllSeqs = length(sread(referenceSequences(object)))
        pos = rep(1, numAllSeqs)
        chr = rep(NA, numAllSeqs)
        strand = factor(rep(NA, numAllSeqs), levels=c("+", "-"))
        ensemblId = rep(NA, numAllSeqs)
	names(pos) = names(sread(referenceSequences(object)))
	names(chr) = names(sread(referenceSequences(object)))
	names(strand) = names(sread(referenceSequences(object)))
	names(ensemblId) = names(sread(referenceSequences(object)))
        for (i in 1:numSeqs) {
  	    refName = names(dnaSet)[i]
            amp = dnaSet[[i]]
            ind = geneInfo$hgnc_symbol == genes[i]
            gene = gSeqs[[geneInfo[ind, "ensembl_gene_id"]]]
        
            m = matchPattern(amp, gene)
            if (length(m) != 1) {
                stop(paste("Could not match amplicon:", names(dnaSet)[i]))
            }
            pos[refName] = geneInfo$start_position[ind] - upDownStream + start(m) - 1
            chr[refName] = geneInfo$chromosome_name[ind]
            ensemblId[refName] = geneInfo$ensembl_gene_id[ind]
            if (geneInfo$strand[ind] == 1) {
                strand[refName] = "+"
            } else {
                strand[refName] = "-"
            }
        }
        newAlignedDataFrame = data.frame(pData(alignData(referenceSequences(object))), 
            ensemblId = ensemblId)
        newAlignedData = AlignedRead(sread = sread(referenceSequences(object)), 
            id=id(referenceSequences(object)), chromosome=chr, strand=strand,
            position = as.integer(pos), alignData = AlignedDataFrame(data = newAlignedDataFrame, 
            metadata = data.frame(row.names = colnames(newAlignedDataFrame))))
        referenceSequences(object) = newAlignedData
        return(object)
    }
)


#################################################################################
########## DISPLAY, GETTER AND SETTER METHODS ###################################
#################################################################################


# this version of show adds some lines about the amplicons to the usual output
# from the eSet version

setMethod("show",
    signature(object="AVASet"),
    function(object){
        if(sum(variantFilterPerc(object)) == 0) {
            message("Variants: ")
        } else {
            message("Variants: ", 
		paste("(", names(variantFilterPerc(object))," filter = ", variantFilterPerc(object),")", sep=""))
        }
        callNextMethod(object)
        message("\nAmplicons: ")
        message("assayDataAmp:", paste(nrow(assayDataAmp(object)$forwCount), "features, ",
            ncol(assayDataAmp(object)$forwCount), "samples"))
        message("  element names:", assayDataElementNames(assayDataAmp(object)))  
        message("featureDataAmp: ")
        show(object@featureDataAmp)
        message("\nReference sequences: ")
        show(referenceSequences(object))
    }
)

# these versions of featureData and fData return the filtered featureData of the AVASet if the
# filter is set to a value > 0; otherwise it calls the original Biobase version

setMethod("fData",
    signature(object="AVASet"),
    function(object){
        if(sum(variantFilterPerc(object)) > 0) {
            #filter the featureData
            return ( subset( object@featureData@data, rownames(object@featureData@data) %in% variantFilter(object) ) )
	} else {
            callNextMethod(object)
	}
    }
)

setMethod("featureData",
    signature(object="AVASet"),
    function(object){
        if(sum(variantFilterPerc(object)) > 0) {
            #filter the featureData
            return ( subset( object@featureData, rownames(object@featureData@data) %in% variantFilter(object) ) )
	} else {
            callNextMethod(object)
	}
    }
)


# this version of assayData returns the filtered assayData of the AVASet if the
# filter is set to a value > 0; otherwise it calls the original Biobase version

setMethod("assayData",
    signature(object="AVASet"),
    function(object){
        if(sum(variantFilterPerc(object)) > 0){
            # filter the assayData
            filteredForwardCount=subset(object@assayData[[1]], rownames(object@assayData[[1]]) %in% variantFilter(object))
            filteredForwardDepth=subset(object@assayData[[2]], rownames(object@assayData[[2]]) %in% variantFilter(object))
            filteredReverseCount=subset(object@assayData[[3]], rownames(object@assayData[[3]]) %in% variantFilter(object))
            filteredReverseDepth=subset(object@assayData[[4]], rownames(object@assayData[[4]]) %in% variantFilter(object))
            return(assayDataNew("list",
                variantForwCount=filteredForwardCount,
                totalForwCount=filteredForwardDepth,
                variantRevCount=filteredReverseCount,
                totalRevCount=filteredReverseDepth))
        } else {
            callNextMethod(object)
        }
    }
)

# extend the Biobase version of the "["-method to the addditional amplicon slot

setMethod("[",
    signature("AVASet"),
    function(x, i, j, ..., drop=FALSE){
        # change the assayData and featureData of the additional amplicon slot
        if(missing(drop)) {
            drop=FALSE
        }
        if(missing(i) && missing(j)) {
            if(length(list(...)) != 0) { 
                stop("specify genes or samples to subset; use '", substitute(x),
                    "$", names(list(...))[[1]],
                    "' to access phenoData variables")
            }
            return(x)
        }

        # call original BioBase method to change variant data
        # change amplicon assayData manually
        orig=assayDataAmp(x)
        if(missing(i)) {
            x=callNextMethod(x, , j, drop=drop)
            orig=lapply(orig, function(obj) obj[ , j, ..., drop = drop])
        } else {
            if (missing(j)) {
                # call original BioBase method to change variant data
                x=callNextMethod(x, i, , drop=drop) 
            } else { 
                # call original BioBase method to change variant data
                x=callNextMethod(x,i,j,drop=drop)
                orig=lapply(orig, function(obj) obj[ , j, ..., drop = drop])
            }
        }
        assayDataAmp(x)=orig
        return(x)
    }
)

## provide another subset method for samples, variants AND amplicons ([] only works for
## samples and variants)
setMethod("subset",
    signature("AVASet"),
          
    function(x, subset, dimension="amplicons"){

      object = x
      if(!missing(subset)){

          if(dimension=="amplicons"){

            ## catch some bad input
            if(is.vector(subset, mode="character")){
             	if(!all(subset %in% rownames(fDataAmp(object)))){
                   stop("invalid argument: given amplicons not found")
             	}
            }else{
                if(is.vector(subset, mode="numeric")){
                    if(!all(subset %in% 1:nrow(fDataAmp(object)))){
                        stop("invalid argument: given indices out of range")
                    }
                }else{
                    if(is.vector(subset, mode="logical")){
                        if(length(subset) != nrow(fDataAmp(object))){
                            stop("invalid argument: given subset is of wrong length")
                        }
                    }else
                        stop("invalid argument: given subset of incorrect type")
                }
            }                  

            ## removing amplicons might affect some variants and samples:

            amps = fDataAmp(object)
            vars = fData(object)
            remainingAmps = amps[subset, ]

            ## remove variants from feature data
            remainingVars = vector(mode="character")
            i = 1
            ## Todo: substitute for loop by apply command
            for(v in rownames(vars)){
              var = vars[v, ]
              amps_var = subset(remainingAmps, referenceSeqID == var$referenceSeqID)
              ## in case of one reference for more than one amplicon: test if variant lies within amplicon range
              if(any((var$start >= amps_var$targetStart) & (var$end <= amps_var$targetEnd))){
                remainingVars[i] = v
                i = i + 1
              }
            }
            newFeatureDataAmp = new("AnnotatedDataFrame", data=remainingAmps,
              varMetadata=varMetadata(featureDataAmp(object)))
            newFeatureData = new("AnnotatedDataFrame", data=fData(object)[remainingVars, ],
              varMetadata=varMetadata(featureData(object)))

            ## remove samples/amplicons/variants from assayData
            remainingAssayAmp1 = assayDataAmp(object)$forwCount[rownames(remainingAmps), , drop=FALSE]
            remainingAssayAmp2 = assayDataAmp(object)$revCount[rownames(remainingAmps), , drop=FALSE]
            idx1 = !apply(remainingAssayAmp1, 2, function(col) all(col == 0))
            idx2 = !apply(remainingAssayAmp2, 2, function(col) all(col == 0))
            idx = idx1 | idx2
            remainingSamples = colnames(remainingAssayAmp1)

            newAssayDataAmp=assayDataNew("list",
              forwCount=remainingAssayAmp1[, idx, drop=FALSE],
              revCount=remainingAssayAmp2[, idx, drop=FALSE])
            newAssayData=assayDataNew("list",
              variantForwCount=assayData(object)$variantForwCount[remainingVars, remainingSamples, drop=FALSE],
              totalForwCount=assayData(object)$totalForwCount[remainingVars, remainingSamples, drop=FALSE],
              variantRevCount=assayData(object)$variantRevCount[remainingVars, remainingSamples, drop=FALSE],
              totalRevCount=assayData(object)$totalRevCount[remainingVars, remainingSamples, drop=FALSE])

            ##remove samples from pheno data
            newPhenoData=new("AnnotatedDataFrame", data=pData(object)[remainingSamples, ],
              varMetadata=varMetadata(phenoData(object)))

            ## ubdate object
            assayData(object) = newAssayData
            assayDataAmp(object) = newAssayDataAmp
            featureData(object) = newFeatureData
            featureDataAmp(object) = newFeatureDataAmp
            phenoData(object) = newPhenoData

            return(object)

          }else{
            if(dimension=="variants"){
              ## catch some bad input
              if(is.vector(subset, mode="character")){
             	  if(!all(subset %in% rownames(fData(object)))){
                     stop("invalid argument: given variants not found")
             	  }
              }else{
                  if(is.vector(subset, mode="numeric")){
                      if(!all(subset %in% 1:nrow(fData(object)))){
                          stop("invalid argument: given indices out of range")
                      }
                  }else{
                      if(is.vector(subset, mode="logical")){
                          if(length(subset) != nrow(fData(object))){
                              stop("invalid argument: given subset is of wrong length")
                          }
                      }else
                          stop("invalid argument: given subset of incorrect type")
                  }
              }
              return(object[subset, ])
            }else{
              if(dimension=="samples"){
              ## catch some bad input
              if(is.vector(subset, mode="character")){
             	  if(!all(subset %in% rownames(pData(object)))){
                     stop("invalid argument: given samples not found")
             	  }
              }else{
                  if(is.vector(subset, mode="numeric")){
                      if(!all(subset %in% 1:nrow(pData(object)))){
                          stop("invalid argument: given indices out of range")
                      }
                  }else{
                      if(is.vector(subset, mode="logical")){
                          if(length(subset) != nrow(pData(object))){
                              stop("invalid argument: given subset is of wrong length")
                          }
                      }else
                          stop("invalid argument: given subset of incorrect type")
                  }
              }
                return(object[, subset])
              }else{
                stop("invalid argument: dimension can be either \"samples\", \"variants\" or \"amplicons\"")
              }
            }
          }
        }else{
          warning("no subset given; AVASet object left unchanged")
          return(object)
        }
    }
)

# fDataAmp/featureDataAmp returns the amplicon data from the featureData slot
# (same as fData for variants from the original Biobase eSet)

setMethod("fDataAmp",
    signature(object="AVASet"),
    function(object){
        return(pData(object@featureDataAmp))
    }
)

setMethod("featureDataAmp",
    signature(object="AVASet"),
    function(object){
        return(object@featureDataAmp)
    }
)

# assayDataAmp returns the amplicon data from the assayData slot (same as
# assayData for variants from the original Biobase eSet)

setMethod("assayDataAmp",
    signature(object="AVASet"),
    function(object){
        return(object@assayDataAmp)
    }
)

setMethod("referenceSequences",
    signature(object="AVASet"),
    function(object){
        return(object@referenceSequences)
    }
)

setMethod("dirs",
    signature(object="AVASet"),
    function(object){
        return(object@dirs)
    }
)

setReplaceMethod("featureDataAmp",
    signature(object="AVASet", value="AnnotatedDataFrame"),
    function(object, value){
        object@featureDataAmp=value
        return(object)
    }
)

setReplaceMethod("assayDataAmp",
    signature(object="AVASet", value="AssayData"),
    function(object, value){
        object@assayDataAmp=value
        return(object)
    }
)

setReplaceMethod("referenceSequences",
    signature(object="AVASet", value="AlignedRead"),
    function(object, value){
        object@referenceSequences=value
        return(object)
    }
)

.genomeSequencerMIDs <- function() {
  
    mids = c(
        MID1 = "ACGAGTGCGT",
        MID2 = "ACGCTCGACA",
        MID3 = "AGACGCACTC",
        MID4 = "AGCACTGTAG",
        MID5 = "ATCAGACACG",
        MID6 = "ATATCGCGAG",
        MID7 = "CGTGTCTCTA",
        MID8 = "CTCGCGTGTC",
        MID9 = "TAGTATCAGC",
        MID10 = "TCTCTATGCG",
        MID11 = "TGATACGTCT",
        MID12 = "TACTGAGCTA",
        MID13 = "CATAGTAGTG",
        MID14 = "CGAGAGATAC"
    )

    midsSet = DNAStringSet(mids, use.names=TRUE)

    return(midsSet)
}


.genomeSequencerMIDs_character <- function(mid) {
    
    mids = genomeSequencerMIDs()
    if (all(is.element(mid, names(mids)))) {
        return(mids[is.element(names(mids), mid)])
    } else {
        stop(paste("Argument mid must be on of the following:",
            paste(names(mids), collapse=", ")))
    }
}


setMethod("genomeSequencerMIDs", signature=signature(mid="missing"),
    .genomeSequencerMIDs)

setMethod("genomeSequencerMIDs", signature=signature(mid="character"),
    .genomeSequencerMIDs_character)

########################################################################################################################
#######################################   Helper functions   ###########################################################
########################################################################################################################

.catchBadInput <- function(object, blocks, transcripts, sampleCols, minMut, dir, title){
  
    if(length(blocks) > 0)
        if(!is.vector(blocks, mode="character"))
	    stop("invalid argument: please specify a character vector of block names for each variant")
    if(length(transcripts) > 0)
        if(!is.vector(transcripts, mode="character"))
	    stop("invalid argument: please specify a character vector of containing Ensembl transcript-IDs")
    if(!is.vector(sampleCols, mode="character")){
	stop("invalid argument: lease specify a character vector of column names of the sample data (phenoData) of your
		AVASet/MapperSet object")}
    if(!is.numeric(minMut) | (minMut < 0) | (minMut > 100))
	stop("invalid argument: please specify a numeric vale for minMut between 0 and 100")
    if(!is.character(dir))
	stop("invalid argument: please specify your target directory as a string")
    if(!is.character(title))
        stop("invalid argument: title has to be of type character")

}

## sets one of the buttons ("Reference", "Sample" or "Quality") as missing and one as disabled, level for links from deeper folders
.createNavigation <- function(missing="", disabled="", level=c("", "", "")) {
	
	buttonBack = "<FORM><INPUT TYPE=\"button\" VALUE=\"Back\" onClick=\"history.go(-1);return true;\"></FORM>"

	if(missing == "Reference") {
		buttonReference = ""
	} else {
		buttonReference = paste("<FORM METHOD=\"LINK\" ACTION=\"",
			level[1], "index.html\"><INPUT TYPE=\"submit\" VALUE=\"Variant report by reference\"", 
			ifelse(disabled == "Reference", " disabled", ""), "></FORM>", sep="")
	}

	if(missing == "Sample") {
		buttonSample = ""
	} else {
		buttonSample = paste("<FORM METHOD=\"LINK\" ACTION=\"",
			level[2], "index_samples.html\"><INPUT TYPE=\"submit\" VALUE=\"Variant report by samples\"",
			ifelse(disabled == "Sample", " disabled", ""), "></FORM>", sep="")
	}

	if(missing == "Quality") {
		buttonQuality = ""
	} else {
		buttonQuality = paste("<FORM METHOD=\"LINK\" ACTION=\"",
			level[3], "qualityReport/qualityReport.html\"> <INPUT TYPE=\"submit\" VALUE=\"Quality report\"",
			ifelse(disabled == "Quality", " disabled", ""), "></FORM>", sep="")
	}
    
	return(
		paste("<table border=\"0\">
        <tr>
        <td>", buttonBack, "</td>
        <td>", buttonReference, "</td>
		<td>", buttonSample, "</td>
        <td>", buttonQuality, "</td>
        </tr>
        </table>", sep="")
	)

}

########################################################################################################################
#######################################   HTML report init for GSMSets   ###############################################
########################################################################################################################

.initHtmlReportGSMSet <- function(object, annot, blocks=c(), transcripts=c(), sampleCols, minMut=3, dir="HTMLReport", title="Summary"){

    ## catch some bad input and missing arguments
    if(missing(sampleCols))
        sampleCols = colnames(pData(object))
    .catchBadInput(object, blocks, transcripts, sampleCols, minMut, dir, title)
    if(missing(annot)){
        warning("no variant information specified: loading data from ensembl
            for all variants")
        annot=annotateVariants(object)
    }

    annot = annotatedVariants(annot)
    if(!is.null(blocks) & length(blocks) != length(annot)){
	warning("argument of illegal size: blocks were ommitted")
	blocks = NULL
    }

    vars_ID = intersect(names(annot), rownames(fData(object)))
    vars_withoutAnnot = setdiff(rownames(fData(object)), names(annot))
    
    ## check if annotated variants and the feature data of the AVASet are consistent
    if(length(vars_ID) == 0)
      stop("Variant annotations are not consistent with the AVASet: variant names in annotations differ from the names in the AVASet")
    else{
    	object = object[vars_ID, ]
	annot = annot[vars_ID]
    }
    if(length(vars_withoutAnnot) > 0)
	warning(length(vars_withoutAnnot), " variants lack of annotation and were omitted: ", paste(vars_withoutAnnot, collapse=", "))

    ## sample data
    variantPercs = round(getVariantPercentages(object, direction="both") * 100, digits=2)
    variantPercsForward = round(getVariantPercentages(object, direction="forward") * 100, digits=2)
    variantPercsReverse = round(getVariantPercentages(object, direction="reverse") * 100, digits=2)

    numReads = assayData(object)$totalForwCount + assayData(object)$totalRevCount
    numReadsMut = assayData(object)$variantForwCount + assayData(object)$variantRevCount
    numReadsForward = assayData(object)$totalForwCount
    numReadsMutForward = assayData(object)$variantForwCount
    numReadsReverse = assayData(object)$totalRevCount
    numReadsMutReverse = assayData(object)$variantRevCount

    ## report the reference gene (with exon information (from the
    ## referenceSequences-slot of the object))
    references_short = rep(NA, length(vars_ID))
    references = rep(NA, length(vars_ID))

    ## report maximum variant percentage
    max_perc=apply(variantPercs, 1, max)[vars_ID]

    ## report the variant name
    vars = rownames(fData(object)[vars_ID, ])
    names(vars) = vars_ID

    ## report base in variant and reference sequence and its start and end
    refSeq_base = fData(object)[vars_ID, ]$referenceBases
    ## only show deletions, that have max 5 symbols
    refSeq_base[nchar(refSeq_base) > 5]  = "-"
    var_base = fData(object)[vars_ID,]$variantBase
    ## only show deletions, that have max 5 symbols
    var_base[nchar(var_base) > 5]  = "-"
    seq_start=  fData(object)[vars_ID, ]$start
    seq_end = fData(object)[vars_ID, ]$end

    ## report absolute variant position in genome
    chromosome = fData(object)[vars_ID, "chr"]
    abs_start = fData(object)[vars_ID, "start"]
    abs_end = fData(object)[vars_ID, "end"]

    ## report type of variant (deletion, insertion point)
    mutation_type = vector(mode="character")
    for(i in vars_ID){
 	## insertion
	if((substr(fData(object)[i, "referenceBases"],1,1) == "-") | 
	    (nchar(fData(object)[i, "referenceBases"]) < nchar(fData(object)[i, "variantBase"]))
	) 
	    mutation_type[i] = "insertion"
	## deletion
	else 
	    if((substr(fData(object)[i, "variantBase"],1,1) == "-") | 
	    	(nchar(fData(object)[i, "referenceBases"]) > nchar(fData(object)[i, "variantBase"]))
	    ) 
		mutation_type[i] = "deletion"
	## point mutation
	    else mutation_type[i] = "point"
    }
    mutation_type = mutation_type[vars_ID]

    ## report change of amino acid
    as_change=as.character(sapply(annot, function(x) {
        if (nrow(x$exon) == 0) {
            return(FALSE)
        } else {
            aaChanges=x$exons$coding &
                ((x$exons$AminoRef != x$exons$AminoMut) |
                is.na(x$exons$AminoRef))
            return(sum(aaChanges, na.rm=TRUE) > 0)
        }
    }))
    as_change=sapply(as_change, function(x) if(x == "TRUE") x="yes"
        else x="no")

    ## report if variant is well known
    snps=sapply(annot, function(x)
        if(ncol(x$snps) > 0) {
	    paste(
            	paste("<a href=\"http://www.ensembl.org/Homo_sapiens/Variation/",
                     "Summary?source=dbSNP;v=", x$snps$refsnp_id,
                     "\" target=\"_blank\">", x$snps$refsnp_id, "</a>", sep=""),
		collapse=",")
        } else {
          "-"
        }
    )

    ## titles for the transcript pages
    genes = sapply(annot, function(x) {return(as.character(x$genes$external_gene_id[1]))})

    transTitles = vector(mode="character")    
    transTitles[vars_ID] = paste("Gene: ", genes[vars_ID], " - Variant: ", vars[vars_ID])

    ## build html report
    .htmlReport(vars_ID, variantPercs, variantPercsForward, variantPercsReverse, numReads, numReadsMut, 
	numReadsForward, numReadsMutForward, numReadsReverse, numReadsMutReverse,  max_perc, references, references_short, 
	vars, refSeq_base, var_base, chromosome, abs_start, abs_end, mutation_type, as_change, snps, transTitles,
	object, annot, blocks, transcripts, sampleCols, minMut, dir, title)
}

########################################################################################################################
#######################################   HTML report init for AVASets   ##############################################
########################################################################################################################

.initHtmlReportAVASet <- function(object, annot, blocks=c(), transcripts=c(), sampleCols, minMut=3, dir="HTMLReport", title="Summary"){
  
    ## catch some bad input and missing arguments
    if(missing(sampleCols))
        sampleCols = colnames(pData(object))

    .catchBadInput(object, blocks, transcripts, sampleCols, minMut, dir, title)
    if(missing(annot)){
        message("No variant information specified: loading data from ensembl for all variants")
        annot=annotateVariants(object)
    }

    annot = annotatedVariants(annot)
    if(!is.null(blocks) & length(blocks) != length(annot)){
	warning("argument of illegal size: blocks were ommitted")
	blocks = NULL
    }

    vars_ID = intersect(names(annot), rownames(fData(object)))
    vars_withoutAnnot = setdiff(rownames(fData(object)), names(annot))
    
    ## check if annotated variants and the feature data of the AVASet are consistent
    if(length(vars_ID) == 0)
      stop("Variant annotations are not consistent with the AVASet: variant names in annotations differ from the names in the AVASet")
    else{
    	object = object[vars_ID, ]
	annot = annot[vars_ID]
    }
    if(length(vars_withoutAnnot) > 0)
	warning(length(vars_withoutAnnot), " variants lack of annotation and were omitted: ", paste(vars_withoutAnnot, collapse=", "))
    
    ## sample data
    variantPercs = round(getVariantPercentages(object, direction="both") * 100, digits=2)
    variantPercsForward = round(getVariantPercentages(object, direction="forward") * 100, digits=2)
    variantPercsReverse = round(getVariantPercentages(object, direction="reverse") * 100, digits=2)

    numReads = assayData(object)$totalForwCount + assayData(object)$totalRevCount
    numReadsMut = assayData(object)$variantForwCount + assayData(object)$variantRevCount
    numReadsForward = assayData(object)$totalForwCount
    numReadsMutForward = assayData(object)$variantForwCount
    numReadsReverse = assayData(object)$totalRevCount
    numReadsMutReverse = assayData(object)$variantRevCount

    ## report the reference gene (with exon information (from the
    ## referenceSequences-slot of the object))
    references_short = sapply(annot, function(x) x$genes$external_gene_id)
    references=sapply(fData(object)[vars_ID, ]$referenceSeqID,
        function(x) subset(pData(alignData(referenceSequences(object))),
            pData(alignData(referenceSequences(object)))$refSeqID == x)$name)
    names(references) = vars_ID

    ## report maximum variant percentage
    max_perc=apply(variantPercs, 1, max)[vars_ID]

    ## report the variant name
    vars=fData(object)[vars_ID, ]$name
    names(vars)=vars_ID

    ## report base in variant and reference sequence and its start and end
    refSeq_base=fData(object)[vars_ID, ]$referenceBases
    ## only show deletions, that have max 5 symbols
    refSeq_base[nchar(refSeq_base) > 5]  = "-"
    var_base = fData(object)[vars_ID,]$variantBase
    ## only show deletions, that have max 5 symbols
    var_base[nchar(var_base) > 5]  = "-"
    seq_start=fData(object)[vars_ID, ]$start
    seq_end=fData(object)[vars_ID, ]$end

    ## report absolute variant position in genome
    refSeqInd=match(fData(object)[vars_ID, ]$referenceSeqID,
        as.character(id(referenceSequences(object))))
    chromosome=as.character(
        chromosome(referenceSequences(object)[refSeqInd]))
    posStrand = strand(referenceSequences(object))[refSeqInd] == "+"
    abs_start = numeric(length(chromosome))
    abs_end = numeric(length(chromosome))
    abs_start[posStrand] = position(referenceSequences(object)[refSeqInd])[posStrand] + seq_start[posStrand] - 1
    abs_end[posStrand] = position(referenceSequences(object)[refSeqInd])[posStrand] + seq_end[posStrand] - 1
    endRefSeqs = position(referenceSequences(object)[refSeqInd]) + width(sread(referenceSequences(object)[refSeqInd])) - 1
    abs_start[!posStrand] = endRefSeqs[!posStrand] - seq_end[!posStrand] + 1
    abs_end[!posStrand] = endRefSeqs[!posStrand] - seq_start[!posStrand] + 1
    rm(refSeqInd)

    ## report type of variant (deletion, insertion point)
    mutation_type=sapply(fData(object)[vars_ID, ]$canonicalPattern,
        function(x) {substr(x, 1, 1)})
    mutation_type[mutation_type == "s"]="point"
    mutation_type[mutation_type == "d"]="deletion"
    mutation_type[mutation_type == "i"]="insertion"

    ## report change of amino acid
    as_change=as.character(sapply(annot, function(x) {
        if (nrow(x$exons) == 0) {
            return(FALSE)
        } else {
            aaChanges=x$exons$coding &
                ((x$exons$AminoRef != x$exons$AminoMut) |
                is.na(x$exons$AminoRef))
            return(sum(aaChanges, na.rm=TRUE) > 0)
        }
    }))
    as_change=sapply(as_change, function(x) if(x == "TRUE") x="yes"
        else x="no")

    ## report if variant is well known
    snps=sapply(annot, function(x)
        if(ncol(x$snps) > 0) {
	    paste(
            	paste("<a href=\"http://www.ensembl.org/Homo_sapiens/Variation/",
                     "Summary?source=dbSNP;v=", x$snps$refsnp_id,
                     "\" target=\"_blank\">", x$snps$refsnp_id, "</a>", sep=""),
		collapse=",")
        } else {
          "-"
        }

    )

    ## titles for the transcript pages
    transTitles = vector(mode="character")
    transTitles[vars_ID] = paste("Reference: ", references[vars_ID], " - Variant: ", vars[vars_ID])

    ## build html report
    .htmlReport(vars_ID, variantPercs, variantPercsForward, variantPercsReverse, numReads, numReadsMut, 
	numReadsForward, numReadsMutForward, numReadsReverse, numReadsMutReverse,  max_perc, references, references_short, 
	vars, refSeq_base, var_base, chromosome, abs_start, abs_end, mutation_type, as_change, snps, transTitles,
	object, annot, blocks, transcripts, sampleCols, minMut, dir, title)
}


########################################################################################################################
#######################################   HTML REPORT   ################################################################
########################################################################################################################
## The report is structured into two (MapperSet) or three (AVASet) parts containing variant and quality 
## information:
## (1) The main page sums up given variant information like the name, type, reference gene, position.
##     Using the argument "blocks", the main page can be individually structured by assigning a block name to 
##     each variant. 
##     The main page can be further structured by samples. For a given AVASet object, every sample links to 
##     another short quality report showing only the amplicon coverage for this sample.
## (2) Every variant on the main page links to a page with further details about the affected genes and transcripts
##     (e.g. Ensembl gene-IDs, transcript-IDs, codon sequences, changes of amino acids (if coding)).
## (3) Only in case of AVASet object: A quality report shows the coverage of every amplicon in forward and/or 
##     reverse direction. Further plots display the coverage by MID and PTP (if this information is given in the 
##     pheno data of the object).
########################################################################################################################

.htmlReport <- function(vars_ID, variantPercs, variantPercsForward, variantPercsReverse, numReads, numReadsMut,
        numReadsForward, numReadsMutForward, numReadsReverse, numReadsMutReverse,  max_perc, references, references_short,
        vars, refSeq_base, var_base, chromosome, abs_start, abs_end, mutation_type, as_change, snps, transTitles,
        object, annot, blocks, transcripts, sampleCols, minMut, dir, title){

    message("Creating variant report ... ", appendLF = FALSE)

    ## create directories
    if(!(file.exists(dir)))
        dir.create(dir, recursive=TRUE)
    if(!(file.exists(file.path(dir,"trans"))))
        dir.create(file.path(dir,"trans"), recursive=TRUE)
    if(!(file.exists(file.path(dir,"qualityReport"))))
        dir.create(file.path(dir,"qualityReport"), recursive=TRUE)

    ## load style sheet
    cssFile=file.path(find.package("R2HTML"), "samples/R2HTML.css")
    s=file.copy(cssFile, file.path(dir, "R2HTML.css"), overwrite=TRUE)
    ## load java script for sorting tables
    jsFile=file.path(path.package("R453Plus1Toolbox"),
        "javascript/sorttable.js")
    js=file.copy(jsFile, file.path(dir, "sorttable.js"), overwrite=TRUE)


    ## create a html file for every transcript
	mutation_samples = vector(mode="character", length=length(vars_ID))
	names(mutation_samples) = vars_ID

    aminos = getAminoAbbr()
    trans_links = rep("", length(vars_ID))
    names(trans_links) = vars_ID

    for(v in vars_ID){
        var=annot[[v]]
        num_trans=length(var$transcripts$ensembl_transcript_id)
	    if(num_trans == 0)
		  trans_links[v]="not available"
	    else{
            transHTML=HTMLInitFile(file.path(dir,"trans"),
                filename=paste("trans",v,sep=""), extension="html",
                Title="Transcript Summary", CSSFile="../R2HTML.css")
            cat("<script src=\"../sorttable.js\"></script>", file=transHTML, append=TRUE)
            HTML.title(transTitles[v], HR=1, file=transHTML)
            ## add buttons
			if(class(object) == "AVASet") {
				cat(.createNavigation(level=c("../", "../", "../")), file=transHTML, append=TRUE)
			} else {
				cat(.createNavigation(missing="Quality", level=c("../", "../", "../")), file=transHTML, append=TRUE)
			}

           	## for the links in the summary page
            trans_links[v]=paste("<a href=\"trans/trans", v,
                ".html\">transcripts</a>", sep="")

            ## special case: all variants on introns (no exon data)
            if(ncol(var$exons) == 0){
                introns = rep(TRUE, num_trans)
                report_trans = data.frame(
                    row.names= seq(1, num_trans),
                    GeneID=paste(
                        c(rep("<a href=\"http://www.ensembl.org/Homo_sapiens/Gene/Summary?g=", num_trans)),
                        var$transcripts$ensembl_gene_id,
                        rep("\" target=\"_blank\">",num_trans),
                        var$transcripts$ensembl_gene_id,
                        rep("</a>", num_trans), sep=""),
		    	    TranscriptID_tmp = var$transcripts$ensembl_transcript_id,
                    TranscriptID=paste(c(
                        rep("<a href=\"http://www.ensembl.org/Homo_sapiens/Transcript/Summary?t=", num_trans)),
                        var$transcripts$ensembl_transcript_id,
                        rep("\" target=\"_blank\">",num_trans),
                        var$transcripts$ensembl_transcript_id,
                        rep("</a>", num_trans), sep=""),
                    Coding="",
                    UTR5="",
                    UTR3="",
                    ExonRank="",
                    Codon="",
                    RefCodonSeq="",
                    MutCodonSeq="",
                    RefAmino="",
                    RefAminoLong="",
                    MutAmino="",
                    MutAminoLong="",
                    stringsAsFactors=FALSE
                )       
            } else { ## usual case: some/all variants on exons 
                var=merge.data.frame(var$transcripts, var$exons,
                    by="ensembl_transcript_id", all=TRUE)
                    num_trans=length(var$ensembl_transcript_id) # update number of transcript in case of more exons than transcripts
                    var$space.y=factor(var$space.y, 
                        levels=c(levels(var$space.y),"")
                )
                var$ensembl_gene_id.y = factor(var$ensembl_gene_id.y,
                  	levels=c(levels(var$ensembl_gene_id.y), ""))
                var$ensembl_exon_id = factor(var$ensembl_exon_id,
                   	levels=c(levels(var$ensembl_exon_id), ""))
                var[is.na(var)]=""
                introns=var$rank == ""
                var$numCodon=paste(var$numCodonStart, var$numCodonEnd, sep="-")
                var$numCodon[var$numCodonStart == var$numCodonEnd]=
                    as.character(var$numCodonStart[var$numCodonStart == var$numCodonEnd])
                ################# Detect Deletions
                ind=nchar(var$AminoRef) == 1 & nchar(var$AminoMut) == 1
                refAminoLong = rep("", length(ind))
                mutAminoLong = rep("", length(ind))
                refAminoLong[ind] = aminos[var$AminoRef[ind]]
                mutAminoLong[ind] = aminos[var$AminoMut[ind]]
                #################
                report_trans = data.frame(
                   	row.names=seq(1,num_trans), 
                   	GeneID=paste(c(rep("<a href=\"http://www.ensembl.org/Homo_sapiens/Gene/Summary?g=", num_trans)),
                        var$ensembl_gene_id.x,
                        rep("\" target=\"_blank\">",num_trans),
                        var$ensembl_gene_id.x, rep("</a>", num_trans), sep=""),
		    	    TranscriptID_tmp = var$ensembl_transcript_id,
                    TranscriptID = paste(c(rep("<a href=\"http://www.ensembl.org/Homo_sapiens/Transcript/Summary?t=", num_trans)), 
			        var$ensembl_transcript_id, rep("\" target=\"_blank\">",num_trans), 
			        var$ensembl_transcript_id, rep("</a>", num_trans), sep=""),
                    Coding=var$coding,
                    UTR5=var$utr_5,
                    UTR3=var$utr_3,
                    ExonRank=paste(c(rep("<a href=\"http://www.ensembl.org/Homo_sapiens/Transcript/Exons?db=core;g=", num_trans)), 
			        var$ensembl_gene_id.x, rep(";t=" ,num_trans), 
			        var$ensembl_transcript_id,rep("\" target=\"_blank\">",num_trans), var$rank, rep("</a>", num_trans), sep=""),
                    Codon=var$numCodon,
                    RefCodonSeq=var$codonRef,
                    MutCodonSeq=var$codonMut,
                    RefAmino=var$AminoRef,
                    RefAminoLong=refAminoLong,
                    MutAmino=var$AminoMut,
                    MutAminoLong=mutAminoLong,
                    stringsAsFactors=FALSE
                )		
            }

            report_trans[introns,"ExonRank"]="Intron"

            colnames(report_trans)[5]="5\'UTR"
            colnames(report_trans)[6]="3\'UTR"

    	    ## order transcript data frame according to a given list of transcript-IDs
    	    varTranscripts = intersect(transcripts, report_trans$TranscriptID_tmp)
            transOrder = which(report_trans$TranscriptID_tmp %in% varTranscripts)
            transOrder = c(transOrder, setdiff(1:num_trans, transOrder))
	    report_trans = report_trans[transOrder, ]
	    rownames(report_trans) = 1:num_trans
	    report_trans$TranscriptID_tmp = NULL

	    ## create HTML-file
            HTML(report_trans, file=transHTML, classtable="sortable")
			
            ## additional table for the sample data
            ## only use valid sample columns
            sampleCols = sampleCols[sampleCols %in% colnames(pData(object))]
            samples = data.frame(pData(object)[, sampleCols])

	    varFreq = t(variantPercs[v, ,drop =FALSE])
	    varFreqForward = t(variantPercsForward[v, ,drop =FALSE])
	    varFreqReverse = t(variantPercsReverse[v, ,drop =FALSE])

            samples$varFreq = varFreq
            samples$varFreqForward = varFreqForward
            samples$varFreqReverse = varFreqReverse	
            samples$Reads = t(numReads[v, , drop=FALSE])
            samples$ReadsMut = paste(t(numReadsMut[v, , drop=FALSE]), " (", varFreq, "%)", sep="") 
            samples$ReadsForward = t(numReadsForward[v, , drop=FALSE])
            samples$ReadsMutForward = paste(t(numReadsMutForward[v, , drop=FALSE]), " (", varFreqForward, "%)", sep="")
            samples$ReadsReverse = t(numReadsReverse[v, , drop=FALSE])
            samples$ReadsMutReverse = paste(t(numReadsMutReverse[v, , drop=FALSE]), " (", varFreqReverse, "%)", sep="")
            colnames(samples) = c(sampleCols, "VarFreq", "VarFreqForward", "VarFreqReverse",  
                "DepthTotal", "VarReads", "DepthForward", "VarReadsForward", "DepthReverse", "VarReadsReverse")
                
            ## for a AVASet object, every sample entry links to its quality report
            sampleNames = rownames(pData(object))
            sampleNums = 1:length(rownames(pData(object)))
            if(class(object) == "AVASet")
              rownames(samples) = paste("<a href=\"../qualityReport/", sampleNums, "/coverageSample_", sampleNums, ".html\">",sampleNames , "</a>", sep="")
            else
                rownames(samples) = sampleNames

            HTML(samples[, c(sampleCols, "DepthTotal", "VarReads", "DepthForward", "VarReadsForward", "DepthReverse", "VarReadsReverse")], 
                file=transHTML, classtable="sortable")
            HTMLEndFile(file=transHTML)

            ## compute column entries concerning samples having mutations in more than minMut% of the reads (for main table; see below)
            rownames(samples) = sampleNames
            mutation_samples[v] = toString( rownames(subset(samples, VarFreqForward >= minMut & VarFreqReverse >= minMut)) )
            if(mutation_samples[v] == "")
                mutation_samples[v] = "-"
	    }
    }


	## MAIN HTML 1 (no special structure or structured by given blocks)
        ## create main html file with gene overview and variant summary
	titleSummary = "Amplicon Summary"
        geneHTML = HTMLInitFile(dir, filename="index", extension="html",
            Title = titleSummary, CSSFile="R2HTML.css")
       	cat("<script src=\"sorttable.js\"></script>", file=geneHTML,
            append=TRUE)
        HTML.title(title, HR=1, file=geneHTML)
	## link to a page that structures the data by samples (see below)
	if(class(object) == "AVASet") {
		cat(.createNavigation(disabled="Reference"), file=geneHTML, append=TRUE)
	} else {
		cat(.createNavigation(missing="Quality", disabled="Reference"), file=geneHTML, append=TRUE)
	}
        ## create data frame to build a html table
        report_summary = data.frame( 
	        row.names = 1:length(references),
            Reference = references,
            Reference_tmp = as.character(references_short),
            Variant = vars,
            RefSeq = refSeq_base,
            VarSeq = var_base,
            AAChange = as_change,
            Mutation = mutation_type,
            KnownSNP = snps,
            Max = max_perc,
	        Min = mutation_samples,	    
            Position = paste("<a href=\"http://www.ensembl.org/Homo_sapiens/",
                "Location/View?db=core;r=", chromosome, ":",abs_start, "-",
                abs_end, "\" target=\"_blank\">", chromosome, ":", abs_start,
                "-", abs_end, "</a>", sep=""),
            Details = trans_links,
            stringsAsFactors = FALSE
        ) 	
	## remove column containing samples with a minumum amount of reads if the threshold (minMut) equals zero
	if(minMut == 0) report_summary$Min=NA

	colnames(report_summary) = c("Reference", "Reference_tmp", "Variant",
	    "RefSeq", "VarSeq", "AAChange", "Mutation", "KnownSNP", "Max%", 
	    paste("Min", minMut, "%", sep=""), "Position", "Details")

	## remove unneeded columns (columns consisting of NA-values, temporary columns)
	removedCols = which(is.na(report_summary), arr.ind=TRUE)
	removedCols = c(2, removedCols[, "col"])

        ## if there is no blocks-argument create just one big table
        ## otherwise create a single table for every given gene
        if(is.null(blocks)) {
            HTML(report_summary[-(removedCols)], file=geneHTML, classtable="sortable")
	## otherwise split the data frame into several blocks and create a html table for each block
        } else {
	    report_summary = report_summary[-(removedCols)]
            for(b in unique(blocks)){
		idx = which(blocks == b)
                HTML.title(b, file=geneHTML)
                HTML(report_summary[idx,],
                    file=geneHTML, classtable="sortable")
            }
        }
        HTMLEndFile(file=geneHTML)
    #save(report_summary, file="/tmp/rep_sum.RData")
    ## create reference overview table
    #for(ref in unique(

	## MAIN HTML 2 (structured by samples)
        ## create main html file with gene overview and variant summary
        geneHTML2 = HTMLInitFile(dir, filename="index_samples", extension="html",
            Title = titleSummary, CSSFile="R2HTML.css")
       	cat("<script src=\"sorttable.js\"></script>", file=geneHTML2,
            append=TRUE)
        HTML.title(title, HR=1, file=geneHTML2)
	## link to the main page and to a quality report (only for AVASet objects)
	if(class(object) == "AVASet") {
		cat(.createNavigation(disabled="Sample"), file=geneHTML2, append=TRUE)
	} else {
		cat(.createNavigation(missing="Quality", disabled="Sample"), file=geneHTML2, append=TRUE)
	}
        ## sample overview: add sample pData with anchors to each sample
        samplesWithLinks = pData(object)
        sampleNames = rownames(samplesWithLinks)
        sampleNums = 1:length(rownames(samplesWithLinks))
        sampleAnchors = paste("<a href=\"#anchor", sampleNums, " \">", sampleNames, "</a>", sep="")
        rownames(samplesWithLinks) = sampleAnchors
        ## add links to quality reports for a AVASet
        ## (extra column "amplicon coverage" for the pData table at the beginning and a link for each sample name)
	if(class(object) == "AVASet"){
            samplesWithLinks$QualityReport = paste("<a href=\"qualityReport/", sampleNums, "/coverageSample_", sampleNums, ".html\"> amplicon coverage </a>", sep="")
            reportSampleTitles = paste("<a href=\"qualityReport/", sampleNums, "/coverageSample_", sampleNums, ".html\">,<font color=\"#FFFFFF\">",sampleNames, "</font> </a>", sep="")
        }
        else
          reportSampleTitles = sampleNames
        names(reportSampleTitles) = sampleNames
        
        HTML(samplesWithLinks, file=geneHTML2, classtable="sortable")
        ## create data frame to build a html table for every sample
	for(sample in 1:length(sampleNames)){
	    ## only show those variants that are relevant for the sample (i.e. that meet the minMut requirement)
	    sampleVars = which(variantPercsForward[vars_ID, sample] > minMut & variantPercsReverse[vars_ID, sample] > minMut)
	    if(length(sampleVars) > 0){
                report_summary = data.frame( 
	    	    row.names = 1:length(references[sampleVars]),
            	    Reference = references[sampleVars],
            	    Reference_tmp = as.character(references_short)[sampleVars],
            	    Variant = vars[sampleVars],
            	    RefSeq = refSeq_base[sampleVars],
            	    VarSeq = var_base[sampleVars],
            	    AAChange = as_change[sampleVars],
            	    Mutation = mutation_type[sampleVars],
            	    KnownSNP = snps[sampleVars],
            	    Max = max_perc[sampleVars],
	    	    Min = mutation_samples[sampleVars],	    
            	    Position = paste("<a href=\"http://www.ensembl.org/Homo_sapiens/",
                        "Location/View?db=core;r=", chromosome, ":",abs_start, "-",
                        abs_end, "\" target=\"_blank\">", chromosome, ":", abs_start,
                        "-", abs_end, "</a>", sep="")[sampleVars],
            	    Details = trans_links[sampleVars],
            	    stringsAsFactors = FALSE
            	) 

	    	## remove column containing samples with a minumum amount of reads if the threshold (minMut) equals zero
	    	if(minMut == 0) report_summary$Min=NA

	    	colnames(report_summary) = c("Reference", "Reference_tmp", "Variant",
	            "RefSeq", "VarSeq", "AAChange", "Mutation", "KnownSNP", "Max%", 
	            paste("Min", minMut, "%", sep=""), "Position", "Details")

	    	## remove unneeded columns (columns consisting of NA-values, temporary columns)
	    	removedCols = which(is.na(report_summary), arr.ind=TRUE)
	    	removedCols = c(2, removedCols[, "col"])
	    }
            ## set anchor to jump to this sample from sample overview on top of the page
            cat(paste("<a name=\"anchor", sample, "\"></a>", sep=""), file=geneHTML2, append=TRUE)
	    ## set sample title (in case of AVASet: link to sample quality report)
            HTML.title(reportSampleTitles[sample], file=geneHTML2)

	    ## export data frame with variant information into the html file
            ## only if at least one variant passed the minMut filter
	    if(length(sampleVars) > 0)
            	HTML(report_summary[-(removedCols)], file=geneHTML2, classtable="sortable")

	}
        HTMLEndFile(file=geneHTML2)
        message("done")
        
	if(class(object) == "AVASet")
	    .qualityReport(object, dir)

    }

########################################################################################################################
#######################################   Quality report   #############################################################
########################################################################################################################

## quality report showing amplicon coverage by amplicons, mids and ptps
## only available for AVASets
.qualityReport <- function(object, dir){

    message("Creating quality report ... ", appendLF = FALSE)
    dirReport = "qualityReport"

    ## a script to hide elements like tables (e.g. when pressing a button)
    hideTableScript =
      "<script language=\"javascript\">
       function toggle_it(itemID){
       if ((document.getElementById(itemID).style.display == 'none'))
       {
         document.getElementById(itemID).style.display = 'inline';
       } else {
         document.getElementById(itemID).style.display = 'none';
       }
       }
       </script>"


    ## save amplicon assaydata as csv file
    write.csv(t(assayDataAmp(object)$forwCount), file=file.path(dir, dirReport, "ampCovForward.csv"))
    write.csv(t(assayDataAmp(object)$revCount), file=file.path(dir, dirReport, "ampCovReverse.csv"))
    write.csv(t(assayDataAmp(object)$forwCount) + t(assayDataAmp(object)$revCount), file=file.path(dir, dirReport, "ampCov.csv"))
              
    ## prepare pData for amplicon, filtered ptp and mid coverage plots and tables
    ## Amplicons
    pDataAMP = pData(object)
    sampleNames = rownames(pDataAMP)
	sampleNums = 1:length(rownames(pDataAMP)) 
    rownames(pDataAMP) = paste("<a href=\"", sampleNums, "/coverageSample_", sampleNums, ".html\">",sampleNames , "</a>", sep="")
    pDataAMP$QualityReport = paste("<a href=\"", sampleNums, "/coverageSample_", sampleNums, ".html\">amplicon coverage</a>", sep="")
	## MID
    if(!is.element("MID1", colnames(pData(object))) || all(is.na(pData(object)$MID1)) || all(pData(object)$MID1 == "NA")){
      mids = NA
      warning("No MIDs found. Left out amplicon coverage report for MIDs.")
    }
    else{
      ind = grep(",", pData(object)$MID1, fixed=TRUE)
      if (length(ind) != 0)
        pDataMID = pData(object[, -ind])
      else
        pDataMID = pData(object)
	  pDataMID$sampleNumber = 1:length(rownames(pDataMID))  ## retain sample numbers beyond ordering
      pDataMID = pDataMID[order(pDataMID$MID1), ]    ## sort samples by mid
      mids = unique(pDataMID$MID1)
      sampleNames = rownames(pDataMID)
	  sampleNums = pDataMID$sampleNumber
	  pDataMID$sampleNumber <- NULL
      rownames(pDataMID) = paste("<a href=\"", sampleNums, "/coverageSample_", sampleNums, ".html\">",sampleNames , "</a>", sep="")
      pDataMID$QualityReport = paste("<a href=\"", sampleNums, "/coverageSample_", sampleNums, ".html\">amplicon coverage</a>", sep="")
    }
    ## PTP
    if(!is.element("PTP_AccNum", colnames(pData(object))) || all(is.na(pData(object)$PTP_AccNum))  || all(pData(object)$PTP_AccNum == "NA") ){
      ptps = NA
      warning("No PTP information found. Left out amplicon coverage report for PTPs.")
    }
    else{
      ind = union(grep(",", pData(object)$PTP_AccNum, fixed=TRUE), grep(",", pData(object)$Lane, fixed=TRUE))
      if (length(ind) != 0)
        pDataPTP = pData(object[, -ind])
      else
        pDataPTP = pData(object)
      pDataPTP$sampleNumber = 1:length(rownames(pDataPTP))  ## retain sample numbers beyond ordering
      pDataPTP = pDataPTP[order(pDataPTP$PTP_AccNum), ]
      ptps = unique(paste(as.character(pDataPTP$PTP_AccNum), as.character(pDataPTP$Lane), sep=""))
      sampleNames = rownames(pDataPTP)
	  sampleNums = pDataPTP$sampleNumber
      pDataPTP$sampleNumber <- NULL
      rownames(pDataPTP) = paste("<a href=\"", sampleNums, "/coverageSample_", sampleNums, ".html\">",sampleNames , "</a>", sep="")
      pDataPTP$QualityReport = paste("<a href=\"", sampleNums, "/coverageSample_", sampleNums, ".html\">amplicon coverage</a>", sep="")
    }
    
    ## set size of output images (heuristic, depending on the number of amplicons, mids ans ptps)
    numAmps = nrow(fDataAmp(object))
    pngWidth = max(300, 30 * (numAmps + 5))
    pngHeight = 600
    pdfWidth = pngWidth / 72
    pdfHeight = pngHeight / 72
    if(!all(is.na(mids))){
      numMIDs = length(mids)
      pngWidthMID = max(300, 30 * (numMIDs + 5))
      pngHeightMID = 600
      pdfWidthMID = pngWidthMID / 72
      pdfHeightMID = pngHeightMID / 72
    }
    if(!all(is.na(ptps))){
      numPTPs = length(ptps)
      pngWidthPTP = max(300, 30 * (numPTPs + 5))
      pngHeightPTP = 600
      pdfWidthPTP = pngWidthPTP / 72
      pdfHeightPTP = pngHeightPTP / 72
    }

    ## Coverage in both directions (together)
    filenameCov1 = "coverageAmp1"
    ## PNG file
    png(filename=file.path(dir, dirReport, paste(filenameCov1, ".png", sep="")), width=pngWidth, height=pngHeight)
    plotAmpliconCoverage(avaSet=object, bothDirections=FALSE, type="amplicon")
    dev.off()
    ## PDF file
    pdf(file=file.path(dir, dirReport, paste(filenameCov1, ".pdf", sep="")), width=pdfWidth, height=pdfHeight)
    plotAmpliconCoverage(avaSet=object, bothDirections=FALSE, type="amplicon")
    dev.off()
    ## Coverage in both directions (separated)
    filenameCov2 = "coverageAmp2"
    ## PNG file
    png(filename=file.path(dir, dirReport, paste(filenameCov2, ".png", sep="")), width=pngWidth*2, height=pngHeight)
    plotAmpliconCoverage(avaSet=object, bothDirections=TRUE, type="amplicon")
    dev.off()
    ## PDF file
    pdf(file=file.path(dir, dirReport, paste(filenameCov2, ".pdf", sep="")), width=pdfWidth*2, height=pdfHeight)
    plotAmpliconCoverage(avaSet=object, bothDirections=TRUE, type="amplicon")
    dev.off()

    if(!all(is.na(mids))){
        ## Coverage in both directions by MID (together)
        filenameCov3 = "coverageMID1"
        ## PNG file
        png(filename=file.path(dir, dirReport, paste(filenameCov3, ".png", sep="")), width=pngWidthMID, height=pngHeightMID)
        plotAmpliconCoverage(avaSet=object, bothDirections=FALSE, type="mid")
        dev.off()
        ## PDF file
        pdf(file=file.path(dir, dirReport, paste(filenameCov3, ".pdf", sep="")), width=pdfWidthMID, height=pdfHeightMID)
        plotAmpliconCoverage(avaSet=object, bothDirections=FALSE, type="mid")
        dev.off()
        ## Coverage in both directions by MID (separated)
        filenameCov4 = "coverageMID2"
        ## PNG file
        png(filename=file.path(dir, dirReport, paste(filenameCov4, ".png", sep="")), width=pngWidthMID*2, height=pngHeightMID)
        plotAmpliconCoverage(avaSet=object, bothDirections=TRUE, type="mid")
        dev.off()
        ## PDF file
        pdf(file=file.path(dir, dirReport, paste(filenameCov4, ".pdf", sep="")), width=pdfWidthMID*2, height=pdfHeightMID)
        plotAmpliconCoverage(avaSet=object, bothDirections=TRUE, type="mid")
        dev.off()
    }

    if(!all(is.na(ptps))){
        ## Coverage in both directions by PTP (together)
        filenameCov5 = "coveragePTP1"
        ## PNG file
        png(filename=file.path(dir, dirReport, paste(filenameCov5, ".png", sep="")), width=pngWidthPTP, height=pngHeightPTP)
        plotAmpliconCoverage(avaSet=object, bothDirections=FALSE, type="ptp")
        dev.off()
        ## PDF file
        pdf(file=file.path(dir, dirReport, paste(filenameCov5, ".pdf", sep="")), width=pdfWidthPTP, height=pdfHeightPTP)
        plotAmpliconCoverage(avaSet=object, bothDirections=FALSE, type="ptp")
        dev.off()
        ## Coverage in both directions by PTP (separated)
        filenameCov6 = "coveragePTP2"
        ## PNG file
        png(filename=file.path(dir, dirReport, paste(filenameCov6, ".png", sep="")), width=pngWidthPTP*2, height=pngHeightPTP)
        plotAmpliconCoverage(avaSet=object, bothDirections=TRUE, type="ptp")
        dev.off()
        ## PDF file
        pdf(file=file.path(dir, dirReport, paste(filenameCov6, ".pdf", sep="")), width=pdfWidthPTP*2, height=pdfHeightPTP)
        plotAmpliconCoverage(avaSet=object, bothDirections=TRUE, type="ptp")
        dev.off()
    }


    ## coverage for each sample
    samples = rownames(pData(object))
    for(s in 1:length(samples)){
	dirSample = file.path(dirReport, s)
        if(!(file.exists(file.path(dir, dirSample))))
            dir.create(file.path(dir, dirSample), recursive=TRUE)
	## PNG file
    	png(filename=file.path(dir, dirSample, paste("coverageSample_", s, ".png", sep="")), width=pngWidth, height=pngHeight)
        plotAmpliconCoverage(avaSet=object[, s], bothDirections=TRUE)
	dev.off()
	## PDF file
    	pdf(file=file.path(dir, dirSample, paste("coverageSample_", s, ".pdf", sep="")), width=pdfWidth, height=pdfHeight)
        plotAmpliconCoverage(avaSet=object[, s], bothDirections=TRUE)
	dev.off()
    }

    ## HTML file for general amplicon coverage (all samples altogether)
    qualityHTML = HTMLInitFile(file.path(dir, dirReport), filename="qualityReport", extension="html",
        Title = "Quality report", CSSFile="../R2HTML.css")
    cat("<script src=\"../sorttable.js\"></script>", file=qualityHTML, append=TRUE)
    cat(hideTableScript, file=qualityHTML, append=TRUE)
    
    ## Amplicon Coverage
    HTML.title("Quality report", HR=1, file=qualityHTML)

    ## add navigation buttons
	cat(.createNavigation(disabled="Quality", level=c("../", "../", "../")), file=qualityHTML, append=TRUE)

	## add page internal navigation buttons
	if(!all(is.na(mids)))
      buttonCovMID =
        "<FORM METHOD=\"LINK\" ACTION=\"#anchorCovMID\">
	 <INPUT TYPE=\"submit\" VALUE=\"Amplicon coverage by MID\">
	 </FORM>"
    else
      buttonCovMID = ""
    if(!all(is.na(ptps)))
      buttonCovPTP =
        "<FORM METHOD=\"LINK\" ACTION=\"#anchorCovPTP\">
	 <INPUT TYPE=\"submit\" VALUE=\"Amplicon coverage by PTP\">
	 </FORM>"
    else
      buttonCovPTP = ""
	if(!all(is.na(mids)) | !all(is.na(ptps))) {
		cat(paste("Go to:<br />
		<table border=\"0\">
        <tr>
        <td>", buttonCovMID, "</td>
        <td>", buttonCovPTP, "</td>
        </tr>
        </table>", sep = ""), file=qualityHTML, append=TRUE)
	}

    ## add some links for download of assaydata
    cat("Download .csv files of:", file=qualityHTML, append=TRUE)
    cat("<br />", file=qualityHTML, append=TRUE)    
    cat("<a href=\"ampCovForward.csv\"> amplicon coverage (forward direction) </a>", file=qualityHTML, append=TRUE)
    cat("<br />", file=qualityHTML, append=TRUE)    
    cat("<a href=\"ampCovReverse.csv\"> amplicon coverage (reverse direction) </a>", file=qualityHTML, append=TRUE)
    cat("<br />", file=qualityHTML, append=TRUE)    
    cat("<a href=\"ampCov.csv\"> amplicon coverage (both directions) </a>", file=qualityHTML, append=TRUE)

    border = 0
    ## Amplicon coverage by amplicons
    HTML.title("Amplicon coverage",file=qualityHTML)
    ## insert image with link to a pdf version
    cat(paste("<a href=\"", filenameCov1, ".pdf\" target=\"_blank\">
	<img src=\"", filenameCov1, ".png\" width=", pngWidth, "height=", pngHeight, "border=", border, "alt=Coverage />
	</a>", sep=""), file=qualityHTML, append=TRUE)
    cat("<br />", file=qualityHTML, append=TRUE)
    ## insert image with link to a pdf version
    cat(paste("<a href=\"", filenameCov2, ".pdf\" target=\"_blank\">
	<img src=\"", filenameCov2, ".png\" width=", pngWidth*2, "height=", pngHeight, "border=", border, "alt=Coverage />
	</a>", sep=""), file=qualityHTML, append=TRUE)
    ## insert table with sample information
    ## (add possibility to hide the table; No better solution  yet for smuggling of table id and style in Border-value)
    buttonHideTab =
      "<FORM> <INPUT TYPE=\"button\" VALUE=\"Show/hide sample information\" onClick=\"toggle_it('sampleTabAmp')\"> </FORM>"
    cat(buttonHideTab, file=qualityHTML, append=TRUE)
    HTML(pDataAMP, file=qualityHTML, classtable="sortable", Border="1 id=\"sampleTabAmp\" style=\"display:none;\"")
    
    ## Amplicon Coverage by MIDs
    if(!all(is.na(mids))){
        cat("<a name=\"anchorCovMID\"></a>", file=qualityHTML, append=TRUE)
        HTML.title("Amplicon coverage by MID",file=qualityHTML)
        ## insert image with link to a pdf version
        cat(paste("<a href=\"", filenameCov3, ".pdf\" target=\"_blank\">
            <img src=\"", filenameCov3, ".png\" width=", pngWidthMID, "height=", pngHeightMID, "border=", border, "alt=Coverage />
            </a>", sep=""), file=qualityHTML, append=TRUE)
        cat("<br />", file=qualityHTML, append=TRUE)
        ## insert image with link to a pdf version
        cat(paste("<a href=\"", filenameCov4, ".pdf\" target=\"_blank\">
            <img src=\"", filenameCov4, ".png\" width=", pngWidthMID*2, "height=", pngHeightMID, "border=", border, "alt=Coverage />
            </a>", sep=""), file=qualityHTML, append=TRUE)
        cat("<br />", file=qualityHTML, append=TRUE)
        ## insert table with sample information
        ## (add possibility to hide the table; No better solution  yet for smuggling of table id and style in Border-value)
        buttonHideTab =
          "<FORM> <INPUT TYPE=\"button\" VALUE=\"Show/hide sample information\" onClick=\"toggle_it('sampleTabMID')\"> </FORM>"
        cat(buttonHideTab, file=qualityHTML, append=TRUE)
        HTML(pDataMID, file=qualityHTML, classtable="sortable", Border="1 id=\"sampleTabMID\" style=\"display:none;\"")
    }
    ## Amplicon Coverage by PTPs    
    if(!all(is.na(ptps))){
        cat("<a name=\"anchorCovPTP\"></a>", file=qualityHTML, append=TRUE)
        HTML.title("Amplicon coverage by PTP",file=qualityHTML)
        ## insert image with link to a pdf version
        cat(paste("<a href=\"", filenameCov5, ".pdf\" target=\"_blank\">
            <img src=\"", filenameCov5, ".png\" width=", pngWidthPTP, "height=", pngHeightPTP, "border=", border, "alt=Coverage />
            </a>", sep=""), file=qualityHTML, append=TRUE)
        cat("<br />", file=qualityHTML, append=TRUE)
            ## insert image with link to a pdf version
        cat(paste("<a href=\"", filenameCov6, ".pdf\" target=\"_blank\">
            <img src=\"", filenameCov6, ".png\" width=", pngWidthPTP*2, "height=", pngHeightPTP, "border=", border, "alt=Coverage />
            </a>", sep=""), file=qualityHTML, append=TRUE)
        cat("<br />", file=qualityHTML, append=TRUE)
        ## insert table with sample information
        ## (add possibility to hide the table; No better solution  yet for smuggling of table id and style in Border-value)
        buttonHideTab =
          "<FORM> <INPUT TYPE=\"button\" VALUE=\"Show/hide sample information\" onClick=\"toggle_it('sampleTabPTP')\"> </FORM>"
        cat(buttonHideTab, file=qualityHTML, append=TRUE)
        HTML(pDataPTP, file=qualityHTML, classtable="sortable", Border="1 id=\"sampleTabPTP\" style=\"display:none;\"")
    }
    HTMLEndFile(file=qualityHTML)

    ## Amplicon coverage for every sample (create html file for each sample)
    for(sample in 1:length(samples)){

		dirSample = file.path(dirReport, sample)
        sampleQualityHTML = HTMLInitFile(file.path(dir, dirSample), filename=paste("coverageSample_", sample, sep=""), 
          extension="html", Title = "Quality report", CSSFile="../../R2HTML.css")
        cat("<script src=\"../../sorttable.js\"></script>", file=sampleQualityHTML, append=TRUE)
    	HTML.title(paste("Amplicon coverage for sample", sample), HR=1, file=sampleQualityHTML)
    	## link to the main pages
		cat(.createNavigation(level=c("../../", "../../", "../../")), file=sampleQualityHTML, append=TRUE)

        ## insert image with link to a pdf version
        cat(paste("<a href=\"coverageSample_", sample, ".pdf\" target=\"_blank\">
            <img src=\"coverageSample_", sample, ".png\" width=", pngWidth, "height=", pngHeight, "border=", border, "alt=Coverage />
            </a>", sep=""), file=sampleQualityHTML, append=TRUE)
        HTMLEndFile(file=sampleQualityHTML)

    }
    message("done")

}

setMethod("htmlReport",
    signature=signature(object="AVASet"),
    .initHtmlReportAVASet)

setMethod("htmlReport",
    signature=c(object="MapperSet"),
    .initHtmlReportGSMSet)

.plotVariationFrequency_data.frame <- function(object, plotRange, col, sequenceCex, xlab, ylab, ylim, ...) {

    if (missing(sequenceCex)) {
        sequenceCex = 0.5
    }
    if (missing(col)) {
        col = c("green", "blue", "black", "red", "purple", "gray", "blue")
    }
    if (missing(ylab)) {
        ylab = "Variation (%)"
    }
    # xlab default value is defined below, because plotRange has be checked first.
  
    refSeq = object[,2]                        # Refposn.
    refPos = floor(object[,1])                 # X
    subMatrix = as.matrix(object[, 3:8])       # A.. C.., G.., T.., N.., X...
    numReads = object[, 9]                     # X.1
    rm(object)

    # match given ploting range (bp pos in amplicon ref) to row in data.frame
    if (min(refPos) > plotRange[1] | max(refPos) < plotRange[2]) {
       stop("plotRange is out of bounce.")
    }
    plotRange = match(plotRange[1], refPos):match(plotRange[2], refPos)

    if (missing(xlab)) {
        xlab = paste("Reference sequence (position ", refPos[plotRange[1]], "-",
            refPos[plotRange[length(plotRange)]], ")", sep="")
    }


    # define the y-axis in 5% steps
    if (missing(ylim)) {
        maxPercent = ceiling(max(t(subMatrix)[, plotRange])/10 * 2) * 10 / 2
        if (maxPercent > 80) {
            maxPercent = 100
        }
        ylim = c(0, maxPercent)
    } else {
        maxPercent = ylim[2]
    }
 

    par(mar=c(5.1, 4.1, 4.1, 5.1))
    barplot(t(subMatrix)[, plotRange],
        col=col[1:6], ylim=ylim, ylab=ylab, space=0, xlab=xlab, ...)

    tickPos = seq(0.5, length(plotRange)-0.5, 1)
    axis(side=1, at=tickPos, labels=FALSE, tick=TRUE, tcl=-0.3)
    for (i in 1:length(plotRange)) {
        mtext(refSeq[plotRange[i]], side=1, line=0,
            outer=FALSE, at=tickPos[i], cex=sequenceCex)
    }

    maxNumReads = ceiling(max(numReads[plotRange])/100) * 100
    scaleFac = maxPercent/maxNumReads
    lines(x=1:length(plotRange), y=numReads[plotRange]*scaleFac,
        col=col[7])
    myTicks = c(0, maxNumReads/2*scaleFac, maxNumReads*scaleFac)
    myLabels = as.character(c(0, maxNumReads/2, maxNumReads))
    axis(side=4, at=myTicks, tick=TRUE, labels=myLabels)
    mtext(side=4, line=3, "Number of reads (coverage)", las=0, col=col[7])
}


.plotVariationFrequency_file <- function(object, plotRange, ...) {

    if (file.exists(object)) {
        df = read.table(object, skip=6, header=TRUE, sep="\t", stringsAsFactors=FALSE)
        .plotVariationFrequency_data.frame(df, plotRange, ...)
    } else {
        stop(paste("File", object, "does not exist."))
    }
   
}


setMethod("plotVariationFrequency", 
    signature=signature(object="character", plotRange="numeric"),
    .plotVariationFrequency_file)

# Getter and setter methods
setMethod("dataTCAG", signature(object="Roche454QualityFilterMetrics"),
    function(object) {
        return(object@dataTCAG)
    }
)

setReplaceMethod("dataTCAG",
    signature=signature(object="Roche454QualityFilterMetrics",
        value="data.frame"),
    function(object, value) {
        object@dataTCAG = value
        return(object)
    }
)

setMethod("dataCATG", signature(object="Roche454QualityFilterMetrics"),
    function(object) {
        return(object@dataCATG)
    }
)

setReplaceMethod("dataCATG",
    signature=signature(object="Roche454QualityFilterMetrics",
        value="data.frame"),
    function(object, value) {
        object@dataCATG = value
        return(object)
    }
)

setMethod("dataTotal", signature(object="Roche454QualityFilterMetrics"),
    function(object) {
        return(df = cbind(object@dataTotal,
            object@dataTCAG[,c(-1,-2)] + object@dataCATG[,c(-1,-2)]))
    }
)

setReplaceMethod("dataTotal",
    signature=signature(object="Roche454QualityFilterMetrics",
        value="data.frame"),
    function(object, value) {
        object@dataTotal = value
        return(object)
    }
)


# subsequent methods are removed due to warning by R CMD check

# Subsetting
#setMethod("[", signature=signature(
#        x="Roche454QualityFilterMetrics",
#        i="ANY",
#        j="missing"),          
#    function(x, i, ..., drop) {
#        if (missing(drop)) {
#            drop = FALSE
#        }         
#        dataTotal(x) = dataTotal(x)[i, 1:3, drop=drop]
#        dataTCAG(x) = dataTCAG(x)[i, , drop=drop]
#        dataCATG(x) = dataCATG(x)[i, , drop=drop]
#        return(x)
#    }
#)


# Other convenience functions
#setMethod(show, signature(object="Roche454QualityFilterMetrics"),
#    function(object) {
#        show(dataTotal(object))
#    }
#)

#setMethod("sampleNames", signature(object="Roche454QualityFilterMetrics"),
#    function(object) {
#      return(rownames(object@dataTotal))
#    }
#)

#setMethod("combine", signature(
#        x="Roche454QualityFilterMetrics",
#        y="Roche454QualityFilterMetrics"),
#    function(x, y) {
#      if (any(is.element(sampleNames(x), sampleNames(y)))) {
#        stop(paste("Cannot combine objects: Objects contain the same regions",
#                  "from the same pico titer plate."))
#      }
#      qfm = new("Roche454QualityFilterMetrics")
#      qfm@dataTotal = rbind(x@dataTotal, y@dataTotal)
#      qfm@dataTCAG = rbind(x@dataTCAG, y@dataTCAG)
#      qfm@dataCATG = rbind(x@dataCATG, y@dataCATG)
#      return(qfm)
#    }
#)

.readsOnTarget <- function(alnReads, targetRegion) {

    result = list()
    for (i in 1:length(alnReads)) {
        rd = GRanges(IRanges(start=alnReads[[i]]$pos,
                             end=alnReads[[i]]$pos + alnReads[[i]]$qwidth - 1),
                             seqnames=alnReads[[i]]$rname)
      
        result[[i]] = overlapsAny(rd, targetRegion)
    }

    return(result)
}

setMethod("readsOnTarget",
    signature=signature(alnReads="list", targetRegion="GRanges"),
    .readsOnTarget)



# this version sets all duplicates TRUE as soon as one matches the target
#onTargetComplex <- function(samDF, chipDesignBed, trackPos=1) {
#    require("rtracklayer")
#
#    refBed=import.ucsc(chipDesignBed, subformat="bed")
#    irCD=refBed[[trackPos]]$ranges
#    spacesCD=gsub("chr", "", refBed[[trackPos]]$space)
#    rlCD=IRangesList(irCD)
#    names(rlCD)=spacesCD
#
#    du1=duplicated(samDF[,"name"], fromLast=FALSE)
#    du2=duplicated(samDF[,"name"], fromLast=TRUE)
#    du=du1 | du2
#    samu=samDF[!du,]
#    samd=samDF[du,]
#
#    iru=IRanges(start=samu[,"start"], end=samu[,"end"])
#    rdu=RangedData(iru, space=samu[,"chr"])
#    rlu=ranges(rdu)
#    mau=match(rlu, rlCD)
#    mau=!is.na(mau)
#    samu[,ncol(samu)+1]=mau
#    colnames(samu)[ncol(samu)]="onTarget"
#        
#    ird=IRanges(start=samd[,"start"], end=samd[,"end"])
#    rdd=RangedData(ird, space=samd[,"chr"])
#    rld=ranges(rdd)
#    mad=match(rld, rlCD)
#    mad=!is.na(mad)
#    samd[,ncol(samd)+1]=mad
#    colnames(samd)[ncol(samd)]="onTarget"
#
#    for(name in unique(samd[,"name"])){
#      actrows=rownames(samd[samd[,"name"] == name,])
#      samd[actrows,"onTarget"]=any(samd[actrows,ncol(samd)])
#    }
#
#    sam=rbind(samu, samd)
#    reads=nrow(sam)
#    on=sum(sam[,"onTarget"])
#    off=sum(!sam[,"onTarget"])
#    onperc=on/reads*100
#    offperc=off/reads*100
#}

writeSFF <- function(sffContainer, filename) {
  
  ll <- list(flowgramFormat=flowgramFormat(sffContainer), flowChars=flowChars(sffContainer), 
             keySequence=keySequence(sffContainer), clipQualityLeft=clipQualityLeft(sffContainer), 
             clipQualityRight=clipQualityRight(sffContainer), clipAdapterLeft=clipAdapterLeft(sffContainer), 
             clipAdapterRight=clipAdapterRight(sffContainer), flowgrams=flowgrams(sffContainer), 
             flowIndexes=flowIndexes(sffContainer), reads=as.character(reads(sffContainer), use.names=TRUE), 
             qualityScores=sapply(sapply(as.character(quality(reads(sffContainer))), function(x) charToRaw(x)), 
                                  function(x) as.integer(x)-as.integer(33)))
  
  .Call("writeSFFfromR", ll, filename)
  
}

setMethod("addRead", signature(x="SFFContainer", read="SFFRead"),
  function(x, read) {
    if(name(read) %in% names(reads(x))) {
      stop("A read with name ", name(read), " already exists.")
    } else if (flowgramFormat(read) != flowgramFormat(x)) {
      stop("Flowgram formats did not match: ", flowgramFormat(read), " vs. ", flowgramFormat(x))
    } else if (flowChars(read) != flowChars(x)) {
      stop("Flow chars did not match: ", flowChars(read), " vs. ", flowChars(x))
    } else if (keySequence(read) != keySequence(x)) {
      stop("Key sequences did not match: ", keySequence(read), " vs. ", keySequence(x))
    } else {
      x@clipQualityLeft[name(read)] = clipQualityLeft(read)
      x@clipQualityRight[name(read)] = clipQualityRight(read)
      x@clipAdapterLeft[name(read)] = clipAdapterLeft(read)
      x@clipAdapterRight[name(read)] = clipAdapterRight(read)
      x@flowgrams[[name(read)]] = flowgram(read)
      x@flowIndexes[[name(read)]] = flowIndexes(read)
      
      pq = PhredQuality(quality(read))
      names(pq) = name(read)
      newRead = QualityScaledDNAStringSet(read(read), pq)
      names(newRead) = name(read)
      
      origReads = reads(x)
      
      # didn't work
      # newobj = append(origReads, newRead)
      # x@reads = newobj
      # workaround
      s1 = append(as(newRead, "XStringSet"), as(origReads, "XStringSet"))
      s2 = as(append(newRead@quality, origReads@quality), class(origReads@quality))
      x@reads = QualityScaledDNAStringSet(s1, s2)
      return(x)
    }
  }
)

setMethod("getRead", signature(x="SFFContainer", readname="character"),
  function(x, readname) {
    if(! readname %in% names(reads(x))) {
      stop(cat("There is no read with name ", readname, " in the SFFContainer.", sep=""))
    } else {
      cql = clipQualityLeft(x)[readname]
      cqr = clipQualityRight(x)[readname]
      cal = clipAdapterLeft(x)[readname]
      car = clipAdapterRight(x)[readname]
      names(cql) = names(cqr) = names(cal) = names(car) = NULL
      read = new("SFFRead", 
                 name=readname,
                 flowgramFormat=flowgramFormat(x),
                 flowChars=flowChars(x),
                 keySequence=keySequence(x),
                 clipQualityLeft=cql,
                 clipQualityRight=cqr,
                 clipAdapterLeft=cal,
                 clipAdapterRight=car,
                 flowgram=flowgrams(x)[[readname]],
                 flowIndexes=flowIndexes(x)[[readname]],
                 read=reads(x)[[readname]],
                 quality=quality(reads(x))[[readname]]
      )
      return(read)
    }
  }
)

setMethod("name", signature(x="SFFContainer"),
  function(x) {
    return(x@name)
  }
)
setReplaceMethod("name", signature(x="SFFContainer", value="character"),
  function(x, value) {
    x@name = value
    return(x)
  }
)

setMethod("flowgramFormat", signature(x="SFFContainer"),
  function(x) {
    return(x@flowgramFormat)
  }
)
setReplaceMethod("flowgramFormat", signature(x="SFFContainer", value="numeric"),
  function(x, value) {
    x@flowgramFormat = value
    return(x)
  }
)

setMethod("keySequence", signature(x="SFFContainer"),
  function(x) {
    return(x@keySequence)
  }
)
setReplaceMethod("keySequence", signature(x="SFFContainer", value="character"),
  function(x, value) {
    x@keySequence = value
    return(x)
  }
)

setMethod("flowChars", signature(x="SFFContainer"),
  function(x) {
    return(x@flowChars)
  }
)
setReplaceMethod("flowChars", signature(x="SFFContainer", value="character"),
  function(x, value) {
    x@flowChars = value
    return(x)
  }
)

setMethod("clipQualityLeft", signature(x="SFFContainer"),
  function(x) {
    return(x@clipQualityLeft)
  }
)
setReplaceMethod("clipQualityLeft", signature(x="SFFContainer", value="numeric"),
  function(x, value) {
    x@clipQualityLeft = value
    return(x)
  }
)

setMethod("clipQualityRight", signature(x="SFFContainer"),
  function(x) {
    return(x@clipQualityRight)
  }
)
setReplaceMethod("clipQualityRight", signature(x="SFFContainer", value="numeric"),
  function(x, value) {
    x@clipQualityRight = value
    return(x)
  }
)

setMethod("clipAdapterLeft", signature(x="SFFContainer"),
  function(x) {
    return(x@clipAdapterLeft)
  }
)
setReplaceMethod("clipAdapterLeft", signature(x="SFFContainer", value="numeric"),
  function(x, value) {
    x@clipAdapterLeft = value
    return(x)
  }
)

setMethod("clipAdapterRight", signature(x="SFFContainer"),
  function(x) {
    return(x@clipAdapterRight)
  }
)
setReplaceMethod("clipAdapterRight", signature(x="SFFContainer", value="numeric"),
  function(x, value) {
    x@clipAdapterRight = value
    return(x)
  }
)

setMethod("flowgrams", signature(x="SFFContainer"),
  function(x) {
    return(x@flowgrams)
  }
)
setReplaceMethod("flowgrams", signature(x="SFFContainer", value="list"),
  function(x, value) {
    x@flowgrams = value
    return(x)
  }
)

setMethod("flowIndexes", signature(x="SFFContainer"),
  function(x) {
    return(x@flowIndexes)
  }
)
setReplaceMethod("flowIndexes", signature(x="SFFContainer", value="list"),
  function(x, value) {
    x@flowIndexes = value
    return(x)
  }
)

setMethod("reads", signature(x="SFFContainer"),
  function(x) {
    return(x@reads)
  }
)
setReplaceMethod("reads", signature(x="SFFContainer", value="QualityScaledDNAStringSet"),
  function(x, value) {
    x@reads = value
    return(x)
  }
)

setMethod("show",
  signature(object="SFFContainer"),
  function(object){
    cat("Name: \n")
    cat(object@name, "\n")
    cat("\n")
    cat("Reads: \n")
    show(object@reads)
    cat("\n")
  }
)

setMethod("[",
  signature("SFFContainer"),
  function(x, i, ...){
    if(is.numeric(i)) {
      sffNew <- new("SFFContainer")
      name(sffNew) <- name(x)
      flowgramFormat(sffNew) <- flowgramFormat(x)
      flowChars(sffNew) <- flowChars(x)
      keySequence(sffNew) <- keySequence(x)
      clipQualityLeft(sffNew) <- clipQualityLeft(x)[i]
      clipQualityRight(sffNew) <- clipQualityRight(x)[i]
      clipAdapterLeft(sffNew) <- clipAdapterLeft(x)[i]
      clipAdapterRight(sffNew) <- clipAdapterRight(x)[i]
      flowgrams(sffNew) <- flowgrams(x)[i]
      flowIndexes(sffNew) <- flowIndexes(x)[i]
      reads(sffNew) <- reads(x)[i]
    } else {
      sffNew <- new("SFFContainer") 
      name(sffNew) <- name(x)
      flowgramFormat(sffNew) <- flowgramFormat(x)
      flowChars(sffNew) <- flowChars(x)
      keySequence(sffNew) <- keySequence(x)
      clipQualityLeft(sffNew) <- clipQualityLeft(x)[i]
      clipQualityRight(sffNew) <- clipQualityRight(x)[i]
      clipAdapterLeft(sffNew) <- clipAdapterLeft(x)[i]
      clipAdapterRight(sffNew) <- clipAdapterRight(x)[i]
      flowgrams(sffNew) <- flowgrams(x)[i]
      flowIndexes(sffNew) <- flowIndexes(x)[i]
      reads <- reads(x)[names(reads(x)) %in% i]
    }
    return(sffNew)
  }
)

setMethod("sff2fastq", 
  signature(x="SFFContainer"), 
  function(x, outdir, fname){
    if(missing(outdir)) {
      outdir = getwd()
    }
    outdir = gsub(paste(.Platform$file.sep, "+$", sep=""), "", outdir)
    if(missing(fname)) {
      fname = name(x)
      fname = gsub("(\\.sff)?$", ".fastq", fname)
    }
    fp = file.path(outdir, fname)
    writeXStringSet(reads(x), filepath=fp, format="fastq", qualities=quality(reads(x)))
    cat("Written file", fp, "\n")
  }
)

##################################################################################################
## getVariantPercentages returns the variant coverage (forward and/or reverse) in percent
## optionally: filters variants whose coverage is higher or equal than a given threshold
## output: data frame with the (filtered) coverage for each variant
##################################################################################################

.getVariantPercentages <- function(object, direction="both"){

    numFilters = length(filter)

    if(!is.character(direction) || !(direction %in% c("forward", "reverse", "both")))
        stop("invalid argument: please specify \"forward\", \"reverse\" or \"both\" as direction")

    if(direction == "both")
    	percs = (assayData(object)[[1]] + assayData(object)[[3]]) /
             	(assayData(object)[[2]] + assayData(object)[[4]])
    if(direction == "forward")
    	percs = assayData(object)[[1]] / assayData(object)[[2]]
    if(direction == "reverse")
    	percs = assayData(object)[[3]] / assayData(object)[[4]]

#    percs[is.na(percs)] = 0
    return(as.matrix(percs))
}


######################################################################################################
## The method setVariantFilter sets the filter for variant coverage to a given value between 0 and 1
## additionally it sets the data frame filter for variants for future access to feature and assayData.
## It returns an updated AVAset / MapperSet
######################################################################################################

.setVariantFilter <- function(object, filter = 0){

    numFilters = length(filter)

    ## catch bad filter input (only filter values between 0 and 1 allowed)
    if(!is.numeric(filter) || !(numFilters %in% c(1,2)))
        stop("invalid argument: please specify one or two numeric filter values between 0 and 1")
    if(any(filter < 0) || any(filter > 1))
        stop("argument out of range: please specify one or two numeric filter values between 0 and 1")

    ## reset filter
    variantFilterPerc(object) = 0

    numVars = nrow(fData(object))
    numSamples = nrow(pData(object))
    variantsPercFiltered = vector("list", 2)
    for(i in 1:numFilters){

	f = filter[i]
	## if one filter value is 0: take all variants without filtering
        if(f == 0){
	    variantsPercFiltered[[i]] = matrix(1, numVars, numSamples)
	    rownames(variantsPercFiltered[[i]]) = rownames(assayData(object)[[1]])
	    colnames(variantsPercFiltered[[i]]) = colnames(assayData(object)[[1]])
        } else {

            ## build the filter:
	    ## if just one filter value specified: apply filter to the sum of both directions (forward and reverse)
	    if(numFilters == 1)
            	variantsPerc = (assayData(object)[[1]] + assayData(object)[[3]]) /
                               (assayData(object)[[2]] + assayData(object)[[4]])
    	    ## if two filter values specified: apply filter to each direction (forward / reverse)
	    else
		if(i == 1)
                    variantsPerc = assayData(object)[[1]] / assayData(object)[[2]]
		else
                    variantsPerc = assayData(object)[[3]] / assayData(object)[[4]]

	    ## remove NA-values and mark those variants, whose percentage is larger than the filter (perc >= filter in forw.&rev. for at least one sample)
            variantsPerc[is.na(variantsPerc)] = 0
            variantsPerc[variantsPerc < f] = 0
            variantsPerc[variantsPerc >= f] = 1
	    variantsPercFiltered[[i]] = variantsPerc
        }
    }
    ## if just one filter value specified: just return the (first) filter
    if(numFilters == 1){
        names(filter) = c("forward & reverse")
        variantFilterPerc(object) = filter
	validVariants = apply(variantsPercFiltered[[1]] == 1, 1, any)
    	variantFilter(object) = names(validVariants[validVariants])
        message("combined forward & reverse filter set to ", filter)
    ## if two filter values specified: return the variant names that meet both filter requirements
    }else{
        names(filter) = c("forward", "reverse")
        variantFilterPerc(object) = filter
   	## the filter finally consists of the variant names that meet the filter requirements (perc >= filter in forw.&rev. for at least one sample)
	validVariants = apply(variantsPercFiltered[[1]] & variantsPercFiltered[[2]], 1, any)
    	variantFilter(object) = names(validVariants[validVariants])
        message("forward filter set to ", filter[1])
        message("reverse filter set to ", filter[2])
    }

    return(object)
}


setMethod("getVariantPercentages",
    signature(object="AVASet"),
    .getVariantPercentages)

setMethod("getVariantPercentages",
    signature(object="MapperSet"),
    .getVariantPercentages)

setMethod("setVariantFilter",
    signature(object="AVASet"),
	.setVariantFilter)	

setMethod("setVariantFilter",
    signature(object="MapperSet"),
	.setVariantFilter)


## getter and setter for the AVASet

setMethod("variantFilterPerc",
    signature(object="AVASet"),
    function(object){
        return(object@variantFilterPerc)
    }
)

setMethod("variantFilter",
    signature(object="AVASet"),
    function(object){
        return(object@variantFilter)
    }
)
setReplaceMethod("variantFilterPerc",
    signature(object="AVASet", value="numeric"),
    function(object, value){
        object@variantFilterPerc=value
        return(object)
    }
)

setReplaceMethod("variantFilter",
    signature(object="AVASet", value = "character"),
    function(object, value){
        object@variantFilter=value
        return(object)
    }
)

## getter and setter for the MapperSet

setMethod("variantFilterPerc",
    signature(object="MapperSet"),
    function(object){
        return(object@variantFilterPerc)
    }
)

setMethod("variantFilter",
    signature(object="MapperSet"),
    function(object){
        return(object@variantFilter)
    }
)

setReplaceMethod("variantFilterPerc",
    signature(object="MapperSet", value="numeric"),
    function(object, value){
        object@variantFilterPerc=value
        return(object)
    }
)

setReplaceMethod("variantFilter",
    signature(object="MapperSet", value = "character"),
    function(object, value){
        object@variantFilter=value
        return(object)
    }
)

.ava2vcf <- function(object, filename, annot) {
      
  # get the index of the reference for each variant
  refSeqInd <- match(fData(object)$referenceSeqID, as.character(id(referenceSequences(object))))
  
  # get the chromosome and strand for each variant
  chromosome <- as.character(chromosome(referenceSequences(object)[refSeqInd]))
  posStrand <- strand(referenceSequences(object))[refSeqInd] == "+"
  if(sum(is.na(chromosome)) > 0 | sum(is.na(posStrand)) > 0) {
   stop("Chromosome and/or strand information for references is missing, maybe you have to run alignShortReads first.") 
  }

  ### calculate the absolute position on the refrence genome for each variant ###
  
  # calculate the absolute positions of the reference sequences
  startRefSeqs <- position(referenceSequences(object)[refSeqInd])
  endRefSeqs <-  startRefSeqs + width(sread(referenceSequences(object)[refSeqInd])) - 1

  seq_start <- fData(object)$start
  seq_end <- fData(object)$end
  
  abs_start <- numeric(length(chromosome))
  # as insertions have .5 positions we have to correct them
  abs_start[posStrand] <- startRefSeqs[posStrand] + ceiling(seq_start[posStrand]) - 1  
  abs_start[!posStrand] <- endRefSeqs[!posStrand] - floor(seq_end[!posStrand]) + 1
  
  # not needed as vcf only uses start positions
  # abs_end = numeric(length(chromosome))
  # as insertions have .5 positions we have to correct them
  # abs_end[posStrand] <- startRefSeqs[posStrand] + ceiling(seq_end[posStrand]) - 1
  # abs_end[!posStrand] <- endRefSeqs[!posStrand] - floor(seq_start[!posStrand]) + 1
  
  mutation_type <- sapply(fData(object)$canonicalPattern, function(x) {substr(x, 1, 1)})
  mutation_type[mutation_type == "s"] <- "point"
  mutation_type[mutation_type == "d"] <- "deletion"
  mutation_type[mutation_type == "i"] <- "insertion"
  
  #get reference and alt alleles
  ref <- fData(object)$referenceBases
  alt <- fData(object)$variantBase
  
  # plus strand
  
  # nothing to do for point mutations on plus strand

  # add base before deletion on plus strand for deletions
  ind <- mutation_type == "deletion" & posStrand
  if(sum(ind) > 0) {
    ref[ind] <- paste(getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE), ref[ind], sep="")
    alt[ind] <- getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE)
    # new start position is that of preceding base
    abs_start[ind] <- abs_start[ind] - 1
  }

  # add base before insertion on plus strand for insertions
  ind <- mutation_type == "insertion" & posStrand
  if(sum(ind) > 0) {
    ref[ind] <- getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE)
    alt[ind] <- paste(getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE), alt[ind], sep="")
    # new start position is that of preceding base
    abs_start[ind] <- abs_start[ind] - 1
  }
 
  # minus strand
  # for mutations with a reference on the minus strand we have to change bases and directions
  
  # invert point mutations on minus strand
  ind <- mutation_type == "point" & !posStrand
  if(sum(ind) > 0) {
    ref[ind] <- as.character(reverseComplement(DNAStringSet(ref[ind])))
    alt[ind] <- as.character(reverseComplement(DNAStringSet(alt[ind])))
  }

  # add base before deletion and create reverse complement
  ind <- mutation_type == "deletion" & !posStrand
  if(sum(ind) > 0) {
    ref[ind] <- paste(getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE), as.character(reverseComplement(DNAStringSet(ref[ind]))), sep="")
    alt[ind] <- getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE)
    # new end position is that of preceding base
    abs_start[ind] <- abs_start[ind] - 1
  }

  # add base before (after in + coordinates) insertion on minus strand and reverse complement
  ind <- mutation_type == "insertion" & !posStrand
  if(sum(ind) > 0) {
    ref[ind] <- getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE)
    alt[ind] <- paste(getSeq(Hsapiens, names=chromosome[ind], start=abs_start[ind]-1, width=1, as.character=TRUE), as.character(reverseComplement(DNAStringSet(alt[ind]))), sep="")
    # new end position is that of preceding base
    abs_start[ind] <- abs_start[ind] - 1
  }
  
  # get known SNPs
  if(!missing(annot)) {
    anna <- annotatedVariants(annot)
    snps <- sapply(anna, function(x)
        if(ncol(x$snps) > 0) {
          # only recognize variants with exactly one known SNP
          rss <- grep("rs", x$snps$refsnp_id, fixed=TRUE)
          if(length(rss == 1)) {
            as.character(x$snps$refsnp_id[rss])
          } else {
            "."
          }
        } else {
          "."
        }
    )
    snpflags <- !(snps == ".")

  }
  
  ### create vcf object ###
  
  # header
  h_meta <- DataFrame(Value=c("VCFv4.1", gsub("-", "", Sys.Date(), fixed=TRUE), "R453Plus1Toolbox", "BSgenome.Hsapiens.UCSC.hg19"), 
                      row.names=c("fileFormat", "fileDate", "source", "reference"))
  if(!missing(annot)) {
    h_info <- DataFrame(Number=c("1", "4", "A", "A", "0"), Type=c("Integer", "Integer", "Float", "Integer", "Flag"), 
                        Description=c("Total Depth", 
                                      "Number of 1) forward ref alleles; 2) reverse ref; 3) forward non-ref; 4) reverse non-ref alleles",
                                      "Allele Frequency, for each ALT allele, in the same order as listed",
                                      "Allele count in genotypes, for each ALT allele, in the same order as listed",
                                      "dbSNP membership, build 135")) # INFO zur variante, welche sind möglich
    rownames(h_info) <- c("DP", "DP4", "AF", "AC", "DB")
  } else {
    h_info <- DataFrame(Number=c("1", "4", "A", "A"), Type=c("Integer", "Integer", "Float", "Integer"), 
                        Description=c("Total Depth", 
                                      "Number of 1) forward ref alleles; 2) reverse ref; 3) forward non-ref; 4) reverse non-ref alleles",
                                      "Allele Frequency, for each ALT allele, in the same order as listed",
                                      "Allele count in genotypes, for each ALT allele, in the same order as listed"
                        )) # INFO zur variante, welche sind möglich
    rownames(h_info) <- c("DP", "DP4", "AF", "AC")
  }

  h_format <- DataFrame(Number=c("1", "."), Type=c("Integer", "Integer"), 
                        Description=c("Read Depth", "Allelic depths for the alt alleles in the order listed")) #FORMAT zu den einzelnen samples
  rownames(h_format) <- c("DP", "AD")
  
  header <- VCFHeader(reference="BSgenome.Hsapiens.UCSC.hg19", samples=as.character(pData(object)$SampleID), 
                      header=DataFrameList(META=h_meta, INFO=h_info, FORMAT=h_format))
  metadata <- list(header=header)

  # body
  rowRanges <- GRanges(seqnames=Rle(chromosome), ranges=IRanges(start=abs_start, end=abs_start), strand="+", paramRangeID=rownames(fData(object)))
  names(rowRanges) <- rownames(object)
  if(!missing(annot)) {
    rowRanges$SNP <- snps
  }
        
#  colData <- DataFrame(Samples=pData(object)$Annotation, row.names=pData(object)$SampleID)
   colData <- DataFrame(SampleID=pData(object)$SampleID, row.names=sampleNames(object))

  if(!missing(annot)) {
    info <- DataFrame(DP=rowSums(assayData(object)$totalForwCount + assayData(object)$totalRevCount),
                      DP4=paste(rowSums(assayData(object)$totalForwCount - assayData(object)$variantForwCount),
                                rowSums(assayData(object)$totalRevCount - assayData(object)$variantRevCount),
                                rowSums(assayData(object)$variantForwCount),
                                rowSums(assayData(object)$variantRevCount),
                                sep=","),
                      AF=round(rowSums(assayData(object)$variantForwCount + assayData(object)$variantRevCount) / 
                         rowSums(assayData(object)$totalForwCount + assayData(object)$totalRevCount), 4),
                      AC=rowSums(assayData(object)$variantForwCount + assayData(object)$variantRevCount),
                      DB=snpflags,
                      row.names=rownames(fData(object)))
  } else {
    info <- DataFrame(DP=rowSums(assayData(object)$totalForwCount + assayData(object)$totalRevCount),
                      DP4=paste(rowSums(assayData(object)$totalForwCount - assayData(object)$variantForwCount),
                                rowSums(assayData(object)$totalRevCount - assayData(object)$variantRevCount),
                                rowSums(assayData(object)$variantForwCount),
                                rowSums(assayData(object)$variantRevCount),
                                sep=","),
                      AF=round(rowSums(assayData(object)$variantForwCount + assayData(object)$variantRevCount) / 
                         rowSums(assayData(object)$totalForwCount + assayData(object)$totalRevCount), 4),
                      AC=rowSums(assayData(object)$variantForwCount + assayData(object)$variantRevCount),
                      row.names=rownames(fData(object)))
  }
  
  format <- SimpleList(DP=(assayData(object)$totalForwCount + assayData(object)$totalRevCount), 
                       AD=(assayData(object)$variantForwCount + assayData(object)$variantRevCount))
  
  fixed <- DataFrame(REF=DNAStringSet(ref), ALT=CharacterList(as.list(alt)), QUAL=0, FILTER=as.character("."))
  
  # merge it all together
  vcf <- VCF(rowRanges=rowRanges, colData=colData, metadata=metadata, fixed=fixed, geno=format, info=info)

  # return VCF object or write it to file
  if(missing(filename)) {
    return(vcf)
  } else {
    writeVcf(vcf, filename) 
  }
}

setMethod("ava2vcf",
    signature=signature(object="AVASet"),
    .ava2vcf)

.plotAmpliconCoverage <- function(avaSet, type="amplicon", bothDirections=TRUE,
    cex.names, cex.axis, las, col, ...) {
  
    args = list(...)
   
##########################################
# Plot number of reads for each amplicon #
##########################################
    if (type == "amplicon") {

        if (ncol(avaSet) == 1) {

            # default values for main and ylab
            main = sampleNames(avaSet)
            ylab = "Number of reads (coverage)"

                 
            data = cbind(assayDataAmp(avaSet)$forwCount,
                assayDataAmp(avaSet)$revCount)
            data = data[sort(rownames(data)), , drop=FALSE]
            data = t(data)

            maxCov = max(data[1, ] + data[2, ])
            maxCov = maxCov + (0.2 * maxCov)    ## add some extra space for the legend

            par(oma=c(4,1,0,0))
            if (is.element("main", names(args)) & is.element("ylab", names(args))) {
                barplot(data, ylim=c(0, maxCov),
                    cex.names=cex.names, names.arg=colnames(data), las=las, col=col, ...)
            } else if (is.element("main", names(args)) & !is.element("ylab", names(args))) {
                barplot(data, ylab=ylab, ylim=c(0, maxCov),
                    cex.names=cex.names, names.arg=colnames(data), las=las, col=col, ...)
            } else if (!is.element("main", names(args)) & is.element("ylab", names(args))) {
                barplot(data, main=main, ylim=c(0, maxCov),
                    cex.names=cex.names, names.arg=colnames(data), las=las, col=col, ...)
            } else {
                barplot(data, ylab=ylab, main=main, ylim=c(0, maxCov),
                    cex.names=cex.names, names.arg=colnames(data), las=las, col=col, ...)
            }
            legend(x="topright", legend=c("forward reads", "reverse reads"), fill=col)       

            
        } else {

            # default values for main and ylab
            main = "Amplicon coverage"
            ylab = "Number of reads (coverage)"

            numAmps = nrow(assayDataAmp(avaSet)$forwCount)
            ampNames = sort(rownames(assayDataAmp(avaSet)$forwCount))

            if (bothDirections) {
                              
                data = matrix(NA, nrow=ncol(avaSet), ncol=2*numAmps)
                colnames(data) = rep("", numAmps*2)
                data[, seq(1, 2*numAmps, 2)] =
                    t(assayDataAmp(avaSet)$forwCount[ampNames, ])
                data[, seq(2, 2*numAmps, 2)] =
                    t(assayDataAmp(avaSet)$revCount[ampNames, ])
                colnames(data)[seq(1, 2*numAmps, 2)] = ampNames
                colnames(data)[seq(2, 2*numAmps, 2)] = ampNames

                maxCov = max(data)
                maxCov = maxCov + (0.2 * maxCov)    ## add some extra space for the legend

                par(oma=c(4,1,0,0))
                if (is.element("main", names(args)) & is.element("ylab", names(args))) {
                    boxplot(data, las=las, ylim=c(0, maxCov),
                        cex.axis=cex.axis, col=col, ...)
                } else if (is.element("main", names(args)) & !is.element("ylab", names(args))) {
                    boxplot(data, las=las, ylim=c(0, maxCov), ylab=ylab,
                        cex.axis=cex.axis, col=col, ...)
                } else if (!is.element("main", names(args)) & is.element("ylab", names(args))) {
                    boxplot(data, las=las, main=main, ylim=c(0, maxCov),
                        cex.axis=cex.axis, col=col, ...)
                } else {
                    boxplot(data, las=las, main=main, ylim=c(0, maxCov), ylab=ylab,
                        cex.axis=cex.axis, col=col, ...)
                }
                legend("topright", legend=c("forward reads", "reverse reads"), fill=col)
                
            } else {
                
                data = t(assayDataAmp(avaSet)$forwCount[ampNames, ]) +
                    t(assayDataAmp(avaSet)$revCount[ampNames, ])
                colnames(data) = ampNames

                par(oma=c(4,1,0,0))
                if (is.element("main", names(args)) & is.element("ylab", names(args))) {
                    boxplot(data, las=las,
                    cex.axis=cex.axis, col=col[1], ...)
                } else if (is.element("main", names(args)) & !is.element("ylab", names(args))) {
                    boxplot(data, las=las, ylab=ylab,
                    cex.axis=cex.axis, col=col[1], ...)
                } else if (!is.element("main", names(args)) & is.element("ylab", names(args))) {
                    boxplot(data, las=las, main=main,
                    cex.axis=cex.axis, col=col[1], ...)
                } else {
                    boxplot(data, las=las, main=main, ylab=ylab,
                    cex.axis=cex.axis, col=col[1], ...)
                }
            } 
        }


#####################################
# Plot number of reads for each MID #
#####################################
    } else if (type == "mid") {

        #default values for main and ylab
        main = "Number of reads by MID"
        ylab = "Number of reads (per sample)"
      
        if (any(!is.na(pData(avaSet)$MID2)) &
            any(pData(avaSet)$MID1 != pData(avaSet)$MID2)) {
            stop("MID1 and MID2 must be equal.")
        }
        ind = grep(",", pData(avaSet)$MID1, fixed=TRUE)
        if (length(ind) != 0) {
            avaSet = avaSet[, -ind]
            warning(paste(length(ind), "samples were excluded due to mutiple MIDs."))
        }

        if (bothDirections) {
          
            midFac = factor(c(paste(as.character(pData(avaSet)$MID1), "_f", sep=""),
                paste(as.character(pData(avaSet)$MID1), "_r", sep="")))
            data = c(apply(assayDataAmp(avaSet)$forwCount, 2, sum),
                apply(assayDataAmp(avaSet)$revCount, 2, sum))

            maxCov = max(data)
            maxCov = maxCov + (0.2 * maxCov)    ## add some extra space for the legend

            par(oma=c(2,1,0,0))
            if (is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ midFac, las=las, col=col, ylim=c(0, maxCov),
                    names=substring(levels(midFac), first=1, last=nchar(levels(midFac))-2),
                    cex.axis=cex.axis, ...)
            } else if (is.element("main", names(args)) & !is.element("ylab", names(args))) {
                boxplot(data ~ midFac, las=las, col=col, ylim=c(0, maxCov), ylab=ylab,
                    names=substring(levels(midFac), first=1, last=nchar(levels(midFac))-2),
                    cex.axis=cex.axis, ...)
            } else if (!is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ midFac, las=las, col=col, main=main, ylim=c(0, maxCov),
                    names=substring(levels(midFac), first=1, last=nchar(levels(midFac))-2),
                    cex.axis=cex.axis, ...)
            } else {
                boxplot(data ~ midFac, las=las, col=col, main=main, ylim=c(0, maxCov), ylab=ylab,
                    names=substring(levels(midFac), first=1, last=nchar(levels(midFac))-2),
                    cex.axis=cex.axis, ...)
            }
            legend("topright", legend=c("forward reads", "reverse reads"), fill=col)

        } else {
          
            midFac = factor(as.character(pData(avaSet)$MID1))
            data = apply(assayDataAmp(avaSet)$forwCount, 2, sum) +
                apply(assayDataAmp(avaSet)$revCount, 2, sum)

            par(oma=c(2,1,0,0))
            if (is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ midFac, las=las, col=col[1], 
                    cex.axis=cex.axis, ...)
            } else if (is.element("main", names(args)) & !is.element("ylab", names(args))) {
                boxplot(data ~ midFac, las=las, col=col[1], ylab=ylab,
                    cex.axis=cex.axis, ...)
            } else if (!is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ midFac, las=las, col=col[1], main=main, 
                    cex.axis=cex.axis, ...)
            } else {
                boxplot(data ~ midFac, las=las, col=col[1], main=main, ylab=ylab,
                    cex.axis=cex.axis, ...)
            }

        }


##############################################
# Plot Number of reads for each PTP / region #
##############################################
    } else if (type == "ptp") {

        # default values for main and ylab
        main = "Number of reads by pico titer plate / region"
        ylab = "Number of reads (per sample)"
      
        ind = union(grep(",", pData(avaSet)$PTP_AccNum, fixed=TRUE),
            grep(",", pData(avaSet)$Lane, fixed=TRUE))
        if (length(ind) != 0) {
            avaSet = avaSet[, -ind]
            warning(paste(length(ind), "samples were excluded due to mutiple PTPs/Lanes."))
        }

        if (bothDirections) {
            ptpFac = paste(as.character(pData(avaSet)$PTP_AccNum),
                as.character(pData(avaSet)$Lane), sep="")
            ptpFac = factor(c(paste(ptpFac, "_f", sep=""), paste(ptpFac, "_r", sep="")))
            data = c(apply(assayDataAmp(avaSet)$forwCount, 2, sum),
                apply(assayDataAmp(avaSet)$revCount, 2, sum))

            maxCov = max(data)
            maxCov = maxCov + (0.2 * maxCov)    ## add some extra space for the legend

            par(oma=c(2,1,0,0))

            if (is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ ptpFac, las=las, col=col, ylim=c(0, maxCov),
                    names=substring(levels(ptpFac), first=1, last=nchar(levels(ptpFac))-2),
                    cex.axis=cex.axis, ...)
            } else if (is.element("main", names(args)) & !is.element("ylab", names(args))) {
                boxplot(data ~ ptpFac, las=las, col=col, ylim=c(0, maxCov), ylab=ylab,
                    names=substring(levels(ptpFac), first=1, last=nchar(levels(ptpFac))-2),
                    cex.axis=cex.axis, ...)
            } else if (!is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ ptpFac, las=las, col=col, main=main, ylim=c(0, maxCov),
                    names=substring(levels(ptpFac), first=1, last=nchar(levels(ptpFac))-2),
                    cex.axis=cex.axis, ...)
            } else {
                boxplot(data ~ ptpFac, las=las, col=col, main=main, ylim=c(0, maxCov), ylab=ylab,
                    names=substring(levels(ptpFac), first=1, last=nchar(levels(ptpFac))-2),
                    cex.axis=cex.axis, ...)
            }
            legend("topright", legend=c("forward reads", "reverse reads"), fill=col)

        } else {
          
            ptpFac = factor(paste(as.character(pData(avaSet)$PTP_AccNum),
                as.character(pData(avaSet)$Lane), sep=""))
            data = apply(assayDataAmp(avaSet)$forwCount, 2, sum) +
                apply(assayDataAmp(avaSet)$revCount, 2, sum)

            par(oma=c(2,1,0,0))
            if (is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ ptpFac, las=las, col=col[1], 
                    cex.axis=cex.axis, ...)
            } else if (is.element("main", names(args)) & !is.element("ylab", names(args))) {
                boxplot(data ~ ptpFac, las=las, col=col[1], ylab=ylab,
                    cex.axis=cex.axis, ...)
            } else if (!is.element("main", names(args)) & is.element("ylab", names(args))) {
                boxplot(data ~ ptpFac, las=las, col=col[1], main=main,
                    cex.axis=cex.axis, ...)
            } else {
                boxplot(data ~ ptpFac, las=las, col=col[1], main=main, ylab=ylab,
                    cex.axis=cex.axis, ...)
            }
        }


    } else {
        print("Argument type must be amplicon, mid or ptp.")
    }

}


setMethod("plotAmpliconCoverage", 
    signature=signature(avaSet="AVASet", type="character", bothDirections="logical"),
    .plotAmpliconCoverage)


setMethod("plotAmpliconCoverage",
    signature=signature(avaSet="AVASet", type="character", bothDirections="missing"),
    .plotAmpliconCoverage)


setMethod("plotAmpliconCoverage",
    signature=signature(avaSet="AVASet", type="missing", bothDirections="logical"),
    .plotAmpliconCoverage)
    


setMethod("plotAmpliconCoverage",
    signature=signature(avaSet="AVASet", type="missing", bothDirections="missing"),
    .plotAmpliconCoverage)

###############################################################################
## All functions below are seperated according to the standard plot, the base 
## pair plot or both The main plotting function .plotChimericReads at the end 
## calls .initializePlot, .plotReference, .annotateReference and 
## .plotReads (<-calls .plotMutations)
###############################################################################
###############################################################################

###############################################################################
################# base pair and standard plot functions #######################
###############################################################################

## .addComma helps reading (large) numbers in plots by adding commas
.addComma <- function(x){

    #paste(read.fwf(textConnection(x), commasAt, as.is = TRUE), collapse = ",")

    x = as.character(x)
    y = character()
    for(i in seq(nchar(x), 1, -3))
	if((i-3) > 0)
	    y = paste(",",substring(x,i-2,i), y, sep="")
        else
	    y = paste(substring(x,1,i), y, sep="")
    return(y)
}

## return complement strand symbol for characters ("+", or "-")
.complementStrand <- function(strand){
    if(strand == "+")
        return("-")
    if(strand == "-")
        return("+")
    stop("complementStrand: unrecognized input")
}

## sometimes the pathogenic and reciproc case are equally oriented: both show chrA <-> chrB
## this function reverses case2 with the result: pathogenic shows chrA <-> chrB, reciproc shows chrB <-> chrA
## this ensures a more intuitive visualization
.reverseC2 <- function(brpData){
    ## reverse commonBpsC2 (reference sequense, strands and local coordinates)
    refLength = length(subject(Views(commonAlignC2(brpData)[[1]])))
    commonBps = commonBpsC2(brpData)[[1]]
    tmp = commonBps$localStart
    commonBps$localStart = refLength - commonBps$localEnd + 1
    commonBps$localEnd = refLength - tmp + 1
    commonBps$strand[1] = .complementStrand(commonBps$strand[1])
    commonBps$strand[2] = .complementStrand(commonBps$strand[2])
    commonBps$refSeq = as.character(reverseComplement(DNAStringSet(commonBps$refSeq)))
    tmp = commonBps[1, ]
    commonBps[1, ] = commonBps[2, ]
    commonBps[2, ] = tmp

    ## reverse alignedReadsC2 (sequences, chromosomes, strands) 
    alnChrs = sapply(strsplit(as.character(chromosome(alignedReadsC2(brpData)[[1]])), split="/"), function(x) paste(x[2], x[1], sep="/"))
    alnStrands = sapply(strsplit(as.character(strand(alignedReadsC2(brpData)[[1]])), split="/"), function(x) paste(.complementStrand(x[1]), .complementStrand(x[2]), sep="/"))
    alnReads = reverseComplement(sread(alignedReadsC2(brpData)[[1]]))

    ## realign the reversed reads to the reversed reference sequence
    substitutionMatrix = nucleotideSubstitutionMatrix(match=1, mismatch=-1)
    alignment = pairwiseAlignment(pattern=alnReads, subject=commonBps$refSeq[1],
        type="global-local", substitutionMatrix=substitutionMatrix, gapOpening=0, gapExtension=1)

    ## return an updated breakpoints-object(remember: the function requires, that the object holds only one breakpoint!)
    commonBpsC2(brpData) = list(commonBps)
    commonAlignC2(brpData) = list(alignment)
    alignedReadsC2(brpData) = list(AlignedRead(sread=alnReads, chromosome=factor(alnChrs), position=start(Views(alignment)), strand=factor(alnStrands)))
    return(brpData)
}

## initialize plot window
.initializePlot <- function(refStart_left, refEnd_right, plotHeight, plotBasePairs, legendColours, legendSymbols, fontScale, fontFamily){

    if(!plotBasePairs){
   	plotTol_left = 0.8		## percentage of the reference length to be added to the left plot limit
    	plotTol_right = 0.2		## percentage of the reference length to be added to the right plot limit
    }else{
    	plotTol_left = 0.3		## percentage of the reference length to be added to the left plot limit
    	plotTol_right = 0		## percentage of the reference length to be added to the right plot limit
    }

    X_min = refStart_left
    X_max = refEnd_right

    ## add some free space on the left and right limit of the plot
    xtol1 = (X_max - X_min) * plotTol_left
    xtol2 = (X_max - X_min) * plotTol_right
    X_min = X_min - xtol1
    X_max = X_max + xtol2

    plot(c(X_min, X_max), c(0, plotHeight), type="n", xaxt="n", yaxt="n", xlab="", ylab="") #, frame.plot=FALSE)

    ## add legend
    legendSize = length(legendColours)
    if(legendSize > 0){
    	legendLabels = c("deletion", "insertion", "mismatch", "breakpoint")
    	legend(x=X_min, y=plotHeight, legend=legendLabels[1:legendSize], pch=legendSymbols, col=legendColours, horiz=TRUE, cex=fontScale)
    }

    ## set character width and height to align sequences and basepairs in a basePair-plot
    if(plotBasePairs){
      characterWidth = strwidth("A", cex=fontScale, family=fontFamily)
      characterHeight = strheight("A", cex=fontScale, family=fontFamily)    
    }else{
      characterWidth = strwidth("A", cex=fontScale)
      characterHeight = strheight("A", cex=fontScale)    
    }
    return(list(characterWidth, characterHeight))

}

## annotate plot of reference sequence with chromosome, strand and position labels
.annotateReference <- function(refPos, refHeight, refStart_left, refEnd_left, refStart_right, refEnd_right, 
	chr_left, chr_right, part_left, part_right, strand_left, strand_right, labels, chrProps, fontScale, plotBasePairs){

    y = refPos + (refHeight / 2)
    ## chromosome labels
    text(x=refStart_left, y=y, label=paste("chr.", chr_left, sep=""), pos=4, cex=fontScale)
    text(x=refEnd_right, y=y, label=paste("chr.", chr_right, sep=""), pos=2, cex=fontScale)

    ## gene symbols
    text(x=refEnd_left, y=y, label=chrProps[part_left, "geneSymbol"], pos=2, cex=fontScale, font=3)
    text(x=refStart_right, y=y, label=as.character(chrProps[part_right, "geneSymbol"]), pos=4, cex=fontScale, font=3)

    ## strand labels
    ##forward
    y = refPos +  refHeight
    text(x=refStart_left, y=y, label=paste(strand_left, "5'"),pos=2, cex=fontScale)
    text(x=refEnd_right, y=y, label=paste("3'", strand_right),pos=4, cex=fontScale)
    ##reverse
    y = refPos
    if(strand_left == "+")
        text(x=refStart_left, y=y, label="- 3'",pos=2, cex=fontScale)
    else
        text(x=refStart_left, y=y, label="+ 3'",pos=2, cex=fontScale)
    if(strand_right == "+")
        text(x=refEnd_right, y=y, label="5' -",pos=4, cex=fontScale)
    else
        text(x=refEnd_right, y=y, label="5' +",pos=4, cex=fontScale)

    ## position labels (not for base pair plot)
    if(!plotBasePairs){
	offset = 0.5
    	y = refPos
    	text(x=refStart_left, y=y, pos = 1, offset=offset, label=.addComma(labels[1]), cex=fontScale)
    	text(x=refEnd_left, y=y, pos = 1, offset=offset, label=.addComma(labels[2]), cex=fontScale)
    	y = refPos + refHeight
    	text(x=refStart_right, pos = 3, offset=offset, y=y, label=.addComma(labels[3]), cex=fontScale)
    	text(x=refEnd_right, pos = 3, offset=offset, y=y, label=.addComma(labels[4]), cex=fontScale)		
    }
}

## .getMutations reads indels and mismatches from the alignment and modifies the coordinates for correct plotting
.getMutations <- function(alignment, from, to){

    alnStart = start(Views(alignment)) - 1

    ## deletions, insertions and mismatches from alignment
    dels = deletion(alignment)[[1]]
    ins = insertion(alignment)[[1]]
    mms = mismatchTable(alignment)

    ## adjust coordinates of mismatches and deletions according to insertions (and deletions) removed prior to them
    if(nrow(mms) > 0)
        for(m in 1:nrow(mms)){
	    mms[m, c("PatternStart", "PatternEnd")] = 
		mms[m, c("PatternStart", "PatternEnd")] - sum(width(restrict(ins, 1, mms$PatternStart[m]))) +
		    sum(width(restrict(dels, 1, mms$PatternStart[m])))
	}
    if(length(dels) > 0)
        for(d in 1:length(dels))
          dels[d] = shift(dels[d], -sum(width(restrict(ins, 1, start(dels[d])))))

    ## adjust coordinates of dels/ins/mismatches according to the beginning of alignment on the reference sequence
    ## take care of dels ("-") inserted prior to a deletion
    if(length(dels) > 0)
      dels = shift(dels, seq(0, length(dels) - 1) + alnStart)
    dels = as.data.frame(dels)
    ins = shift(ins, alnStart)
    ins = as.data.frame(ins)
    mms[ , c("PatternStart", "PatternEnd")] = mms[ , c("PatternStart", "PatternEnd")] + alnStart
    ## only draw dels/ins/mismatches within a given range (from/to; +/- maxBasePairs around the breakpoint)
    dels = subset(dels, start >= from & end <= to)
    ins = subset(ins, start >= from & end <= to)
    mms = subset(mms, PatternStart >= from & PatternEnd <= to)

    if(nrow(dels) > 0)
    	dels = unlist(c(apply(dels, 1, function(x) x["start"]:x["end"]))) - (from - 1)
    else
	dels=c()
    if(nrow(ins) > 0)
    	ins = ins$start - (from - 1)
    else
	ins=c()
    if(nrow(mms) > 0)
    	mms = mms$PatternStart - (from - 1)
    else
	mms=c()

    return(list(dels, ins, mms))

}

###############################################################################
################# standard plot functions #####################################
###############################################################################

## draw reference sequence in the center of the plot
.plotReference <- function(refHeight, refPos, refStart_left, refStart_right, refEnd_left, refEnd_right, 
	part_left, part_right, chrProps, colours){

    y = c(refPos, refPos + refHeight, refPos + refHeight, refPos)

    ## left part
    x = c(refStart_left, refStart_left, refEnd_left, refEnd_left)
    polygon(x, y, col=chrProps[part_left, "colourRef"])
    ## right part
    x = c(refStart_right, refStart_right, refEnd_right, refEnd_right)
    polygon(x, y, col=chrProps[part_right, "colourRef"])

    ## gap between both parts
    x = c(refEnd_left, refEnd_left, refStart_right, refStart_right)
    polygon(x, y, density=c(20,30), angle=c(-45,45), col=colours[10], border=colours[10])
}

## plots deletions, insertions and mismatches for standard plot
.plotMutations = function (dels, ins, mms, y, colours){

    points(x=dels, y=rep(y,length(dels)), pch="|", col=colours[7])	
    points(x=ins, y=rep(y,length(ins)), pch="|", col=colours[8])	
    points(x=mms, y=rep(y,length(mms)), pch="|", col=colours[9])	

}

## draw reads above and below the reference sequence
.plotReads <- function(alignments, names, directions, numReads, refPos, refHeight, readDist, localBrp_left, localBrp_right,
	refEnd_right, refStart_left, plotMut, colours, fontScale, characterHeight){

    yDist_above = 1
    yDist_below = 1

    for(i in 1:numReads){

   	## x-positions
	start = start(Views(alignments[i]))
	end = end(Views(alignments[i]))

    	## y-position (depends on direction: distinguish between two directions: 3'-5' = below reference, 5'-3' = above reference)
	## forward (5'-3')
       	if(directions[i] == "forward"){
    	    y = refPos + refHeight + 3*characterHeight + (yDist_above * readDist)
    	    yDist_above = yDist_above + 1	
	## reverse (3'-5')
	}else{
    	    y = refPos - 3*characterHeight - (yDist_below * readDist)
    	    yDist_below = yDist_below + 1	
	}

	## plot read and mutations
    	lines(x=c(start, end), y=c(y, y), col=colours[6])
	points(x=start, y=y, pch=20, col=colours[6])
	points(x=end, y=y, pch=20, col=colours[6])
	points(x=c(localBrp_left, localBrp_right - 1), y=c(y, y), pch="|", col=colours[10])

    	if(plotMut){
    	    muts = .getMutations(alignments[i], refStart_left, refEnd_right)
       	    .plotMutations(muts[[1]], muts[[2]], muts[[3]], y, colours)
	}

	## label each read with its name
	labelOffset = 2
      	    text(x=refStart_left, y=y, label=names[i], pos=2, offset = labelOffset, cex=fontScale)
	
    }	
}

###############################################################################
################# base pair plot functions ####################################
###############################################################################

## draw reference sequence in the center of the plot
.plotReferenceBasePairs <- function(numReadsAbove, numReadsBelow, readDist, refHeight, refPos, refSeq, localBrp_left, localBrp_right,
	refStart_left, refStart_right, refEnd_left, refEnd_right, part_left, part_right, from, to, chrProps, colours, labels, 
	fontScale, fontFamily, characterWidth, characterHeight, maxBasePairs_left, maxBasePairs_right){

    x_left = c(refStart_left, refStart_left, refEnd_left,  refEnd_left)
    x_right = c(refStart_right, refStart_right, refEnd_right, refEnd_right)
    x_gap = c(refEnd_left, refEnd_left, refStart_right, refStart_right)
    yReads = c(refPos - 2*characterHeight - readDist * numReadsBelow, refPos + refHeight + 3*characterHeight + readDist * numReadsAbove,
	refPos + refHeight + 3*characterHeight + readDist * numReadsAbove, refPos - 2*characterHeight - readDist * numReadsBelow)
    yRef = c(refPos - characterHeight,refPos + refHeight + characterHeight,refPos + refHeight + characterHeight,refPos - characterHeight)

    ## draw coloured frames for reads (left and right)
    polygon(x_left, yReads, col=chrProps[part_left, "colourReads"], border="black")
    polygon(x_right, yReads, col=chrProps[part_right, "colourReads"], border="black")

    ## draw coloured frames for reference sequence (left and right)
    polygon(x_left, yRef, col=chrProps[part_left, "colourRef"], border="black")
    polygon(x_right, yRef, col=chrProps[part_right, "colourRef"], border="black")

    ## draw coloured frame for the gap
    polygon(x_gap, yReads, col=colours[5], border=colours[5])

    rect(xleft=refStart_left, ybottom=refPos - 2*characterHeight - readDist * numReadsBelow,
   	xright=refEnd_right, ytop=refPos + refHeight + 3*characterHeight + readDist * numReadsAbove, col=NA, border="black")

    ## draw basepairs of the reference sequence; compute reverse/reverseComplement for 3'-5' direction
    text(x=refStart_left, y=refPos + refHeight, label=substr(refSeq, from, to), pos=4, offset=0, cex=fontScale, family=fontFamily, col=colours[6])
    text(x=refStart_left, y=refPos, label=substr(complement(DNAString(refSeq)), from, to), pos=4, offset=0, cex=fontScale, family=fontFamily, col=colours[6])
}

## plots deletions and insertions for basePair-plot
.plotMutationsBasePairPlot <- function(x, y, dels, ins, mms, fontScale, fontFamily, characterWidth, characterHeight, colours){

    ## mark mutations as triangles
    mutHeight = characterHeight * 0.6
    mutWidth = characterWidth * 0.6
    y = y + characterHeight

    ## print deletions as (red) triangle 
    if(length(dels) > 0){
    	dels = dels * characterWidth
    	xd = x - 0.5 * characterWidth  ## draw in the center of a symbol
    	sapply(dels, function(d) 
	    polygon(x=c(xd + d, xd + d - mutWidth, xd + d + mutWidth), y=c(y, y + mutHeight, 
	    y + mutHeight), col=colours[7], border=colours[7]) 
	)
    }
    ## print insertions as (green) triangle
    if(length(ins) > 0){
    	ins = ins * characterWidth
	xi = x - characterWidth
    	sapply(ins, function(i) 
	    polygon(x=c(xi + i, xi + i - mutWidth, xi + i + mutWidth), y=c(y, y + mutHeight, 
	    y + mutHeight), col=colours[8], border=colours[8]) 
	)
    }
    ## print matches as (black) triangle
    if(length(mms) > 0){
    	mms = mms * characterWidth
    	xm = x - 0.5 * characterWidth  ## draw in the center of a symbol
    	sapply(mms, function(m) 
	    polygon(x=c(xm + m, xm + m - mutWidth, xm + m + mutWidth), y=c(y, y + mutHeight, 
	    y + mutHeight), col=colours[9], border=colours[9]) 
	)
    }
}

## draw reads above and below the reference sequence
.plotReadsBasePairs <- function(alignments, names, directions, numReads, refPos, refHeight, readDist, refStart_left, localBrp_left, localBrp_right, 
	from, to, plotMut, colours, fontScale, fontFamily, characterWidth, characterHeight){

    yDist_above = 1
    yDist_below = 1

    for(i in 1:numReads){

	seq = toString(aligned(alignments)[i])

   	## x-position
        x = refStart_left

    	## y-position (depends on direction: distinguish between two directions: 3'-5' = below reference, 5'-3' = above reference)
	## forward (5'-3')
       	if(directions[i] == "forward"){
    	    y = refPos + refHeight + characterHeight + (yDist_above * readDist)
    	    yDist_above = yDist_above + 1	
	## reverse (3'-5')
	}else{
    	    y = refPos - characterHeight - (yDist_below * readDist)
    	    yDist_below = yDist_below + 1	
	    seq = toString(complement(DNAString(seq)))
	}

	## only plot basepairs within a given range (from, to; +/- maxBasePairs around the breakpoint)
	seq = substr(seq, from, to)

	## plot read and mutations
    	text(x=x, y=y, label= seq, pos=4, offset=0, cex=fontScale, family=fontFamily, col=colours[6])
    	if(plotMut){
    	    muts = .getMutations(alignments[i], from, to)
	    .plotMutationsBasePairPlot(x, y, muts[[1]], muts[[2]], muts[[3]], fontScale, fontFamily, characterWidth, characterHeight, colours)	    
	}

	## label each reads with its name
	labelOffset = 2
       	text(x=x, y=y, label=names[i], pos=2, offset = labelOffset, cex=fontScale)
	
    }	
}

##############################################################################
################# Main function ##############################################
##############################################################################

.plotChimericReads <- function(brpData, geneSymbols=FALSE, plotMut=TRUE, plotBasePairs=FALSE, maxBasePairs=50, legend=FALSE, title="", 
    col=c("red", "green", "black", "orange")){

    ## catch some bad input
    if(length(brpData) > 1)
	stop("invalid argument: please pass just one breakpoint cluster for plotting (try subsetting your breakpoint data)")
    if(!is.logical(plotMut))
	stop("invalid argument: please specify a logical value for plotMut")    
    if(!is.logical(plotBasePairs))
	stop("invalid argument: please specify a logical value for plotBasePairs")
    if(!is.logical(geneSymbols))
	if(!is.vector(geneSymbols, mode="character") | (length(geneSymbols) != 2))
	    stop("invalid argument: please specify a logical value or a vector of two strings for geneSymbols")
    if(!is.numeric(maxBasePairs) | (maxBasePairs < 1))
	stop("invalid argument: please specify value > 0 for maxBasePairs")
    if(maxBasePairs > 1000)
	warning(paste("sure you want to plot up to", 2*maxBasePairs, "base pairs of each read?"))
    if(!is.character(title))
	stop("invalid argument: title has to be of type character")
    if(!is.vector(col, mode="character") | (length(col) != 4))
	stop("invalid argument: please specify four valid colours for deletions, insertions, mismatches and breakpoints")

    ## some (heuristic) constant variables:
    ## scale and font family (font family only for basepairs in basePair-plot)
    fontScale = 0.7
    fontFamily = "mono"
    ## distance between reads, height of the reference sequence 
    if(!plotBasePairs){
      	readDistInit = 4
       	refHeight = 5
    }else{
    	readDistInit = 0.1	## distance of reads for initialization of plot window (later depending on character height, but not known yet)
    	refHeight = 0.1
    }
    ## maximum distance to a breakpoint say it belongs to another one (only needed to distinguish between both parts of an inversion)
    maxInvDist = 1000

    ## define plot colours
    ## in this order: reference background (left), reads background (left), reference background (right), reads background (right),
    ## breakpoints (standard plot only), gap (base pair plot only), reference/reads
    ## colors for deletions, insertions, mismatches and breqakpoints given by function argument (default: red, green, black, orange)
    colours = c("lightsteelblue", "lightsteelblue1", "lightyellow2", "ivory", "lightgrey", "black", col)


    ## initialize biomaRt
    if(class(geneSymbols) == "logical")
	if(geneSymbols)
    	    ensembl = useMart("ensembl", dataset="hsapiens_gene_ensembl")

    ## see if data requires two plots (case2 contains breakpoint and alignment data)
    if(nrow(commonBpsC2(brpData)[[1]]) == 0)
      numPlots = 1
    else
      numPlots = 2

    ## split plot window into two parts if necessary (horizontally for basepair-plot vertically for standard plot)
    if(numPlots == 2)
    	if(plotBasePairs)
    	    par(mfrow=c(2, 1), oma=c(0, 0, 2, 0), mar = c(0.5, 0.5, 0.5, 0.5))
    	else
            ## for horizontal alignment    
    	    par(mfrow=c(1,2), oma=c(0, 0, 2, 0), mar = c(0.5, 0.5, 0.5, 0.5))
#            ## for vertical alignment    
#    	    par(mfrow=c(2,1), oma=c(0, 0, 2, 0), mar = c(0.5, 0.5, 0.5, 0.5))
    else
    	par(mfrow=c(1,1), oma=c(0, 0, 2, 0), mar = c(0.5, 0.5, 0.5, 0.5))

    ## if the pathogenic and reciproc case are equally oriented (both show chrA <-> chrB), reverse case2 (reciproc case)
    ## in case of an inversion (chrA == chrB), compare the first two breakpoints instead of the first two chromosomes
    if(numPlots == 2){
    	if(commonBpsC1(brpData)[[1]]$chr[1] != commonBpsC1(brpData)[[1]]$chr[2]){
	    if(commonBpsC1(brpData)[[1]]$chr[1] == commonBpsC2(brpData)[[1]]$chr[1])
            	brpData = .reverseC2(brpData)
   	}else{
	    if(abs(commonBpsC1(brpData)[[1]]$breakpoint[1] - commonBpsC2(brpData)[[1]]$breakpoint[1]) < 100)
            	brpData = .reverseC2(brpData)
    	}
    }

    ## retrieve read names and directions (forward: 5'-3', reverse: 3'-5')
    ## according its the direction a read will be plotted below or above the reference sequence
    ##-> count the reads below and above the reference sequence to correctly initialize the plot window
    seqNames_all = list(length=numPlots)
    directions_all = list(length=numPlots)
    numReadsAbove_all = vector(mode="numeric", length=numPlots)
    numReadsBelow_all = vector(mode="numeric", length=numPlots)

    refStrands = paste(commonBpsC1(brpData)[[1]]$strand, collapse="/") ## note: strand information is formatted e.g. as "+/-" in class AlignedRead
    seqNames_all[[1]] = names(sread(alignedReadsC1(brpData)[[1]]))
    directions_all[[1]] = sapply(strand(alignedReadsC1(brpData)[[1]]), function(x) if(x != refStrands) return("reverse") else return("forward")) 
    numReadsAbove_all[1] = sum(directions_all[[1]] == "forward")
    numReadsBelow_all[1] = sum(directions_all[[1]] == "reverse")
    if(numPlots == 2){
    	refStrands = paste(commonBpsC2(brpData)[[1]]$strand, collapse="/") ## note: strand information is formatted e.g. as "+/-" in class AlignedRead
    	seqNames_all[[2]] = names(sread(alignedReadsC2(brpData)[[1]]))
    	directions_all[[2]] = sapply(strand(alignedReadsC2(brpData)[[1]]), function(x) if(x != refStrands) return("reverse") else return("forward")) 
    	numReadsAbove_all[2] = sum(directions_all[[2]] == "forward")
    	numReadsBelow_all[2] = sum(directions_all[[2]] == "reverse")
    }
    numReadsAbove_max = max(numReadsAbove_all)
    numReadsBelow_max = max(numReadsBelow_all)

    ## plot reference sequence and reads
    for(d in 1:numPlots){

	## get information from breakpoint data (list: 1.) breakpoints, 2.) alignments, 3.) reads)
    	if(d == 1){
	    breakpoints = commonBpsC1(brpData)[[1]]
 	    alignment = commonAlignC1(brpData)[[1]]
	}else{
	    breakpoints = commonBpsC2(brpData)[[1]]
 	    alignment = commonAlignC2(brpData)[[1]]
	}
    	numReads = length(alignment)

	## initialize all information about the reads (chromosomes and strands etc.)
	chr_left = part_left = breakpoints$chr[1]
    	chr_right = part_right = breakpoints$chr[2]
    	strand_left = breakpoints$strand[1]
    	strand_right = breakpoints$strand[2]
	geneSymbol_left = ""
	geneSymbol_right = ""
	seqNames = seqNames_all[[d]]
	directions = directions_all[[d]]
	numReadsAbove = numReadsAbove_all[[d]]
	numReadsBelow = numReadsBelow_all[[d]]

	## initialize all information about the reference (breakpoint etc.)
	refSeq = breakpoints$refSeq[1]
	brp_left = breakpoints$breakpoint[1]
	brp_right = breakpoints$breakpoint[2]
	localBrp_left = breakpoints$localEnd[1]
	localBrp_right = breakpoints$localStart[2]
	refLength_left = localBrp_left
	refLength_right = breakpoints$localEnd[2] - localBrp_right + 1

    	## set position labels (from original alignment data)
	if(strand_left == "+")
    	    label_start = brp_left - refLength_left
	else
    	    label_start = brp_left + refLength_left
	if(strand_right == "+")
    	    label_end = brp_right + refLength_right
	else
    	   label_end = brp_right - refLength_right
    	labels = c(label_start, brp_left, brp_right, label_end)

    	## load gene symbols from biomaRt
	if(class(geneSymbols) != "logical"){
	    geneSymbol_left = geneSymbols[1]
	    geneSymbol_right = geneSymbols[2]
	}else{
	    if(geneSymbols){
    	    	geneSymbol_left = getBM(attributes=c("hgnc_symbol"), 
        	    filters = c("chromosome_name", "start", "end"), 
		    values = list(chr_left, labels[2], labels[2]), 
		    mart = ensembl)
    	    	geneSymbol_right = getBM(attributes=c("hgnc_symbol"), 
        	    filters = c("chromosome_name", "start", "end"), 
		    values = list(chr_right, labels[3], labels[3]), 
		    mart=ensembl)
		## paste gene symbols if more than one reported
		geneSymbol_left = paste(geneSymbol_left$hgnc_symbol, collapse=",")
		geneSymbol_right = paste(geneSymbol_right$hgnc_symbol, collapse=",")
	    }
	}
	## assign colours and geneSymbols to the two parts (chromosomes) of the chimeric reads (only once for first plot)
	## in case of inversion (equal chromosome): assign a name (part1/2) to each part according to location of the breakpoint
        if(d == 1){
	    if(chr_left == chr_right){
		brp_part1 = brp_left
		part_left = "part1"
		part_right= "part2"
	    }
            chrProps = data.frame(
	        colourRef=c(colours[1], colours[3]), 
	        colourReads=c(colours[2], colours[4]),
	        geneSymbol=c(geneSymbol_left, geneSymbol_right),
	        row.names=c(part_left, part_right),
	        stringsAsFactors=FALSE
	    )
	}else{
	    if(chr_left == chr_right)
	        if(abs(brp_left - brp_part1) < maxInvDist){
	            part_left = "part1"
	    	    part_right= "part2"
		}else{
		    part_left = "part2"
		    part_right= "part1"
		}
   	}

	## initialize coordinates of the reference sequence
	refStart_left = 1
	refEnd_left = localBrp_left
	refStart_right = localBrp_right - 1
	refEnd_right = nchar(refSeq)

	## for base pair plots: only plot a subset of the base pairs around th breakpoint
    	maxBasePairs_left = min(maxBasePairs, refLength_left)
    	maxBasePairs_right = min(maxBasePairs, refLength_right)

    	from = localBrp_left - maxBasePairs_left + 1
    	to = localBrp_right + maxBasePairs_right - 1

	## initialize plot window
	if(!plotBasePairs){
    	    plotHeight = ((numReadsAbove_max + numReadsBelow_max + 2) * readDistInit) + refHeight
	    refPos = ((numReadsBelow_max + 1) * readDistInit) - (refHeight / 2)
	    ## colours for the legend: colours for dels, ins, mismatches and brp (legend only for first plot)
	    if(!legend | d == 2) 
		legendColours = c()
	    else 
		legendColours = colours[7:10]
	    legendSymbols = rep("|", 4)
	    init = .initializePlot(refStart_left, refEnd_right, plotHeight, plotBasePairs, legendColours, legendSymbols, fontScale, fontFamily)
	}
	else{
    	    plotHeight = ((numReadsAbove + numReadsBelow + 2) * readDistInit) + refHeight
	    refPos = ((numReadsBelow + 1) * readDistInit) - (refHeight / 2)
	    if(!legend | d == 2) 
		legendColours = c()
	    else 
		legendColours = colours[7:9]
	    legendSymbols = rep(6, 3)	## triangle
	    init = .initializePlot(refStart_left, refStart_right + maxBasePairs_right, plotHeight, plotBasePairs, legendColours, 
		legendSymbols, fontScale, fontFamily)
	}
    	characterWidth = init[[1]]
	characterHeight = init[[2]]
	readDist = characterHeight * 3

	## when plotting base pairs, adjust reference coordinates according to the character widths and plotting region
	if(plotBasePairs){
	    gap = refStart_right - refEnd_left
	    refEnd_left = refStart_left + (characterWidth * maxBasePairs_left)	## ends after symbol at poisiton maxBasePairs
	    refStart_right = refEnd_left + (characterWidth * gap)			## starts at symbol at poisiton maxBasePairs + 1
	    refEnd_right = refStart_right + (characterWidth * maxBasePairs_right)  	## ends after last symbol at poisiton 2*maxBasePairs
	}

	## draw reference
    	if(!plotBasePairs)
	    .plotReference(refHeight, refPos, refStart_left, refStart_right, refEnd_left, refEnd_right, part_left, part_right, chrProps, colours)
	else
	    .plotReferenceBasePairs(numReadsAbove, numReadsBelow, readDist, refHeight, refPos, refSeq, localBrp_left, localBrp_right,
		refStart_left, refStart_right, refEnd_left, refEnd_right, part_left, part_right, from, to, chrProps, colours, labels, 
		fontScale, fontFamily, characterWidth, characterHeight, maxBasePairs_left, maxBasePairs_right)
	.annotateReference(refPos, refHeight, refStart_left, refEnd_left, refStart_right, refEnd_right, 
	    chr_left, chr_right, part_left, part_right, strand_left, strand_right, labels, chrProps, fontScale, plotBasePairs)

    	## draw reads
    	if(!plotBasePairs)
	    .plotReads(alignment, seqNames, directions, numReads, refPos, refHeight, readDist, localBrp_left, localBrp_right,
	        refEnd_right, refStart_left, plotMut, colours, fontScale, characterHeight)
	else
	    .plotReadsBasePairs(alignment, seqNames, directions, numReads, refPos, refHeight, readDist, refStart_left, 
		localBrp_left, localBrp_right, from, to, plotMut, colours, fontScale, fontFamily, characterWidth, characterHeight)

    }

    ## Finally, the plot receives its title (if any)
    mtext(title, side=3, outer=TRUE) 
}

setMethod("plotChimericReads",
    signature(brpData="Breakpoints"),
    .plotChimericReads
)

### Getter & Setter ###
setMethod("name", signature(x="SFFRead"),
  function(x) {
    return(x@name)
  }
)
setReplaceMethod("name", signature(x="SFFRead", value="character"),
  function(x, value) {
    x@name = value
    return(x)
  }
)

setMethod("read", signature(x="SFFRead"),
  function(x) {
    return(x@read)
  }
)
setReplaceMethod("read", signature(x="SFFRead", value="QualityScaledDNAStringSet"),
  function(x, value) {
    x@read = value
    return(x)
  }
)

setMethod("flowgramFormat", signature(x="SFFRead"),
  function(x) {
    return(x@flowgramFormat)
  }
)
setReplaceMethod("flowgramFormat", signature(x="SFFRead", value="numeric"),
  function(x, value) {
    x@flowgramFormat = value
    return(x)
  }
)

setMethod("keySequence", signature(x="SFFRead"),
  function(x) {
    return(x@keySequence)
  }
)
setReplaceMethod("keySequence", signature(x="SFFRead", value="character"),
  function(x, value) {
    x@keySequence = value
    return(x)
  }
)

setMethod("flowChars", signature(x="SFFRead"),
  function(x) {
    return(x@flowChars)
  }
)
setReplaceMethod("flowChars", signature(x="SFFRead", value="character"),
  function(x, value) {
    x@flowChars = value
    return(x)
  }
)

setMethod("clipQualityLeft", signature(x="SFFRead"),
  function(x) {
    return(x@clipQualityLeft)
  }
)
setReplaceMethod("clipQualityLeft", signature(x="SFFRead", value="numeric"),
  function(x, value) {
    x@clipQualityLeft = value
    return(x)
  }
)

setMethod("clipQualityRight", signature(x="SFFRead"),
  function(x) {
    return(x@clipQualityRight)
  }
)
setReplaceMethod("clipQualityRight", signature(x="SFFRead", value="numeric"),
  function(x, value) {
    x@clipQualityRight = value
    return(x)
  }
)

setMethod("clipAdapterLeft", signature(x="SFFRead"),
  function(x) {
    return(x@clipAdapterLeft)
  }
)
setReplaceMethod("clipAdapterLeft", signature(x="SFFRead", value="numeric"),
  function(x, value) {
    x@clipAdapterLeft = value
    return(x)
  }
)

setMethod("clipAdapterRight", signature(x="SFFRead"),
  function(x) {
    return(x@clipAdapterRight)
  }
)
setReplaceMethod("clipAdapterRight", signature(x="SFFRead", value="numeric"),
  function(x, value) {
    x@clipAdapterRight = value
    return(x)
  }
)

setMethod("flowgram", signature(x="SFFRead"),
  function(x) {
    return(x@flowgram)
  }
)
setReplaceMethod("flowgram", signature(x="SFFRead", value="numeric"),
  function(x, value) {
    x@flowgram = value
    return(x)
  }
)

setMethod("flowIndexes", signature(x="SFFRead"),
  function(x) {
    return(x@flowIndexes)
  }
)
setReplaceMethod("flowIndexes", signature(x="SFFRead", value="numeric"),
  function(x, value) {
    x@flowIndexes = value
    return(x)
  }
)

setMethod("show",
  signature(object="SFFRead"),
  function(object){
    cat("Readname:\n")
    show(object@name)
    cat("\n")
    cat("Flow chars:\n")
    show(object@flowChars)
    cat("\n")
    cat("Key sequence:\n")
    show(object@keySequence)
    cat("\n")
    cat("Read:\n")
    show(object@read)
    cat("\n")
    cat("Quality:\n")
    show(object@quality)
    cat("\n")
    cat("Clip quality left:  ", object@clipQualityLeft, "\n", sep="")
    cat("Clip quality right: ", object@clipQualityRight, "\n", sep="")
    cat("Clip adapter left:  ", object@clipAdapterLeft, "\n", sep="")
    cat("Clip adapter right: ", object@clipAdapterRight, "\n", sep="")
    cat("\n")
    cat("Flowgram: \n")
    lfl = length(object@flowgram)
    if(lfl > 20) {
      cat(object@flowgram[1:10], "...", object@flowgram[(lfl-10):lfl], "\n", sep=" ")
    } else {
      cat(object@flowgram)
    }
    cat("\n")
    cat("Flow indexes: \n")
    lfi = length(object@flowIndexes)
    if(lfi > 20) {
      cat(object@flowIndexes[1:10], "...", object@flowIndexes[(lfi-10):lfi], "\n", sep=" ")
    } else {
      cat(object@flowIndexes)
    }
    cat("\n")
  }
)

setClass("Roche454BaseCallerMetrics",
    representation=list(
        dataTCAG="data.frame",
        dataCATG="data.frame",
        lengthTCAG="matrix",
        scoreTCAG="matrix",
        lengthCATG="matrix",
        scoreCATG="matrix"),
    prototype=prototype(
        dataTCAG=data.frame(),
        dataCATG=data.frame(),
        lengthTCAG=matrix(),
        scoreTCAG=matrix(),
        lengthCATG=matrix(),
        scoreCATG=matrix()),
    )


setClass("Roche454QualityFilterMetrics",
    representation=list(
        dataTotal="data.frame",
        dataTCAG="data.frame",
        dataCATG="data.frame"),
    prototype=prototype(
        dataTotal=data.frame(),
        dataTCAG=data.frame(),
        dataCATG=data.frame())
    )


setClass("AnnotatedVariants",
    representation=list(
        annotatedVariants="list"),
    prototype=prototype(
        annotatedVariants=list())
    )


setClass("Breakpoints",
    representation=list(
        seqsC1="list",
	seqsC2="list",
        commonBpsC1="list",
	commonBpsC2="list",
	commonAlignC1="list",
	commonAlignC2="list",
	alignedReadsC1="list",
	alignedReadsC2="list"),
    prototype=prototype(
        seqsC1=list(),
        seqsC2=list(),
        commonBpsC1=list(),
        commonBpsC2=list(),
        commonAlignC1=list(),
        commonAlignC2=list(),
        alignedReadsC1=list(),
        alignedReadsC2=list())
    )



# AVASet: class definition for the AVASet subclass of the original
# Biobase eSet
# extend object by two slots for amplicon data, two slots for variant filtering
# and a slot for the reference sequences
setClass("AVASet",
    contains = "eSet",
    representation=list(
        assayDataAmp="AssayData",
        featureDataAmp="AnnotatedDataFrame",
        referenceSequences="AlignedRead",
        variantFilterPerc="numeric",
	variantFilter="character",
	dirs="character"), 	
    prototype=prototype(
        assayDataAmp=assayDataNew(), 
	featureDataAmp=new("AnnotatedDataFrame"),
	referenceSequences=new("AlignedRead"),
	variantFilterPerc=0,
	variantFilter="",
	dirs="",
	new("VersionedBiobase", versions=c(classVersion("eSet"),
            AVASet="0.1"))
    )
)

# MapperSet: class definition for the MapperSet subclass of the original
# Biobase eSet
# extend object by two slots for variant filtering
setClass("MapperSet",
    contains = "eSet",
    representation=list(
        variantFilterPerc="numeric",
	variantFilter="character", 	
	dirs="character"), 	
    prototype=prototype(
	variantFilterPerc=0,
	variantFilter="",
	dirs="",
	new("VersionedBiobase", versions=c(classVersion("eSet"),
            MapperSet="0.1"))
    )
)

# SFFRead: representation of a single read from a Roche Standard Flowgram Format (SFF) file
setClass("SFFRead",
  representation=list(
    name               = "character",
    flowgramFormat     = "numeric",
    flowChars          = "character",
    keySequence        = "character",
    clipQualityLeft    = "numeric",
    clipQualityRight   = "numeric",
    clipAdapterLeft    = "numeric",
    clipAdapterRight   = "numeric",
    flowgram           = "numeric",
    flowIndexes        = "numeric",
    read               = "DNAString",
    quality            = "BString"
  ),
  prototype=prototype(
    name               = "",
    flowgramFormat     = 1,
    flowChars          = "",
    keySequence        = "",
    clipQualityLeft    = 0,
    clipQualityRight   = 0,
    clipAdapterLeft    = 0,
    clipAdapterRight   = 0,
    flowgram           = 0,
    flowIndexes        = 0,
    read               = DNAString(),
    quality            = BString()
  ),
  validity=function(object) {
    if(length(read(object)) != length(quality(object)) | length(read(object)) != length(flowIndexes(object)))
      return("Read, quality and flowIndexes must be of equal length.")
    if(nchar(flowChars(object)) != length(flowgram(object)))
      stop("Flowgram and flowChars must be of equal length.")
    return(TRUE)
  }
)

# SFFContainer: representation of a complete Roche Standard Flowgram Format (SFF) file
setClass("SFFContainer",
  representation=list(
    name               = "character",
    flowgramFormat     = "numeric",
    flowChars          = "character",
    keySequence        = "character",
    clipQualityLeft    = "numeric",
    clipQualityRight   = "numeric",
    clipAdapterLeft    = "numeric",
    clipAdapterRight   = "numeric", 
    flowgrams          = "list",
    flowIndexes        = "list",
    reads              = "QualityScaledDNAStringSet"
  ),
  prototype=prototype(
    name               = "",
    flowgramFormat     = 1,
    flowChars          = "",
    keySequence        = "",
    clipQualityLeft    = numeric(0),
    clipQualityRight   = numeric(0),
    clipAdapterLeft    = numeric(0),
    clipAdapterRight   = numeric(0),
    flowgrams          = list(),
    flowIndexes        = list(),
    reads              = QualityScaledDNAStringSet(DNAStringSet(), PhredQuality(""))
  ),
  validity=function(object) {
    if(length(reads(object)) != length(flowgrams(object)) |
       length(flowgrams(object)) != length(flowIndexes(sff)))
      return("Number of reads, flowgram length and flow indexes length must be equal")
    return(TRUE)
  }
)

.calculateTiTv <- function(object) {

  if (!is.element("variantBase", names(fData(object))) |
      !is.element("referenceBases", names(fData(object)))) {
    stop("Column variantBase or referenceBases is missing.")
  }

  df = fData(object)
  indSNP = nchar(df$referenceBases) == 1 & nchar(df$variantBase) == 1
  df = df[indSNP, ]

  subMat = matrix(NA, nrow=4, ncol=4,
      dimnames=list(c("A", "G", "C", "T"), c("A", "G", "C", "T")))
  subMat["A", "G"] = sum(df$referenceBases == "A" & df$variantBase == "G")
  subMat["A", "C"] = sum(df$referenceBases == "A" & df$variantBase == "C")
  subMat["A", "T"] = sum(df$referenceBases == "A" & df$variantBase == "T")
  subMat["G", "A"] = sum(df$referenceBases == "G" & df$variantBase == "A")
  subMat["G", "C"] = sum(df$referenceBases == "G" & df$variantBase == "C")
  subMat["G", "T"] = sum(df$referenceBases == "G" & df$variantBase == "T")
  subMat["C", "A"] = sum(df$referenceBases == "C" & df$variantBase == "A")
  subMat["C", "G"] = sum(df$referenceBases == "C" & df$variantBase == "G")
  subMat["C", "T"] = sum(df$referenceBases == "C" & df$variantBase == "T")
  subMat["T", "A"] = sum(df$referenceBases == "T" & df$variantBase == "A")
  subMat["T", "G"] = sum(df$referenceBases == "T" & df$variantBase == "G")
  subMat["T", "C"] = sum(df$referenceBases == "T" & df$variantBase == "C")
    
  # A transition is a A <-> G or a C <-> T substitution
  # http://www.mun.ca/biology/scarr/Transitions_vs_Transversions.html
  # there are twice as many transversion
  # expected values: http://www.broadinstitute.org/gsa/wiki/index.php/VariantEval
  ti = sum(subMat["A", "G"] + subMat["G", "A"]
      + subMat["C", "T"] + subMat["T", "C"])
  tv = sum(subMat, na.rm=TRUE) - ti

  return(list(subMat=subMat, TiTv= ti/tv))
}


setMethod("calculateTiTv", signature=signature(object="AVASet"),
    .calculateTiTv)

setMethod("calculateTiTv", signature=signature(object="MapperSet"),
    .calculateTiTv)

##############################
### Read length statistics ###
##############################
.readLengthStats <- function(object) {
  readLengths = width(object)
  stats = vector(mode="numeric", length=5)
  stats[1] = mean(readLengths)
  stats[2] = median(readLengths)
  stats[3] = min(readLengths)
  stats[4] = max(readLengths)
  stats[5] = sd(readLengths)
  stats = round(stats, digits=2)
  names(stats) = c("mean", "median", "min", "max", "sd")
  return(stats)
}

.readLengthStats_sr <- function(object) {
  return(.readLengthStats(sread(object)))
}

.readLengthStats_sff <- function(object) {
  return(.readLengthStats(reads(object)))
}

setMethod("readLengthStats", signature=signature(object="DNAStringSet"), .readLengthStats)
setMethod("readLengthStats", signature=signature(object="ShortRead"), .readLengthStats_sr)
setMethod("readLengthStats", signature=signature(object="SFFContainer"), .readLengthStats_sff)


.readLengthHist <- function(object, cutoff, xlab, ylab, col, breaks, ...) {
  args = list(...)
  readLengths = width(object)
  cut = quantile(readLengths, cutoff)
  readLengths = readLengths[readLengths <= cut]
  par(xaxs = "i")
  par(yaxs = "i")
  if (!is.element("main", names(args))) {
    main = "Histogram of the lengths of the sequences"
    hist(readLengths, main=main, xlab=xlab, ylab=ylab, xlim=c(0, max(readLengths)), breaks=breaks, 
    col=col, ...)
  } else {
    hist(readLengths, xlab=xlab, ylab=ylab, xlim=c(0, max(readLengths)), breaks=breaks, col=col, ...)
  }
}

.readLengthHist_sr <- function(object, cutoff, xlab, ylab, col, breaks, ...) {
  .readLengthHist(sread(object), cutoff, xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
}

.readLengthHist_sff <- function(object, cutoff, xlab, ylab, col, breaks, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Histogram of the lengths of the sequences \n", name(object), sep="")
    .readLengthHist(reads(object), cutoff, main=main, xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  } else {
    .readLengthHist(reads(object), cutoff, xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  }
}

setMethod("readLengthHist", signature(object="DNAStringSet"), .readLengthHist)
setMethod("readLengthHist", signature(object="ShortRead"), .readLengthHist_sr)
setMethod("readLengthHist", signature(object="SFFContainer"), .readLengthHist_sff)


###############################
### Base quality statistics ###
###############################
.baseQualityStats <- function(object) {
  qualityScores = sapply(as.character(quality(object)), function(x) as.integer(charToRaw(x))-33)
  stats = vector(mode="numeric", length=4)
  stats[1] = sum(sapply(qualityScores, sum)) / sum(width(object))
  stats[2] = min(sapply(qualityScores, min))
  stats[3] = max(sapply(qualityScores, max))
  stats[4] = sqrt(
    sum(sapply(qualityScores, function(x){sum((x-stats[1])^2)})) / 
    (sum(width(object))-1)
    )
  stats = round(stats, digits=2)
  names(stats) = c("mean", "min", "max", "sd")
  return(stats)
}

.baseQualityStats_sr <- function(object) {
  nu = as(quality(object), "numeric")
  stats = vector(mode="numeric", length=4)
  stats[1] = mean(nu)
  stats[2] = min(nu)
  stats[3] = max(nu)
  stats[4] = sd(nu)
  stats = round(stats, digits=2)
  names(stats) = c("mean", "min", "max", "sd")
  return(stats)

}

.baseQualityStats_sff <- function(object) {
  return(.baseQualityStats(reads(object)))
}

setMethod("baseQualityStats", signature(object="QualityScaledDNAStringSet"), .baseQualityStats)
setMethod("baseQualityStats", signature(object="ShortReadQ"), .baseQualityStats_sr)
setMethod("baseQualityStats", signature(object="SFFContainer"), .baseQualityStats_sff)


# Overall quality histogram
.baseQualityHist <- function(object, xlab, ylab, col, breaks, ...) {
  args = list(...)
  onestring = paste(as.character(quality(object)), collapse="")
  qualityScores = as.integer(charToRaw(onestring))-33
  par(xaxs = "i")
  par(yaxs = "i")
  if (!is.element("main", names(args))) {
    main = "Histogram of base quality scores"
    hist(qualityScores, main=main, xlab=xlab, ylab=ylab, breaks=breaks, col=col, ...)
  } else {
    hist(qualityScores, xlab=xlab, ylab=ylab, breaks=breaks, col=col, ...)
  }
}

.baseQualityHist_sr <- function(object, xlab, ylab, col, breaks, ...) {
  .baseQualityHist(as(object, "QualityScaledDNAStringSet"), xlab=xlab, ylab=ylab, col=col, 
    breaks=breaks, ...)
}

.baseQualityHist_sff <- function(object, xlab, ylab, col, breaks, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Histogram of base quality scores\n", name(object), sep="")
    .baseQualityHist(reads(object), main=main, xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  } else {
    .baseQualityHist(reads(object), xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  }
}

setMethod("baseQualityHist", signature(object="QualityScaledDNAStringSet"), .baseQualityHist)
setMethod("baseQualityHist", signature(object="ShortReadQ"), .baseQualityHist_sr)
setMethod("baseQualityHist", signature(object="SFFContainer"), .baseQualityHist_sff)


#mean quality per sequence histogram
.sequenceQualityHist <- function(object, xlab, ylab, col, ...) {
  args = list(...)
  qualityScores = sapply(as.character(quality(object)), function(x) as.integer(charToRaw(x))-33)
  if (!is.element("main", names(args)) & !is.element("breaks", names(args))) {
    main = "Histogram of mean quality scores per sequence"
    hist(sapply(qualityScores, mean), main=main, xlab=xlab, ylab=ylab, col=col, breaks=40, ...)
  } else if (!is.element("main", names(args)) & is.element("breaks", names(args))) {
    main = "Histogram of mean quality scores per sequence"
    hist(sapply(qualityScores, mean), main=main, xlab=xlab, ylab=ylab, col=col, ...)
  } else if (is.element("main", names(args)) & !is.element("breaks", names(args))) {
    hist(sapply(qualityScores, mean), xlab=xlab, ylab=ylab, col=col, breaks=40, ...)
  } else {
    hist(sapply(qualityScores, mean), xlab=xlab, ylab=ylab, col=col, ...)
  }
}

.sequenceQualityHist_sr <- function(object, xlab, ylab, col, ...) {
  args = list(...)
  ma = as(quality(object), "matrix")
  if (!is.element("main", names(args)) & !is.element("breaks", names(args))) {
    main = "Histogram of mean quality scores per sequence"
    hist(apply(ma, 1, mean), main=main, xlab=xlab, ylab=ylab, col=col, breaks=max(ma), ...)
  } else if (!is.element("main", names(args)) & is.element("breaks", names(args))) {
    main = "Histogram of mean quality scores per sequence"
    hist(apply(ma, 1, mean), main=main, xlab=xlab, ylab=ylab, col=col, ...)
  } else if (is.element("main", names(args)) & !is.element("breaks", names(args))) {
    hist(apply(ma, 1, mean), xlab=xlab, ylab=ylab, col=col, breaks=max(ma), ...)
  } else {
    hist(apply(ma, 1, mean), xlab=xlab, ylab=ylab, col=col, ...)
  }
}

.sequenceQualityHist_sff <- function(object, xlab, ylab, col, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Histogram of mean quality scores per sequence\n", name(object), sep="")
    .sequenceQualityHist(reads(object), main=main, xlab=xlab, ylab=ylab, col=col, ...)
  } else {
    .sequenceQualityHist(reads(object), xlab=xlab, ylab=ylab, col=col, ...)
  } 
}

setMethod("sequenceQualityHist", signature(object="QualityScaledDNAStringSet"), .sequenceQualityHist)
setMethod("sequenceQualityHist", signature(object="ShortReadQ"), .sequenceQualityHist_sr)
setMethod("sequenceQualityHist", signature(object="SFFContainer"), .sequenceQualityHist_sff)


#boxplot of quality per position
.positionQualityBoxplot <- function(object, range, binsize, xlab, ylab, col, ...) {
  args = list(...)
  qs = sapply(as.character(quality(object)), function(x) as.integer(charToRaw(x))-33)
  nreads = length(object)
  readlengths = width(object)
  nbases = max(readlengths)
  ma = matrix(NA, nrow=nreads, ncol=nbases) # empty matrix, size based on longest read
  for(i in 1:nreads) {
    ma[i, seq(along=qs[[i]])] = qs[[i]]
  }
  if (missing(range)) {
    range = c(1, nbases)
  }
  if(length(range) == 1) {
    start = 1
    end = range
  } else {
    start = range[1]
    end = range[2]
  }
  if(start > dim(ma)[2] | end > dim(ma)[2]) {
    stop("Values are out of range")
  }
  ma = ma[, start:end]
  goalsize = binsize * ceiling(dim(ma)[2] / binsize) # size dividable by binsize
  mb = matrix(NA, nrow=nreads, ncol=goalsize-dim(ma)[2]) # fill with NAs
  mc = cbind(ma, mb)
  md = matrix(mc, ncol=goalsize/binsize)
  if (!is.element("main", names(args))) {
    main = "Boxplot of quality per position"
    boxplot(md, outline=FALSE, col=col, axes=FALSE, main=main, xlab=xlab, ylab=ylab, frame=TRUE, 
    ylim=c(0, max(md, na.rm=TRUE)), ...)
  } else {
    boxplot(md, outline=FALSE, col=col, axes=FALSE, xlab=xlab, ylab=ylab, frame=TRUE, 
    ylim=c(0, max(md, na.rm=TRUE)), ...)
  }
  myLabels <- pretty(0:dim(ma)[2], n=4, min.n=1)
  myTicks <- myLabels/binsize
  axis(1, at=myTicks, labels=myLabels)
  axis(2, at=c(0, 10, 20, 30, 40), labels=TRUE)
}

.positionQualityBoxplot_sr <- function(object, range, binsize, xlab, ylab, col, ...) {
  args = list(...)
  ma = as(quality(object), "matrix")
  if (missing(range)) {
    range = c(1, dim(ma)[2])
  }
  if(length(range) == 1) {
    start = 1
    end = range
  } else {
    start = range[1]
    end = range[2]
  }
  if(start > dim(ma)[2] | end > dim(ma)[2]) {
    stop("Values are out of range")
  }
  ma = ma[, start:end]
  goalsize = binsize * ceiling(dim(ma)[2] / binsize) # size dividable by binsize
  mb = matrix(NA, nrow=dim(ma)[1], ncol=goalsize-dim(ma)[2]) # fill with NAs
  mc = cbind(ma, mb)
  md = matrix(mc, ncol=goalsize/binsize)
  maxq = max(md, na.rm=TRUE)
  if (!is.element("main", names(args))) {
    main = "Boxplot of quality per position"
    boxplot(md, outline=FALSE, col=col, axes=FALSE, main=main, xlab=xlab, ylab=ylab, frame=TRUE, 
    ylim=c(0, maxq), ...)
  } else {
    boxplot(md, outline=FALSE, col=col, axes=FALSE, xlab=xlab, ylab=ylab, frame=TRUE, 
    ylim=c(0, maxq), ...)
  }  
  myLabels <- pretty(0:dim(ma)[2], n=4, min.n=1)
  myTicks <- myLabels/binsize
  axis(1, at=myTicks, labels=myLabels)
  axis(2, at=c(0, 10, 20, 30, 40), labels=TRUE)
}

.positionQualityBoxplot_sff <- function(object, range, binsize, xlab, ylab, col, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Boxplot of quality per position\n", name(object), sep="")
    .positionQualityBoxplot(reads(object), range, binsize, main=main, xlab=xlab, ylab=ylab, col=col, ...)
  } else {
    .positionQualityBoxplot(reads(object), range, binsize, xlab=xlab, ylab=ylab, col=col, ...)
  }
}

setMethod("positionQualityBoxplot", signature(object="QualityScaledDNAStringSet"), .positionQualityBoxplot)
setMethod("positionQualityBoxplot", signature(object="ShortReadQ"), .positionQualityBoxplot_sr)
setMethod("positionQualityBoxplot", signature(object="SFFContainer"), .positionQualityBoxplot_sff)


####################################
### Base distribution statistics ###
####################################
.baseFrequency <- function(object) {
  af1 = alphabetFrequency(object, baseOnly=TRUE, collapse=TRUE)
  af2 = alphabetFrequency(object, as.prob=TRUE, baseOnly=TRUE, collapse=TRUE) * 100
  af1["total"] = sum(af1)
  af2["total"] = 100
  af2 = round(af2, digits=2)
  ff = data.frame(absolute=af1, relative=af2)
  return(ff)
}

.baseFrequency_sr <- function(object) {
  return(.baseFrequency(sread(object)))
}

.baseFrequency_sff <- function(object) {
  re = reads(object)
  return(.baseFrequency(re))
}

setMethod("baseFrequency", signature=signature(object="DNAStringSet"), .baseFrequency)
setMethod("baseFrequency", signature=signature(object="ShortRead"), .baseFrequency_sr)
setMethod("baseFrequency", signature=signature(object="SFFContainer"), .baseFrequency_sff)


.nucleotideCharts <- function(object, range, linetypes, linecols, xlab, ylab, ...) {
  args = list(...)
  readLengths = width(object)
  if(length(range) == 1) {
    start = 1
    end = quantile(readLengths, range)
  } else {
    start = range[1]
    end = range[2]
  }
  abc = alphabetByCycle(object, c("A", "C", "G", "T", "N"))
  nucsPerPos = colSums(abc)
  if(start > dim(abc)[2] | end > dim(abc)[2]) {
    stop("Values are out of range")
  }
  if (!is.element("main", names(args))) {
    main = "Nucleotide frequency"
    plot(x=start:end, y=abc["A", start:end]/nucsPerPos[start:end], type=linetypes["A"], 
      col=linecols["A"], main=main, xlab=xlab, ylab=ylab, ylim=c(0,1), ...)
  } else {
    plot(x=start:end, y=abc["A", start:end]/nucsPerPos[start:end], type=linetypes["A"], 
      col=linecols["A"], xlab=xlab, ylab=ylab, ylim=c(0,1), ...)
  }
  lines(x=start:end, y=abc["C", start:end]/nucsPerPos[start:end], type=linetypes["C"], 
    col=linecols["C"])
  lines(x=start:end, y=abc["G", start:end]/nucsPerPos[start:end], type=linetypes["G"], 
    col=linecols["G"])
  lines(x=start:end, y=abc["T", start:end]/nucsPerPos[start:end], type=linetypes["T"], 
    col=linecols["T"])
  lines(x=start:end, y=abc["N", start:end]/nucsPerPos[start:end], type=linetypes["N"], 
    col=linecols["N"])
  legend(x="top", horiz=TRUE, legend=c("A", "C", "G", "T", "N"), 
    fill=c(linecols["A"], linecols["C"], linecols["G"], linecols["T"], linecols["N"]))
}

.nucleotideCharts_sr <- function(object, range, linetypes, linecols, xlab, ylab, ...) {
  .nucleotideCharts(sread(object), range, linetypes=linetypes, linecols=linecols, xlab=xlab, 
    ylab=ylab, ...)
}

.nucleotideCharts_sff <- function(object, range, linetypes, linecols, xlab, ylab, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Nucleotide frequency\n", name(object), sep="")
    .nucleotideCharts(reads(object), range, linetypes=linetypes, linecols=linecols, main=main, 
      xlab=xlab, ylab=ylab, ...)
  } else {
    .nucleotideCharts(reads(object), range, linetypes=linetypes, linecols=linecols,
      xlab=xlab, ylab=ylab, ...)
  } 
}

setMethod("nucleotideCharts", signature(object="DNAStringSet"), .nucleotideCharts)
setMethod("nucleotideCharts", signature(object="ShortRead"), .nucleotideCharts_sr)
setMethod("nucleotideCharts", signature(object="SFFContainer"), .nucleotideCharts_sff)


.gcContent <- function(object) {
  af = alphabetFrequency(object, baseOnly=TRUE, collapse=TRUE)
  gc = ((af["G"] + af["C"]) / (af["A"] + af["C"] + af["G"] + af["T"])) * 100
  names(gc) = "GCcontent"
  gc = round(gc, digits=2)
  return(gc)
}

.gcContent_sr <- function(object) {
  return(.gcContent(sread(object)))
}

.gcContent_sff <- function(object) {
  reads = reads(object)
  return(.gcContent(reads))
}

setMethod("gcContent", signature(object="DNAStringSet"), .gcContent)
setMethod("gcContent", signature(object="ShortRead"), .gcContent_sr)
setMethod("gcContent", signature(object="SFFContainer"), .gcContent_sff)


.gcPerPosition <- function(object, range, type, col, xlab, ylab, ...) {
  args = list(...)
  readLengths = width(object)
  if(length(range) == 1) {
    start = 1
    end = quantile(readLengths, range)
  } else {
    start = range[1]
    end = range[2]
  }
  abc = alphabetByCycle(object, c("A", "C", "G", "T", "N"))
  nucsPerPos = colSums(abc)
  if(start > dim(abc)[2] | end > dim(abc)[2]) {
    stop("Values are out of range")
  }
  if (!is.element("main", names(args))) {
    main = "GC content per base"
    plot(x=start:end, y=(abc["G", start:end] + abc["C", start:end])/nucsPerPos[start:end], 
      type=type, col=col, main=main, xlab=xlab, ylab=ylab, ylim=c(0,1), ...)
  } else {
    plot(x=start:end, y=(abc["G", start:end] + abc["C", start:end])/nucsPerPos[start:end], 
      type=type, col=col, xlab=xlab, ylab=ylab, ylim=c(0,1), ...)
  }
}

.gcPerPosition_sr <- function(object, range, type, col, xlab, ylab, ...) {
  .gcPerPosition(sread(object), range, type=type, col=col, xlab=xlab, ylab=ylab, ...)
}

.gcPerPosition_sff <- function(object, range, type, col, xlab, ylab, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("GC content per base\n", name(object), sep="")
    .gcPerPosition(reads(object), range, main=main, type=type, col=col, xlab=xlab, ylab=ylab, ...)
  } else {
    .gcPerPosition(reads(object), range, type=type, col=col, xlab=xlab, ylab=ylab, ...)
  }
}

setMethod("gcPerPosition", signature(object="DNAStringSet"), .gcPerPosition)
setMethod("gcPerPosition", signature(object="ShortRead"), .gcPerPosition_sr)
setMethod("gcPerPosition", signature(object="SFFContainer"), .gcPerPosition_sff)


.gcContentHist <- function(object, xlab, ylab, col, breaks, ...) {
  args = list(...)
  af = alphabetFrequency(object, baseOnly=TRUE)
  gc = ((af[,"G"] + af[,"C"]) / (af[,"A"] + af[,"C"] + af[,"G"] + af[,"T"])) * 100
  if (!is.element("main", names(args))) {
    main = "Histogram of GC content per sequence"
    hist(gc, main=main, xlim=c(0,100), xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  } else {
    hist(gc, xlim=c(0,100), xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  }
}

.gcContentHist_sr <- function(object, xlab, ylab, col, breaks, ...) {
  .gcContentHist(sread(object), xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
}

.gcContentHist_sff <- function(object, xlab, ylab, col, breaks, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Histogram of GC content per sequence\n", name(object), sep="")
    .gcContentHist(reads(object), main=main, xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  } else {
    .gcContentHist(reads(object), xlab=xlab, ylab=ylab, col=col, breaks=breaks, ...)
  }
}

setMethod("gcContentHist", signature(object="DNAStringSet"), .gcContentHist)
setMethod("gcContentHist", signature(object="ShortRead"), .gcContentHist_sr)
setMethod("gcContentHist", signature(object="SFFContainer"), .gcContentHist_sff)


###########################
### Sequence complexity ###
###########################

### DUST approach ###
.complexity.dust <- function(object, xlab, ylab, xlim, col, breaks, ...) {
  args = list(...)
  scores = vector(mode="numeric", length=length(object))
  tfq = trinucleotideFrequency(object) # matrix mit nreads rows and 64 cols
  rls = width(object)
  scaling = ((rls - 2) * (rls - 2 - 1)) / ((rls - 2 - 1) * 2)
  scores = rowSums(tfq * (tfq - 1)) * 100 / (2 * scaling * (rls - 2 - 1))
  if (!is.element("main", names(args))) {
    main = "Histogram of DUST complexity scores"
    hist(scores, main=main, xlab=xlab, ylab=ylab, xlim=xlim, col=col, breaks=breaks, ...)
  } else {
    hist(scores, xlab=xlab, ylab=ylab, xlim=xlim, col=col, breaks=breaks, ...)
  }
  abline(v=7, col="blue")
  return(scores)
}

.complexity.dust_sr <- function(object, xlab, ylab, xlim, col, breaks, ...) {
  .complexity.dust(sread(object), xlab=xlab, ylab=ylab, xlim=xlim, col=col, breaks=breaks, ...)
}

.complexity.dust_sff <- function(object, xlab, ylab, xlim, col, breaks, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Histogram of DUST complexity scores\n", name(object), sep="")
    .complexity.dust(reads(object), main=main, xlab=xlab, ylab=ylab, xlim=xlim, col=col, 
      breaks=breaks, ...)
  } else {
    .complexity.dust(reads(object), xlab=xlab, ylab=ylab, xlim=xlim, col=col, breaks=breaks, ...)
  }
}

setMethod("complexity.dust", signature(object="DNAStringSet"), .complexity.dust)
setMethod("complexity.dust", signature(object="ShortRead"), .complexity.dust_sr)
setMethod("complexity.dust", signature(object="SFFContainer"), .complexity.dust_sff)

### Entropy approach ###
.complexity.entropy <- function(object, xlab, ylab, xlim, col, breaks, ...) {
  args = list(...)
  scores = vector(mode="numeric", length=length(object))
  tfq = trinucleotideFrequency(object) # matrix with nreads rows and 64 cols
  rls = width(object)
  fac1 = tfq / (rls - 2)
  fac2 = ifelse(fac1 == 0, 0, log(fac1, base=ifelse(rls < 66, rls - 2, 64)))
  scores = (-100) * rowSums(fac1 * fac2)
  if (!is.element("plot", names(args)) || args$plot == TRUE) {
    if (!is.element("main", names(args))) {
      main = "Histogram of Entropy complexity scores"
      hist(scores, main=main, xlab=xlab, ylab=ylab, xlim=xlim, breaks=breaks, col=col, ...)
    } else {
      hist(scores, xlab=xlab, ylab=ylab, xlim=xlim, breaks=breaks, col=col, ...)
    }
    abline(v=70, col="blue")
  }
  return(scores)
}

.complexity.entropy_sr <- function(object, xlab, ylab, xlim, col, breaks, ...) {
  .complexity.entropy(sread(object), xlab=xlab, ylab=ylab, xlim=xlim, col=col, breaks=breaks, ...)
}

.complexity.entropy_sff <- function(object, xlab, ylab, xlim, col, breaks, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Histogram of Entropy complexity scores\n", name(object), sep="")
    .complexity.entropy(reads(object), main=main, xlab=xlab, ylab=ylab, xlim=xlim, col=col, 
      breaks=breaks, ...)
  } else {
    .complexity.entropy(reads(object), xlab=xlab, ylab=ylab, xlim=xlim, col=col, breaks=breaks, ...)
  }
}

setMethod("complexity.entropy", signature(object="DNAStringSet"), .complexity.entropy)
setMethod("complexity.entropy", signature(object="ShortRead"), .complexity.entropy_sr)
setMethod("complexity.entropy", signature(object="SFFContainer"), .complexity.entropy_sff)

###############################
### Dinucleotide odds ratio ###
###############################
.dinucleotideOddsRatio <- function(object, xlab, col, ...) {
  args = list(...)
  dfq = dinucleotideFrequency(object) # matrix with nreads rows and 16 cols
  nfq = oligonucleotideFrequency(object, width=1) # matrix with nreads rows and 4 cols
  dinucs = c("AA", "AC", "AG", "AT", "CA", "CC", "CG", "CT", "GA", "GC", "GG", "GT", "TA", "TC", 
    "TG", "TT")
  rcdinucs = c("TT", "GT", "GC", "AT", "TG", "GG", "CG", "AG", "TC", "GC", "CC", "AC", "TA", "GA", 
    "CA", "AA")
  scorema = matrix(nrow=length(object), ncol=length(dinucs))
  colnames(scorema) = dinucs
  for(i in 1:length(dinucs)) {
    div1 = dfq[, dinucs[i]] + dfq[, rcdinucs[i]]
    div2 = nfq[, substr(dinucs[i], 1, 1)] + nfq[, substr(rcdinucs[i], 2, 2)]
    div3 = nfq[, substr(dinucs[i], 2, 2)] + nfq[, substr(rcdinucs[i], 1, 1)]
    div4 = 2 * (rowSums(nfq)^2) / rowSums(dfq)
    scorema[, dinucs[i]] = (div1 / (div2 * div3)) * div4
  }
  means = apply(scorema, 2, mean)
  if (!is.element("main", names(args))) {
    main = "Dinucleotide odds ratio"
    bar = barplot(means-1, main=main, xlab=xlab, horiz=TRUE, xlim=c(-1,1), col=col, las=1, 
      axes=FALSE, axisnames=FALSE, ...)
  } else {
    bar = barplot(means-1, xlab=xlab, horiz=TRUE, xlim=c(-1,1), col=col, las=1, axes=FALSE, 
      axisnames=FALSE, ...)
  }
  abline(v=0, col="black")
  axis(1, at=pretty(-1:1, n=4, min.n=1), labels=pretty(0:2, n=4, min.n=1))
  text(x=means-1+ifelse(means < 1, -0.08, 0.08), y=bar[, 1], labels=dinucs)
  return(scorema)
}

.dinucleotideOddsRatio_sr <- function(object, xlab, col, ...) {
  .dinucleotideOddsRatio(sread(object), xlab=xlab, col=col, ...)
}

.dinucleotideOddsRatio_sff <- function(object, xlab, col, ...) {
  args = list(...)
  if (!is.element("main", names(args))) {
    main = paste("Dinucleotide odds ratio\n", name(object), sep="")
    .dinucleotideOddsRatio(reads(object), main=main, xlab=xlab, col=col, ...)
  } else {
    .dinucleotideOddsRatio(reads(object), xlab=xlab, col=col, ...)
  }
}

setMethod("dinucleotideOddsRatio", signature(object="DNAStringSet"), .dinucleotideOddsRatio)
setMethod("dinucleotideOddsRatio", signature(object="ShortRead"), .dinucleotideOddsRatio_sr)
setMethod("dinucleotideOddsRatio", signature(object="SFFContainer"), .dinucleotideOddsRatio_sff)


###########################
### Flowgram statistics ###
###########################

.flowgramBarplot <- function(x, range, xlab, ylab, col, ...) {
  args = list(...)
  nflows = length(flowgram(x))
  if(length(range) != 2 | range[1] < 0 | range[2] > nflows | range[1] > range[2]) {
    stop("Illegal range values.")
  }
  flowChars = unlist(strsplit(flowChars(x), NULL))[range[1]:range[2]]
  flowgram = flowgram(x)[range[1]:range[2]]
  colors = col[match(flowChars, names(col))]
  if (!is.element("main", names(args))) {
    main = paste("Flowgram intensity\n", name(x), sep="")
    barplot(flowgram, main=main, xlab=xlab, ylab=ylab, col=colors, border=colors, width=1, space=0, ...)
  } else {
    barplot(flowgram, xlab=xlab, ylab=ylab, col=colors, border=colors, width=1, space=0, ...)
  }
  myLabels <- pretty(range[1]:range[2], n=5)
  myTicks <- myLabels-range[1]
  axis(1, at=myTicks, labels=myLabels)
  legend(x="top", horiz=TRUE, legend=c("A", "C", "G", "T"), 
    fill=c(col["A"], col["C"], col["G"], col["T"]))
}

setMethod("flowgramBarplot", signature(x="SFFRead"), .flowgramBarplot)


.homopolymerHist <- function(x, range, xlab, ylab, col, ...) {
  args = list(...)
  flowChars = unlist(strsplit(flowChars(x), NULL))[range[1]:range[2]]
  flowgram = round(flowgram(x)[range[1]:range[2]]/100)
  longest = max(flowgram)
  ta = table(factor(flowgram[flowChars == "A"], levels=0:longest))
  tc = table(factor(flowgram[flowChars == "C"], levels=0:longest))
  tg = table(factor(flowgram[flowChars == "G"], levels=0:longest))
  tt = table(factor(flowgram[flowChars == "T"], levels=0:longest))
  ma = matrix(c(ta,tc,tg,tt), nrow=4, byrow=TRUE)
  rownames(ma) = c("A", "C", "G", "T")
  colnames(ma) = 0:longest
  if (!is.element("main", names(args))) {
    main = paste("Homopolymers in read:\n", name(x), sep="")
    barplot(ma, beside=TRUE, main=main, xlab=xlab, ylab=ylab, col=col, ...)
  } else {
    barplot(ma, beside=TRUE, xlab=xlab, ylab=ylab, col=col, ...)
  }
  legend(x="topright", legend=c("A", "C", "G", "T"), 
    fill=c(col["A"], col["C"], col["G"], col["T"]))
  #as.data.frame(table(flowgram[flowChars == "A"], exclude=c(0,1)))
}

setMethod("homopolymerHist", signature(x="SFFRead"), .homopolymerHist)

.coverageOnTarget <- function(alnReads, targetRegion) {

    result = list()
    for (i in 1:length(alnReads)) {
        rd = GRanges(IRanges(start=alnReads[[i]]$pos,
                             end=alnReads[[i]]$pos + alnReads[[i]]$qwidth - 1),
                             seqnames=alnReads[[i]]$rname)
        cov = coverage(rd)

        chrs = as.character(intersect(names(cov), seqnames(targetRegion)))
        result[[i]] = numeric()
        for (chr in chrs) {
            ind = end(targetRegion[seqnames(targetRegion) == chr]) > sum(width(cov[[chr]]))
            end(targetRegion[seqnames(targetRegion) == chr])[ind] = sum(width(cov[[chr]]))
            tmpTC = unlist(aggregate(cov[[chr]], targetRegion[seqnames(targetRegion) == chr], FUN=as.numeric))
            names(tmpTC) = rep(chr, length(tmpTC))
            result[[i]] = as.integer(c(result[[i]], tmpTC))
        }
    }

    return(result)
}


setMethod("coverageOnTarget", 
    signature=signature(alnReads="list", targetRegion="GRanges"),
    .coverageOnTarget)



# targetRegion muss Klasse "RangedData" sein
# readStatus ab<C3><A4>ndern in GSMSet
#
#targetCoverage <- function(gsmSet, targetRegion) {
#
#    first = TRUE
#    readStatus = getReadStatus(gsmSet)
#    
#    for (i in 1:length(readStatus)) {
#        rStat = readStatus[[i]]
#        rStat = rStat[rStat$Status == "Full", ]
#
#        arRanges = RangedData(
#            ranges=IRanges(start=rStat$Start, end=rStat$Stop),
#            space=rStat$Accno.1)
#
#        chrs = intersect(names(arRanges), names(targetRegion))
#        tRanges = targetRegion[chrs]
#        rRanges = arRanges[chrs]
#
#        cov = countOverlaps(query=rRanges, subject=tRanges)
#        sov = subsetByOverlaps(query=rRanges, subject=tRanges)
#
#        targetCov = list()
#        for (i in 1:length(chrs)) {
#            targetCov[[i]] = coverage(rRanges[chrs[i]])[[chrs[i]]]
#            targetCov[[i]] = targetCov[[i]][ranges(tRanges[chrs[i]])[[chrs[i]]]]
#        }
#        names(targetCov) = chrs
#
#        df = data.frame(
#            NoMappedReads=nrow(rStat),
#            NoTargetReads=sum(sapply(cov, sum)),
#            NoMappedBases=sum(width(arRanges)),
#            NoTargetBases=sum(sapply(targetCov, sum)),
#            MeanTargetCoverage=mean(unlist(lapply(targetCov, as.integer))),
#            MedianTargetCoverage=median(unlist(lapply(targetCov, as.integer))))
#
#        if (first) {
#            summaryStats = df
#            first = FALSE
#        } else {
#            summaryStats = rbind(summaryStats, df)
#        }
#    }
#
#    rownames(summaryStats) = names(readStatus)
#    return(summaryStats)
#}

setMethod("annotatedVariants", signature(object="AnnotatedVariants"),
    function(object) {
        return(object@annotatedVariants)
    }
)

setReplaceMethod("annotatedVariants",
    signature=signature(object="AnnotatedVariants", value="list"),
    function(object, value) {
        object@annotatedVariants = value
        return(object)
    }
)

setMethod("names", signature(x="AnnotatedVariants"),
    function(x) {
        return(names(x@annotatedVariants))
    }
)

setReplaceMethod("names",
    signature=signature(x="AnnotatedVariants", value="character"),
    function(x, value) {
        names(x@annotatedVariants) = value
        return(x)
    }
)

setMethod("show", signature(object="AnnotatedVariants"),
    function(object) {
        cat(paste("An object of class AnnotatedVariants with",
            length(annotatedVariants(object)), "variants.\n"))
    }
)

# These are temporary methods, that are likely to be replaced by
# methods from the Rsamtools package in near future.


# takes a character vector with CIGAR strings, returns a list
extendedCIGARToList <- function(cigars) {
    indList = gregexpr("[A-Z]", cigars)
    cigarList = list()

    for (l in 1:length(indList)) {
        ind = c(0, indList[[l]])
        s = cigars[[l]]
        m = integer()
        for (i in 2:length(ind)) {
            m[i-1] = as.integer(substr(s, ind[i-1]+1, ind[i]-1))
            names(m)[i-1] = substr(s, ind[i], ind[i])
        }
        cigarList[[l]] = m
    }
    names(cigarList) = names(cigars)
    return(cigarList)
}

# takes a list with converted CIGAR string (see above) and returns a character vector
listToExtendedCIGAR <- function(cigarList){

    cigars = vector(mode="character")

    for (l in 1:length(cigarList))
	 cigars[l] = paste(paste(cigarList[[l]], names(cigarList[[l]]), sep=""), collapse="")

    names(cigars) = names(cigarList)
    return(cigars)
}

.detectBreakpoints <- function(chimericReads, bpDist=100, minClusterSize=4, removeSoftClips=TRUE, bsGenome) {

    # use library BSgenome.Hsapiens.UCSC.hg19 as default genome for common breakpoint detection
    if(missing(bsGenome)){
      if (!requireNamespace("BSgenome.Hsapiens.UCSC.hg19", quietly = TRUE)) {
        stop("library BSgenome.Hsapiens.UCSC.hg19 not found (please try to pass your own genome via function parameters)")
      }
      bsGenome = BSgenome.Hsapiens.UCSC.hg19::Hsapiens
    }
    
    # check sam/bam input
    reqFields = c("qname", "rname", "pos", "cigar", "seq", "strand", "qwidth")
    chimericReads[["log"]] = NULL # remove optional filtering log entry
    reads = lapply(names(chimericReads[[1]]), function(elt) {
        if (is.factor(chimericReads[[1]][[elt]])) {
            factor(do.call(c, unname(lapply(chimericReads, function (x) {
                as.character(x[[elt]])
            }))))
                                            
        } else
            do.call(c, unname(lapply(chimericReads, "[[", elt)))
        }
    )
    names(reads) = names(chimericReads[[1]])
    if (!all(is.element(reqFields, names(reads)))) {
        stop(paste("The following SAM fields are necessary:", reqFields))
    }
    reads = do.call("DataFrame", reads)
    reads = reads[order(reads$qname), ]
    
    # remove soft clips, which may be found at the beginning/end of some reads
    if(removeSoftClips){
      message("Removing soft-clipped bases ... ", appendLF = FALSE)
      reads = .removeSoftclips(reads)
      message("done")
    }

    # Build a data frame with one line for each chimeric reads. We can
    # assume that we only have chimeric reads with exactly two local
    # alignments. The data frame has these columns:
    #   p5Chr - Chromosome that the 5' end aligns to.
    #   p5Start - Start position of the 5' alignment. (ref. coordinates)
    #   p5End - End position of the 5' alignment. (reference coordinates)
    #   p5Strand - Strand of the 5' alignment.
    #   p5LocalStart - Start position of the 5' alignment. (read coordinates)
    #   p5LocalEnd - End position of the 5' alignment. (read coordinates)
    #   p5Breakpoint - Breakpoint of the 5' alignment
    #   p3Chr, p3Start, p3End, p3LocalStart, p3LocalEnd, p3Breakpoint - same for 3'
    #
    # Notes: For local coordinates, the first base of the 5' end has
    # coordinate 1. In SAM files, sequences that were aligned to the minus
    # strand of the reference are stored as reverse complement. In that case,
    # the local coordinates do not directly apply to the sequence in the sam
    # file but to its reverse complement.

    crList = split(reads, reads$qname)
    if (!all(sapply(crList, nrow) == 2)) {
        stop("Some of the given reads do not have exactly two
        local alignments.")
    }
    cReads = lapply(crList, function(x) {

        # extract number of matches, dels and ins from cigar
        cigars = extendedCIGARToList(x$cigar)
        dels = sapply(cigars, function(x) sum(x[names(x)=="D"]))
        ins = sapply(cigars, function(x) sum(x[names(x)=="I"]))
        matches = sapply(cigars, function(x) sum(x[names(x)=="M"]))

        # compute global start and end positions
        chr = as.character(x$rname)
        start = x$pos
        end = as.integer(start + matches + dels - 1)

        # compute local start and end position
        # counting starts at the reads 5 prime end
        localStart = as.integer(sapply(cigars, function(cgr) {
            ind = min(which(names(cgr) == "M"))
            cgr = c(0, cgr)
            return(sum(cgr[1:ind]) + 1)
        }))
        localEnd = as.integer(sapply(cigars, function(cgr) {
            ind = max(which(names(cgr) == "M"))
            return(sum(cgr[1:ind]))
        }) - dels)
        ind = x$strand == "-"
        if (sum(ind) > 0) {
            tmp = localStart
            localStart[ind] = x$qwidth[ind] - localEnd[ind] + 1
            localEnd[ind] = x$qwidth[ind] - tmp[ind] + 1
            x$seq[ind] = reverseComplement(x$seq[ind])
            rm(tmp)
        }

        # the smallest local start position is the 5' alignment
        ind5prime = localStart == min(localStart)

        return(data.frame(
            read=x$qname[1],
            p5Chr=chr[ind5prime],
            p5Start=start[ind5prime],
            p5End=end[ind5prime],
            p5Strand=as.character(x$strand[ind5prime]),
            p5LocalStart=localStart[ind5prime],
            p5LocalEnd=localEnd[ind5prime],
            p3Chr=chr[!ind5prime],
            p3Start=start[!ind5prime],
            p3End=end[!ind5prime],
            p3Strand=as.character(x$strand[!ind5prime]),
            p3LocalStart=localStart[!ind5prime],
            p3LocalEnd=localEnd[!ind5prime],
            seq=toString(x$seq[ind5prime]),
            stringsAsFactors=FALSE))
    })
    crInfo = data.frame()
    for (i in 1:length(cReads)) {
        crInfo = rbind(crInfo, cReads[[i]])
    }
    rm(cReads)
    rownames(crInfo) = crInfo$read

    # add breakpoints
    crInfo$p5Breakpoint = NA
    crInfo$p3Breakpoint = NA
    # 5 prime: positive strand
    ind = crInfo$p5Strand == "+"
    crInfo$p5Breakpoint[ind] = crInfo$p5End[ind]
    # 5 prime: negative strand
    ind = crInfo$p5Strand == "-"
    crInfo$p5Breakpoint[ind] = crInfo$p5Start[ind]
    # 3 prime: positive strand
    ind = crInfo$p3Strand == "+"
    crInfo$p3Breakpoint[ind] = crInfo$p3Start[ind]
    # 3 prime: negative strand
    ind = crInfo$p3Strand == "-"
    crInfo$p3Breakpoint[ind] = crInfo$p3End[ind]


    
    # cluster chimeric reads
    dMat = matrix(NA, nrow=nrow(crInfo), ncol=nrow(crInfo))
    rownames(dMat) = rownames(crInfo)
    colnames(dMat) = rownames(crInfo)
    for (i in 1:nrow(dMat)) {
        for (j in i:ncol(dMat)) {

            # first, consider strands: 
            # case 1: both alignment of both reads are from the same strand
            if (crInfo$p5Strand[i] == crInfo$p3Strand[i] &
                crInfo$p5Strand[j] == crInfo$p3Strand[j]) {

                # case 1.1: if the 5' alignments of both reads have the
                #   same strand, both must have the same chromosome
                #   -> same case, same strand
                if (crInfo$p5Strand[i] == crInfo$p5Strand[j] &
                    crInfo$p3Strand[i] == crInfo$p3Strand[j]) {

                    if (crInfo$p5Chr[i] == crInfo$p5Chr[j] &
                        crInfo$p3Chr[i] == crInfo$p3Chr[j]) {

                        dMat[i,j] = (crInfo$p5Breakpoint[i] - crInfo$p5Breakpoint[j])^2 +
                            (crInfo$p3Breakpoint[i] - crInfo$p3Breakpoint[j])^2

                    } else {
                        dMat[i,j] = Inf
                    }

                # case 1.2: if the 5' alignments of both reads are from
                #   different strand, the chromosome of the 5' alignment
                #   of one read must equal the chromosome of the 3'
                #   alignment of the other read
                #   -> same case, different strands
                } else if (crInfo$p5Strand[i] != crInfo$p5Strand[j] &
                    crInfo$p3Strand[i] != crInfo$p3Strand[j]) {

                    if (crInfo$p5Chr[i] == crInfo$p3Chr[j] &
                        crInfo$p3Chr[i] == crInfo$p5Chr[j]) {

                        dMat[i,j] = (crInfo$p5Breakpoint[i] - crInfo$p3Breakpoint[j])^2 +
                            (crInfo$p3Breakpoint[i] - crInfo$p5Breakpoint[j])^2

                    } else {
                        dMat[i,j] = Inf
                    }
                      

                }

            # case 2: alignments of both reads are from different strands
            } else if (crInfo$p5Strand[i] != crInfo$p3Strand[i] &
                crInfo$p5Strand[j] != crInfo$p3Strand[j]) {

                # In this case (strand change), both reads must have the
                # strands for the 5' and 3' alignment.
                if (crInfo$p5Strand[i] == crInfo$p5Strand[j] &
                    crInfo$p3Strand[i] == crInfo$p3Strand[j]) {

                    # case 2.0: Inversions. We know that both reads
                    #   are from the same case, but we cannot
                    #   distinguish between the strands
                    if (crInfo$p5Chr[i] == crInfo$p3Chr[i] &
                        crInfo$p5Chr[j] == crInfo$p3Chr[j] &
                        crInfo$p5Chr[i] == crInfo$p5Chr[j]) {

                        # same strand
                        d1 = (crInfo$p5Breakpoint[i] - crInfo$p5Breakpoint[j])^2 +
                            (crInfo$p3Breakpoint[i] - crInfo$p3Breakpoint[j])^2
                        # different strands
                        d2 = (crInfo$p5Breakpoint[i] - crInfo$p3Breakpoint[j])^2 +
                            (crInfo$p3Breakpoint[i] - crInfo$p5Breakpoint[j])^2
                
                        dMat[i,j] = min(d1, d2)

                    # case 2.1: Translocations. Alignments of both
                    #  reads are from the same chromomsomes and strands
                    #  -> same case, same strand
                    } else if (crInfo$p5Chr[i] == crInfo$p5Chr[j] &
                        crInfo$p3Chr[i] == crInfo$p3Chr[j]) {
                        
                        dMat[i,j] = (crInfo$p5Breakpoint[i] - crInfo$p5Breakpoint[j])^2 +
                            (crInfo$p3Breakpoint[i] - crInfo$p3Breakpoint[j])^2

                    # case 2.2: Translocations. Alignments of both
                    #  reads are from the same strands, but the chr
                    #  of the 5' alignment of one reads equals the
                    #  chr of the 3' alignmnet of the other read.
                    #  -> same case, different strands
                    } else if (crInfo$p5Chr[i] == crInfo$p3Chr[j] &
                        crInfo$p3Chr[i] == crInfo$p5Chr[j]) {

                        dMat[i,j] = (crInfo$p5Breakpoint[i] - crInfo$p3Breakpoint[j])^2 +
                            (crInfo$p3Breakpoint[i] - crInfo$p5Breakpoint[j])^2

                    # different chromosomes
                    } else {
                        dMat[i,j] = Inf
                    }

                
                # Reads are not from the same case
                } else {
                    dMat[i,j] = Inf
                }

            # case 3: both alignments of one read are from the same strand,
            #   whereas the aligment of the other reads are from different
            #   strands -> reads originate from different variants
            } else {
                dMat[i,j] = Inf
            }
        }
    }
    dMat[lower.tri(dMat, diag=FALSE)] = t(dMat)[lower.tri(dMat, diag=FALSE)]
    dMat[dMat == Inf] = max(10^10, 2 * bpDist)

    if (ncol(dMat) > 1) {
        hc = hclust(as.dist(dMat), method="complete")
        bpCluster = cutree(hc, h=bpDist)
        bpClusterSize = sort(table(bpCluster), decreasing=TRUE)
    } else {
        bpCluster = 1
        names(bpCluster) = colnames(dMat)
        bpClusterSize = 1
        names(bpClusterSize) = "1"
    }

    ## only use those cluster for further analysis whose size is greater or equal "minClusterSize" (see function arguments)
    bpClusterSize = bpClusterSize[bpClusterSize >= minClusterSize]
    cluster = names(bpClusterSize)

    ## take care, that reads in "crInfo" and "reads" are in the same order
    crInfo = crInfo[unique(reads$qname), ]
    
    message("Determining consensus breakpoints ... ", appendLF = FALSE)

    ## tolerance: bases read beyond the start and end of the reference sequence
    ## minimum tolerance for single read cluster
    baseTol = 250
    minTol = 10
  
    ## commonBps stores later all information about common Breakpoints and the alignments to the common reference sequences
    seqsC1 = list()
    seqsC2 = list()
    commonBpsC1 = list()
    commonBpsC2 = list()
    commonAlignC1 = list()
    commonAlignC2 = list()
    alignedReadsC1 = list()
    alignedReadsC2 = list()
    
    ## for each cluster, determine a breakpoint, that is shared by all reads (sort cluster from large to small)
    for(clusterInd in cluster){

      clReads = names(bpCluster[bpCluster == clusterInd])
      alnData = crInfo[clReads, ]
      clReads = reads[reads$qname %in% clReads, ]

      numReads = nrow(alnData)
      refSeqs = vector(mode="character")
      localBreakpoints = list()
      brps = list()
      chrs = list()
      strands = list()

      ## determine all possible breakpoints for every read (more than one in case of overlapping read parts)
      start = vector(length=(numReads*2), mode="numeric")
      end = vector(length=(numReads*2), mode="numeric")
      lStart = vector(length=(numReads*2), mode="numeric")
      lEnd = vector(length=(numReads*2), mode="numeric")
      breakpoint = vector(length=(numReads*2), mode="numeric")      

      ## use less tolerance left and right of the reference sequence for single read cluster
      if(numReads > 1)
        tol = baseTol
      else
        tol = minTol

      for(i in 1:numReads){

        
        ## read local, global and breakpoint coordinates (add it to the SAM information in "reads" below)
        if(alnData$p5Start[i] == clReads$pos[2*i-1])
          ind = c(2*i-1, 2*i)
        else
          ind = c(2*i, 2*i-1)          
        start[ind] = c(alnData$p5Start[i], alnData$p3Start[i])
        end[ind] = c(alnData$p5End[i], alnData$p3End[i])
        lStart[ind] = c(alnData$p5LocalStart[i], alnData$p3LocalStart[i])
        lEnd[ind] = c(alnData$p5LocalEnd[i], alnData$p3LocalEnd[i])
        breakpoint[ind] = c(alnData$p5Breakpoint[i], alnData$p3Breakpoint[i])

        ## see if both parts are overlapping or gapped
        seqDiff = alnData$p5LocalEnd[i] - alnData$p3LocalStart[i] + 1

        ## sequences overlap
        if(seqDiff >= 0){
          breakpoints1 = 0:seqDiff
          breakpoints2 = seqDiff:0          
        ## no overlapping (maybe gap between sequences)
        }else{
          breakpoints1 = 0
          breakpoints2 = 0	   	
        }
        
        strand1 = as.character(alnData$p5Strand[i])
        strand2 = as.character(alnData$p3Strand[i])
        chr1 = alnData$p5Chr[i]
        chr2 = alnData$p3Chr[i]
        
        bsGenomeList = list(bsGenome[[paste("chr", chr1, sep="")]], bsGenome[[paste("chr", chr2, sep="")]])
        
        for(j in 1:length(breakpoints1)){

          ## first part of the reference sequence (reverse sequence for neg. strands)
          if(strand1 == "-"){
            refStart1 = alnData$p5Start[i] + breakpoints1[j]
            refEnd1 = alnData$p5End[i] + tol
            refSeqPart1 = toString(subseq(bsGenomeList[[1]], refStart1, refEnd1))
            refSeqPart1 = toString(reverseComplement(DNAString(refSeqPart1)))
            brp1 = refStart1 
          }else{
            refStart1 = alnData$p5Start[i] - tol
            refEnd1 = alnData$p5End[i] - breakpoints1[j]
            refSeqPart1 = toString(subseq(bsGenomeList[[1]], refStart1, refEnd1))
            brp1 = refEnd1
          }
          
          ## second part of the reference sequence (reverse sequence for neg. strands)
          if(strand2 == "-"){
            refStart2 = alnData$p3Start[i] - tol
            refEnd2 = alnData$p3End[i] - breakpoints2[j]
            refSeqPart2 = toString(subseq(bsGenomeList[[2]], refStart2, refEnd2))
            refSeqPart2 = toString(reverseComplement(DNAString(refSeqPart2)))
            brp2 = refEnd2
          }else{
            refStart2 = alnData$p3Start[i] + breakpoints2[j]
            refEnd2 = alnData$p3End[i] + tol
            refSeqPart2 = toString(subseq(bsGenomeList[[2]], refStart2, refEnd2))
            brp2 = refStart2
          }
          
          ## put reference sequence together (first part, gap, second part)
          ## and remember its global and local coordinates, strands, chromosomes
          gap = substr(alnData$seq[i], alnData$p5LocalEnd[i] + 1,  alnData$p3LocalStart[i] - 1)
          brps[[i]] = c(brp1, brp2)
          localBreakpoints[[i]] = c(nchar(refSeqPart1), nchar(refSeqPart1) + nchar(gap) + 1)
          chrs[[i]] = c(chr1, chr2)
          strands[[i]] = c(strand1, strand2)
          refSeqs[i] = paste(refSeqPart1, gap, refSeqPart2, sep="")
        }
      }
      
      ## align reads to reference sequences
      numRefSeqs = length(refSeqs)
      scores = vector(length=numRefSeqs)
      alignments = list(length=numRefSeqs)
      alignedReads = list(length=numRefSeqs)

      for(r in 1:numRefSeqs){
        
        refSeq = refSeqs[r]
        refBps = brps[[r]]
        refp5Chr = chrs[[r]]
        alnReads = vector(length=numReads)
        alnChrs = vector(length(numReads), mode="character")
        alnStrands = vector(length(numReads), mode="character")
        
        ## use the reverse complement of sequences on complement strand
        ## (i.e. 5' chromosome differs between read and reference,
        ## or (in case of inversions) p5Breakpoint differs from 5' reference breakpoint:
        ## distance between breakpoints > 1.000bp (empirical threshold))
        for(j in 1:numReads){
          if(alnData$p5Chr[j] != refp5Chr[1] | abs(alnData$p5Breakpoint[j] - refBps[1]) > 1000){
            alnChrs[j] = paste(alnData$p3Chr[j], alnData$p5Chr[j],sep="/")
            alnStrands[j] = paste(alnData$p3Strand[j], alnData$p5Strand[j],sep="/")
            alnReads[j] = toString(reverseComplement(DNAString(alnData$seq[j])))
          }else{
            alnChrs[j] = paste(alnData$p5Chr[j], alnData$p3Chr[j],sep="/")
            alnStrands[j] = paste(alnData$p5Strand[j], alnData$p3Strand[j],sep="/")
            alnReads[j] = toString(alnData$seq[j])
          }
        }
        alnReads = DNAStringSet(alnReads)
        names(alnReads) = alnData$read
        
        ## alignment
        substitutionMatrix = nucleotideSubstitutionMatrix(match=1, mismatch=-1)
        alignment = pairwiseAlignment(pattern=alnReads, subject=refSeq, 
          type="global-local", substitutionMatrix=substitutionMatrix, gapOpening=0, gapExtension=1)
        scores[r] = sum(score(alignment))
        alignments[[r]] = alignment
        alignedReads[[r]] = AlignedRead(sread=alnReads, chromosome=factor(alnChrs), position=start(Views(alignment)), strand=factor(alnStrands))
      
      }
      ## add local, global and breakpoint coordinates to the alignment information in "reads"
      clReads$start = as.integer(start)
      clReads$end = as.integer(end)
      clReads$localStart = as.integer(lStart)
      clReads$localEnd = as.integer(lEnd)
      clReads$breakpoint = as.integer(breakpoint)
      rm(list=c("start", "end", "lStart", "lEnd", "breakpoint"))
      
      ## select breakpoints with maximum score, and the corresponding alignment
      maxScoreInd = which(scores == max(scores))[1]
      seqsC1[[clusterInd]] = alnData
      seqsC2[[clusterInd]] = data.frame()
      commonBpsC1[[clusterInd]] = data.frame(
                   "chr" = chrs[[maxScoreInd]],
                   "strand" = strands[[maxScoreInd]],
                   "breakpoint" = brps[[maxScoreInd]],
                   "localStart" = c(1, localBreakpoints[[maxScoreInd]][2]),
                   "localEnd" = c(localBreakpoints[[maxScoreInd]][1], nchar(refSeqs[maxScoreInd])),
                   "refSeq" = refSeqs[maxScoreInd],
                   stringsAsFactors = FALSE)
      commonBpsC2[[clusterInd]] = data.frame()
      commonAlignC1[[clusterInd]] = alignments[[maxScoreInd]]
      commonAlignC2[[clusterInd]] = NA
      alignedReadsC1[[clusterInd]] = alignedReads[[maxScoreInd]]
      alignedReadsC2[[clusterInd]] = AlignedRead()

    }
    if(length(cluster >= 1)){
      bpNames = paste("BP", 1:length(cluster), sep="")
      names(seqsC1) = bpNames
      names(seqsC2) = bpNames
      names(commonBpsC1) = bpNames
      names(commonBpsC2) = bpNames
      names(commonAlignC1) = bpNames
      names(commonAlignC2) = bpNames
      names(alignedReadsC1) = bpNames
      names(alignedReadsC2) = bpNames
    }    

    commonBps = new("Breakpoints", seqsC1=seqsC1, commonBpsC1=commonBpsC1, commonAlignC1=commonAlignC1, alignedReadsC1=alignedReadsC1, seqsC2=seqsC2, commonBpsC2=commonBpsC2, commonAlignC2=commonAlignC2, alignedReadsC2=alignedReadsC2)
    message("done")
    
    message("Detected ", length(cluster), " cluster of size >= ", minClusterSize)
    
    return(commonBps)

    
}

setMethod("detectBreakpoints",
          signature=signature(chimericReads="list"),
          .detectBreakpoints)


## Methods for preprocessing:


## Some reads may contain soft-clipped bases at the beginning of the first part of the read or 
## at the end of the second part. This function removes these unaligned (sub-)sequences
## and adjusts the cigar string, the sequence, the sequence width (qwidth) and the 
## local start/end coordinates

.removeSoftclips <- function(seqs){

   ## initialize vectors for updated entries (for some reason, updating entries directly in the IRanges DataFrame is relatively slow)
   numSeqs = nrow(seqs)
   newCigar = vector(length=numSeqs, mode="character")
   newSeq = vector(length=numSeqs, mode="character")
   newWidth = vector(length=numSeqs, mode="numeric")
   ## keep sequence names (if existent)
   if(!is.null(names(seqs$seq)))
      names(newSeq) = names(seqs$seq)

   ## 1. compute local start from cigar string and convert it for neg. strands

   # extract number of matches, dels and ins from cigar
   cigars = extendedCIGARToList(seqs$cigar)
   dels = sapply(cigars, function(x) sum(x[names(x)=="D"]))
   ins = sapply(cigars, function(x) sum(x[names(x)=="I"]))
   matches = sapply(cigars, function(x) sum(x[names(x)=="M"]))

   # compute local start and end position
   # counting starts at the reads 5 prime end
   localStart = as.integer(sapply(cigars, function(cgr) {
       ind = min(which(names(cgr) == "M"))
       cgr = c(0, cgr)
       return(sum(cgr[1:ind]) + 1)
   }))
   localEnd = as.integer(sapply(cigars, function(cgr) {
       ind = max(which(names(cgr) == "M"))
       return(sum(cgr[1:ind]))
   }) - dels)
   ind = seqs$strand == "-"
   if (sum(ind) > 0) {
       tmp = localStart
       localStart[ind] = seqs$qwidth[ind] - localEnd[ind] + 1
       localEnd[ind] = seqs$qwidth[ind] - tmp[ind] + 1
       rm(tmp)
   }
   
   for(i in seq(1, numSeqs, 2)){

     lStart1 = localStart[i]
     lStart2 = localStart[i + 1]

     ## 2. find out order of reads
     if(lStart1 < lStart2){
       part1 = i
       part2 = i + 1
     }else{
       part1 = i + 1
       part2 = i
     }

     ## 3. remove soft clip from the cigar string and the sequence itself
     ##    unify reads by reversing cigar string and sequence for reads on the negative strand
     ##    -> soft clip is the the beginning of the first part and at the end of the second part
     cig1 = extendedCIGARToList(seqs$cigar[part1])[[1]]
     cig2 = extendedCIGARToList(seqs$cigar[part2])[[1]]
     seq1 = seqs$seq[part1]
     seq2 = seqs$seq[part2]             
     sc1 = 0
     sc2 = 0

     if(seqs$strand[part1] == "-")
       cig1 = rev(cig1)
     if(seqs$strand[part2] == "-")
       cig2 = rev(cig2)

     ## test if any soft clips exist and remove their entry
     if(names(cig1[1]) == "S"){
       sc1 = cig1[1]               
       cig1 = cig1[-1]
     }
     if(names(cig2[length(cig2)]) == "S"){
       sc2 = cig2[length(cig2)]
       cig2 = cig2[-(length(cig2))]
     }

     ## adjust coordinates and cigar entries if soft clip is existent
     if(sc1 > 0 || sc2 > 0){

       ## adjust other cigar entries before/after the soft clip
       cig1[length(cig1)] = cig1[length(cig1)] - sc2
       cig2[1] = cig2[1] - sc1

       ## remove soft clip from sequence
       if(seqs$strand[part1] == "-")
         seq1 = reverseComplement(seq1)
       if(seqs$strand[part2] == "-")    
         seq2 = reverseComplement(seq2)                              
       seq1 = subseq(seq1, sc1 + 1, width(seq1) - sc2)
       seq2 = seq1
       if(seqs$strand[part1] == "-"){
         cig1 = rev(cig1)
         seq1 = reverseComplement(seq1)
       }
       if(seqs$strand[part2] == "-"){
         cig2 = rev(cig2)                 
         seq2 = reverseComplement(seq2)
       }
       
       ## update breakpoints object
       newCigar[c(part1, part2)] = c(listToExtendedCIGAR(list(cig1))[1], listToExtendedCIGAR(list(cig2))[1]) 
       newSeq[c(part1, part2)] = c(as.character(seq1), as.character(seq2))
       newWidth[c(part1, part2)] = c(seqs$qwidth[part1] - sc1 - sc2, seqs$qwidth[part2] - sc1 - sc2)
       
     }else{
       ## if no soft clip exists, keep old coordinates and cigar strings
       newCigar[c(part1, part2)] = c(seqs$cigar[part1], seqs$cigar[part2])
       newSeq[c(part1, part2)] = c(toString(seqs$seq[part1]), toString(seqs$seq[part2]))
       newWidth[c(part1, part2)] = c(seqs$qwidth[part1], seqs$qwidth[part2])
     }
   }

   ## update given IRanges DataFrame
   seqs$cigar = newCigar
   seqs$seq = DNAStringSet(newSeq, use.names=TRUE)
   seqs$qwidth = as.integer(newWidth)
   return(seqs)
}



## computes local coordinates (start/end of mapped region within a read) from a given cigar 
## note: cigar must be given as vector (see "extendedCIGARToList")

.computeLocalCoordinates <- function(cigar){
  
  dels = sum(cigar[names(cigar)=="D"])

  ## local start
  ind = min(which(names(cigar) == "M"))
  tmp = c(0, cigar)
  localStart = sum(tmp[1:ind]) + 1

  ## local end
  dels = sum(cigar[names(cigar)=="D"])
  ind = max(which(names(cigar) == "M"))
  localEnd = sum(cigar[1:ind]) - dels

  return(c(localStart, localEnd))
}

# private function doing the annotation - object must be a proper data frame
.annotateVariantBlock <- function(object, ensembl, ensemblSnp) {
#    dataset = "hsapiens_gene_ensembl"
#    snpDataset = "hsapiens_snp"

    object$chromosome = gsub("chr", "", object$chromosome)
    # create final annotation data structure
    variants = list()
    for (v in row.names(object)) {
        variants[[v]] = list(genes=data.frame(),
            transcripts=data.frame(),
            exons=data.frame(),
            snps=data.frame())
    }


    # variants as IRanges
    variantRanges = GRanges(IRanges(start=object$start, end=object$end),
        ID=row.names(object), strand=object$strand,
        seqSur=object$seqSur, seqRef=object$seqRef,
        seqMut=object$seqMut, seqnames=object$chromosome)


    # fetch all genomic information from ENSEMBL
    martRegions = paste(object$chromosome, object$start, object$end, sep=":")
#    ensembl = useMart("ensembl", dataset=dataset)
    bmAttributes = c(
        "ensembl_gene_id",
        "chromosome_name",
        "start_position",
        "end_position",
        "strand",
        "ensembl_transcript_id",
        "transcript_start",
        "transcript_end",
        "ensembl_exon_id",
        "exon_chrom_start",
        "exon_chrom_end",
        "rank",
        "5_utr_start",
        "5_utr_end",
        "3_utr_start",
        "3_utr_end",
        "cds_start",
        "cds_end",
        "cds_length"
    )
    cat("Fetching gene structure information from Ensembl ... ")
    bmInfo = getBM(attributes=bmAttributes, filters="chromosomal_region",
        values=martRegions, mart=ensembl)
    if (nrow(bmInfo) == 0) {
        cat("done.\n")
        return(variants)
    }
    bmInfoNames = getBM(attributes=c("ensembl_transcript_id",
        "external_gene_name", "external_transcript_name"),
        filters="ensembl_transcript_id",
        values=unique(bmInfo$ensembl_transcript_id),
        mart=ensembl)
    mInd = match(bmInfo$ensembl_transcript_id,
        bmInfoNames$ensembl_transcript_id)
    bmInfo$external_gene_id = bmInfoNames$external_gene_name[mInd]
    bmInfo$external_transcript_id = bmInfoNames$external_transcript_name[mInd]
    cat("done.\n")
  
  
    # annotate gene information
    cat("Processing affected genes ... ")
    cols = c("ensembl_gene_id", "start_position",
        "end_position", "chromosome_name", "strand", "external_gene_id")
    geneDF = bmInfo[!duplicated(bmInfo[,"ensembl_gene_id",]), cols]
    geneRanges = GRanges(IRanges(start=geneDF$start_position, end=geneDF$end_position),
        ensembl_gene_id=geneDF$ensembl_gene_id,
        strand=geneDF$strand,
        external_gene_id=geneDF$external_gene_id,
        seqnames=geneDF$chromosome_name)
    matchList = findOverlaps(query=ranges(geneRanges),
        subject=ranges(variantRanges))
    
    for (hit in setdiff(0:length(matchList), 0)) {
               varInd = subjectHits(matchList)[hit]
               geneInd = queryHits(matchList)[hit]
               variants[[variantRanges$ID[varInd]]]$genes =
               rbind(variants[[variantRanges$ID[varInd]]]$genes,
                   as.data.frame(geneRanges[geneInd]))
    }
    cat("done.\n")
  

    # annotate transcript information
    cat("Processing affected transcripts ... ")
    cols = c("ensembl_gene_id", "ensembl_transcript_id",
        "transcript_start", "transcript_end", "chromosome_name",
        "strand", "external_transcript_id")
    transcriptDF = bmInfo[!duplicated(bmInfo[,"ensembl_transcript_id",]), cols]
    transcriptRanges = GRanges(IRanges(start=transcriptDF$transcript_start,
        end=transcriptDF$transcript_end),
        ensembl_gene_id=transcriptDF$ensembl_gene_id,
        ensembl_transcript_id=transcriptDF$ensembl_transcript_id,
        external_transcript_id=transcriptDF$external_transcript_id,
        strand=transcriptDF$strand,
        seqnames=transcriptDF$chromosome_name)
    matchList = findOverlaps(query=ranges(transcriptRanges),
        subject=ranges(variantRanges))
    
    for (hit in setdiff(0:length(matchList), 0)) {
      varInd = subjectHits(matchList)[hit]
      transInd = queryHits(matchList)[hit]
      variants[[variantRanges$ID[varInd]]]$transcripts =
        rbind(variants[[variantRanges$ID[varInd]]]$transcripts,
              as.data.frame(transcriptRanges[transInd]))
    }
    
    cat("done.\n")
  

    # annotate exon information
    cat("Processing affected exons including coding regions and UTRs ... ")
    cols = c("ensembl_gene_id", "ensembl_transcript_id", "ensembl_exon_id",
        "exon_chrom_start", "exon_chrom_end", "rank",
        "5_utr_start", "5_utr_end", "3_utr_start", "3_utr_end",
        "cds_start", "cds_end", "cds_length", "chromosome_name", "strand")
    exonDF = bmInfo[!duplicated(
        bmInfo[,c("ensembl_transcript_id","ensembl_exon_id")]), cols]
    exonRanges = GRanges(
        IRanges(start=exonDF$exon_chrom_start, end=exonDF$exon_chrom_end),
        ensembl_gene_id=exonDF$ensembl_gene_id,
        ensembl_transcript_id=exonDF$ ensembl_transcript_id,
        ensembl_exon_id=exonDF$ensembl_exon_id,
        rank=exonDF$rank,
        X5_utr_start=exonDF[,"5_utr_start"],
        X5_utr_end=exonDF[,"5_utr_end"],
        X3_utr_start=exonDF[,"3_utr_start"],
        X3_utr_end=exonDF[,"3_utr_end"],
        cds_start=exonDF$cds_start,
        cds_end=exonDF$cds_end,
        cds_length=exonDF$cds_length,
        strand=exonDF$strand,
        seqnames=exonDF$chromosome_name)
    matchList = findOverlaps(query=ranges(exonRanges),
        subject=ranges(variantRanges))
    
    for (hit in setdiff(0:length(matchList), 0)) {
      varInd = subjectHits(matchList)[hit]
      exonInd = queryHits(matchList)[hit]

            tmpExon = as.data.frame(exonRanges[exonInd])
            tmpVar = as.data.frame(variantRanges[varInd])

            cds = .codingRegion(tmpExon)
      
            # Is mutation in 5'-UTR?
            if (is.na(tmpExon$X5_utr_start)) {
                tmpExon$utr_5 = FALSE
            } else {
                mm = as.matrix(findOverlaps(
                    IRanges(tmpExon$X5_utr_start, tmpExon$X5_utr_end),
                    IRanges(tmpVar$start, tmpVar$end)))
                tmpExon$utr_5 = nrow(mm) == 1
            }
      
            # Is mutation in 3'-UTR?
            if (is.na(tmpExon$X3_utr_start)) {
                tmpExon$utr_3 = FALSE
            } else {
                mm = as.matrix(findOverlaps(
                    IRanges(tmpExon$X3_utr_start, tmpExon$X3_utr_end),
                    IRanges(tmpVar$start, tmpVar$end)))
                tmpExon$utr_3 = nrow(mm) == 1
            }
      
            # Is mutation in coding region?
            if (is.na(cds[1])) {
                tmpExon$coding = FALSE
            } else {
                mm = as.matrix(findOverlaps(
                    IRanges(cds[1], cds[2]),
                    IRanges(tmpVar$start, tmpVar$end)))
                tmpExon$coding = nrow(mm) == 1
            }
      
            # If mutation is in coding region, we compute affected codons
            if (tmpExon$coding) {
                if (tmpExon$strand == "+") {
                    relCdsPosStart = tmpVar$start - cds[1] + tmpExon$cds_start
                    relCdsPosEnd = tmpVar$end - cds[1] + tmpExon$cds_start
                    seqSur = DNAString(as.character(tmpVar$seqSur))
                    seqMut = DNAString(as.character(tmpVar$seqMut))
                } else {
                    relCdsPosStart = cds[2] - tmpVar$end + tmpExon$cds_start
                    relCdsPosEnd = cds[2] - tmpVar$start + tmpExon$cds_start
                    seqSur = reverseComplement(
                        DNAString(as.character(tmpVar$seqSur)))
                    seqMut = reverseComplement(
                        DNAString(as.character(tmpVar$seqMut)))
                }
                if (tmpVar$strand == "-") {
                    seqSur = reverseComplement(seqSur)
                    seqMut = reverseComplement(seqMut)
                }

                # substitution
                if (substr(tmpVar$seqRef, 1, 1) != "-" &
                        substr(tmpVar$seqMut, 1, 1) != "-") {
                    tmpExon$numCodonStart = ceiling(relCdsPosStart / 3)
                    tmpExon$numCodonEnd = ceiling(relCdsPosEnd / 3)
                    relCodonPosStart = relCdsPosStart -
                        ((tmpExon$numCodonStart - 1) * 3)
                    relCodonPosEnd = relCdsPosEnd -
                        ((tmpExon$numCodonEnd - 1) * 3)

                    # two bases at each end
                    #tmpExon$codonRef = as.character(substr(seqSur,
                    #    4-relCodonPosStart, length(seqSur)+1-relCodonPosEnd))
                    # three bases at each end
                    tmpExon$codonRef = as.character(substr(seqSur,
                        5-relCodonPosStart, length(seqSur)-relCodonPosEnd))
                    tmpExon$codonMut = tmpExon$codonRef
                    substr(tmpExon$codonMut, relCodonPosStart,
                        nchar(tmpExon$codonMut)-(3-relCodonPosEnd)) =
                        as.character(seqMut)

                    tmpExon$AminoRef = as.character(translate(
                        DNAString(tmpExon$codonRef)))
                    tmpExon$AminoMut = as.character(translate(
                        DNAString(tmpExon$codonMut)))

                # Deletion
                } else if (substr(tmpVar$seqMut, 1, 1) == "-") {
                    tmpExon$numCodonStart = ceiling(relCdsPosStart / 3)
                    tmpExon$numCodonEnd = ceiling(relCdsPosEnd / 3)
                    relCodonPosStart = relCdsPosStart -
                        ((tmpExon$numCodonStart - 1) * 3)
                    relCodonPosEnd = relCdsPosEnd -
                        ((tmpExon$numCodonEnd - 1) * 3)
                
                    tmpExon$codonRef = as.character(substr(seqSur,
                        5-relCodonPosStart, length(seqSur)-relCodonPosEnd))
                    tmpExon$codonMut = tmpExon$codonRef
                    substr(tmpExon$codonMut, relCodonPosStart,
                        nchar(tmpExon$codonMut)-(3-relCodonPosEnd)) =
                        as.character(seqMut)
                    
                    tmpExon$AminoRef = as.character(translate(
                        DNAString(tmpExon$codonRef)))
                    codonMutSeq = gsub("-", "", tmpExon$codonMut)
                    if (nchar(codonMutSeq) == 0) {
                        tmpExon$AminoMut = ""
                    } else if (nchar(codonMutSeq) == 3) {
                        tmpExon$AminoMut = as.character(translate(
                            DNAString(codonMutSeq)))
                    } else if (nchar(codonMutSeq) > 3) { # 4 bases remain
                        tmpExon$AminoMut = paste(as.character(translate(
                            DNAString(substr(codonMutSeq, 1, 3)))),
                            "+ frame-shift")
                    } else if (nchar(codonMutSeq) == 1) { # 1 base remains
                        nextCodon = paste(codonMutSeq,
                            as.character(substr(seqSur,
                            length(seqSur)-relCodonPosEnd + 1,
                            length(seqSur)-relCodonPosEnd + 2)), sep="")
                        nextAS = as.character(translate(DNAString(nextCodon)))
                        if (nextAS == "*") {
                            tmpExon$AminoMut = nextAS
                        } else {
                            tmpExon$AminoMut = paste(nextAS, "+ frame-shift")
                        }
                    } else if (nchar(codonMutSeq) == 2) { # 2 base remain
                        nextCodon = paste(codonMutSeq,
                            as.character(substr(seqSur,
                            length(seqSur) - relCodonPosEnd + 1,
                            length(seqSur) - relCodonPosEnd + 1)), sep="")
                        nextAS = as.character(translate(DNAString(nextCodon)))
                        if (nextAS == "*") {
                            tmpExon$AminoMut = nextAS
                        } else {
                            tmpExon$AminoMut = paste(nextAS, "+ frame-shift")
                        }
                    }

                # Insertion
                } else if (substr(tmpVar$seqRef, 1, 1) == "-") {
                    tmpExon$numCodonStart = ceiling(relCdsPosStart / 3)
                    tmpExon$numCodonEnd = ceiling(relCdsPosEnd / 3)
                    relCodonPosStart = relCdsPosStart -
                        ((tmpExon$numCodonStart - 1) * 3)
                    relCodonPosEnd = relCdsPosEnd -
                        ((tmpExon$numCodonEnd - 1) * 3)
                    if (relCodonPosStart == 3) { # => relCodonPosEnd == 1
                        tmpExon$numCodonStart = tmpExon$numCodonStart + 1
                        lIns = length(seqMut)
                        fillCodon = c(0, 2, 1)[(lIns %% 3)+1]
                        tmpExon$codonRef = as.character(substr(seqSur,
                            start=4, stop=3+lIns+fillCodon))
                        tmpExon$codonMut = tmpExon$codonRef
                        substr(tmpExon$codonMut, 1, lIns) = as.character(seqMut)
                        
                    } else if (relCodonPosStart == 2) {
                        lIns = length(seqMut)
                        fillCodon = c(0, 2, 1)[((lIns+2) %% 3)+1]
                        tmpExon$codonRef = as.character(substr(seqSur,
                            start=2, stop=3+lIns+fillCodon))
                        tmpExon$codonMut = tmpExon$codonRef
                        substr(tmpExon$codonMut, 3, lIns+2) =
                            as.character(seqMut)
                        
                    } else if (relCodonPosStart == 1) {
                        lIns = length(seqMut)
                        fillCodon = c(0, 2, 1)[((lIns+1) %% 3)+1]
                        tmpExon$codonRef = as.character(substr(seqSur,
                            start=3, stop=3+lIns+fillCodon))
                        tmpExon$codonMut = tmpExon$codonRef
                        substr(tmpExon$codonMut, 2, lIns+1) =
                            as.character(seqMut)
                    }                    

                    tmpExon$AminoMut = as.character(translate(
                        DNAString(tmpExon$codonMut)))
                    codonRefSeq = gsub("-", "", tmpExon$codonRef)
                    if (nchar(codonRefSeq) == 0) {
                        tmpExon$AminoRef = ""
                    } else if (nchar(codonRefSeq) == 3) {
                        tmpExon$AminoRef = as.character(translate(
                            DNAString(codonRefSeq)))
                    } else if (nchar(codonRefSeq) > 3) { # 4 bases remain
                        tmpExon$AminoRef = paste(as.character(translate(
                            DNAString(substr(codonRefSeq, 1, 3)))),
                            "+ frame-shift")
                    } else { # 1 or 2 bases remain
                        tmpExon$AminoRef = "frame-shift"
                    }
                }

            # Mutation is not in coding region
            } else {
                tmpExon$numCodonStart = NA
                tmpExon$numCodonEnd = NA
                tmpExon$codonRef = NA
                tmpExon$codonMut = NA
                tmpExon$AminoRef = NA
                tmpExon$AminoMut = NA
            }

            variants[[variantRanges$ID[varInd]]]$exons =
              rbind(variants[[variantRanges$ID[varInd]]]$exons,
                    tmpExon[, c("seqnames", "start", "end", "width", "ensembl_gene_id",
                                "ensembl_transcript_id", "ensembl_exon_id", "rank",
                                "strand", "utr_5", "utr_3", "coding", "numCodonStart",
                                "numCodonEnd", "codonRef", "codonMut", "AminoRef", "AminoMut")])
        
    }
    cat("done.\n")


    # download SNP information
#    ensemblSnp = useMart("snp", dataset=snpDataset)
    bmAttributes = c(
        "refsnp_id",
        "chr_name",
        "chrom_start",
        "chrom_strand",
        "allele")
    cat("Fetching SNP information from Ensembl ... ")
    bmInfo = getBM(attributes=bmAttributes, filters="chromosomal_region",
        values=martRegions, mart=ensemblSnp)
    cat("done.\n")
#    bmInfo$A = sapply(strsplit(bmInfo$allele, "/"), function(x) {x[1]})
#    bmInfo$B = sapply(strsplit(bmInfo$allele, "/"), function(x) {x[2]})


    # annotate SNP informations
    cat("Processing known SNPs ... ")
    snpDF = bmInfo
    if (nrow(snpDF) > 0) {
        snpDF$A = sapply(strsplit(snpDF$allele, "/"), function(x) {x[1]})
        snpDF$B = sapply(strsplit(snpDF$allele, "/"), function(x) {x[2]})        
        snpRanges = GRanges(IRanges(start=snpDF$chrom_start,
            end=snpDF$chrom_start + pmax(nchar(snpDF$A), nchar(snpDF$B)) - 1),
            refsnp_id=snpDF$refsnp_id,
            chrom_strand=snpDF$chrom_strand,
            allele=snpDF$allele,
            A=snpDF$A,
            B=snpDF$B,
            seqnames=snpDF$chr_name)
        matchList = findOverlaps(query=ranges(snpRanges),
            subject=ranges(variantRanges))
     
        for (hit in setdiff(0:length(matchList), 0)) {
            varInd = subjectHits(matchList)[hit]
            snpInd = queryHits(matchList)[hit]
            
                tmpSNP = as.data.frame(snpRanges[snpInd])
                tmpVar = as.data.frame(variantRanges[varInd])

                # We cannot process entries of class "HGMD_MUTATION", because
                # no alleles are given.
                if (tmpSNP$allele != "HGMD_MUTATION") {

                    # test whether the known SNP is the variant
                
                    sA = as.character(tmpSNP$A)
                    vA = as.character(tmpVar$seqRef)
                    if (substr(vA, start=1, stop=1) == "-") {
                        vA = "-"
                    }
                    sB = as.character(tmpSNP$B)
                    vB = as.character(tmpVar$seqMut)
                    if (substr(vB, start=1, stop=1) == "-") {
                        vB = "-"
                    }
                    if ((tmpSNP$chrom_strand == "+" & tmpVar$strand == "-") |
                        (tmpSNP$chrom_strand == "-" & tmpVar$strand == "+")) {
                        sA = as.character(complement(DNAString(sA)))
                        sB = as.character(complement(DNAString(sB)))
                    }                
                    if ((sA == vA & sB == vB & tmpSNP$start == tmpVar$start &
                            tmpSNP$end == tmpVar$end) |
                            (sA == vA & sB == vB & tmpSNP$start == tmpVar$end &
                            tmpSNP$end == tmpVar$start) |
                            (sB == vA & sA == vB & tmpSNP$start == tmpVar$start &
                            tmpSNP$end == tmpVar$end) |
                            (sB == vA & sA == vB & tmpSNP$start == tmpVar$end &
                            tmpSNP$end == tmpVar$start)) {
                        tmpSNP$identical = TRUE
                    } else {
                        tmpSNP$identical = FALSE
                    }
                } else { # HGMD_MUTATIONs
                    tmpSNP$identical = FALSE
                }

                variants[[variantRanges$ID[varInd]]]$snps =
                  rbind(variants[[variantRanges$ID[varInd]]]$snps,
                        tmpSNP)
            
        }
    }
    cat("done.\n")
  
    annoVars = new("AnnotatedVariants")
    annotatedVariants(annoVars) = variants
    return(annoVars)
}



.annotateVariants_AVASet <- function(object){
    refSeqs=referenceSequences(object)

    if(any(is.na(chromosome(refSeqs)))){
      stop("Aligned reference sequences are required for variant annotation. Please use the function alignShortReads first to perform the alignment.")
      return(NULL)
    }
    
    mInd=match(fData(object)$referenceSeqID, as.character(id(refSeqs)))

    variantData=data.frame(
        start=NA,
        end=NA,
        chromosome=chromosome(refSeqs)[mInd],
        strand=strand(refSeqs)[mInd],
        seqRef=fData(object)$referenceBases,
        seqSur=substr(sread(refSeqs)[mInd], fData(object)$start - 3,
            fData(object)$end + 3),
        seqMut=fData(object)$variantBase,
        row.names=row.names(fData(object)),
        stringsAsFactors=FALSE
    )

    RefStart = position(refSeqs)[mInd]
    RefEnd = position(refSeqs)[mInd] + width(refSeqs)[mInd] - 1
    ind = variantData$strand == "+"

    # plus strand    
    variantData$start[ind] = RefStart[ind] + fData(object)$start[ind] - 1
    variantData$end[ind] = RefStart[ind] + fData(object)$end[ind] - 1
    # minus strand
    variantData$start[!ind] = RefEnd[!ind] - fData(object)$end[!ind] + 1
    variantData$end[!ind] = RefEnd[!ind] - fData(object)$start[!ind] + 1
    
    return(annotateVariants(variantData))
}


.annotateVariants_MapperSet <- function(object, bsGenome){

    chrs = as.character(fData(object)$chr)
    starts = fData(object)$start
    ends = fData(object)$end

    ## determine surroundings of +/-3 based in reference sequence
    if (missing(bsGenome)) {
        if (!requireNamespace("BSgenome.Hsapiens.UCSC.hg19", quietly = TRUE)) {
            stop("Package BSgenome.Hsapiens.UCSC.hg19 is required if no genome is specified.")
        }
        bsGenome = BSgenome.Hsapiens.UCSC.hg19::Hsapiens
    }    
    surr = vector(mode="character", length=length(chrs))
    for(i in 1:length(chrs))
	surr[i] = toString(subseq(bsGenome[[paste("chr", chrs[i], sep="")]], starts[i] - 3, ends[i] + 3))

    ## prepare data frame with variant data
    variantData = data.frame(
        start = starts,
        end = ends,
        chromosome = chrs,
        strand = fData(object)$strand,
        seqRef = fData(object)$referenceBases,
        seqSur = surr,
        seqMut = fData(object)$variantBase,
        row.names=row.names(fData(object)),
        stringsAsFactors=FALSE
    )

    return(annotateVariants(variantData))
}


.annotateVariants_data.frame <- function(object) {

    blockSize = 40
    dataset = "hsapiens_gene_ensembl"
    snpDataset = "hsapiens_snp"

    ensembl = useMart("ENSEMBL_MART_ENSEMBL", dataset=dataset, host="http://grch37.ensembl.org")
    ensemblSnp = useMart("ENSEMBL_MART_SNP", dataset=snpDataset, host="http://grch37.ensembl.org")
    # ensemblSnp = useMart("snp", dataset=snpDataset, host="http://grch37.ensembl.org")
    
    av = new("AnnotatedVariants")
    i = 1
    while (i <= nrow(object)) {
        j = min(i + blockSize - 1, nrow(object))
        message(paste("Annotating variants ", i, " to ", j, ".", sep=""))
        subDf = object[i:j, ]
        subAv = .annotateVariantBlock(subDf, ensembl, ensemblSnp)
        annotatedVariants(av) = c(annotatedVariants(av), annotatedVariants(subAv))              
        i = i + blockSize
    }
    return(av)
}


setMethod("annotateVariants", signature=signature(object="AVASet", bsGenome="missing"),
    .annotateVariants_AVASet)

setMethod("annotateVariants", signature=signature(object="MapperSet", bsGenome="missing"),
    .annotateVariants_MapperSet)

setMethod("annotateVariants", signature=signature(object="MapperSet", bsGenome="BSgenome"),
    .annotateVariants_MapperSet)

setMethod("annotateVariants", signature=signature(object="data.frame", bsGenome="missing"),
    .annotateVariants_data.frame)






# private (not exported) function
#
# returns the absolute position of the coding region c(start, end)
# c(NA, NA) means that the exon has no coding region
.codingRegion <- function(exon) {

    cdsStart = NA
    cdsEnd = NA

    # some transcripts are not protein coding at all - fetch these cases first
    if (is.na(exon$cds_end) & is.na(exon$X5_utr_end) & is.na(exon$X3_utr_end)) {
        return(c(cdsStart, cdsEnd))
    }

    # whole exon is coding
    if (is.na(exon$X5_utr_end) & is.na(exon$X3_utr_end)) {
        cdsStart = exon$start
        cdsEnd = exon$end

    # exon has 5'-UTR
    } else if (!is.na(exon$X5_utr_end) & is.na(exon$X3_utr_end)) {
        if (exon$strand == "+") {
            if (exon$X5_utr_end == exon$end) {
                cdsStart = NA
                cdsEnd = NA
            } else {
                cdsStart = exon$X5_utr_end + 1
                cdsEnd = exon$end
            }
        } else {
            if (exon$X5_utr_start == exon$start) {
                cdsStart = NA
                cdsEnd = NA
            } else {
                cdsStart = exon$start
                cdsEnd = exon$X5_utr_start - 1
            }
        }

    # exon has 3'-UTR
    } else if (is.na(exon$X5_utr_end) & !is.na(exon$X3_utr_end)) {
        if (exon$strand == "+") {
            if (exon$X3_utr_start == exon$start) {
                cdsStart = NA
                cdsEnd = NA
            } else {
                cdsStart = exon$start
                cdsEnd = exon$X3_utr_start - 1
            }
        } else {
            if (exon$X3_utr_end == exon$end) {
                cdsStart = NA
                cdsEnd = NA
        } else {
                cdsStart = exon$X3_utr_end + 1
                cdsEnd = exon$end
        }
    }

    # exon has 5'-UTR and 3'-UTR => coding region must be in between
    } else if (!is.na(exon$X5_utr_end) & !is.na(exon$X3_utr_end)) {
        if (exon$strand == "+") {
            cdsStart = exon$X5_utr_end + 1
            cdsEnd =exon$X3_utr_start - 1
        } else {
            cdsStart = exon$X3_utr_end + 1
            cdsEnd = exon$X5_utr_start - 1
        }
    }
  
    return(c(cdsStart, cdsEnd))
}

.read454BaseCallerMetrics_character <- function(object){

    bcm = read.table(file=object, sep=",",
                     colClasses=c("character","character", "character"),
                     fill=TRUE, col.names=c("V1", "V2", "V3"))

    # Extract the user specified analysis name, i.e. the last part of the
    # analysis directory name.
    ind = bcm[,1] == "Rig Result Directory:"
    analysisName = unlist(strsplit(bcm[ind, 2], "_"))
    analysisName = analysisName[length(analysisName)]

    # Extract the pico titer plate ID.
    ind = bcm[,1] == "PTPBarCode:"
    picoTiterPlateID = as.numeric(bcm[ind,2])

    # Extract reads' length and bases' quality for each
    # region and key
    regInd = which(bcm[,1] == "region:")
    keyInd = which(bcm[,1] == "key:")
    lengthInd = which(bcm[,1] == "Length")
    scoreInd = which(bcm[,1] == "Score")

    dataTCAG = data.frame()
    dataCATG = data.frame()
    lengthTCAG = matrix()
    scoreTCAG = matrix()
    lengthCATG = matrix()
    scoreCATG = matrix()
    
    for (r in 1:length(regInd)) {
        l = as.numeric(bcm[(lengthInd[r] + 1):(scoreInd[r]-1), 1])
        cl = as.numeric(bcm[(lengthInd[r] + 1):(scoreInd[r]-1), 2])

        if (r == length(regInd)) {
          endS = nrow(bcm)        
        } else {
          endS = regInd[r+1] - 1
        }
        s = as.numeric(bcm[(scoreInd[r] + 1):endS, 1])
        cs = as.numeric(bcm[(scoreInd[r] + 1):endS, 2])

        if (bcm[keyInd[r], 2] == "TCA") {
            ind = nrow(dataTCAG) + 1

            m = max(l, nrow(lengthTCAG))
            old = lengthTCAG
            lengthTCAG = matrix(0, nrow=m, ncol=ind)
            lengthTCAG[1:nrow(old), ind-1] = old
            lengthTCAG[l, ind] = cl
        
            m = max(s+1, nrow(scoreTCAG))
            old = scoreTCAG
            scoreTCAG = matrix(0, nrow=m, ncol=ind)
            scoreTCAG[1:nrow(old), ind-1] = old
            scoreTCAG[s+1, ind] = cs

            allSeqsLengths = rep(1:nrow(lengthTCAG), lengthTCAG[, ind])
            allBaseScores = rep(1:nrow(scoreTCAG), scoreTCAG[, ind])
            dataTCAG[ind, "AnalysisName"] = analysisName
            dataTCAG[ind, "PicoTiterPlateID"] = picoTiterPlateID
            dataTCAG[ind, "Region"] = bcm[regInd[r], 2]
            dataTCAG[ind, "NumberOfReads"] = length(allSeqsLengths)
            dataTCAG[ind, "NumberOfBases"] = length(allBaseScores)
            dataTCAG[ind, "ReadLengthMean"] = mean(allSeqsLengths)
            dataTCAG[ind, "ReadLengthStd"] = sd(allSeqsLengths)
            dataTCAG[ind, "ReadLengthMedian"] = median(allSeqsLengths)
            dataTCAG[ind, "ReadLengthMAD"] = mad(allSeqsLengths, constant=1)
            dataTCAG[ind, "ReadScoreMean"] = mean(allBaseScores)
            dataTCAG[ind, "ReadScoreStd"] = sd(allBaseScores)
            dataTCAG[ind, "ReadScoreMedian"] = median(allBaseScores)
            dataTCAG[ind, "ReadScoreMAD"] = mad(allBaseScores, constant=1)
            rm(allSeqsLengths)
            rm(allBaseScores)
 
        } else if (bcm[keyInd[r], 2] == "CAT") {
            ind = nrow(dataCATG) + 1

            m = max(l, nrow(lengthCATG))
            old = lengthCATG
            lengthCATG = matrix(0, nrow=m, ncol=ind)
            lengthCATG[1:nrow(old), ind-1] = old
            lengthCATG[l, ind] = cl
        
            m = max(s+1, nrow(scoreCATG))
            old = scoreCATG
            scoreCATG = matrix(0, nrow=m, ncol=ind)
            scoreCATG[1:nrow(old), ind-1] = old
            scoreCATG[s+1, ind] = cs

            allSeqsLengths = rep(1:nrow(lengthCATG), lengthCATG[, ind])
            allBaseScores = rep(1:nrow(scoreCATG), scoreCATG[, ind])
            dataCATG[ind, "AnalysisName"] = analysisName
            dataCATG[ind, "PicoTiterPlateID"] = picoTiterPlateID
            dataCATG[ind, "Region"] = bcm[regInd[r], 2]
            dataCATG[ind, "NumberOfReads"] = length(allSeqsLengths)
            dataCATG[ind, "NumberOfBases"] = length(allBaseScores)
            dataCATG[ind, "ReadLengthMean"] = mean(allSeqsLengths)
            dataCATG[ind, "ReadLengthStd"] = sd(allSeqsLengths)
            dataCATG[ind, "ReadLengthMedian"] = median(allSeqsLengths)
            dataCATG[ind, "ReadLengthMAD"] = mad(allSeqsLengths, constant=1)
            dataCATG[ind, "ReadScoreMean"] = mean(allBaseScores)
            dataCATG[ind, "ReadScoreStd"] = sd(allBaseScores)
            dataCATG[ind, "ReadScoreMedian"] = median(allBaseScores)
            dataCATG[ind, "ReadScoreMAD"] = mad(allBaseScores, constant=1)
            rm(allSeqsLengths)
            rm(allBaseScores)

        } else {
            stop(paste("Unknown key sequence:", bcm[keyInd[r], 2]))
        }      
    }

    names = paste(dataTCAG[,"AnalysisName"], dataTCAG[,"Region"], sep="_")
    rownames(dataTCAG) = names
    rownames(dataCATG) = names
    colnames(lengthTCAG) = names
    colnames(scoreTCAG) = names
    colnames(lengthCATG) = names
    colnames(scoreCATG) = names
    
    bcm = new("Roche454BaseCallerMetrics")
    bcm@dataTCAG = dataTCAG
    bcm@dataCATG = dataCATG
    bcm@lengthTCAG = lengthTCAG
    bcm@scoreTCAG = scoreTCAG
    bcm@lengthCATG = lengthCATG
    bcm@scoreCATG = scoreCATG

    return(bcm)
}

setMethod("read454BaseCallerMetrics", signature="character",
    .read454BaseCallerMetrics_character)

#dyn.load("readSFF.so")

readSFF = function(files) {

  # check or create the list of files
  if (file.exists(files[1])) {
    tmp = file.info(files[1])
    if (tmp$isdir) { # file is directory
      directory <- files[1]
      directory <- sub("[\\/]$", "", directory)
      sfffiles <- try (dir(directory, full.names=TRUE))
      if (inherits(sfffiles, "try-error") || length(sfffiles) == 0) stop("Can't read directory: ", directory)
      sfffiles <- sfffiles[tolower(substring(sfffiles, nchar(sfffiles)-3)) == ".sff"]
    } else { # read invididual files	
      sfffiles <- files
    }
  } else {
    stop("Please provide a valid file or directory name.")
  }	

  containerlist = list()
  for(file in sfffiles) {
    if(!file.exists(file)) {
      cat("Error reading file: ", file ," - skipping.\n", sep="")
      next
    }

    cat("Reading file ", basename(file), " ...", sep="")
    result = .Call("readSFFfromR", file, PACKAGE="R453Plus1Toolbox")
  
    sffcontainer = new("SFFContainer")
  
    name(sffcontainer) = basename(file)
    flowgramFormat(sffcontainer) = result[["flowgramFormat"]]
    flowChars(sffcontainer) = result[["flowChars"]]
    keySequence(sffcontainer) = result[["keySequence"]]
  
    clipQualityLeft(sffcontainer) = result[["clipQualityLeft"]]
    clipQualityRight(sffcontainer) = result[["clipQualityRight"]]
    clipAdapterLeft(sffcontainer) = result[["clipAdapterLeft"]]
    clipAdapterRight(sffcontainer) = result[["clipAdapterRight"]]
    flowgrams(sffcontainer) = result[["flowgrams"]]
    flowIndexes(sffcontainer) = result[["flowIndexes"]]
    qualityScores = PhredQuality(sapply(result[["qualityScores"]], function(x) rawToChar(as.raw(x + 33)))) 
    reads(sffcontainer) = QualityScaledDNAStringSet(DNAStringSet(result[["reads"]]), qualityScores)
  
    containerlist[[basename(file)]] = sffcontainer 
    cat(" done! \n", sep="")
  }

  if(length(containerlist) > 1) {
    return(containerlist)
  } else {
    return(containerlist[[1]])
  }
  
}

.removeLinker <- function(reads, linker,
   removeReadsWithoutLinker, minOverlap, penalty) {

    w = length(linker)
  
    if (missing(removeReadsWithoutLinker)) {
        removeReadsWithoutLinker = FALSE
    }
   
    if (missing(minOverlap)) {
        minOverlap = round(w/2)
    }
    if (missing(penalty)) {
        penalty = 2
    }
    
    readsPrefix = subseq(reads, start=1, end=pmin(width(reads), w+5))

    submat = matrix(-penalty, nrow=5, ncol=5,
        dimnames=list(c("A","C","G","T","N"), c("A","C","G","T","N")))
    diag(submat)[1:4] = 1

    pA = pairwiseAlignment(pattern=readsPrefix, subject=linker,
          type="overlap", substitutionMatrix=submat,
          gapOpening=0, gapExtension=penalty)

    ind = score(pA) >= minOverlap
    trimReads = reads
    linkerEnd = end(pA@pattern@range)+1
    linkerEnd[!ind] = 1
    trimReads = subseq(trimReads, start=linkerEnd)

    if (removeReadsWithoutLinker) {
      trimReads = trimReads[ind]
    }
    return(trimReads)
}

setMethod("removeLinker",
    signature=signature(reads="XStringSet", linker="DNAString", 
        removeReadsWithoutLinker="logical", minOverlap="numeric", penalty="numeric"),
    .removeLinker)

setMethod("removeLinker",
    signature=signature(reads="XStringSet", linker="DNAString", 
        removeReadsWithoutLinker="missing", minOverlap="missing", penalty="missing"),
    .removeLinker)

qualityReportSFF <- function(sfffiles, outfile="qcreport.pdf") {
  basename <- gsub(".pdf", "", outfile, fixed=TRUE)
  tmp <<- environment()
  file.copy(system.file("extdata", "qualityReport.Rnw", package="R453Plus1Toolbox"), paste(basename, ".Rnw", sep=""))
  Sweave(paste(basename, ".Rnw", sep=""))
  texi2dvi(file=paste(basename, ".tex", sep=""), pdf=TRUE)
}

#################################################################################
########## METHODS TO CREATE A MAPPERSET ########################################
#################################################################################

setMethod("MapperSet",
    signature(dirs="character"),
    function(dirs, samplenames){

        ## catch some bad input
        ## first, check if all files in the given folders exist
        for(dir in dirs){
            file = file.path(dir, "mapping/454HCDiffs.txt")
	    if(!file.exists(file))
	    	stop("File not found: ", file)
	}

	## if no sample names are given, use the dirs
	if(!missing(samplenames)){
	    if(!(is.vector(samplenames, mode="character")))
	    	stop("invalid argument: please give a vector of characters as samplenames")
	    if(length(samplenames) != length(dirs))
		stop("invalid argument: number of sample names not consistent with the number of dirs")
    	}else
	    samplenames = dirs
        
        names(dirs) = samplenames

        message("Reading data ... ")
        mapperData1 = .readGSMAssayFeatureData(dirs, samplenames)
        mapperData2 = .readGSMPhenoData(dirs, samplenames)
        message("done")

        mapperSet = new("MapperSet",
            assay1=mapperData1[[1]],
	    assay2=mapperData1[[2]],
            assay3=mapperData1[[3]],
	    assay4=mapperData1[[4]],
            featureDf=mapperData1[[5]],
            featureMeta=mapperData1[[6]],
            phenoDf=mapperData2[[1]],
            phenoMeta=mapperData2[[2]],
            dirs=dirs
        )
        return(mapperSet)

    }
)

## extend the eSet initialize method

setMethod("initialize",
    signature(.Object="MapperSet"),
    function(.Object,
        assayData=assayDataNew("list", 
	    variantForwCount=assay1,
	    totalForwCount=assay2,
	    variantRevCount=assay3,
	    totalRevCount=assay4),
        featureData=new("AnnotatedDataFrame", data=featureDf,
            varMetadata=featureMeta),
        phenoData=new("AnnotatedDataFrame", data=phenoDf, varMetadata=phenoMeta),
        assay1=new("matrix"),
        assay2=new("matrix"),
        assay3=new("matrix"),
        assay4=new("matrix"),
        featureDf=new("data.frame"),
        featureMeta=new("data.frame"),
        phenoDf=new("data.frame"),
        phenoMeta=new("data.frame"),
        dirs="",
        ...)
    {
        .Object@dirs = dirs
        callNextMethod(.Object, assayData = assayData, featureData=featureData,
            phenoData=phenoData, ...)
    }
)

#################################################################################
########## IMPORT OF FEATURE AND PHENODATA (internal/private methods) ###########
#################################################################################

.readGSMAssayFeatureData <- function(dirs, samplenames) {

    firstDir = TRUE
    numSamples = length(dirs)
    sampleNAmes = NULL

    ## initialize assay and feature data (in case no file can be read)
    assayMutForward = matrix(NA, 1, numSamples)
    assayMutReverse = matrix(NA, 1, numSamples)
    assayForwardDepth = matrix(NA, 1, numSamples)
    assayReverseDepth = matrix(NA, 1, numSamples)
    featureData = data.frame(
        chr = NA,
        strand = NA,
        start = NA,
        end = NA,
        referenceBases = NA,
        variantBase = NA,
        regName = NA,
        knownSNP = NA,
        stringsAsFactors=FALSE
    )

    for(dir in dirs){
      

        message("... from ", dir)
	## read file
        file = file.path(dir, "mapping/454HCDiffs.txt")
       	text = readLines(file)

	## count reads with mutation
    	st = grep("Reads with Difference", text)
    	en = grep("Other Reads", text)
	
	##if the file is empty: only fill the assayData with zero values (no new featureData added)
	if(length(st) == 0 | length(en) == 0){
    	   assayMutForward = cbind(assayMutForward, matrix(0, nrow(assayMutForward),1))
    	   assayMutReverse = cbind(assayMutReverse, matrix(0, nrow(assayMutReverse),1))
    	   assayForwardDepth = cbind(assayForwardDepth, matrix(0, nrow(assayForwardDepth),1))
    	   assayReverseDepth = cbind(assayReverseDepth, matrix(0, nrow(assayReverseDepth),1))
	}
	else{

	    sampleNames = c(sampleNames, dir)

    	    countMutForward = vector(mode="numeric", length=length(st))
    	    countMutReverse = vector(mode="numeric", length=length(st))
    	    for(i in 1:length(st)) {
            	reads = text[(st[i]+2):(en[i]-1)]
	    	direction = grep("[A-Za-z0-9]", reads, value=TRUE)
	    	direction = gsub("[A-Za-z0-9 \\(\\)\\ ]", "", direction)
	    	direction = as.vector(sapply(direction, function(x) substring(x,1,1)))
            	countMutForward[i] = sum(direction == "+")
            	countMutReverse[i] = sum(direction == "-")
    	    }

	    ## count reads without mutation
    	    st = grep("Other Reads", text)
    	    en = grep(">chr", text)
    	    en = c(en[-1], length(text))
    	    countNoMutForward = vector(mode="numeric", length=length(st))
    	    countNoMutReverse = vector(mode="numeric", length=length(st))
    	    for(i in 1:length(st)) {
            	reads = text[(st[i]+1):(en[i]-1)]
	    	direction = grep("[A-Za-z0-9]", reads, value=TRUE)
	    	direction = gsub("[A-Za-z0-9 \\(\\)\\ ]", "", direction)
	    	direction = as.vector(sapply(direction, function(x) substring(x,1,1)))
            	countNoMutForward[i] = sum(direction == "+")
            	countNoMutReverse[i] = sum(direction == "-")
    	    }

	    ## read start and strand of reference sequence
    	    refLines = grep("^chr", text, value=TRUE)
    	    strands = vector(mode="character", length=length(refLines))
    	    refStarts = vector(mode="numeric", length=length(refLines))
    	    refSeqs = vector(mode="numeric", length=length(refLines))
    	    for(i in 1:length(refLines)) {
            	entries = refLines[i]
            	line = unlist(strsplit(entries, " +"))
            	strands[i] = substring(line[2], nchar(line[2]))
            	refStarts[i] = substring(line[2], 1, nchar(line[2])-1)
            	refSeqs[i] = gsub("-", "", line[3])
    	    }

	    ## construct eSet information containing assayData, featureData and phenoData	
    	    featLines = grep("^>chr", text, value=TRUE)
    	    con = textConnection(featLines)
    	    tab = read.csv(con, sep="\t", 
	    	col.names=c("Chr", "Start", "End", "RefNuc", "VarNuc", "TotalDepth", 
	    	    "VarFreq", "RefAA", "VarAA", "CodingFrame", "RegionName", "KnownSNPs"), 
	    	header=FALSE,
    	  	colClasses=c("character", "numeric", "numeric", "character", "character", 
	    	"numeric", "character", "character", "character", "character", "character", "character")
     	    )
    	    close(con)
	    chrs = gsub(">", "", tab[, "Chr"])
    	    tab[, "Chr"] = substr(chrs, 4, nchar(chrs))
    	    tab[,"mutReadsForward"] = countMutForward
    	    tab[,"mutReadsReverse"] = countMutReverse
    	    tab[,"forwardDepth"] = countMutForward + countNoMutForward
    	    tab[,"reverseDepth"] = countMutReverse + countNoMutReverse
    	    tab[,"Strand"] = strands

    	    ## case1 (first file): only initialize data
    	    if(firstDir){

    	    	## assay data
    	    	assayMutForward = matrix(countMutForward)
    	    	assayMutReverse = matrix(countMutReverse)
    	    	assayForwardDepth = matrix(countMutForward + countNoMutForward)
    	    	assayReverseDepth = matrix(countMutReverse + countNoMutReverse)

    	    	## feature data
    	        featureData = data.frame(
                   chr = tab[, "Chr"],
		   strand = tab[, "Strand"],
                   start = as.numeric(tab[, "Start"]),
                   end = as.numeric(tab[, "End"]),
                   referenceBases = tab[, "RefNuc"],
                   variantBase = tab[, "VarNuc"],
                   regName = tab[, "RegionName"],
                   knownSNP = tab[, "KnownSNPs"],
		   stringsAsFactors=FALSE
    	        )
	    	firstDir = FALSE
    	    }
    	    ## case2 (more than one file): merge new assay and feature data with older data
    	    else{

    	    	## assay data
    	    	assayMutForwardNew = matrix(countMutForward)
    	    	assayMutReverseNew = matrix(countMutReverse)
    	    	assayForwardDepthNew = matrix(countMutForward + countNoMutForward)
    	    	assayReverseDepthNew = matrix(countMutReverse + countNoMutReverse)

	    	## feature data
            	featureDataNew = data.frame(
                    chr = tab[, "Chr"],
		    strand = tab[, "Strand"],
                    start = tab[, "Start"],
                    end = tab[, "End"],
                    referenceBases = tab[, "RefNuc"],
                    variantBase = tab[, "VarNuc"],
                    regName = tab[, "RegionName"],
                    knownSNP = tab[, "KnownSNPs"],
		    stringsAsFactors=FALSE
	     	)
		## determine common rows between the old and the new feature data (then merge common data and add new entries)
		join <- function(x) do.call("paste", c(as.data.frame(x), sep = "\r"))
		idx1 = which(join(featureData) %in% join(featureDataNew))
		idx2 = which(join(featureDataNew) %in% join(featureData))
		idx1_not = setdiff(1:nrow(featureData), idx1)
		idx2_not = setdiff(1:nrow(featureDataNew), idx2)

	    	## extend assay data (by columns, initialization with 0)
    	    	assayMutForward = cbind(assayMutForward, matrix(0, nrow(assayMutForward),1))
    	    	assayMutReverse = cbind(assayMutReverse, matrix(0, nrow(assayMutReverse),1))
    	    	assayForwardDepth = cbind(assayForwardDepth, matrix(0, nrow(assayForwardDepth),1))
    	    	assayReverseDepth = cbind(assayReverseDepth, matrix(0, nrow(assayReverseDepth),1))

	    	## add entries for common mutations (assure, that both sets are in the same order (-> idx3))
		f1 = featureData[idx1, ]
	 	f2 = featureDataNew[idx2, ]
		a1 = assayMutForwardNew[idx2]
		a2 = assayMutReverseNew[idx2]
		a3 = assayForwardDepthNew[idx2]
		a4 = assayReverseDepthNew[idx2]
		idx3 = unlist(sapply(f1$start, function(x) which(f2$start == x)))

	        assayMutForward[idx1, ncol(assayMutForward)] = a1[idx3]
	        assayMutReverse[idx1, ncol(assayMutReverse)] = a2[idx3]
	        assayForwardDepth[idx1, ncol(assayForwardDepth)] = a3[idx3]
	        assayReverseDepth[idx1, ncol(assayReverseDepth)] = a4[idx3]

	        ## add entries for new mutations (by columns, insert 0 for older entries)
	   	## assay data slot 1
	    	aMutForwardNew = assayMutForwardNew[idx2_not]
	    	aMutForwardNew = cbind(matrix(0, length(aMutForwardNew), (ncol(assayMutForward)) - 1), aMutForwardNew)
	    	assayMutForward = rbind(assayMutForward, aMutForwardNew)
	    	## assay data slot 2
	    	aMutReverseNew = assayMutReverseNew[idx2_not]
	    	aMutReverseNew = cbind(matrix(0, length(aMutReverseNew), (ncol(assayMutReverse)) - 1), aMutReverseNew)
	    	assayMutReverse = rbind(assayMutReverse, aMutReverseNew)
	    	## assay data slot 3
	    	aForwardDepthNew = assayForwardDepthNew[idx2_not]
	    	aForwardDepthNew = cbind(matrix(0, length(aForwardDepthNew), (ncol(assayForwardDepth)) - 1), aForwardDepthNew)
	    	assayForwardDepth = rbind(assayForwardDepth, aForwardDepthNew)
	    	## assay data slot 4
	    	aReverseDepthNew = assayReverseDepthNew[idx2_not]
	    	aReverseDepthNew = cbind(matrix(0, length(aReverseDepthNew), (ncol(assayReverseDepth)) - 1), aReverseDepthNew)
	    	assayReverseDepth = rbind(assayReverseDepth, aReverseDepthNew)

	    	## extend feature data (by rows)
	    	fDataNew = featureDataNew[idx2_not, ]
	        featureData = rbind(featureData, fDataNew)
    	    }
        }
    }

    ## change style of varNuc: repeat "-" according to the number of bases involved in a deletion
    for(i in 1:nrow(featureData)){
	if(!is.na(featureData[i, "variantBase"]) & featureData[i, "variantBase"] == "-") 
	    featureData[i, "variantBase"] = paste(rep("-", nchar(featureData[i, "referenceBases"])), collapse="")
	if(!is.na(featureData[i, "referenceBases"]) & featureData[i, "referenceBases"] == "-") 
	    featureData[i, "referenceBases"] = paste(rep("-", nchar(featureData[i, "variantBase"])), collapse="")
    }

    ## distribute some new row and columnnames for feature and assay data
    rownames(featureData) = 1:nrow(featureData)
    rownames(assayMutForward) = rownames(featureData)
    rownames(assayMutReverse) = rownames(featureData)
    rownames(assayForwardDepth) = rownames(featureData)
    rownames(assayReverseDepth) = rownames(featureData)
    colnames(assayMutForward) = samplenames
    colnames(assayMutReverse) = samplenames
    colnames(assayForwardDepth) = samplenames
    colnames(assayReverseDepth) = samplenames

    ## feature meta
    featureMeta = data.frame(row.names=colnames(featureData),
        labelDescription=c("Chromosome","Strand", "Start", "End", "Reference nucleotide",
      	    "Variant nucleotide", "Region name", "Known SNPs"))

    return(
        list(
	    assayMutForward,
	    assayForwardDepth,
	    assayMutReverse,
	    assayReverseDepth,
	    featureData,
	    featureMeta
	)
    )


}

.readGSMPhenoData <- function(dirs, samplenames) {

    numSamples = length(dirs)

    ## pheno data
    accessionNumbers = rep(NA, numSamples)
    names(accessionNumbers) = dirs
    for(dir in dirs){

        file = file.path(dir, "mapping/454NewblerMetrics.txt")

        if(!file.exists(file))
            warning("File not found:", file, "; pheno data omitted")
        else{
            ## read file
            text = readLines(file)
            ## locate accession number
            an = grep(".sff", text, value=TRUE)
	    if(length(an) > 0){
            	an = an[1]
            	an = unlist(strsplit(an, split="/"))
            	an = an[length(an)]
            	an = unlist(strsplit(an, split=".sff"))
            	accessionNumbers[dir] = an[1]
	    }
	}
    }

    phenoData = matrix(accessionNumbers, numSamples, 1)
    phenoData = data.frame(accessionNumber=phenoData, row.names=samplenames)

    ## pheno meta
    phenoMeta = data.frame(row.names=colnames(phenoData),
        labelDescription=c("Accession number"))

    return(
        list(
            phenoData,
            phenoMeta
        )
    )
}




#################################################################################
########## GETTER AND SETTER METHODS ############################################
#################################################################################

## provide another subset method for samples and variants
## Todo: common method for AVASet & MapperSet
setMethod("subset",
    signature("MapperSet"),
          
    function(x, subset, dimension="variants"){

      object = x
      if(!missing(subset)){

            if(dimension=="variants"){
              ## catch some bad input
              if(is.vector(subset, mode="character")){
             	  if(!all(subset %in% rownames(fData(object)))){
                     stop("invalid argument: given variants not found")
             	  }
              }else{
                  if(is.vector(subset, mode="numeric")){
                      if(!all(subset %in% 1:nrow(fData(object)))){
                          stop("invalid argument: given indices out of range")
                      }
                  }else{
                      if(is.vector(subset, mode="logical")){
                          if(length(subset) != nrow(fData(object))){
                              stop("invalid argument: given subset is of wrong length")
                          }
                      }else
                          stop("invalid argument: given subset of incorrect type")
                  }
              }
              return(object[subset, ])
            }else{
              if(dimension=="samples"){
              ## catch some bad input
              if(is.vector(subset, mode="character")){
             	  if(!all(subset %in% rownames(pData(object)))){
                     stop("invalid argument: given samples not found")
             	  }
              }else{
                  if(is.vector(subset, mode="numeric")){
                      if(!all(subset %in% 1:nrow(pData(object)))){
                          stop("invalid argument: given indices out of range")
                      }
                  }else{
                      if(is.vector(subset, mode="logical")){
                          if(length(subset) != nrow(pData(object))){
                              stop("invalid argument: given subset is of wrong length")
                          }
                      }else
                          stop("invalid argument: given subset of incorrect type")
                  }
              }
                return(object[, subset])
              }else{
                stop("invalid argument: dimension can be either \"samples\" or \"variants\"")
              }
            }
          
        }else{
          warning("no subset given; MapperSet object left unchanged")
          return(object)
        }
    }
)

# this methods reads the file "454ReadStatus.txt" in the GSM project directory and returns its content in a dataframe
# the file contains information about the alignment of each read (chr, pos, strand, etc.)

setMethod("getReadStatus", signature(object="MapperSet"),
          function(object){
            dirs = dirs(object)
            readStatus = list()
            files = file.path(dirs, "mapping", "454ReadStatus.txt")
            for (i in 1:length(files)) {
              readStatus[[i]] = read.table(files[i], skip=1, header=TRUE, stringsAsFactors=FALSE, fill=TRUE)
              names(readStatus)[i] = names(dirs)[i]
            }
            return(readStatus)
          })

# this version of assayData returns the filtered assayData of the MapperSet if the
# filter is set to a value > 0; otherwise it calls the original Biobase version

setMethod("assayData",
    signature(object="MapperSet"),
    function(object){
        if(sum(variantFilterPerc(object)) > 0){
            # filter the assayData
            filteredForwardCount=subset(object@assayData[[1]], rownames(object@assayData[[1]]) %in% variantFilter(object))
            filteredForwardDepth=subset(object@assayData[[2]], rownames(object@assayData[[2]]) %in% variantFilter(object))
            filteredReverseCount=subset(object@assayData[[3]], rownames(object@assayData[[3]]) %in% variantFilter(object))
            filteredReverseDepth=subset(object@assayData[[4]], rownames(object@assayData[[4]]) %in% variantFilter(object))
            return(assayDataNew("list",
                variantForwCount=filteredForwardCount,
                totalForwCount=filteredForwardDepth,
                variantRevCount=filteredReverseCount,
                totalRevCount=filteredReverseDepth))
        } else {
            callNextMethod(object)
        }
    }
)

# these versions of fetureData and fData return the filtered featureData of the MapperSet if the
# filter is set to a value > 0; otherwise it calls the original Biobase version

setMethod("fData",
    signature(object="MapperSet"),
    function(object){
        if(sum(variantFilterPerc(object)) > 0) {
            #filter the featureData
            return ( subset( object@featureData@data, rownames(object@featureData@data) %in% variantFilter(object) ) )
        } else {
            callNextMethod(object)
        }
    }
)

setMethod("featureData",
    signature(object="MapperSet"),
    function(object){
        if(sum(variantFilterPerc(object)) > 0) {
            #filter the featureData
            return ( subset( object@featureData, rownames(object@featureData@data) %in% variantFilter(object) ) )
        } else {
            callNextMethod(object)
        }
    }
)

setMethod("dirs",
    signature(object="MapperSet"),
    function(object){
        return(object@dirs)
    }
)

# class Breakpoints
setGeneric("seqsC1", function(object) {
    standardGeneric("seqsC1")})
setGeneric("seqsC2", function(object) {
    standardGeneric("seqsC2")})
setGeneric("commonBpsC1", function(object) {
    standardGeneric("commonBpsC1")})
setGeneric("commonBpsC2", function(object) {
    standardGeneric("commonBpsC2")})
setGeneric("commonAlignC1", function(object) {
    standardGeneric("commonAlignC1")})
setGeneric("commonAlignC2", function(object) {
    standardGeneric("commonAlignC2")})
setGeneric("alignedReadsC1", function(object) {
    standardGeneric("alignedReadsC1")})
setGeneric("alignedReadsC2", function(object) {
    standardGeneric("alignedReadsC2")})

setGeneric("seqsC1<-", function(object, value) {
    standardGeneric("seqsC1<-")})
setGeneric("seqsC2<-", function(object, value) {
    standardGeneric("seqsC2<-")})
setGeneric("commonBpsC1<-", function(object, value) {
    standardGeneric("commonBpsC1<-")})
setGeneric("commonBpsC2<-", function(object, value) {
    standardGeneric("commonBpsC2<-")})
setGeneric("commonAlignC1<-", function(object, value) {
    standardGeneric("commonAlignC1<-")})
setGeneric("commonAlignC2<-", function(object, value) {
    standardGeneric("commonAlignC2<-")})
setGeneric("alignedReadsC1<-", function(object, value) {
    standardGeneric("alignedReadsC1<-")})
setGeneric("alignedReadsC2<-", function(object, value) {
    standardGeneric("alignedReadsC2<-")})



setGeneric("read454QualityFilterMetrics", function(object) {
    standardGeneric("read454QualityFilterMetrics")})

setGeneric("dataTotal", function(object) {
    standardGeneric("dataTotal")})

setGeneric("dataTotal<-", function(object, value) {
    standardGeneric("dataTotal<-")})

setGeneric("dataTCAG", function(object) {
    standardGeneric("dataTCAG")})

setGeneric("dataTCAG<-", function(object, value) {
    standardGeneric("dataTCAG<-")})

setGeneric("dataCATG", function(object) {
    standardGeneric("dataCATG")})

setGeneric("dataCATG<-", function(object, value) {
    standardGeneric("dataCATG<-")})

setGeneric("read454BaseCallerMetrics", function(object) {
    standardGeneric("read454BaseCallerMetrics")})

setGeneric("lengthTCAG", function(object) {
    standardGeneric("lengthTCAG")})

setGeneric("lengthTCAG<-", function(object, value) {
    standardGeneric("lengthTCAG<-")})

setGeneric("scoreTCAG", function(object) {
    standardGeneric("scoreTCAG")})

setGeneric("scoreTCAG<-", function(object, value) {
    standardGeneric("scoreTCAG<-")})

setGeneric("lengthCATG", function(object) {
    standardGeneric("lengthCATG")})

setGeneric("lengthCATG<-", function(object, value) {
    standardGeneric("lengthCATG<-")})

setGeneric("scoreCATG", function(object) {
    standardGeneric("scoreCATG")})

setGeneric("scoreCATG<-", function(object, value) {
    standardGeneric("scoreCATG<-")})

setGeneric("annotatedVariants", function(object) {
    standardGeneric("annotatedVariants")})

setGeneric("annotatedVariants<-", function(object, value) {
    standardGeneric("annotatedVariants<-")})

setGeneric("plotVariationFrequency", function(object, plotRange, ...) {
    standardGeneric("plotVariationFrequency")})

setGeneric("alignShortReads", function(object, bsGenome, seqNames, ensemblNotation) {
    standardGeneric("alignShortReads")})



setGeneric("genomeSequencerMIDs", function(mid) {
    standardGeneric("genomeSequencerMIDs")})

setGeneric("sequenceCaptureLinkers", function(name) {
    standardGeneric("sequenceCaptureLinkers")})

setGeneric("demultiplexReads", function(reads, mids, numMismatches, trim) {
    standardGeneric("demultiplexReads")})

setGeneric("removeLinker", function(reads, linker, removeReadsWithoutLinker, minOverlap, penalty) {
    standardGeneric("removeLinker")})

setGeneric("filterChimericReads", function(alnReads, targetRegion, linkerSeq, minDist, dupReadDist) {
    standardGeneric("filterChimericReads")})

setGeneric("detectBreakpoints", signature=c("chimericReads"),
           function(chimericReads, bpDist=100, minClusterSize=4, removeSoftClips=TRUE, bsGenome){
             standardGeneric("detectBreakpoints")})

setGeneric("mergeBreakpoints", function(breakpoints, maxDist, mergeBPs) {
    standardGeneric("mergeBreakpoints")})


setGeneric("readsOnTarget", function(alnReads, targetRegion) {
    standardGeneric("readsOnTarget")})

setGeneric("coverageOnTarget", function(alnReads, targetRegion) {
    standardGeneric("coverageOnTarget")})

setGeneric("calculateTiTv", function(object) {
    standardGeneric("calculateTiTv")})

setGeneric("read454Metrics", function(rochePath, ...) {
    standardGeneric("read454Metrics")})


setGeneric("annotateVariants", function(object, bsGenome) {
    standardGeneric("annotateVariants")})


## generics for plotting functions

setGeneric("plotVariants", signature=c("data", "gene"),
    function(data, gene, transcript=NA, regions, mutationInfo, groupBy="pos", horiz=FALSE, cex=1, title="", legend=TRUE)
        standardGeneric("plotVariants"))

setGeneric("plotChimericReads", signature=("brpData"),
    function(brpData, geneSymbols=FALSE, plotMut=TRUE, plotBasePairs=FALSE, maxBasePairs=50, legend=FALSE, title="", 
	col=c("red", "green", "black", "orange"))
        standardGeneric("plotChimericReads"))

setGeneric("plotAmpliconCoverage", signature=c("avaSet", "type", "bothDirections"),
     function(avaSet, type="amplicon", bothDirections=TRUE,
         cex.names=0.8, cex.axis=0.8, las=3,
         col=c(rgb(217/255, 214/255, 209/255), rgb(173/255, 38/255, 36/255)),
         ...) {
         standardGeneric("plotAmpliconCoverage")})

setGeneric("htmlReport", signature=c("object"),
    function(object, annot, blocks=c(), transcripts=c(), sampleCols, minMut=3, dir="HTMLReport", title="Summary")
        standardGeneric("htmlReport"))


## generics for filterVariants

setGeneric("getVariantPercentages", signature=c("object"),
    function(object, direction="both")
        standardGeneric("getVariantPercentages"))

setGeneric("setVariantFilter", signature=c("object"),
    function(object, filter=0)
        standardGeneric("setVariantFilter"))

setGeneric("variantFilterPerc", signature = c("object"),
    function(object)
        standardGeneric("variantFilterPerc"))

setGeneric("variantFilter", signature = c("object"),
     function(object)
        standardGeneric("variantFilter"))


## generics for the AVASet:

setGeneric("AVASet", function(dirname, avaBin, file_sample, file_amp, file_reference, file_variant, file_variantHits) 
    standardGeneric("AVASet"))

setGeneric("readSampleData_AVACLI", function(samples) 
    standardGeneric("readSampleData_AVACLI"))

setGeneric("readVariants_AVACLI", function(variants, variantHits, samples) 
    standardGeneric("readVariants_AVACLI"))

setGeneric("readReferenceSequences_AVACLI", function(references) 
    standardGeneric("readReferenceSequences_AVACLI"))

setGeneric("readAmplicons_AVACLI", function(amps, amps_align, samples) 
    standardGeneric("readAmplicons_AVACLI"))

setGeneric("readSampleData", function(dir_projectDef) 
    standardGeneric("readSampleData"))

setGeneric("readVariants", function(dir_variants,samples,refSeqs) 
    standardGeneric("readVariants"))

setGeneric("readReferenceSequences", function(dir_projectDef) 
    standardGeneric("readReferenceSequences"))

setGeneric("readAmplicons", function(dir_projectDef, dir_align, samples) 
    standardGeneric("readAmplicons"))

setGeneric("readAmpliconPositions", signature=c("object", "dnaSet","genes"), 
    function(object, dnaSet, genes, dataset="hsapiens_gene_ensembl") 
	standardGeneric("readAmpliconPositions"))

setGeneric("fDataAmp", function(object) 
    standardGeneric("fDataAmp"))

setGeneric("featureDataAmp", function(object) 
    standardGeneric("featureDataAmp"))

setGeneric("assayDataAmp", function(object) 
    standardGeneric("assayDataAmp"))

setGeneric("referenceSequences", signature = c("object"), 
    function(object) 
	standardGeneric("referenceSequences"))

setGeneric("getAlignedReads", signature = c("object"), 
    function(object, amplicons, dir) 
	standardGeneric("getAlignedReads"))

setGeneric("featureDataAmp<-", signature = c("object","value"), 
    function(object, value) 
	standardGeneric("featureDataAmp<-"))

setGeneric("assayDataAmp<-", signature = c("object","value"),
    function(object, value) 
	standardGeneric("assayDataAmp<-"))

setGeneric("referenceSequences<-", signature = c("object","value"), 
    function(object, value) 
	standardGeneric("referenceSequences<-"))

setGeneric(name="ava2vcf",
           def=function(object, filename, annot) standarGeneric("ava2vcf"),
           signature=c("object"))


#generics for the MapperSet:

setGeneric("MapperSet", function(dirs, samplenames)
    standardGeneric("MapperSet"))

setGeneric("getReadStatus", function(object)
    standardGeneric("getReadStatus"))



## generics for the AVASet and MapperSet:

setGeneric("dirs", signature = c("object"), 
    function(object) 
	standardGeneric("dirs"))

setGeneric("variantFilterPerc<-", signature = c("object","value"), 
    function(object, value) 
	standardGeneric("variantFilterPerc<-"))

setGeneric("variantFilter<-", signature = c("object","value"), 
    function(object, value) 
	standardGeneric("variantFilter<-"))
	
	
## generics for SFFRead and SFFContainer

setGeneric("name<-", function(x, value) standardGeneric("name<-"))

setGeneric("flowgramFormat", function(x) standardGeneric("flowgramFormat"))
setGeneric("flowgramFormat<-", function(x, value) standardGeneric("flowgramFormat<-"))

setGeneric("flowChars", function(x) standardGeneric("flowChars"))
setGeneric("flowChars<-", function(x, value) standardGeneric("flowChars<-"))

setGeneric("keySequence", function(x) standardGeneric("keySequence"))
setGeneric("keySequence<-", function(x, value) standardGeneric("keySequence<-"))

setGeneric("clipQualityLeft", function(x) standardGeneric("clipQualityLeft"))
setGeneric("clipQualityLeft<-", function(x, value) standardGeneric("clipQualityLeft<-"))

setGeneric("clipQualityRight", function(x) standardGeneric("clipQualityRight"))
setGeneric("clipQualityRight<-", function(x, value) standardGeneric("clipQualityRight<-"))

setGeneric("clipAdapterLeft", function(x) standardGeneric("clipAdapterLeft"))
setGeneric("clipAdapterLeft<-", function(x, value) standardGeneric("clipAdapterLeft<-"))

setGeneric("clipAdapterRight", function(x) standardGeneric("clipAdapterRight"))
setGeneric("clipAdapterRight<-", function(x, value) standardGeneric("clipAdapterRight<-"))

setGeneric("flowgram", function(x) standardGeneric("flowgram"))
setGeneric("flowgram<-", function(x, value) standardGeneric("flowgram<-"))

setGeneric("flowgrams", function(x) standardGeneric("flowgrams"))
setGeneric("flowgrams<-", function(x, value) standardGeneric("flowgrams<-"))

setGeneric("flowIndexes", function(x) standardGeneric("flowIndexes"))
setGeneric("flowIndexes<-", function(x, value) standardGeneric("flowIndexes<-"))

setGeneric("read", function(x) standardGeneric("read"))
setGeneric("read<-", function(x, value) standardGeneric("read<-"))

setGeneric("quality<-", function(x, value) standardGeneric("quality<-"))

setGeneric("reads", function(x) standardGeneric("reads"))
setGeneric("reads<-", function(x, value) standardGeneric("reads<-"))

setGeneric("addRead", function(x, read) standardGeneric("addRead"))
setGeneric("getRead", function(x, readname) standardGeneric("getRead"))

setGeneric(name="sff2fastq", 
           def=function(x, outdir, fname) standardGeneric("sff2fastq"),
           signature=c("x"))


## generics for quality functions
setGeneric("readLengthStats", function(object) standardGeneric("readLengthStats"))
setGeneric(name="readLengthHist", 
           def=function(object, cutoff=0.99, xlab="Read length", ylab="Number of sequences", 
             col="firebrick1", breaks=100, ...) standardGeneric("readLengthHist"),
           signature=c("object"))

setGeneric("baseQualityStats", function(object) standardGeneric("baseQualityStats"))
setGeneric(name="baseQualityHist", 
           def=function(object, xlab="Quality score", ylab="Number of bases", 
             col="firebrick1", breaks=40, ...) standardGeneric("baseQualityHist"),
           signature=c("object"))
setGeneric(name="sequenceQualityHist", 
           def=function(object, xlab="Mean of quality scores per sequence", 
             ylab="Number of sequences", col="firebrick1", ...) 
             standardGeneric("sequenceQualityHist"),
           signature=c("object"))
setGeneric(name="positionQualityBoxplot", 
           def=function(object, range, binsize=10, 
             xlab=paste("Read position in bp (Bin size: ", binsize, "bp)", sep=""), 
             ylab="Quality score", col="firebrick1", ...) standardGeneric("positionQualityBoxplot"),
           signature=c("object"))

setGeneric("baseFrequency", function(object) standardGeneric("baseFrequency"))
setGeneric(name="nucleotideCharts", 
           def=function(object, range=0.95, linetypes=c(A="l", C="l", G="l", T="l", N="l"), 
             linecols=c(A="black", C="red", G="blue", T="green", N="grey70"), xlab="Position",
             ylab="Frequency", ...) standardGeneric("nucleotideCharts"),
           signature=c("object"))
setGeneric("gcContent", function(object) standardGeneric("gcContent"))
setGeneric(name="gcPerPosition", 
           def=function(object, range=0.95, type="l", col="blue", xlab="Position", ylab="Frequency",
             ...) standardGeneric("gcPerPosition"),
           signature=c("object"))
setGeneric(name="gcContentHist", 
           def=function(object, xlab="GC content", ylab="Number of sequences", col="firebrick1",
             breaks=50, ...) standardGeneric("gcContentHist"),
           signature=c("object"))

setGeneric(name="complexity.dust", 
           def=function(object, xlab="Complexity score (0=high, 100=low)", 
             ylab="Number of sequences", xlim=c(0, 100), col="firebrick1", breaks=100, ...) 
             standardGeneric("complexity.dust"),
           signature=c("object"))
setGeneric(name="complexity.entropy", 
           def=function(object, xlab="Complexity score (0=low, 100=high)", 
             ylab="Number of sequences", xlim=c(0, 100), col="firebrick1", breaks=100, ...) 
             standardGeneric("complexity.entropy"),
           signature=c("object"))

setGeneric(name="dinucleotideOddsRatio", 
           def=function(object, xlab="Under-/over-representation of dinucleotides", 
             col="firebrick1", ...) standardGeneric("dinucleotideOddsRatio"),
           signature=c("object"))

setGeneric(name="flowgramBarplot",
           def=function(x, range=c(0, length(flowgram(x))), xlab="Flow sequence", ylab="Flow intensity",
             col=c(A="black", C="red", G="blue", T="green"), ...)
             standardGeneric("flowgramBarplot"),
           signature=c("x"))

setGeneric(name="homopolymerHist",
           def=function(x, range=c(0, length(flowgram(x))), xlab="Homopolymer length",
             ylab="Number of homopolymers", col=c(A="black", C="red", G="blue", T="green"), ...)
             standardGeneric("homopolymerHist"),
           signature=c("x"))

.filterChimericReads <- function(alnReads, targetRegion, linkerSeq,
                                 minDist, dupReadDist) {

    if (missing(minDist)) {
        minDist = 1000
    }
    if (missing(dupReadDist)) {
        dupReadDist = 1
    }

    # check sam/bam input
    reads = lapply(names(alnReads[[1]]), function(elt) {
        if (is.factor(alnReads[[1]][[elt]])) {
            factor(do.call(c, unname(lapply(alnReads, function (x) {
                as.character(x[[elt]])
            }))))
                                            
        } else
            do.call(c, unname(lapply(alnReads, "[[", elt)))
        }
    )
    names(reads) = names(alnReads[[1]])
    reqFields = c("qname", "rname", "pos", "cigar")
    if (!all(is.element(reqFields, names(reads)))) {
        stop(paste("The following SAM fields are necessary:", reqFields))
    }
    reads = do.call("DataFrame", reads)


    # initiate data frame for logging info
    log = data.frame(AlignedReads=length(unique(reads$qname)),
        stringsAsFactors=FALSE)

    
    # 1: extract reads with exactly two local alignments
    message("Extrating chimeric reads with exactly two local alignments.")
    rTable = table(reads$qname)
    crNames = names(rTable[rTable == 2])
    reads = reads[is.element(reads$qname, crNames),]
    log$ChimericReads = sum(rTable > 1)
    log$TwoLocalAlignments = sum(rTable == 2)

    
    # intermediate step : add these four columns to the data frame
    # start      - start position of the alignment in reference coordinates
    #              (equal to the SAM field pos)
    # end        - end position of the alignment in reference coordinates
    #              (corrected for insertions in the query read)
    # localStart - start position of the alignment in query seq. coordinates
    # localEnd   - end position of the alignment in query seq. coordinates
    #              (corrected for deletions in the query read)
    # All fields refer to the positive strand as usual in SAM.
    cigars = extendedCIGARToList(reads$cigar)
    dels = sapply(cigars, function(x) sum(x[names(x)=="D"]))
    ins = sapply(cigars, function(x) sum(x[names(x)=="I"]))
    matches = sapply(cigars, function(x) sum(x[names(x)=="M"]))
    reads$start = reads$pos
    reads$end = as.integer(reads$start + matches + dels - 1)
    reads$localStart = as.integer(sapply(cigars, FUN=
        function(x) {
            ind = min(which(names(x) == "M"))
            x = c(0, x)
            return(sum(x[1:ind]) + 1)
        }
    ))
    reads$localEnd = as.integer(sapply(cigars, FUN=
        function(x) {
            ind = max(which(names(x) == "M"))
            return(sum(x[1:ind]))
        }
    ) - dels)
    

    # 2: remove all reads that do not align to the chip's target region
    if (!missing(targetRegion)) {
        message("Removing reads that do not align to the target region.")
        readRanges = GRanges(
            IRanges(start=reads$start, end=reads$end),
            seqnames=reads$rname, name=reads$qname)
        ind = suppressWarnings(overlapsAny(readRanges, GRanges(targetRegion)))
        targetReads <- readRanges$name[ind]
        reads = reads[is.element(reads$qname, targetReads),]
        log$TargetRegion = length(unique(reads$qname))
    } else {
        targetRegion = "missing" # only for parameter slot in the returned object
    }
    reads$seq = compact(reads$seq)


    # 3: remove all reads that have a linker sequence in the middle
    if (!missing(linkerSeq)) {
        message("Removing reads with a linker sequence in the middle.")
        dupInd = duplicated(reads$qname)
        seqs = reads$seq[!dupInd]
        names(seqs) = reads$qname[!dupInd]
        readsLinker = character()

        f = vmatchPattern(linkerSeq, seqs, max.mismatch=4)
        indF = elementNROWS(f) > 0
        seqLength = width(seqs)[indF]
        for (i in setdiff(0:sum(indF), 0)) {
            if (! (any(startIndex(f)[indF][[i]] < 10) |
                any(seqLength[i] - endIndex(f)[indF][[i]] < 10))) {
                readsLinker = c(readsLinker, names(seqs)[indF][i])
            }
        }
      
        r = vmatchPattern(linkerSeq, reverseComplement(seqs), max.mismatch=4)
        indR = elementNROWS(r) > 0
        seqLength = width(seqs)[indR]
        for (i in setdiff(0:sum(indR), 0)) {
            if (! (any(startIndex(r)[indR][[i]] < 10) |
                any(seqLength[i] - endIndex(r)[indR][[i]] < 10))) {
                readsLinker = c(readsLinker, names(seqs)[indR][i])
            }
        }
    
        ind = !is.element(reads$qname, readsLinker)
        reads = reads[ind, ]
        log$NoLinker = length(unique(reads$qname))
    } else {
        linkerSeq = "missing" # only for parameter slot in the returned object
    }


    # 4: Remove local alignments that are too close together (minDist)
    message("Removing reads with close local alignments (perhaps small indels).")
    cList = list()
    readNames = unique(reads$qname)
    for (n in readNames) {
        ind = reads$qname == n
        if (sum(ind) == 2) {
            if (reads[ind, "rname"][1] == reads[ind, "rname"][2]) {
                # Reads from same chr. must not overlap in reference seq.
                # and must insure a minimal distance minDist
                ir = IRanges(start=reads[ind, "start"],
                    end=reads[ind, "end"])
                if (countOverlaps(ir[1], ir[2]) == 0  &
                    min(abs(start(ir[1]) - end(ir[2])),
                        abs(end(ir[1]) - start(ir[2]))) >= minDist) {
                    cList[[n]] = reads[ind, ]
                    cList[[n]]$seq = compact(cList[[n]]$seq)
                }
            } else {
                cList[[n]] = reads[ind, ]
                cList[[n]]$seq = compact(cList[[n]]$seq)
            }
        }
    }
    log$MinimumDistance = length(cList)


    # intermediate step - extract Breakpoints:
    # Each chimeric read with two local alignments has 4 points where an
    # alignment starts/ends. The two points in the middle are the breakpoint.
    # Before we can compute the breakpoint, we need the alignment positions
    # with respect to the reads:
    # Positions are given for the sequences stored in the sam file. Seqs that
    # were aligned to the minus strand of the reference are stored as reverse
    # complement. Here, we compute start (end) and localStart (localEnd)
    # coordinates by defining the 5 prime end as start point. So, two local
    # alignments of the same read are defined in the same coordinates even if
    # one part maps to the + and the other to the - strand.
    cList = lapply(cList,
        function (df) {
            ind = df$strand == "-"
            df$localStart5 = df$localStart
            df$localEnd5 = df$localEnd
            df$start5 = df$start
            df$end5 = df$end
            if (sum(ind) > 0) {
                df$localEnd5[ind] = (df$qwidth[ind] + 1) - df$localStart[ind]
                df$localStart5[ind] = (df$qwidth[ind] + 1) - df$localEnd[ind]
                df$start5[ind] = df$end[ind]
                df$end5[ind] = df$start[ind]
            }

            bp1 = which(df$localStart5 == max(df$localStart5))[1]
            bp2 = which(df$localEnd5 == min(df$localEnd5))[1]
            
            if (bp1 == 1 & bp2 == 2) {
                df$breakpoint = c(df$start5[bp1], df$end5[bp2])
            } else if (bp1 == 2 & bp2 == 1) {
                df$breakpoint = c(df$end5[bp2], df$start5[bp1])
            } else {
                df$breakpoint = NA
            }
            return(df)
        }
    )


    # 5: Remove duplicated reads,
    # (dup. reads := reads with same chr., same strand and same 5' start pos.
    message("Removing duplicated reads (perhaps duplication due to PCR).")
    checkDF = data.frame(
        chr = sapply(cList, function(x) {
            ind = which(x$localStart5 == min(x$localStart5))[1]
            return(as.character(x$rname[ind]))
        }),
        strand = sapply(cList, function(x) {
            ind = which(x$localStart5 == min(x$localStart5))[1]
            return(as.character(x$strand[ind]))
        }),
        pos = sapply(cList, function(x) {
            ind = which(x$localStart5 == min(x$localStart5))[1]
            return(x$start5[ind])
        }),
        qwidth = sapply(cList, function(x) {x$qwidth[1]})
    )
    row.names(checkDF) = names(cList)
    dupNames = character()
    sCheckDF = split(checkDF, checkDF$chr)
    for (i in 1:length(sCheckDF)) {
        df = split(sCheckDF[[i]], sCheckDF[[i]]$strand)
        for (j in 1:length(df)) {
            if (nrow(df[[j]]) > 0) {
                for (k in 1:nrow(df[[j]])) {
                    ind = abs(df[[j]]$pos[k] - df[[j]]$pos[-k]) <= dupReadDist
                    allNames = sort(
                        c(row.names(df[[j]][k,]), row.names(df[[j]][-k,])[ind]))                    
                    longestRead = which(df[[j]][allNames, "qwidth"] ==
                        max(df[[j]][allNames, "qwidth"]))
                    dupNames = union(dupNames, allNames[-longestRead])
                }
            }
        }
    }
    cList = cList[setdiff(names(cList), dupNames)]
    log$Unique5PrimeStart = length(cList)


    # return list with filtered chimeric reads
    filteredChimReads = names(cList)
    for (i in 1:length(alnReads)) {
        ind = is.element(alnReads[[i]]$qname, filteredChimReads)
        for (j in 1:length(alnReads[[i]])) {
            alnReads[[i]][[j]] = alnReads[[i]][[j]][ind]
        }
    }
    alnReads[["log"]] = log
    return(alnReads)
}


setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="IntegerRangesList", linkerSeq="DNAString",
        minDist="numeric", dupReadDist="numeric"),
    .filterChimericReads)

setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="IntegerRangesList", linkerSeq="DNAString",
        minDist="missing", dupReadDist="missing"),
    .filterChimericReads)

setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="IntegerRangesList", linkerSeq="missing",
        minDist="numeric", dupReadDist="numeric"),
    .filterChimericReads)

setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="IntegerRangesList", linkerSeq="missing",
        minDist="missing", dupReadDist="missing"),
    .filterChimericReads)

setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="missing", linkerSeq="DNAString",
        minDist="numeric", dupReadDist="numeric"),
    .filterChimericReads)

setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="missing", linkerSeq="DNAString",
        minDist="missing", dupReadDist="missing"),
    .filterChimericReads)

setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="missing", linkerSeq="missing",
        minDist="numeric", dupReadDist="numeric"),
    .filterChimericReads)

setMethod("filterChimericReads",
    signature=signature(alnReads="list", targetRegion="missing", linkerSeq="missing",
        minDist="missing", dupReadDist="missing"),
    .filterChimericReads)

.alignShortReads_DNAStringSet <- function(object, bsGenome, seqNames, ensemblNotation) {

    if (is.null(names(object))) {
        names(object) = as.character(1:length(object))
    }

    pDictPlus = PDict(object, max.mismatch=NA,
        tb.start=1, tb.end=min(width(object)))
    pDictMinus = PDict(reverseComplement(object), max.mismatch=NA,
        tb.start=1, tb.end=min(width(object)))

    matches = data.frame(
        chr=as.character(rep(NA, length(object))),
        start=as.integer(rep(NA, length(object))),
        strand=as.character(rep(NA, length(object))),
        unique=rep(TRUE, length(object)),
        stringsAsFactors=FALSE)
    row.names(matches) = names(object)
  
    for (seqname in seqNames) {
        subject = bsGenome[[seqname]]
        index = list(
            "+"=matchPDict(pDictPlus, subject),
            "-"=matchPDict(pDictMinus, subject))

        for (s in c("+", "-")) {
            sindex = index[[s]]
            for (n in names(sindex)) {
                if (length(sindex[[n]]) == 1) {
                    if (matches[n, "unique"] & is.na(matches[n, "chr"])) {
                        matches[n, "chr"] = seqname
                        matches[n, "start"] = start(sindex[[n]])
                        matches[n, "strand"] = s
                    } else {
                        matches[n,] = c(NA, NA, NA, FALSE)
                    }
                } else if (length(sindex[[n]]) > 1) {
                    matches[n,] = c(NA, NA, NA, FALSE)
                }
            }
        }
    }

    if (ensemblNotation) {
        matches$chr = gsub("chr", "", matches$chr)
    }

    return(AlignedRead(object,
        id=BStringSet(names(object)),
        chromosome=matches$chr,
        position=matches$start,
        strand=factor(matches$strand)))
}


.alignShortReads_AVASet <- function(object, bsGenome, seqNames, ensemblNotation) {

   aln = alignShortReads(sread(referenceSequences(object)),
       bsGenome, seqNames, ensemblNotation)

   if (!all(names(sread(referenceSequences(object))) == names(sread(aln)))) {
       stop("Some sequences do not match.")
   }
   
   referenceSequences(object)@position = position(aln)
   referenceSequences(object)@chromosome = chromosome(aln)
   referenceSequences(object)@strand = strand(aln)
   
   return(object)
}



setMethod("alignShortReads", 
    signature=signature(object="DNAStringSet", bsGenome="BSgenome",
        seqNames="character", ensemblNotation="logical"),
    .alignShortReads_DNAStringSet)

setMethod("alignShortReads", 
    signature=signature(object="AVASet", bsGenome="BSgenome",
        seqNames="character", ensemblNotation="logical"),
    .alignShortReads_AVASet)

setMethod("alignShortReads", 
    signature=signature(object="DNAStringSet", bsGenome="BSgenome",
        seqNames="character", ensemblNotation="missing"),
    function(object, bsGenome, seqNames) {
        return(.alignShortReads_DNAStringSet(object, bsGenome, seqNames, FALSE))
    })

setMethod("alignShortReads", 
    signature=signature(object="AVASet", bsGenome="BSgenome",
        seqNames="character", ensemblNotation="missing"),
    function(object, bsGenome, seqNames) {
        return(.alignShortReads_AVASet(object, bsGenome, seqNames, FALSE))
    })

setMethod("alignShortReads", 
    signature=signature(object="DNAStringSet", bsGenome="BSgenome",
        seqNames="missing", ensemblNotation="logical"),
    function(object, bsGenome, ensemblNotation) {
        return(.alignShortReads_DNAStringSet(object, bsGenome, names(bsGenome)[1:24], ensemblNotation))
    })

setMethod("alignShortReads", 
    signature=signature(object="AVASet", bsGenome="BSgenome",
        seqNames="missing", ensemblNotation="logical"),
    function(object, bsGenome, ensemblNotation) {
        return(.alignShortReads_AVASet(object, bsGenome, names(bsGenome)[1:24], ensemblNotation))
    })

setMethod("alignShortReads", 
    signature=signature(object="DNAStringSet", bsGenome="BSgenome",
        seqNames="missing", ensemblNotation="missing"),
    function(object, bsGenome) {
        return(.alignShortReads_DNAStringSet(object, bsGenome, names(bsGenome)[1:24], FALSE))
    })

setMethod("alignShortReads", 
    signature=signature(object="AVASet", bsGenome="BSgenome",
        seqNames="missing", ensemblNotation="missing"),
    function(object, bsGenome) {
        return(.alignShortReads_AVASet(object, bsGenome, names(bsGenome)[1:24], FALSE))
    })






#getAmpliconPositions <- function(dnaSet, genes, dataset="hsapiens_gene_ensembl") {
#  
#  ensembl = useMart("ensembl", dataset=dataset)
#  geneInfo = getBM(attributes=c("hgnc_symbol", "ensembl_gene_id",
#                   "start_position", "end_position", "strand", "chromosome_name"),
#                   filter="hgnc_symbol", values=unique(genes), mart=ensembl)
#
#  geneSeqs = getSequence(id=geneInfo$ensembl_gene_id, type="ensembl_gene_id",
#                         seqType="gene_exon_intron", mart=ensembl)
#
#  gSeqs = DNAStringSet(geneSeqs$gene_exon_intron)
#  names(gSeqs) = geneSeqs$ensembl_gene_id
#  
#  ampPos = data.frame(amplicon=names(dnaSet), start=NA, end=NA, chr=NA,
#              stringsAsFactors=FALSE)
#  for (i in 1:nrow(ampPos)) {
#    amp = dnaSet[[ampPos[i, "amplicon"]]]
#    ind = geneInfo$hgnc_symbol == genes[i]
#    if (geneInfo$strand[ind] == -1) {
#      gene = reverseComplement(gSeqs[[geneInfo[ind, "ensembl_gene_id"]]])
#    } else {
#      gene = gSeqs[[geneInfo[ind, "ensembl_gene_id"]]]
#    }
#
#    m = matchPattern(amp, gene)
#    if (length(m) != 1) {
#      stop(paste("Could not match amplicon:", names(dnaSet)[i]))
#    }
#    
#    ampPos[i, "start"] = geneInfo$start_position[ind] + start(m) - 1
#    ampPos[i, "end"] = geneInfo$start_position[ind] + end(m) - 1
#    ampPos[i, "chr"] = geneInfo$chromosome_name[ind]
#    ampPos[i, "symbol"] = geneInfo$hgnc_symbol[ind]
#    ampPos[i, "ensemblid"] = geneInfo$ensembl_gene_id[ind]
#  }
#
#  return(list(ampPos=ampPos, gSeqs=gSeqs))
#
#}

.sequenceCaptureLinkers <- function() {
  
    linkers = c(
        gSel3 = "CTCGAGAATTCTGGATCCTC",
        gSel4P = "GAGGATCCAGAATTCTCGAGTT"
    )

    linkersSet = DNAStringSet(linkers, use.names=TRUE)

    return(linkersSet)
}


.sequenceCaptureLinkers_character <- function(name) {
    
    linkers = sequenceCaptureLinkers()
    if (all(is.element(name, names(linkers)))) {
        return(linkers[is.element(names(linkers), name)])
    } else {
        stop(paste("Argument name must be on of the following:",
            paste(names(linkers), collapse=", ")))
    }
}


setMethod("sequenceCaptureLinkers", signature=signature(name="missing"),
    .sequenceCaptureLinkers)

setMethod("sequenceCaptureLinkers", signature=signature(name="character"),
    .sequenceCaptureLinkers_character)

.plotVariants <- function(data, gene, transcript, bmExons, bmGene, bmReturn, regions, mutationInfo, groupBy, horiz, cex, title, legend){

  ## When sorting by position, take care that different mutations at the same position will not be in the same group
  ## Therefore, combine the position with the mutation type
  if(groupBy == "pos"){
    groupBy = "groups"
    data$groups = paste(data$pos, data$mutation, sep="_")
  }
  data = data[order(data[, groupBy]), ]
  
  ## parameter horiz stands for horizontal alignment of a group of mutations a single position
  ## if horiz is false, these mutations will be plotted vertically
  if(horiz == FALSE){
    ## list of mutations for every group
    mutGroup = !duplicated(data[, groupBy])
    muts = list()
    for(i in 1:sum(mutGroup)){
      g = data[mutGroup, groupBy][i]
      muts[[i]] = as.character(data[data[, groupBy]==g, "mutation"])
    }
    numMuts = length(unique(data[ , groupBy]))
  }else{
    ## if horiz is true every mutation will be plotted horizontally
    mutGroup = 1:nrow(data)
    muts = as.list(data$mutation)
    numMuts = length(data$label)
  }
  maxMuts = max(table(data[, groupBy]))
  
  ## plot boundaries
  xmin = 0
  #xmax = max(max(data$pos), max(regions$end)) * 3
  xmax = max(bmExons$cds_end/3)
  xlength = xmax - xmin
  ymin = 0
  ymax = 30  ## upper margin of the plot (too large: much empty space at the top of the plot; too small: mutation labels do not fit into the plot region)
  y0 = 6
  y1 = 7.5
  y2 = 10
  axis_steps = 100

  ## design parameters
  labels.cex = cex
  lwd = 1.3
  pch = 21
  
  ## initialize plot window
  plot(c(xmin-xlength*0.2, xmax+xlength*0.2), c(ymin,ymax), type="n", xaxt="n", yaxt="n", xlab="", ylab="", frame.plot=FALSE, xaxs="i", yaxs="i", main=title)
  axis(side=1, at=c(1, seq(xmin+axis_steps, xmax, axis_steps)), pos=y0*0.7, cex.axis=cex)

  ## text size information is important to correctly adjust the lines and points to the plotted text
  textHeight = max(strheight(letters, cex=labels.cex))
  textWidth =  max(strwidth(letters, cex=labels.cex))
  textWidth = textWidth + (textWidth * 0.2)

  ## draw annotated regions
  polygon(x=c(xmin,xmin,xmax,xmax), y=c(y0,y1,y1,y0), col="black", border="black", density=0, lwd=lwd)
  text(x=xmin, y=(y0[1]+y1[1])/2, labels=bmGene$hgnc_symbol[1], pos=2, cex=cex)
  text(x=xmax, y=(y0[1]+y1[1])/2, labels=paste(bmExons$cds_length[1]/3,"aa",sep=""), pos=4, cex=cex)
  if(!any(is.na(regions))){
    for(i in 1:nrow(regions)){
      polygon(x=c(regions$start[i], regions$start[i], regions$end[i], regions$end[i]), y=c(y0,y1,y1,y0), border="black", col=as.character(regions$color)[i], lwd=lwd)
    }
  }
  ## Draw exon regions
  for(i in 1:nrow(bmExons)){
    start = bmExons$cds_start[i]/3
    end = bmExons$cds_end[i]/3
    polygon(x=c(start,start,end,end), y=c(y0*0.9,y0*0.7,y0*0.7,y0*0.9), col="lightgrey", lwd=lwd)
    if(textWidth < (end-start)){
      text((start+end)/2, y0*0.8, bmExons$rank[i], cex=cex)
    }
  }

  ## y-positions of labels and mutation marks
  if(horiz== FALSE){
    y3 = y2+((maxMuts+1)*textHeight)
  }else{
    y3 = y2+(2*textHeight)
  }
  
  ## set mutation labels (take care of overlapping labels using xLab function from TeachingDemos package)
  x = data$pos[mutGroup]
  labels = data$label[mutGroup]
  ## if the plotting device is too small, it may not be possible to align all mutation labels without overlaps (gives a warning in function spread.labs)
  ## thus, catch this warning and retry with smaller font size (90% of original)
  overlappingLabels = TRUE
  while(overlappingLabels == TRUE){
    overlappingLabels = FALSE
    catchUpdate = tryCatch(
             xLab <- spread.labs(x=x, mindiff=textWidth, maxiter=100000, min=xmin, max=xmax),
             warning = function(w){
               message("Plot region too small for all mutation labels: Reducing font size. It may be appropriate to change the width of the device and restart the plot.")
               overlappingLabels = TRUE
               labels.cex = labels.cex * 0.9
               textWidth = max(strwidth(letters, cex = labels.cex))
               textWidth = textWidth + (textWidth * 0.2)
               return(list(overlappingLabels, labels.cex, textWidth))
             }
             )
    ## overlaps
    if(is.list(catchUpdate)){
      overlappingLabels = catchUpdate[[1]]
      labels.cex = catchUpdate[[2]]
      textWidth = catchUpdate[[3]]
    }
    ## no overlaps
    if(is.vector(catchUpdate)){
      xLab = catchUpdate
    }
  }

  x = data.frame(pos_axis=x, pos_labels=xLab, color=data$color[mutGroup], group=data[mutGroup, groupBy], stringsAsFactors=FALSE)
  colnames(x) = c("pos_axis", "pos_labels", "color", groupBy)
  text(x$pos_labels, y3+(textHeight/2), labels, srt=90, cex=labels.cex, adj=c(0,0.5), offset=3)

  ## mark mutations (as colored points)
  pt.cex = labels.cex
  ## vertical alignment of mutation marks
  if(horiz == FALSE){
    for(i in 1:numMuts){
      lines(list(x=c(x$pos_axis[i],x$pos_axis[i],x$pos_labels[i],x$pos_labels[i]), y=c(y0,y1,y2,y3)), col=x$color[i], lwd=lwd)
      xm = x$pos_labels[i]
      ym = y2 + textHeight
      for(m in muts[[i]]){
        if(!is.na(m)){
          points(xm, ym, bg=as.character(subset(mutationInfo, mutation == m)$color), col="black", cex=pt.cex, pch=pch, )
          ym = ym + textHeight
        }
      }
  }
  ## horizontal alignment of mutation marks
  }else{
    ## draw single mutation lines
    x2 = x[!(x[, groupBy] %in% x[duplicated(x[groupBy]), groupBy]), ]
    for(i in 1:nrow(x2)){
      lines(list(x=c(x2$pos_axis[i],x2$pos_axis[i],x2$pos_labels[i],x2$pos_labels[i]), y=c(y0,y1,y2,y3)), col=x2$color[i], lwd=lwd)
    }
    ## draw lines for mutation groups (as "forks" like in dendrograms)
    groups = table(x[, groupBy])
    groups = groups[groups > 1]
    anyGroups = any(groups > 1)
    if(anyGroups == TRUE){
      groups = groups[groups > 1]
      x_group= x[x[, groupBy] %in% x[duplicated(x[groupBy]), groupBy], ]
      for(i in 1:length(groups)){
        x2 = x_group[x_group[, groupBy] == names(groups[i]), ]
        lines(x=c(min(x2$pos_labels), min(x2$pos_labels), max(x2$pos_labels), max(x2$pos_labels)), y=c(y3,y2,y2,y3), col=x2$color[1], lwd=lwd)
        #points(x=(max(x2$pos_labels)+min(x2$pos_labels))/2, y=y2, bg=x2$color[1], pch=25, cex=pt.cex*0.75)  ## "anchor point"
        polygon(x=c(min(x2$pos_labels), min(x2$pos_labels), max(x2$pos_labels), max(x2$pos_labels)), y=c(y2,y3,y3,y2), col=x2$color[1], border=NA, density=18, lwd=lwd*0.75)
        lines(list(x=c(x2$pos_axis[1],x2$pos_axis[1],(max(x2$pos_labels)+min(x2$pos_labels))/2), y=c(y0,y1,y2)), col=x2$color[1], lwd=lwd)
      }
    }
    ## draw mutations marks
    for(i in 1:nrow(x)){
      m = muts[[i]]
      xm = x$pos_labels[i]
      ym = y2 + textHeight
      points(xm, ym, bg=as.character(subset(mutationInfo, mutation == m)$color), col="black", cex=pt.cex, pch=pch)
    }
  }
  
  ## draw legend for mutations and annotated regions
  if(legend == TRUE){
    legend(x="topleft", legend=mutationInfo$legend, pt.bg=as.character(mutationInfo$color), col="black", pch=pch, bty="n", cex=cex, pt.cex=cex)
    if(!any(is.na(regions))){  
      legend(x="bottomleft", legend=regions$name, fill=as.character(regions$color), bty="n", cex=cex, horiz=TRUE)
    }
  }

  return(bmReturn)
}

.annotatedVariants_to_dataFrame <- function(varAnnot, transcript){

  ## convert input of class AnnotatedVariants
  data = annotatedVariants(varAnnot)
  mutNames = names(data)
  label = vector(mode="character")
  pos = vector(mode="integer")
  type = vector(mode="character")
  m_count = 1
#  aminos = getAminoAbbr()  ## maybe offer the possibility to label amino acid changes
  for(mn in mutNames){
    
    m = data[[mn]]$exons
    
    ## see if exon data available
    if(nrow(m) > 0){
    
      ## see if variant lies on the given transcript
      m = subset(m, m$ensembl_transcript_id == transcript)
      if(nrow(m) > 0){
        
        for(i in 1:nrow(m)){
          
          ## draw markers for variants that lie in a coding area
          if(m[i,"coding"] == TRUE){
            if(length(grep("-", m[i, "codonMut"])) == 0){
              pos[m_count] = m[i, "numCodonStart"]             
              if(m[i,"AminoRef"] != "*" & m[i,"AminoMut"] != "*"){                
                ## missense point mutation
                if(m[i,"AminoRef"] !=  m[i,"AminoMut"]){
                  type[m_count]= "M"
 #                 label[m_count] = paste(aminos[m[i,"AminoRef"]], aminos[m[i,"AminoMut"]], sep=" -> ")
                }else{
                  ## silent point mutation
                  type[m_count] = "S"
 #                 label[m_count] = aminos[m[i,"AminoRef"]]
                }                
                ## nonsense point mut
              } else {
                if(m[i,"AminoRef"] != "*" & m[i,"AminoMut"] == "*"){
                  type[m_count] = "N"
 #                 label[m_count] = paste(aminos[m[i,"AminoRef"]], aminos[m[i,"AminoMut"]], sep=" -> ")
                }else
                warning("variant ", mn, " is of unknown type")                
              }             
              ## deletion
            } else {
              pos[m_count] = (m[i, "numCodonStart"] +
                              m[i, "numCodonEnd"]) / 2
              type[m_count] = "D"
 #             label[m_count] = paste(aminos[m[i,"AminoRef"]], m[i,"AminoMut"], sep=" -> ")
            }
          }
          m_count = m_count + 1
        }
      }
    }
  }
  
  df = data.frame(label=rep(" ", length(pos)), pos=pos, mutation=type, color=rep("black", length(pos)), stringsAsFactors=FALSE)
  mutationInfo = data.frame(mutation=c("M","N","S","D"), legend=c("Missense","Nonsense","Silent","Deletion"), color=c("cadetblue4","darkred","darkgoldenrod1","blue"), stringsAsFactors=FALSE)
  return(list(df, mutationInfo))
}

.getEnsemblInfo <- function(gene, transcript){
  ## retrieve exon and gene information from Ensembl (if Ensembl gene-id is provided)
  #ensembl=useMart("ensembl", dataset="hsapiens_gene_ensembl", host="http://grch37.ensembl.org")
  ensembl=useEnsembl(biomart="genes", dataset="hsapiens_gene_ensembl", GRCh=37)
  bmAttributes = c("ensembl_gene_id", "ensembl_transcript_id", "rank", "cds_start", "cds_end", "cds_length")
  bmExons = getBM(attributes=bmAttributes, filters="ensembl_gene_id", values=gene, mart=ensembl)
  bmExons = subset(bmExons, !apply(is.na(bmExons), 1, any))
  bmReturn = bmExons
  if(is.na(transcript)){
    ## take largest transcript if no ensembl transcript id is given
    transcript = subset(bmExons, cds_length == max(bmExons$cds_length)[1])$ensembl_transcript_id[1]
    warning("No Ensembl transcript-id given. Will choose largest transcript. It is recommended to specify the transcript via the \"transcript\" parameter to ensure a most appropriate illustration.")
  }
  bmExons = subset(bmExons, ensembl_transcript_id == transcript)
  bmAttributes = c("ensembl_gene_id", "hgnc_symbol")
  bmGene = getBM(attributes=bmAttributes, filters="ensembl_gene_id", values=gene, mart=ensembl)
  return(list(gene, transcript, bmExons, bmGene, bmReturn))
}




## Method for the R453Toolbox class "AnnotatedVariants"
setMethod("plotVariants",
          signature=c(data="AnnotatedVariants", "character"),
          function(data, gene, transcript=NA, regions, horiz=FALSE, cex=1, title="", legend=TRUE){
            
            #library(TeachingDemos)
            #library(biomaRt)

            ## check integrity of transcript
            if(!(is.character(transcript) | is.na(transcript))){
                stop("Invalid argument: transcript has to be an Ensembl transcript-id of type character")
            }
            ## check integrity of regions
            if(missing(regions)){
              regions = NA
            }else{
              if(!is.data.frame(regions))
                stop("Invalid argument: regions has to be of type data.frame")
              if(!all( c("name", "start", "end") %in% colnames(regions)))
                stop("Wrong columns in regions: please provide data as a data.frame with columns \"name\", \"start\", \"end\" and \"color\". See ?plotVariants for details.")
              if(!("color" %in% colnames(regions))){
                warning("No colors specified for given regions. Assigning colors automatically. If you find them inappropriate, please add a column \"color\" to your region data.frame. See parameter \"regions\" in ?plotVariants for details.")
                regions$color = rainbow(nrow(regions))
              }
            }
            ## check integrity of horiz, title, legend, cex
            if(!is.logical(horiz))
              stop("Invalid argument: horiz has to be TRUE or FALSE")
            if(!is.character(title))
              stop("Invalid argument: title has to be of type character")
            if(!is.logical(legend))
              stop("Invalid argument: legend has to be TRUE or FALSE")
            if(!is.numeric(cex))
              stop("Invalid argument: cex has to be a numeric value")

            ## get Ensembl info for given gene / transcript
            bmInfo = .getEnsemblInfo(gene, transcript)
            
            df = .annotatedVariants_to_dataFrame(data, bmInfo[[2]])
            data = df[[1]]
            mutationInfo = df[[2]]
            
            if(nrow(data) >0){
              .plotVariants(data=data, gene=bmInfo[[1]], transcript=bmInfo[[2]], bmExons=bmInfo[[3]], bmGene=bmInfo[[4]], bmReturn=bmInfo[[5]], regions=regions, mutationInfo=mutationInfo, groupBy="pos", cex=cex, horiz=horiz, title=title, legend=legend)
            }else{
              stop("No annotation for given transcript. Stopping here. I'm sorry.")
            }
            
          })


## Method for data in data.frame format
setMethod("plotVariants",
          signature=c(data="data.frame", gene="character"),
          function(data, gene, transcript=NA, regions, mutationInfo, groupBy="pos", horiz=FALSE, cex=1, title="", legend=TRUE){

            #library(TeachingDemos)
            #library(biomaRt)
        
            ## check integrity of data
            data = subset(data, !apply(is.na(data), 1, any))
            if(!all(c("label", "pos", "mutation") %in% colnames(data)))
              stop("Wrong data columns: please provide data as a data.frame with columns \"label\", \"pos\", \"mutation\" (optional: \"color\"). See ?plotVariants for details.")
            if(!("color" %in% colnames(data)))
              data$color = "black"
            
            ## check integrity of transcript
              if(!(is.character(transcript) | is.na(transcript))){
                stop("Invalid argument: transcript has to be an Ensembl transcript-id of type character")
            }
            ## check integrity of regions
            if(missing(regions)){
              regions = NA
            }else{
              if(!is.data.frame(regions))
                stop("Invalid argument: regions has to be of type data.frame")
              if(!all( c("name", "start", "end") %in% colnames(regions)))
                stop("Wrong columns in regions: please provide data as a data.frame with columns \"name\", \"start\", \"end\" and \"color\". See ?plotVariants for details.")
              if(!("color" %in% colnames(regions))){
                warning("No colors specified for given regions. Assigning colors automatically. If you find them inappropriate, please add a column \"color\" to your region data.frame. See parameter \"regions\" in ?plotVariants for details.")
                regions$color = rainbow(nrow(regions))
              }
            }
            ## check integrity of mutationInfo
            if(missing(mutationInfo)){
              warning("Legend and colors for all mutation types were generated automatically. It is suggested to specify your own mutation info. See parameter \"mutationInfo\" in ?plotVariants for details.")
              muts = levels(factor(data$mutation))
              col = rainbow(length(muts))
              mutationInfo = data.frame(mutation=muts, legend=muts, color=col, stringsAsFactors=FALSE)              
            }else{
              if(!is.data.frame(mutationInfo))
                stop("Invalid argument: mutationInfo has to be a data frame")
              if(!all(c("mutation", "legend") %in% colnames(mutationInfo)))
                stop("Wrong columns in mutationInfo: please provide data as a data.frame with columns \"mutation\", \"legend\" and \"color\". See ?plotVariants for details.")
              if(!("color" %in% colnames(mutationInfo))){
                warning("No colors specified for given mutation types. Assigning colors automatically. If you find them inappropriate, please add a column \"color\" to your mutationInfo data.frame. See parameter \"mutationInfo\" in ?plotVariants for details.")
                mutationInfo$color = rainbow(nrow(mutationInfo))
              }
            }
            ## check integrity of groupBy
            if(!is.character(groupBy)){
              stop("Invalid argument: groupBy has to be of type character")
            }else{
              if(!(groupBy %in% colnames(data)))
                stop("Column not found: When grouping mutations by a column other than \"pos\", make sure the column has the same name in your data")
            }
            ## check integrity of horiz, title, legend, cex
            if(!is.logical(horiz))
              stop("Invalid argument: horiz has to be TRUE or FALSE")
            if(!is.character(title))
              stop("Invalid argument: title has to be of type character")
            if(!is.logical(legend))
              stop("Invalid argument: legend has to be TRUE or FALSE")
            if(!is.numeric(cex))
              stop("Invalid argument: cex has to be a numeric value")

            ## get Ensembl info for given gene / transcript
            bmInfo = .getEnsemblInfo(gene, transcript)
            
            return(.plotVariants(data=data, gene=bmInfo[[1]], transcript=bmInfo[[2]], bmExons=bmInfo[[3]], bmGene=bmInfo[[4]], bmReturn=bmInfo[[5]], regions=regions, mutationInfo=mutationInfo, groupBy=groupBy, horiz=horiz, cex=cex, title=title, legend=legend))
          })

getAminoAbbr <- function() {
  aminos = c("Alanine", "Arginine", "Asparagine", "Aspartic acid", "Cysteine", "Glutamic acid", 
             "Glutamine", "Glycine", "Histidine", "Isoleucine", "Leucine", "Lysine", 
             "Methionine", "Phenylalanine", "Proline", "Serine", "Threonine", "Tryptophan", 
             "Tyrosine", "Valine", "Stop")
  names(aminos) = c("A", "R", "N", "D", "C", "E", "Q", "G", "H", "I", "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V", "*")
  return(aminos)
}

## length, names are the same for seqsC1, commonBpsC1, commonAlignC1, alignedReadsC1 (and C2 respectively)

setMethod("length", signature(x="Breakpoints"),
    function(x) {
        length(commonBpsC1(x))
    }
)

setMethod("names", signature(x="Breakpoints"),
    function(x) {
        names(commonBpsC1(x))
    }
)

setReplaceMethod("names", signature(x="Breakpoints", value="ANY"),
    function(x, value) {
        names(seqsC1(x)) = value
        names(seqsC2(x)) = value
        names(commonBpsC1(x)) = value
        names(commonBpsC2(x)) = value
        names(commonAlignC1(x)) = value
        names(commonAlignC2(x)) = value
        names(alignedReadsC1(x)) = value
        names(alignedReadsC2(x)) = value
        return(x)
    }
)

setMethod("summary", signature(object="Breakpoints"),
    function(object) {

	if(length(object) > 0){
          chrA=sapply(commonBpsC1(object), function(x) as.character(x[1, "chr"]))
          chrB=sapply(commonBpsC1(object), function(x) as.character(x[2, "chr"]))
          bpAC1 = sapply(commonBpsC1(object), function(x) x[1, "breakpoint"])
          bpBC1 = sapply(commonBpsC1(object), function(x) x[2, "breakpoint"])
          bpAC2 = sapply(commonBpsC2(object), function(x) if(nrow(x) > 0) x[1, "breakpoint"] else return(NA))
          bpBC2 = sapply(commonBpsC2(object), function(x) if(nrow(x) > 0) x[2, "breakpoint"] else return(NA))

          # get chromosomes from Case 2 temporarly
          chrAC2=sapply(commonBpsC2(object), function(x) if(nrow(x) > 0) as.character(x[1, "chr"]) else return(NA))
          chrBC2=sapply(commonBpsC2(object), function(x) if(nrow(x) > 0) as.character(x[2, "chr"]) else return(NA))

          ind = !is.na(chrAC2)
          # chrA from case 2 is not chrA from case 1
          ind2 = chrA[ind] != chrAC2[ind]
          bpTmp = bpAC2[ind][ind2]
          bpAC2[ind][ind2] = bpBC2[ind][ind2]
          bpBC2[ind][ind2] = bpTmp
          # inversion
          ind = which(chrA == chrB & !is.na(chrAC2))
          for (i in ind) {
            if (abs(bpAC1[i] - bpAC2[i]) > abs(bpAC1[i] - bpBC2[i])) {
              bpTmp = bpAC2[i]
              bpAC2[i] = bpBC2[i]
              bpBC2[i] = bpTmp
            }
          }
          rm(chrAC2, chrBC2)


          # Get number of reads.
          firstCase = sapply(alignedReadsC1(object), function(x) {
            t = table(strand(x))
            if (length(t) == 1) {
              return(paste(t, "0", sep="/"))
            } else if (length(t) == 2) {
              return(paste(t[1], t[2], sep="/"))
            } else {
              stop("More than two strands!")
            }
          })
          secondCase = sapply(alignedReadsC2(object), function(x) {
            if (length(x) > 0) {
              t = table(strand(x))
              if (length(t) == 1) {
                return(paste(t, "0", sep="/"))
              } else if (length(t) == 2) {
                return(paste(t[1], t[2], sep="/"))
              } else {
                stop("More than two strands!")
              }
            } else {
              return("0/0")
            }
          })
          total = sapply(seqsC1(object), function(x) nrow(x)) + sapply(seqsC2(object), function(x) nrow(x))
          
          df = data.frame(
            ChrA=chrA,
            ChrB=chrB,
            BpACase1=bpAC1,
            BpBCase1=bpBC1,
            BpACase2=bpAC2,
            BpBCase2=bpBC2,
            NoReadsCase1=firstCase,
            NoReadsCase2=secondCase,
            NoReadsTotal=total)
        
          return(df)
        }else{
          warning("The given object is empty")
          return(data.frame())
        }
    }
)


setMethod(show, signature(object="Breakpoints"),
    function(object) {
        n = min(6, length(object))
 	if(n > 0){
            df = data.frame(
            	Size=sapply(seqsC1(object), function(x) nrow(x)) + sapply(seqsC2(object), function(x) nrow(x)),
            	ChrA=sapply(commonBpsC1(object), function(x) as.character(x[1, "chr"])),
            	ChrB=sapply(commonBpsC1(object), function(x) as.character(x[2, "chr"])),
            	stringsAsFactors=FALSE
            )
            if (n == 1) {
            	message(paste("Collection of 1 breakpoint:"))
            } else {
            	message(paste("Collection of", length(object), "breakpoints:"))
            }
            print(df[1:n,])
            if (length(object) > n) {
            	message("...")
            }
	}else
	    message("No consensus breakpoints detected: Object is empty")
    }
)

setMethod(table, signature(...="Breakpoints"),
    function(...) {
      if(length(...) > 0){
        size = sapply(seqsC1(...), function(x) nrow(x)) + sapply(seqsC2(...), function(x) nrow(x))
        return(table(size))
      }else{
        stop("The given object is empty")
      }
    }
)


#########
## GETTER
#########

setMethod("[",
    signature("Breakpoints"),
    function(x, i, ...){
        if((is.numeric(i) && all(is.element(i, -length(x):length(x))))  || is.element(i, names(x))) {
	    seqsC1(x) = seqsC1(x)[i]
	    seqsC2(x) = seqsC2(x)[i]
	    commonBpsC1(x) = commonBpsC1(x)[i]
	    commonBpsC2(x) = commonBpsC2(x)[i]
	    commonAlignC1(x) = commonAlignC1(x)[i]
	    commonAlignC2(x) = commonAlignC2(x)[i]
	    alignedReadsC1(x) = alignedReadsC1(x)[i]
	    alignedReadsC2(x) = alignedReadsC2(x)[i]
	    return(x)
	}else
            stop("Index invalid or out of range")
    }
)

setMethod("seqsC1", signature(object="Breakpoints"),
    function(object) {
        return(object@seqsC1)
    }
)
setMethod("seqsC2", signature(object="Breakpoints"),
    function(object) {
        return(object@seqsC2)
    }
)

setMethod("commonBpsC1", signature(object="Breakpoints"),
    function(object) {
        return(object@commonBpsC1)
    }
)
setMethod("commonBpsC2", signature(object="Breakpoints"),
    function(object) {
        return(object@commonBpsC2)
    }
)

setMethod("commonAlignC1", signature(object="Breakpoints"),
    function(object) {
        return(object@commonAlignC1)
    }
)
setMethod("commonAlignC2", signature(object="Breakpoints"),
    function(object) {
        return(object@commonAlignC2)
    }
)

setMethod("alignedReadsC1", signature(object="Breakpoints"),
    function(object) {
        return(object@alignedReadsC1)
    }
)
setMethod("alignedReadsC2", signature(object="Breakpoints"),
    function(object) {
        return(object@alignedReadsC2)
    }
)

#########
## SETTER
#########

setReplaceMethod("seqsC1",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@seqsC1 = value
        return(object)
    }
)
setReplaceMethod("seqsC2",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@seqsC2 = value
        return(object)
    }
)

setReplaceMethod("commonBpsC1",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@commonBpsC1 = value
        return(object)
    }
)
setReplaceMethod("commonBpsC2",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@commonBpsC2 = value
        return(object)
    }
)

setReplaceMethod("commonAlignC1",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@commonAlignC1 = value
        return(object)
    }
)
setReplaceMethod("commonAlignC2",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@commonAlignC2 = value
        return(object)
    }
)

setReplaceMethod("alignedReadsC1",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@alignedReadsC1 = value
        return(object)
    }
)
setReplaceMethod("alignedReadsC2",
    signature=signature(object="Breakpoints", value="list"),
    function(object, value) {
        object@alignedReadsC2 = value
        return(object)
    }
)

%\VignetteIndexEntry{A package for importing and analyzing data from Roche's Genome Sequencer System}
%\VignetteDepends{rtracklayer}
%\VignetteKeywords{Roche, AVA, GSM, something else}
%\VignettePackage{R453Plus1Toolbox}

\documentclass[10pt]{article}

\usepackage{times}
\usepackage{hyperref}
\usepackage{verbatim} %TEMP: only for block comments

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}
\newcommand{\Rfunarg}[1]{{\texttt{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}
\newcommand{\Rcode}[1]{{\texttt{#1}}}

\SweaveOpts{prefix.string=R453Plus1}


\begin{document}
\SweaveOpts{concordance=TRUE}

\title{R453Plus1Toolbox \\ A package for importing and analyzing data from Roche's Genome Sequencer System}
\author{Hans-Ulrich Klein, Christoph Bartenhagen, Christian Ruckert}
\date{December 05, 2013}

\maketitle

\tableofcontents

<<echo=FALSE>>=
options(width=60)
options(continue=" ")
@

\newpage

\section{Introduction}
The R453Plus1 Toolbox comprises useful functions for the analysis of data generated by Roche's 454 
sequencing platform. It adds functions for quality assurance as well as for annotation and 
visualization of detected variants, complementing the software tools shipped by Roche with their 
product. Further, a pipeline for the detection of structural variants is provided. \\

<<preliminaries>>=
library(R453Plus1Toolbox)
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%___Analysis of PCR amplicon projects___%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Analysis of PCR amplicon projects}
This section deals with the analysis of projects investigating massively parallel data generated 
from specifically designed PCR products.


\subsection{Importing a Roche Amplicon Variant Analyzer project}
\label{sec:AVASetFunctions}
The function \Rfunction{AVASet} imports data from Roche's Amplicon Variant Analyzer (AVA). This can be done in three ways, depending on the version
of your AVA software:
\subsubsection{Import from AVA without AVA-CLI (version 2.5 and lower)}
For projects created with the AVA software version $\leq$ 2.5, \Rfunction{AVASet} expects only a \Rfunarg{dirname} pointing 
to the project data, i.e. a directory that contains the following files and subdirectories: \\
\begin{itemize}
\item "Amplicons/\-ProjectDef/\-ampliconsProject.txt"
\item "Amplicons/\-Results/\-Variants/\-currentVariantDefs.txt"
\item "Amplicons/\-Results/\-Variants"
\item "Amplicons/\-Results/\-Align"
\end{itemize}
There is an example project "AVASet" included in the \Rpackage{R453Plus1Toolbox} installation directory:
<<label=createRocheAVASet1>>=
projectDir = system.file("extdata", "AVASet", package = "R453Plus1Toolbox") 
avaSet = AVASet(dirname=projectDir)
@
\subsubsection{Import from AVA with AVA-CLI}
The function \Rfunction{AVASet} can directly access the AVA Command Line Interface (AVA-CLI) from within R. If the AVA software is installed on the same machine 
that runs R, the easiest way to import a project is to specify the project directory with \Rfunarg{dirname} 
and the path to the binaries in the AVA software's installation directory with \Rfunarg{avaBin}. It is usually the directory "bin" 
containing the AVA-CLI command interpreter "doAmplicon".\\
Let's say the AVA software was installed to the directory "/home/User/AVA". Then, the function call looks like:
<<label=createRocheAVASet2, eval=FALSE>>=
projectDir = "My/AVA/Project"
avaSet = AVASet(dirname=projectDir, avaBin="/home/User/AVA/bin")
@

\subsubsection{Import from AVA with projects exported via AVA-CLI}
If the AVA software is not installed on the same machine that runs R, all data must be exported manually using AVA-CLI.
It can be accessed via the command line interpreter "doAmplicon" from the AVA software's installation directory. 
Within the AVA-CLI, load your project with the command "open". \Rfunction{AVASet} expects five files (variant information is optional): \\
\begin{table}[ht]
\begin{scriptsize}
	\begin{center}
		\begin{tabular}{l|l|p{3cm}}
		\hline                    
		\Rfunction{AVASet} argument
		& AVA-CLI command
		& Description \\
		\hline
		\Rfunarg{file\_sample}                     
		& list sample -outputFile sample.csv
		& Table with sample names and annotations \\
		\Rfunarg{file\_amp}                     
		& list amplicon -outputFile amp.csv
		& Table with primer sequences, positions and annotations \\
		\Rfunarg{file\_reference}                     
		& list reference -outputFile reference.csv
		& Reference sequences \\
		\Rfunarg{file\_variant}                     
		& list variant -outputFile variant.csv
		& Detected variants (if available) \\
		\Rfunarg{file\_variantHits}                     
		& report variantHits -outputFile variantHits.csv
		& Variant hits for all samples (if available) \\
		\hline
		\end{tabular}
	\end{center}
	\caption{\Rfunction{AVASet} function arguments for loading projects exported via AVA-CLI.}
	\label{table:AVASet_AVACLI}
\end{scriptsize}
\end{table} 
\\
Note, that all exported tables are expected to be in csv-format. \\
There is an example project "AVASet\_doAmplicon" included in the \Rpackage{R453Plus1Toolbox} installation directory:
<<label=createRocheAVASet3, eval=FALSE>>=
projectDir = system.file("extdata", "AVASet_doAmplicon", package="R453Plus1Toolbox")
avaSetExample = AVASet(dirname=projectDir, file_sample="sample.csv",
 file_amp="amp.csv", file_reference="reference.csv", file_variant="variant.csv", 
 file_variantHits="variantHits.csv")
@
\Rfunction{AVASet} searches the specified \Rfunarg{dirname} for the exported csv-files. \Rfunarg{file\_variant} \Rfunarg{file\_variantHits} can be omitted if no variant information is
available for the project.

\subsection{The AVASet class}
The \Rclass{AVASet} class defines a container to store data imported from projects conducted with 
Roche's AVA software. It extends the \Rpackage{Biobase} \Rclass{eSet} to store 
all relevant information.\\
<<label=showAVASet>>=
avaSet
@

An object of class \Rclass{AVASet} consists of three main components: \\
\begin{enumerate}
	\item Variants: \\
		The variants part stores data about the found variants and is accessible by the functions 
        \Rfunction{assayData}, \Rfunction{featureData} and 
		\Rfunction{phenoData} known from Biobase eSet. \\
		The \Rcode{assayData} slot contains four matrices with variants as rows and samples as 
        columns:
		\begin{itemize}
			\item variantForwCount: Matrix containing the number of reads with the respective 
				variant in forward direction.
			\item variantRevCount: Matrix containing the number of reads with the respective variant 
				in reverse direction.
			\item totalForwCount: Matrix containig the total coverage for every variant location in 
				forward direction.
			\item totalRevCount: Matrix containing the total coverage for every variant location in 
				reverse direction.
		\end{itemize}
<<label=assayDataAVA>>=
assayData(avaSet)$totalForwCount[1:3, ]
@
		The \Rcode{featureData} slot provides additional information on the variants. 
		\Rfunction{fData} returns a data frame with variants as rows and the following columns:
		\begin{itemize}
			\item name/canonicalPattern: Short identifiers of a variant including the position and 
				the bases changed.
			\item referenceSeq: Gives the identifier of the reference sequence (see below).
			\item start/end: The position of the variant relative to the reference sequence.
			\item variantBase/referenceBases: The bases changed in the variant.    
		\end{itemize}
<<label=fDataAVA>>=
fData(avaSet)[1:3, ]
@
		The \Rcode{phenoData} slot provides sample-IDs, multiplexer IDs (MID1, MID2), the pico titer 
		plate (PTP) accession number, the lane, the read group and additional textual annotation for 
		each sample. Most of these informations are imported directly from Roche's software.
<<label=pDataAVA>>=
pData(avaSet)
@
	\item Amplicons: \\
		This part stores information about the used amplicons and is accessible by the functions 
		\Rfunction{assayDataAmp} and \Rfunction{fDataAmp}. \\
		The slot \Rcode{assayDataAmp} contains two matrices with amplicons as rows and samples as 
		columns:
		\begin{itemize}
			\item forwCount: Matrix containing the number of reads for each amplicon and each sample 
				in forward direction.
			\item revCount: Matrix containing the number of reads for each amplicon and each sample 
				in reverse direction.
		\end{itemize}
<<label=assayDataAmpAVA>>=
assayDataAmp(avaSet)$forwCount
@
		The slot \Rcode{featureDataAmp} contains an AnnotatedDataFrame with additional information 
		on each amplicon: 
		\begin{itemize}
			\item ampID: The identifier of the current amplicon.
			\item primer1, primer2: The primer sequences for each amplicon.
			\item referenceSeqID: The identifier of the reference sequence (see below).
			\item targetStart/targetEnd: The coordinates of the target region.
		\end{itemize}
<<label=fDataAmpAVA>>=
fDataAmp(avaSet)
@
As both refer to the same samples, the variants phenoData slot is used for amplicons as well. 
	\item Reference sequences: \\
		This part stores data about the reference sequences the amplicons were selected from. All 
		information is stored into an object of class \Rclass{AlignedRead}. The reads are accessible 
		via	\Rfunction{sread}. To retrieve additional information from Ensembl about the chromosome, 
		the position and the strand of each reference sequence run function 
		\Rfunction{alignShortReads} (see section \ref{sec:Annotations} for details).
<<label=referenceSequences>>=
library(ShortRead)
referenceSequences(avaSet)
sread(referenceSequences(avaSet))
@
\end{enumerate}

The following table sums up the available slots and accessor functions: \\
\begin{table}[ht]
	\begin{center}
		\begin{tabular}{l|p{7cm}}
		\hline

		Function/Slot                    
		& Description \\

		\hline

		\Rfunction{assayData}                     
		& Contains the number of reads and the total coverage for every variant and each sample in forward and reverse direction.\\

		\Rfunction{fData}/\Rfunction{featureData}
		& Contains information about the type, position and reference of each variant. \\

		\Rfunction{pData}/\Rfunction{phenoData}                   
		& Contains sample-IDs, multiplexer IDs (MID1, MID2), the pico titer plate (PTP) accession number, the lane, the read group and additional 
		textual annotation for each sample. \\

		\Rfunction{assayDataAmp}                  
		& Contains the number of reads for every amplicon and each sample in forward/reverse direction.\\

		\Rfunction{fDataAmp}/\Rfunction{featureDataAmp} 
		& Contains the primer sequences, reference sequence and the coordinates of the target region for each amplicon.\\

		\Rfunction{referenceSequences}         
		& Contains the reference sequences for the amplicons together with additional annotations.\\

		\hline
		\end{tabular}
	\end{center}
	\caption{AVASet contents and accessor functions.}
	\label{table:AVASet}
\end{table}



\subsection{Subsetting an AVASet}
\label{sec:Subsetting}
A subset of an \Rclass{AVASet} object can be generated using the common \Rcode{"[]"}-notation: 
<<label=AVASubSet1>>=
avaSubSet = avaSet[1:10, "Sample_1"]
@
The first dimension refers to the variants and the second dimension to the samples, so an 
\Rclass{AVASet} with ten variants and one sample is returned.\\
This is a short and to some extend equivalent version of the function \Rfunction{subset}, which 
expects a \Rfunarg{subset} argument and the respective \Rfunarg{dimension} (either "variants", 
"samples" or "amplicons"):
<<label=AVASubSet2>>=
avaSubSet = subset(avaSet, subset=1:10, dimension="variants")
@ 
The following is equivalent to the \Rcode{"[]"}-example above:
<<label=AVASubSet3>>=
avaSubSet = subset(subset(avaSet, subset=1:10, dimension="variants"), subset="Sample_1", 
 dimension="samples")
@ 
In contrast to the \Rcode{"[]"}-Notation, the function \Rfunction{subset} allows further subsetting 
by amplicons:
<<label=AVASubSet4>>=
avaSubSet = subset(avaSet, subset=c("TET2_E11.04", "TET2_E06"), dimension="amplicons")
@ 
When subsetting by amplicons all variants referring to amplicons that are not in the subset will be 
excluded. \\



\subsection{Setting filters on an AVASet}
\label{sec:Filters}
Another way of generating a subset of an \Rclass{AVASet} object is filtering only those variants, 
whose coverage (in percent) in forward and reverse direction respectively is higher than a given 
\Rfunarg{filter} value in at least one sample. Here, the coverage is defined as the percentual 
amount of the reads with the given variant on the number of all reads covering the variant's 
position. \\
The function \Rfunction{setVariantFilter} returns an updated \Rclass{AVASet} object that meets the 
given requirements:
<<label=filterAVA1>>=
avaSetFiltered1 = setVariantFilter(avaSet, filter=0.05)
@
The above example returns an \Rclass{AVASet}, which only contains variants whose coverage is greater 
than 5\% in at least one sample.\\
Passing a vector of two \Rfunarg{filter} values applies filtering according to forward and reverse 
read direction separately:
<<label=filterAVA2>>=
avaSetFiltered2 = setVariantFilter(avaSet, filter=c(0.1, 0.05))
@
In fact, when filtering an \Rclass{AVASet}, the whole object is still availabe. The filter only 
affects the output given by accessor functions like \Rfunction{fData}, \Rfunction{featureData} and 
\Rfunction{assayData}.\\
The process can be reversed and the filter value(s) can be reset to zero by calling
<<label=filterAVA4>>=
avaSet = setVariantFilter(avaSetFiltered1, filter=0)
@
or simply
<<label=filterAVA5>>=
avaSet = setVariantFilter(avaSetFiltered2)
@



\subsection{Variant coverage}
The function \Rfunction{getVariantPercentages} displays the coverage of the variants for a given 
\Rfunarg{direction} (either "forward", "reverse", or "both"):
<<label=varPercentages1>>=
getVariantPercentages(avaSet, direction="both")[20:25, 1:4]
@ 
In the example above, \Rfunction{getVariantPercentages} is simply a short form of calculating
<<label=varPercentages2, results=hide>>=
(assayData(avaSet)[[1]] + assayData(avaSet)[[3]]) / (assayData(avaSet)[[2]]
 + assayData(avaSet)[[4]])
@ 



\subsection{Annotations and Variant Reports}
\label{sec:Annotations}

Before creating the variant and quality report, the reference sequences must be aligned against a 
reference genome and afterwards the variants have to be annotatetd. \\
The method \Rfunction{alignShortReads} aligns the reference sequences from an \Rclass{AVASet} 
against a given reference genome. Only exact (no errors) and unique matches are returned. In the 
example below the hg19 assembly as provided by UCSC from package 
\Rpackage{BSgenome.Hsapiens.UCSC.hg19} is used as reference:
<<label=alignShortReads, eval=FALSE>>=
library(BSgenome.Hsapiens.UCSC.hg19)
seqNames = names(Hsapiens)[1:24]
avaSet = alignShortReads(avaSet, bsGenome=Hsapiens, 
 seqNames=seqNames, ensemblNotation=TRUE)
@
The function \Rfunction{annotateVariants} annotates genomic variants (mutations) given in a data 
frame or more likely an \Rclass{AVASet}. Annotation includes affected genes, exons and codons. 
Resulting amino acid changes are returned as well as dbSNP identifiers if the mutation is already 
known. All information is fetched from Ensembl via \Rpackage{biomaRt} and returned in an object of 
class \Rclass{AnnotatedVariants}. It is advisible to filter the \Rclass{AVASet} (see section 
\ref{sec:Filters}) prior to that since the annotation process is very time consuming for a large 
number (>500) of variants.
<<label=annotateVariants, eval=FALSE>>=
avaSet = setVariantFilter(avaSet, filter=0.05)
avaAnnot = annotateVariants(avaSet)
@
For an \Rclass{AVASet} with corresponding annotated variants, the function \Rfunction{htmlReport} 
creates a html report containing variant and quality information. \\
The report is structured into three pages:
\begin{enumerate}
    \item Variant report by reference: This page sums up additional information for each variant 
		including name, type, reference gene, position, changed nucleotides and affected samples. In 
		addition, every variant is linked to a page with further details about the affected genes and 
		transcripts (e.g. Ensembl gene-IDs, transcript-IDs, codon sequences, changes of amino acids 
		(if coding)).
    \item Variant report by sample: The upper fraction of this page presents an overview of all 
		samples together with links to individual amplicon coverage plots for each sample. In the 
		lower fraction the found variants are listed for each sample seperately in the same way as 
		described in the variant report by reference above.
    \item Quality report: The report shows the coverage of every amplicon in forward and/or reverse 
		direction. Further plots display the coverage by MID and PTP (if this information is given 
		in the pheno data of the object).
\end{enumerate}
The following command creates a report containing only variants covered by at least 5\% of the reads 
using the argument \Rfunarg{minMut} (\Rfunarg{minMut}=3 is the default value). The argument 
\Rfunarg{blocks} can be used to structure the page by assigning each variant to a block. In this 
example the corresponding genes for each variant are used to create blocks, resulting in only one 
block in the example data set:
<<label=htmlReport, eval=FALSE>>=
blocks = as.character(sapply(annotatedVariants(avaAnnot), 
 function(x) x$genes$external_gene_id))
htmlReport(avaSet, annot=avaAnnot, blocks=blocks, dir="htmlReportExampleAVA", 
 title="htmlReport Example", minMut=3)
@



\subsection{Plotting}

\subsubsection{Plot amplicon coverage}
The function \Rfunction{plotAmpliconCoverage} creates a plot showing the coverage (number of reads) 
per amplicon, MID or PTP. This results in a barplot if the \Rclass{AVASet} contains only one sample 
or in a boxplot for all other cases.
<<label=plotAmpCov1,fig=TRUE,include=FALSE,eps=FALSE,pdf=TRUE>>=
plotAmpliconCoverage(avaSet[, 2], type="amplicon")
@
\begin{figure}[hp]
\begin{center}
  \includegraphics[height=8cm]{R453Plus1-plotAmpCov1}
  \caption{Barplot of the amplicon coverage for sample 2.}
\end{center}
\end{figure}
<<label=plotAmpCov2,fig=TRUE,include=FALSE,eps=FALSE,pdf=TRUE>>=
plotAmpliconCoverage(avaSet, bothDirections=TRUE, type="amplicon")
@
\begin{figure}[hp]
\begin{center}
  \includegraphics[height=8cm]{R453Plus1-plotAmpCov2}
  \caption{Boxplot of the coverage for four amplicons seperated by read direction.}
\end{center}
\end{figure}


\subsubsection{Plot variation frequency}
Given a Roche Amplicon Variant Analyzer Global Alignment export file, the function 
\Rfunction{plotVariationFrequency} creates a plot similar to the variation frequency plot in Roche's 
GS Amplicon Variant Analyzer. The plot shows the reference sequence along the x-axis and indicates 
variants as bars at the appropriate positions. The height of the bars corresponds to the percentage 
of reads carrying the variant. A second y-axis indicates the absolute number of reads covering the 
variant. \Rfunarg{plotRange} defines the start and end base of the reference sequence that should be 
plotted.
<<label=loadXLS>>=
file = system.file("extdata", "AVAVarFreqExport", "AVAVarFreqExport.xls", 
 package="R453Plus1Toolbox")
@
<<label=plotVarFreq,fig=TRUE,include=FALSE,eps=FALSE,pdf=TRUE>>=
plotVariationFrequency(file, plotRange=c(50, 150))
@

\begin{figure}[hp]
\begin{center}
  \includegraphics{R453Plus1-plotVarFreq}
  \caption{Plot of the variation frequency for a given reference sequence.}
\end{center}
\end{figure}


\subsubsection{Plot variant locations}
The function \Rfunction{plotVariants} illustrates the positions and types of mutations within a given \Rfunarg{gene} and \Rfunarg{transcript} (specified by an 
Ensembl gene/transcript id). The plot shows only coding regions (thus, units are amino acids / codons). The coding 
region is further divided into exons labeled with their rank in the transcript. An attribute \Rfunarg{regions} allows to highlight special, predefined areas on the transcript like for example protein domains. \\ \\
The function can be used in two ways: \\
It offers the most functionality when used as a "standalone" function by passing all mutations as a data frame. This mode allows an individual and detailed
annotation of the mutations like labels, colors and user defined mutation types. It requires the columns "label", "pos" "mutation" and "color". It is recommended to add more detailed info 
for each mutation type by preparing a data frame for the parameter \Rfunarg{mutationInfo} which requires the three columns "mutation", "legend" and "color". \\ 
The following example calls \Rfunction{plotVariants} for the gene TET2 having th Ensembl id "ENSG00000168769" and transcript "ENST00000513237" (see Figure 4 below):
<<label=loadXLS>>=
data(plotVariantsExample)
@
<<label=plotVariants,fig=TRUE,include=FALSE,eps=FALSE,pdf=TRUE,eval=FALSE>>=
geneInfo = plotVariants(data=variants, gene="ENSG00000168769", 
 transcript="ENST00000513237", regions=regions, 
 mutationInfo=mutationInfo, horiz=TRUE, cex=0.8)
@
\begin{figure}[hp]
\begin{center}
  \includegraphics{plotVariants.pdf}
  \caption{Plot of the variants for gene TET2 by passing mutations as a data frame. This version includes mutation labels and allows user defined mutation types.}
\end{center}
\end{figure}
Especially for integration into the R453Plus1Toolbox and for compatibility to older versions \Rfunction{plotVariants} also accepts annotated variants of class 
\Rclass{annotatedVariants} (see section \ref{sec:Annotations}). The function then only distinguishes missense, nonsense and silent point mutations and deletions and does not include mutation labels.\\


\subsection{VCF export}
The variant call format (VCF) is a generic file format for storing DNA polymorphism data such as SNPs, insertions, deletions and structural variants, together with rich annotations (\cite{Danecek2011}). 
The following command exports all variants stored in an \Rclass{AVASet} object into a vcf file with the given name. Make sure to run \Rfunction{alignShortReads} first and optional add dbSNP identifiers with \Rfunction{annotateVariants} (see section \ref{sec:Annotations} for details)
<<label=vcfexport, eval=FALSE>>=
ava2vcf(avaSet, filename="variants.vcf", annot=avaAnnot)
@




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%___Analysis of GS Mapper projects___%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Analysis of GS Mapper projects}
Mapping projects allow the alignment of arbitrary reads from one or more sequencing runs to a given 
reference sequence.


\subsection{Importing a GS Reference Mapper project}
The function \Rfunction{MapperSet} imports data from Roche's GS Reference Mapper. The GS Mapper 
software stores information for each sample in a seperate directory, so \Rfunction{MapperSet} 
expects a character vector \Rfunarg{dirs} containing the directories of all samples to read in, i.e. 
directories containing the files: \\
\begin{itemize}
  \item "mapping/454HCDiffs.txt"
  \item "mapping/454NewblerMetrics.txt"
\end{itemize}
Furthermore the parameter \Rfunarg{samplenames} allows the seperate specification of sample names. 
if missing, the directory names are taken. The following example imports a project containing 3 
samples (N01, N03, N04) with a total of 111 variants:
<<label=gsmDir1>>=
dir_sample01 = system.file("extdata", "MapperSet", "N01", package = "R453Plus1Toolbox")
@ 
<<label=gsmDir2>>=
dir_sample03 = system.file("extdata", "MapperSet", "N03", package = "R453Plus1Toolbox")
@ 
<<label=gsmDir3>>=
dir_sample04 = system.file("extdata", "MapperSet", "N04", package = "R453Plus1Toolbox")
<<label=gsmDir4>>=
dirs = c(dir_sample01, dir_sample03, dir_sample04)
@
<<label=createRocheGSMSet>>= 
mapperSet = MapperSet(dirs=dirs, samplenames=c("N01", "N03", "N04"))
@



\subsection{The MapperSet class}
An object of class \Rclass{MapperSet} ia a container to store data imported from a project of 
Roche's GS Reference Mapper Software. It directly extends the \Rpackage{Biobase} \Rclass{eSet} class 
and as such provides the following slots:
\begin{enumerate}
	\item The \Rcode{assayData} slot contains four matrices with variants as rows and samples as 
		columns:
		\begin{itemize}
			\item variantForwCount/variantRevCount: Matrices containing the number of 
				reads with the respective variant in forward/reverse direction.
			\item totalForwCount/totalRevCount: Matrices containing the total read 
				coverage for every variant location in forward/reverse direction.
		\end{itemize}    
	\item The \Rcode{featureData} slot holds the variants as rows together with additional 
		information on each variant within the following columns:
		\begin{itemize}
			\item chromosome/start/end/strand: Give the location of each variant.
			\item referenceBases/variantBase: Show the base(s) changed in each variant.
			\item regName: The name of the region (gene) where the variant is located.
			\item knownSNP: Contains dbSNP reference cluster ids for known SNPs as given by the GS 
				Mapper software (if any).
		\end{itemize}
	\item The \Rcode{phenoData} slot contains additional information about the samples represented 
		as rows:
		\begin{itemize}
			\item By default, the phenoData slot only contains an accession number indicating the 
				PTP of every sample.
		\end{itemize}
\end{enumerate}
<<label=showMapperSet>>=
mapperSet
@

As the \Rclass{MapperSet} is derived from the \Rpackage{Biobase} \Rclass{eSet} the methods used to 
access or to manipulate the elements of a \Rclass{MapperSet} object remain the same:
<<label=assayDataMapper1>>=
assayData(mapperSet)$variantForwCount[1:4, ]
@ 
<<label=assayDataMapper2>>=
assayData(mapperSet)$totalForwCount[1:4, ]
@
<<label=fDataMapper>>=
fData(mapperSet)[1:4, ]
@ 
<<label=pDataMapper>>=
pData(mapperSet)
@ 



\subsection{Setting filters and subsetting a MapperSet}
\label{sec:FiltersMapper}

The \Rclass{MapperSet} uses the same methods for filtering and subsetting as the \Rclass{AVASet} 
(see section \ref{sec:Subsetting} and \ref{sec:Filters} for details).



\subsection{Annotations and Variant Reports}
\label{sec:AnnotationsMapper}

Before creating the variant and quality report, the variants have to be annotatetd using function 
\Rfunction{annotateVariants}. Annotation includes affected genes, exons and codons. Resulting amino 
acid changes are returned as well as dbSNP identifiers, if the mutation is already known. All 
information is fetched from Ensembl via \Rpackage{biomaRt} and returned in an object of class 
\Rclass{AnnotatedVariants}. It is advisible to filter the \Rclass{MapperSet} (see section 
\ref{sec:FiltersMapper}) since the annotation process is very time consuming for a large number 
(>500) of variants.
<<label=annotateVarMapper, eval=FALSE>>=
mapperAnnot = annotateVariants(mapperSet)
@

For a \Rclass{MapperSet} with corresponding annotated variants, the function \Rfunction{htmlReport} 
creates a html report containing detailed variant information. \\
The report is structured into two pages:
\begin{enumerate}
    \item Variant report by reference: This page sums up additional information for each variant 
		including name, type, reference gene, position, changed nucleotides and affected samples. 
		Furthermore every variant is linked to a page with further details about the affected genes 
		and transcripts (e.g. Ensembl gene-IDs, transcript-IDs, codon sequences, changes of amino 
		acids (if coding)). 
    \item Variant report by sample: The upper fraction of this page presents an overview of all 
		samples. In the lower fraction the found variants are listed for each sample seperately in 
		the same way as described in the variant report by reference above.
\end{enumerate}

The following command creates a report containing only variants covered by at least 3\% of the reads 
using the argument \Rfunarg{minMut} (\Rfunarg{minMut}=3 is also the default value):
<<label=htmlReportMapper, eval=FALSE>>=
htmlReport(mapperSet, annot=mapperAnnot, dir="htmlReportExampleMapper", 
 title="htmlReport Example", minMut=3)
@



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%___Detection of structural variants___%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Detection of structural variants}
Structural variants like translocations or inversions can be detected using non-paired reads if at 
least one read spans the breakpoint of the variant. These reads originate from two different 
locations on the reference genome and are called 'chimeric reads'. 


\subsection{Data preparation}
Before breakpoints can be detected, the generated raw sequences must be preprocessed and aligned. Of 
course, data preprocessing depends on the applied laboratory protocols. The exemplary data set used 
in this vignette is a subset of the data set presented by Kohlmann et al. (\cite{Kohlmann2009}) and 
is described in detail therein.\\

In our example data set, each region of the pico titer plate contains reads from three different 
samples which were loaded into that region. To reallocate reads to samples, each sample has a unique 
multiplex sequence prefixing all reads from that sample. This allocation process is called 
\emph{demultiplexing}. In the code section below, the multiplexed sequences are read in and 
demultiplexed according to the given multiplex sequences (MIDs) using the 
\Rfunction{demultiplexReads} method. The standard multiplex sequences used by the Genome Sequence 
MID library kits can be retrieved by calling \Rfunction{genomeSequencerMIDs}. The last two commands 
show that all reads could be successfully demultiplexed.\\

<<label=demultiplex, eval=TRUE>>=
fnaFile = system.file("extdata", "SVDetection", 
 "R_2009_07_30", 
 "D_2009_07_31", 
 "1.TCA.454Reads.fna", package="R453Plus1Toolbox")
seqs = readDNAStringSet(fnaFile, format="fasta")
MIDSeqs = genomeSequencerMIDs(c("MID1", "MID2", "MID3"))
dmReads = demultiplexReads(seqs, MIDSeqs, numMismatches=2, trim=TRUE)
length(seqs)
sum(sapply(dmReads, length))
@

A sequence capture array was used to ensure that the example data set predominantly contains reads 
from certain genomic regions of interest. The applied NimbleGen array captured short segments 
corresponding to all exon regions of 92 distinct target genes. In addition, contiguous genomic 
regions for three additional genes, i.e. \textit{CBFB}, \textit{MLL}, and \textit{RUNX1}, were 
present on the array. During sample preparation, linkers were ligated to the polished fragments in 
the library to provide a priming site as recommended by the NimbleGen protocol. These linker 
sequences were sequenced and are located at the 5 prime end of the reads. In case of long reads, the 
reverse complement of the linker may be located at the 3 prime end. The function 
\Rfunction{removeLinker} can be used to remove these linkers. Aditionally, very short reads are 
discarded in the following code snippet.
<<label=removeLinker, eval=TRUE>>=
minReadLength = 15
gSel3 = sequenceCaptureLinkers("gSel3")[[1]]
trimReads = lapply(dmReads, function (reads) {
reads = reads[width(reads) >= minReadLength]
reads = removeLinker(reads, gSel3)
reads = reads[width(reads) >= minReadLength]
readsRev = reverseComplement(reads)
readsRev = removeLinker(readsRev, gSel3)
reads = reverseComplement(readsRev)
reads = reads[width(reads) >= minReadLength]
return(reads)
})
@

Finally, the preprocessed reads must be aligned against a reference genome. For this purpose, we 
write the reads to a .fasta file and use the BWA-SW (\cite{Li2010}) algorithm for generating local alignments. 
The BWA-SW algorithm can be substituted by other local alignment algorithms. However, BWA-SW has the 
useful feature to only report the best local alignments. Hence, two local alignments do not overlap 
on the query sequence (they may overlap on the reference). This is an assumption made by the 
pipeline implemented in this package.
<<label=writeFASTA, eval=FALSE>>=
write.XStringSet(trimReads[["MID1"]], file="/tmp/N01.fasta", format="fasta")
@


\subsection{Computing and assessing putative structural variants}
As chimeric reads may also be caused by technical issues during sample preparation, the function
\Rfunction{filterChimericReads} implements several filter steps to remove artificial chimeric reads.
\\

The remaining reads are passed to the \Rfunction{detectBreakpoints} method to create clusters 
representing putative breakpoints. Each cluster contains all chimeric reads that span this 
breakpoint. Promising candidates are clusters with more than one read and ideally with reads from 
different strands. Some structural variations like translocations or inversion lead to two related 
breakpoints. In the context of fusion genes, these breakpoints are refered to as \emph{pathogenic} 
and \emph{reciprocal} breakpoint. By the use of read orientation and strand information during 
clustering, it is ensured that reads from the pathogenic breakpoint will not cluster together with 
reads from the reciprocal breakpoint, although their genomic coordinates may be close to each other 
or even equal. After clsutering, consensus breakpoint coordinates are computed for each cluster.\\

In the last step, the function \Rfunction{mergeBreakpoints} searches breakpoints that originate from 
the same structural variation (i.e. the pathogenic and the related reciprocal breakpoint) and merges
them. We observed, that the distance between two related breakpoints may be up to a few hundred 
basepairs, whereas the breakpoint coordinates of single reads spanning the same breakpoint vary
only by a very few bases due to sequencing errors or ambiguities during alignment.\\

In the following example, we use the reads from sample N01 presented in the previous section. The
reads have been aligned using BWA-SW:
<<label=readBam, eval=TRUE>>=
library("Rsamtools")
bamFile = system.file("extdata", "SVDetection", "bam", "N01.bam", 
 package="R453Plus1Toolbox")
parameters = ScanBamParam(what=scanBamWhat())
bam = scanBam(bamFile, param=parameters)
@ 

For the filtering step, we specify a target region, i.e. the used capture array in form of a
\Rclass{IntegerRangesList}. All chimeric reads not overlapping this region with at least one local
alignment are discarded. The following example creates a target region out of a given .bed file
containing region information using functions from package \Rpackage{rtracklayer}. 
<<label=filterReads, eval=TRUE>>=
library("rtracklayer")
bedFile = system.file("extdata", "SVDetection", "chip", 
 "CaptureArray_hg19.bed", package="R453Plus1Toolbox")
chip = import.ucsc(bedFile, subformat="bed")
chip = split(ranges(chip[[1]]), seqnames(chip[[1]]))
names(chip) = gsub("chr", "", names(chip))

linker = sequenceCaptureLinkers("gSel3")[[1]]

filterReads = filterChimericReads(bam, targetRegion=chip, linkerSeq=linker)
filterReads$log
@
The \Rfunarg{linkerSeq} argument allows to specify the linker sequence used during sample 
preparation. All chimeric reads that have this linker sequence between their local alignments are 
removed.\\

Finally, we call the \Rfunction{detectBreakpoints} and \Rfunction{mergeBreakpoints} functions:
<<label=detectBreakpoints, eval=TRUE>>=
bp = detectBreakpoints(filterReads, minClusterSize=1)
bp
breakpointTable(bp)

mbp = mergeBreakpoints(bp)
summary(mbp)
@
One cluster of size 8 and another cluster of size 4 were detected. Both putative breakpoints span 
two regions on chromosome 16. Further, 11 clusters of size one were found. The 
\Rfunction{mergeBreakpoints} function merges the first two clusters. The summary reveals that the 
coordinates of the breakpoints only differ by two bases at each region on chromosome 16. Moreover, 
both strands from both breakpoints were sequenced. Obviously, we detected two related breakpoints 
caused by an inversion on chromsome 16.


\subsection{Visualization of breakpoints}
The function \Rfunction{plotChimericReads} takes the output of the function 
\Rfunction{mergeBreakpoints} and produces a plot of the breakpoint regions together with the 
aligned reads and marks deletions, insertions and mismatches. If a pathogenic and a reciprocal 
breakpoint exist, \Rfunction{plotChimericReads} creates two plots as shown in the example
below.\\

The following example shows the breakpoints (pathogenic and reciprocal) of an inversion on chromosome 16 where 12 reads 
aligned:\\
<<eval=TRUE, label=plotCR1,fig=TRUE,include=FALSE,eps=FALSE,pdf=TRUE,width=10,height=5>>=
plotChimericReads(mbp[1], legend=TRUE)
@
\begin{figure}[hbp]
\begin{center}
  \includegraphics[height=7cm, width=12cm]{R453Plus1-plotCR1}
  \caption{Plot of the breakpoint region.}
\end{center}
\end{figure}


Optionally (if the argument \Rfunarg{plotBasePairs} is \Rcode{TRUE}), \Rfunction{plotChimericReads} 
displays all base pairs within a given region of size \Rfunarg{maxBasePairs} around the breakpoint:

<<eval=TRUE, label=plotCR2,fig=TRUE,include=FALSE,eps=FALSE,pdf=TRUE>>=
plotChimericReads(mbp[1], plotBasePairs=TRUE, maxBasePairs=30)
@
\begin{figure}[hbp]
\begin{center}
  \includegraphics[height=8cm]{R453Plus1-plotCR2}
  \caption{Plot of the breakpoint region including base pairs.}
\end{center}
\end{figure}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%___Analysis and manipulation of SFF files___%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Analysis and manipulation of SFF files}

The Standard Flowgram Format(SFF) is a binary file format designed by Roche to store the homopolymer stretches typical for 454 sequencing. It consists of a common header section, containing general information (e.g. number of reads, nucleotides used for each flow) and for each read a read header (e.g. read length, read name) and read data section (e.g. called bases, flow values, quality scores). 

\subsection{Importing SFF files}
SFF files be imported using the \Rfunction{readSFF} function.
<<label=importSFFfiles, eval=TRUE>>=
file <- system.file("extdata", "SFF", "example.sff", package="R453Plus1Toolbox")
sffContainer <- readSFF(file)
@

\subsection{The SFF container}
The contents of the SFF file are stored in an object of class \Rclass{SFFContainer} with different slots:
<<label=SFFcontainer, eval=TRUE>>=
showClass("SFFContainer")
@

The most import slot is the reads slot containing the called bases and the corresponding quality measures:
<<label=sffreads, eval=TRUE>>=
reads(sffContainer)
@

An \Rclass{SFFContainer} object can be subsetted using the \Rcode{[} operator and some read names or numbers:
<<label=sffsubsetting, eval=TRUE>>=
subSffContainer <- sffContainer[1:5]
@

\subsection{Writing SFF files}
An \Rclass{SFFContainer} can be written back into a file using the \Rfunction{writeSFF} method:
<<label=sffsubsetting, eval=FALSE>>=
writeSFF(subSffContainer, subSffFile.sff)
@

\subsection{Quality control of SFF files}
The \Rfunction{qualityReportSFF} function creates a PDF document from one or more SFF files containing information relevant for quality control (e.g. read length distributions, quality histograms, GC content). Two example plots are shown below:
<<label=positionQualityBoxplot, eval=TRUE, fig=TRUE, include=FALSE, results=hide, eps=FALSE, pdf=TRUE>>=
positionQualityBoxplot(sffContainer)
@
\begin{figure}[hbp]
\begin{center}
  \includegraphics[height=8cm]{R453Plus1-positionQualityBoxplot}
  \caption{Position quality boxplot - One of the plots contained in the PDF quality report.}
\end{center}
\end{figure}
@

<<label=dinucleotideOddsRatio, eval=TRUE, results=hide, fig=TRUE, include=FALSE, eps=FALSE, pdf=TRUE>>=
dinucleotideOddsRatio(sffContainer)
@
\begin{figure}[hbp]
\begin{center}
  \includegraphics[height=8cm]{R453Plus1-dinucleotideOddsRatio}
  \caption{Dinucleotide odds ratio showing the over-/under-representation of dinucleotides - One of the plots contained in the PDF quality report.}
\end{center}
\end{figure}
@


\begin{thebibliography}{}
\bibitem[Kohlmann {\it et~al}., 2009]{Kohlmann2009}
Kohlmann,A. \textit{et~al}. (2009) Targeted next-generation sequencing (NGS) enables for the first time the detection of point mutations, molecular insertions and deletions, as well as leukemia-specific fusion genes in AML in a single procedure.
\textit{Blood (ASH Annual Meeting Abstracts)}, \textbf{114}(22), 294--295.

\bibitem[Li and Durbin, 2010]{Li2010}
Li,H. and Durbin,R. (2010) Fast and accurate long-read alignment with Burrows-Wheeler transform.
\textit{Bioinformatics}, \textbf{26}(5), 589--95. 

\bibitem[Danecek {\it et~al}., 2011]{Danecek2011}
Danecek,P. \textit{et~al}. (2011) The variant call format and VCFtools.
\textit{Bioinformatics}, \textbf{27}(15), 2156--2158. 
\end{thebibliography}

\end{document}

\documentclass[10pt]{article}
\usepackage[section]{placeins}

\begin{document}
\SweaveOpts{concordance=TRUE}
\title {Quality report}

\maketitle

\section{Processed files}

<<label=preliminaries, echo=FALSE, results=hide>>=
library(xtable)
library(R453Plus1Toolbox)
if(exists("sfffiles", envir=tmp)) { filenames <- tmp$sfffiles }
sfffiles <- readSFF(filenames)
if(length(sfffiles) == 1) {
  sfffiles <- list(sfffiles)
}
cols <- ceiling((length(sfffiles) + 2) / 3)
rows <- ceiling(length(sfffiles) / cols)
@

<<label=processedFiles, echo=FALSE, results=tex>>=
print(xtable(data.frame(filenames)))
@

\section{Read length statistics}

<<label=readLengthStats, echo=FALSE, results=tex>>=
df <- data.frame(row.names=c("mean", "median", "min", "max", "sd"))
for(sff in sfffiles) {
  df[, name(sff)] <- readLengthStats(sff)
}
for(i in 1:ceiling(length(sfffiles) / 4)) {
  print(xtable(df[, (i*1 + (i-1)*4):(min(length(sfffiles), i*4))]))
}
@

<<label=readLengthHist, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-readLengthHist.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  readLengthHist(sff, cutoff=0.99)
}
dev.off()
@

\begin{figure}[htb]
\begin{center}
  \includegraphics[width=13cm]{qcreport-readLengthHist}
  \caption{Histogram of the lengths of the sequences.}
\end{center}
\end{figure}

\clearpage


\section{Base quality statistics}

<<label=baseQualityStats, echo=FALSE, results=tex>>=
df <- data.frame(row.names=c("mean", "min", "max", "sd"))
for(sff in sfffiles) {
  df[, name(sff)] <- baseQualityStats(sff)
}
for(i in 1:ceiling(length(sfffiles) / 4)) {
  print(xtable(df[, (i*1 + (i-1)*4):(min(length(sfffiles), i*4))]))
}
@

<<label=baseQualityHist, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-baseQualityHist.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  baseQualityHist(sff)
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-baseQualityHist}
  \caption{Histogram of base quality scores.}
\end{center}
\end{figure}


<<label=sequenceQualityHist, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-sequenceQualityHist.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  sequenceQualityHist(sff)
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-sequenceQualityHist}
  \caption{Histogram of mean quality scores per sequence.}
\end{center}
\end{figure}


<<label=positionQualityBoxplot, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-positionQualityBoxplot.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  positionQualityBoxplot(sff, range=200, binsize=10)
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-positionQualityBoxplot}
  \caption{Boxplot of base quality per position.}
\end{center}
\end{figure}

\clearpage


\section{Base distribution statistics}

\subsection{Absolute nucleotide frequencies:}
<<label=absoluteBaseFrequency, echo=FALSE, results=tex>>=
dfabs <- data.frame(row.names=c("A", "C", "G", "T", "other", "total"))
dfrel <- data.frame(row.names=c("A", "C", "G", "T", "other", "total"))
for(sff in sfffiles) {
  bf <- baseFrequency(sff)
  dfabs[, name(sff)] <- bf[,"absolute"]
  dfrel[, name(sff)] <- bf[,"relative"]
}
for(i in 1:ceiling(length(sfffiles) / 4)) {
  print(xtable(dfabs[, (i*1 + (i-1)*4):(min(length(sfffiles), i*4))]))
}
@

\subsection{Relative nucleotide frequencies:}
<<label=relativeBaseFrequency, echo=FALSE, results=tex>>=
for(i in 1:ceiling(length(sfffiles) / 4)) {
  print(xtable(dfrel[, (i*1 + (i-1)*4):(min(length(sfffiles), i*4))]))
}
@

<<label=nucleotideCharts, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-nucleotideCharts.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  nucleotideCharts(sff, range=c(1,20))
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-nucleotideCharts}
  \caption{Nucleotide frequency per position.}
\end{center}
\end{figure}

\section{GC-content}

\large{Relative frequency of GC-content:}
<<label=gcContent, echo=FALSE, results=tex>>=
df <- data.frame(row.names=c("GC-content"))
for(sff in sfffiles) {
  df[, name(sff)] <- gcContent(sff)
}
for(i in 1:ceiling(length(sfffiles) / 4)) {
  print(xtable(df[, (i*1 + (i-1)*4):(min(length(sfffiles), i*4))]))
}
@


<<label=gcPerPosition, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-gcPerPosition.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  gcPerPosition(sff, range=0.95)
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-gcPerPosition}
  \caption{GC content per position.}
\end{center}
\end{figure}


<<label=gcContentHist, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-gcContentHist.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  gcContentHist(sff, col="blue")
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-gcContentHist}
  \caption{GC content histogram.}
\end{center}
\end{figure}

\clearpage


\section{Sequence complexity}

Sequence complexity measures help to detect regions of low complexity (e.g. dinucleotide repeats ATATATATATATATAT). Both approaches use trinucleotide frequencies in windows of given size and are scaled between 0 and 100. For the DUST approach scores above 7 (vertical line) can be considered low-complexity (TTTTTTTTT has score 100, TATATATA has score around 49, TAGTAGTAGTAG has a score around 32). For the entropy approach lower values imply lower complexity, a value below 70 (vertical bar) can be considered low-complexity (TTTTTTTTT has value 0, TATATATA has a value around 16 and TAGTAGTAGTAG has a score around 26). \\

<<label=complexityDust, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-complexityDust.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  complexity.dust(sff)
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-complexityDust}
  \caption{Histogram of DUST complexity scores.}
\end{center}
\end{figure}


<<label=complexityEntropy, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-complexityEntropy.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  complexity.entropy(sff)
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-complexityEntropy}
  \caption{Histogram of Entropy complexity scores.}
\end{center}
\end{figure}

\clearpage


\section{Dinucleotide odds ratio}

The dinucleotide odds ratio shows the over-/under-representation of dinucleotides given the the numbers of the corresponding nucleotides.

<<label=dinucleotideOddsRatio, fig=FALSE, echo=FALSE, results=hide, include=FALSE>>=
pdf("qcreport-dinucleotideOddsRatio.pdf", width=cols*7, height=rows*7)
par(mfrow=c(rows, cols))
for(sff in sfffiles) {
  dinucleotideOddsRatio(sff)
}
dev.off()
@

\begin{figure}[ht]
\begin{center}
  \includegraphics[width=13cm]{qcreport-dinucleotideOddsRatio}
  \caption{Dinucleotide odds ratio}
\end{center}
\end{figure}

\end{document}

\name{ava2vcf}

\docType{methods}

\alias{ava2vcf}
\alias{ava2vcf,AVASet-method}

\title{Convert an AVASet object into a VCF object}

\description{
  Converts all variants in a given AVASet object into a VCF object and writes it to a file in VCF format if filename is given.
}

\usage{
  ava2vcf(object, filename, annot)
}

\arguments{
  \item{object}{An object of class \link{AVASet}.}
  \item{filename}{The name of the VCF file to write in, if ommitted no file is written.}
  \item{annot}{An object of class \code{AnnotatedVariants}. Optional, if given variants are annotated with informations from dbSNP.}
}

\value{An object of class \code{\link{VCF-class}}}

\examples{
  data("avaSetFiltered")
  vcf <- ava2vcf(avaSetFiltered)
}

\seealso{\code{\link{AnnotatedVariants-class}}, \code{\link{AVASet-class}}, \code{\link{VCF-class}}, \code{\link{writeVcf}}}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{readLengthHist}

\docType{methods}

\alias{readLengthHist}
\alias{readLengthHist,DNAStringSet-method}
\alias{readLengthHist,ShortRead-method}
\alias{readLengthHist,SFFContainer-method}

\title{Histogram Of The Read Lengths}

\description{
  This function plots a histogram of the read lengths.
}

\usage{
  readLengthHist(object, cutoff=0.99, xlab="Read length", ylab="Number of sequences", 
    col="firebrick1", breaks=100, ...)
}

\arguments{
  \item{object}{An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.}
  \item{cutoff}{Reads longer than the cutoff-percent quantile are omitted from the plot.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{col}{The plotting color.}
  \item{breaks}{The number of breaks in the histogram (see \sQuote{hist}).}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{SFFRead-class}
\Rdversion{1.1}
\docType{class}
\alias{SFFRead-class}
\alias{SFFRead}
\alias{read<-,SFFRead,QualityScaledDNAStringSet-method}
\alias{read<-}
\alias{read,SFFRead-method}
\alias{read}
\alias{flowChars<-,SFFRead,character-method}
\alias{flowChars,SFFRead-method}
\alias{flowgramFormat<-,SFFRead,numeric-method}
\alias{flowgramFormat,SFFRead-method}
\alias{keySequence<-,SFFRead,character-method}
\alias{keySequence,SFFRead-method}
\alias{clipAdapterLeft<-,SFFRead,numeric-method}
\alias{clipAdapterLeft,SFFRead-method}
\alias{clipAdapterRight<-,SFFRead,numeric-method}
\alias{clipAdapterRight,SFFRead-method}
\alias{clipQualityLeft<-,SFFRead,numeric-method}
\alias{clipQualityLeft,SFFRead-method}
\alias{clipQualityRight<-,SFFRead,numeric-method}
\alias{clipQualityRight,SFFRead-method}
\alias{flowgram<-,SFFRead,numeric-method}
\alias{flowgram<-}
\alias{flowgram,SFFRead-method}
\alias{flowgram}
\alias{flowIndexes<-,SFFRead,numeric-method}
\alias{flowIndexes,SFFRead-method}
\alias{name<-,SFFRead,character-method}
\alias{name,SFFRead-method}
\alias{quality<-}
\alias{quality<-,SFFRead,BString-method}
\alias{quality,SFFRead-method}

\title{Class \code{"SFFRead"}}
\description{
This class is a container for a single read from files in Roche's Standard Flowgram Format (SFF).
}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("SFFRead", ...)}.
Usually, objects will be created by calling the \code{\link{getRead}} method on an object of class
\code{\linkS4class{SFFContainer}}.
}

\section{Slots}{
  \describe{
    \item{\code{name}:}{Object of class \code{"character"} representing the name of the read.}
    \item{\code{read}:}{Object of class \code{"DNAString"} containing the basecalled 
      nucleotide sequence of the read.}
    \item{\code{flowgramFormat}:}{Object of class \code{"numeric"} representing the format used to 
      encode each of the flowgram values for each read. Currently, only one flowgram format has been 
      adopted and is coded by the value 1.}
    \item{\code{flowChars}:}{Object of class \code{"character"} containing the array of nucleotide 
      bases ('A', 'C', 'G', 'T') that correspond to the nucleotides used for each flow of each 
      read.}
    \item{\code{keySequence}:}{Object of class \code{"character"} representing the nucleotide bases
      of the key sequence used for these reads.}
    \item{\code{clipQualityLeft}:}{Object of class \code{"numeric"} representing the position of the 
      first base after the clipping point for an attached quality sequence. If only a combined 
      (quality+adapter) clipping position is computed it should be stored in clipQualityLeft. If no 
      clipping value is computed the field is set to 0. The position values use 1-based indexing.}
    \item{\code{clipQualityRight}:}{Object of class \code{"numeric"} representing the position of 
      the last base before the clipping point for an attached quality sequence. If only a combined 
      (quality+adapter) clipping position is computed it should be stored in clipQualityRight. If no 
      clipping value is computed the field is set to 0. The position values use 1-based indexing.}
    \item{\code{clipAdapterLeft}:}{Object of class \code{"numeric"} representing the position of the 
      first base after the clipping point for an attached adapter sequence. If only a combined 
      (quality+adapter) clipping position is computed it should be stored in clipQualityLeft. If no 
      clipping value is computed the field is set to 0. The position values use 1-based indexing.}
    \item{\code{clipAdapterRight}:}{Object of class \code{"numeric"} representing the position of 
      the last base before the clipping point for an attached adapter sequence. If only a combined 
      (quality+adapter) clipping position is computed it should be stored in clipQualityRight. If no 
      clipping value is computed the field is set to 0. The position values use 1-based indexing.}
    \item{\code{flowgram}:}{Object of class \code{"numeric"} containing the homopolymer stretch 
      estimates for each flow.}
    \item{\code{flowIndexes}:}{Object of class \code{"numeric"} containing the flow positions for 
      each base in the called sequence, i.e. for each base, the position in the flowgram whose 
      estimate resulted in that base being called.}
    \item{\code{quality}:}{Object of class \code{"BString"} containing
      the quality scores for each of the bases in the sequence, where
      the values use the standard -log10 probability scale.}
  }
}

\section{Methods}{
  \describe{
    \item{read<-}{\code{signature(object = "SFFRead", value = "DNAString")}: 
      Setter-method for the read slot.}
    \item{read}{\code{signature(object = "SFFRead")}: 
      Getter-method for the read slot.}
    \item{flowChars<-}{\code{signature(object = "SFFContainer", value = "character")}: 
      Setter-method for the flowChars slot.}
    \item{flowChars}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the flowChars slot.}
    \item{flowgramFormat<-}{\code{signature(object = "SFFContainer", value = "numeric")}: 
      Setter-method for the flowgramFormat slot.}
    \item{flowgramFormat}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the flowgramFormat slot.}
    \item{keySequence<-}{\code{signature(object = "SFFContainer", value = "character")}: 
      Setter-method for the keySequence slot.}
    \item{keySequence}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the keySequence slot.}
    \item{clipAdapterLeft<-}{\code{signature(object = "SFFRead", value = "numeric")}: 
      Setter-method for the clipAdapterLeft slot.}
    \item{clipAdapterLeft}{\code{signature(object = "SFFRead")}: 
      Getter-method for the clipAdapterLeft slot.}
    \item{clipAdapterRight<-}{\code{signature(object = "SFFRead", value = "numeric")}: 
      Setter-method for the clipAdapterRight slot.}
    \item{clipAdapterRight}{\code{signature(object = "SFFRead")}: 
      Getter-method for the clipAdapterRight slot.}
    \item{clipQualityLeft<-}{\code{signature(object = "SFFRead", value = "numeric")}: 
      Setter-method for the clipQualityLeft slot.}
    \item{clipQualityLeft}{\code{signature(object = "SFFRead")}: 
      Getter-method for the clipQualityLeft slot.}
    \item{clipQualityRight<-}{\code{signature(object = "SFFRead", value = "numeric")}: 
      Setter-method for the clipQualityRight slot.}
    \item{clipQualityRight}{\code{signature(object = "SFFRead")}: 
      Getter-method for the clipQualityRight slot.}
    \item{flowgram<-}{\code{signature(object = "SFFRead", value = "numeric")}: 
      Setter-method for the flowgram slot.}
    \item{flowgram}{\code{signature(object = "SFFRead")}: 
      Getter-method for the flowgram slot.}
    \item{flowIndexes<-}{\code{signature(object = "SFFRead", value = "numeric")}: 
      Setter-method for the flowIndexes slot.}
    \item{flowIndexes}{\code{signature(object = "SFFRead")}: 
      Getter-method for the flowIndexes slot.}
    \item{name<-}{\code{signature(object = "SFFRead", value = "character")}: 
      Setter-method for the name slot.}
    \item{name}{\code{signature(object = "SFFRead")}: 
      Getter-method for the name slot.}
    \item{quality<-}{\code{signature(object = "SFFRead", value = "BString")}: 
      Setter-method for the quality slot.}
    \item{quality}{\code{signature(object = "SFFRead")}: 
      Getter-method for the quality slot.}
  }
}

\author{
Christian Ruckert
}

\seealso{
\code{\link{readSFF}}, \code{\linkS4class{SFFContainer}}
}

\examples{
showClass("SFFRead")
}

\keyword{classes}

\name{calculateTiTv}

\alias{calculateTiTv}
\alias{calculateTiTv,AVASet-method}
\alias{calculateTiTv,MapperSet-method}


\title{Calculate transition transversion ratio}

\description{When many point mutations are detected, the ration of transitions
to transversions can be used as quality measure to assess the number of false
positives.}

\usage{
  \S4method{calculateTiTv}{AVASet}(object)
  \S4method{calculateTiTv}{MapperSet}(object)
}

\arguments{
  \item{object}{An instance of AVASet or MapperSet storing the detected variants.}
}

\value{A \code{list} with two elements: A substitution matrix summarizing all
observed substitutions and the transition/transversion ratio.}

\details{For more information about the Ti/Tv ratio see 
http://www.broadinstitute.org/gsa/wiki/index.php/QC_Methods}

\author{Hans-Ulrich Klein}

\examples{
data(avaSetExample)
ava = setVariantFilter(avaSetExample, c(0.03, 0.03))
calculateTiTv(ava)
}

\keyword{TiTv,Transition,Transversion}

\name{AVASet}

\alias{AVASet}
\alias{AVASet,character,missing,missing,missing,missing,missing,missing-method}
\alias{AVASet,character,character,missing,missing,missing,missing,missing-method}
\alias{AVASet,character,missing,character,character,character,character,character-method}
\alias{AVASet,character,missing,character,character,character,missing,missing-method}


\title{Creating an AVASet}

\description{
    This function imports a project of Roche's Amplicon Variant Analyzer (AVA) Software. 
    It stores all information into an extended version of the Biobase eSet.
}

\usage{AVASet(dirname, avaBin, file_sample, file_amp, file_reference, file_variant, file_variantHits)}

\arguments{
  \item{dirname}{The path of the AVA project. \cr 
	Without AVA-CLI (AVA version < 2.6): A directory that contains the files and subdirectories
	"Amplicons/ProjectDef/ampliconsProject.txt", "Amplicons/Results/Variants/currentVariantDefs.txt", "Amplicons/Results/Variants", 
	"Amplicons/Results/Align". \cr 
	Using AVA-CLI (recommended): Path usually ends with directory "projectfolder"}
  \item{avaBin}{The directory containing the AVA-CLI binary doAmplicon (usually "bin" in the AVA installation directory)}
  \item{file_sample}{Sample information exported with the AVA-CLI. File has to be in CSV format.}
  \item{file_amp}{Amplicons exported with the AVA-CLI. File has to be in CSV format.}
  \item{file_reference}{Reference sequences exported with the AVA-CLI. File has to be in CSV format.}
  \item{file_variant}{Variant information exported with the AVA-CLI. File has to be in CSV format.}
  \item{file_variantHits}{Report of variant hits exported with the AVA-CLI. File has to be in CSV format.}
}

\details{
The five arguments for AVA command line interface (AVA-CLI) exports are optional and useful for exported projects, when no AVA software is installed. 
For exporting, start the AVA-CLI with the command "doAmplicon" and use the commands "open", then "list sample", "list amplicon", "list reference", 
"list variant" and "report variantHits". See \code{\link{AVASet-class}} for more details. \cr
Giving only a project directory and the path to the AVA-CLI binary doAmplicon, \code{AVASet} will 
import all information by accessing the AVA-CLI from within R. 

An AVASet object consists of three slots to store data about \cr
1. variants \cr
\describe{
    \item{variantForwCount/variantRevCount:}{Data frames that contain the number of reads with the respective variant in forward/reverse 
	direction.}
    \item{totalForwCount/totalRevCount:}{Data frames that contain the total coverage for every variant location in forward/reverse 
	direction.}
    \item{referenceSeq:}{Gives the identifier of the reference sequence.}
    \item{variantBase/referenceBases:}{The bases changed in each variant.}
    \item{start/end:}{The position of the variant on the reference sequence.}
    \item{canonicalPattern/name:}{Short identifiers of a variant including the position and the bases changed.}
}
2. amplicons \cr
\describe{
    \item{forwCount/revCount:}{Data frames that contain the number of reads for every amplicon and each sample in 
	forward/reverse direction.}
    \item{primer1,primer2:}{The primer sequences for every amplicon.}
    \item{referenceSeqID:}{The identifier of the reference sequence.}
    \item{targetStart/targetEnd:}{The coordinates of the target region.}
}
3. reference sequences \cr
\describe{
    \item{If additional information has been loaded from Ensembl via \code{alignShortReads}, this slot knows about the chromosome, position and 
	the strand of each reference sequence.}{}
} 

The structure of the variant and amplicon data is derived from the Biobase eSet and thus separated into assayData, phenoData and 
featureData. All information about the reference sequences is stored into an object of class \code{AlignedRead}.

The phenoData of the variants lists the sample-IDs and name, annotation and group of the read data for all samples. If available, the pico titer 
plate(s) (PTP) or MID(s) of each sample are shown as well (using the AVA-CLI, PTPs and MIDs cannot be importet at the moment).
}

\value{
    An instance of the AVASet class.
}
\note{
    It is recommended to use the import via AVA-CLI access. Although deprecated, the import for projects created with older version of the AVA software (< v2.6) is still possible.  
}

\seealso{
    \code{\link{AVASet-class}}, \code{\link{MapperSet-class}}, \code{\link{alignShortReads}}
}

\examples{
# Loading a project from AVA version < 2.6:
# Load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
data(avaSetExample)
avaSetExample

# Loading exported data, that was exported via AVA-CLI
# Load an AVA dataset containing 6 samples, 4 amplicons and 222 variants
# by specifying each file exported from the AVA-CLI
projectDir = system.file("extdata", "AVASet_doAmplicon", package="R453Plus1Toolbox")
avaSetExample = AVASet(dirname=projectDir, file_sample="sample.csv", file_amp="amp.csv", file_reference="reference.csv", file_variant="variant.csv", file_variantHits="variantHits.csv")
avaSetExample

# In case AVA software is installed:
# Saying, for example, the AVA software was installed to the directory "/home/User/AVA",
# the easiest way to import a project via AVA-CLI would look like:
# avaSetExample = AVASet(dirname="myProjectDir", avaBin="/home/User/AVA/bin")


}

\author{Christoph Bartenhagen}

\name{MapperSet}

\alias{MapperSet}
\alias{MapperSet,character-method}

\title{Creating a MapperSet}

\description{
    This function imports a project of Roche's GS Reference Mapper Software.
    It stores all information into an instance of the Biobase ExpressionSet.
}

\usage{MapperSet(dirs, samplenames)}

\arguments{
  \item{dirs}{A character vector containing all sample directories (i.e. directories that contain the files "mapping/454HCDiffs.txt" (required), 
	"mapping/454ReadStatus.txt" (optional), "mapping/454NewblerMetrics.txt"(optional)).}
  \item{samplenames}{A character vector containing samplenames. The order and number of samplenames must be consistent with the filenames to ensure 
	the correctness of the MapperSet. If no samplenames are given, the filenames are used for naming.}
}

\details{An instance of the MapperSet is derived from the Biobase eSet and thus structured into \cr

1. assayData \cr
\describe{
    \item{variantForwCount/variantRevCount:}{Contain the number of reads with the respective difference in forward/reverse direction.} 
    \item{totalForwCount/totalRevCount:}{Contain the total coverage for every variant in forward/reverse direction.}  
}
2. featureData\cr
\describe{ 
    \item{chromosome, start/end:}{Give the location of each variant.}
    \item{referenceBases/variantBase:}{Show the bases changed in each variant.}
    \item{regName:}{The name of the region (gene) where the variant is located.}
    \item{knownSNP:}{Lists Ensembl variant-ids for known SNPs (if any).}
}
3. phenoData \cr
\describe{
    \item{By default, the phenoData contains the accession number of every sample.}{}
}
}

\value{
    An instance of the MapperSet.
}

\seealso{
    \code{\link{AVASet-class}}
}

\examples{
# load a GS Mapper dataset containing 3 samples and 111 variants
data(mapperSetExample)
mapperSetExample
}

\author{Christoph Bartenhagen}

\name{gcPerPosition}

\docType{methods}

\alias{gcPerPosition}
\alias{gcPerPosition,DNAStringSet-method}
\alias{gcPerPosition,ShortRead-method}
\alias{gcPerPosition,SFFContainer-method}

\title{GC-Content Per Position}

\description{
  This function plots the GC-content frequency per base position.
}

\usage{
  gcPerPosition(object, range=0.95, type="l", col="blue", xlab="Position", ylab="Frequency", ...)
}

\arguments{
  \item{object}{An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.}
  \item{range}{An integer vector of length one or two. If length one only bases up to the percent 
    quantile of read lengths defined by the given value are shown. A vector of length two gives the 
    start and end base between which the GC-content is plotted.}
  \item{type}{The type of the plot (see \sQuote{plot}).}
  \item{col}{The plotting color.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{alignShortReads}

\alias{alignShortReads}
\alias{alignShortReads,AVASet,BSgenome,character,logical-method}
\alias{alignShortReads,AVASet,BSgenome,character,missing-method}
\alias{alignShortReads,AVASet,BSgenome,missing,logical-method}
\alias{alignShortReads,AVASet,BSgenome,missing,missing-method}
\alias{alignShortReads,DNAStringSet,BSgenome,character,logical-method}
\alias{alignShortReads,DNAStringSet,BSgenome,character,missing-method}
\alias{alignShortReads,DNAStringSet,BSgenome,missing,logical-method}
\alias{alignShortReads,DNAStringSet,BSgenome,missing,missing-method}

\title{Exact alignment of DNA sequences against a reference}

\description{This method aligns given sequences against a given reference
genome using the \code{matchPDict} method. Only exact (no errors) and unique
matches are returned.}

\usage{alignShortReads(object, bsGenome, seqNames, ensemblNotation)}

\arguments{
  \item{object}{The reads that should be aligned agiven either as a
\code{DNAStringSet} or a \code{AVASet} instance. In the latter case
the reference sequences are extracted and aligned.}
  \item{bsGenome}{A \code{bsGenome} instance providing the reference
sequences.}
  \item{seqNames}{The names of the sequences in \code{bsGenome} that should
be used. If omitted, all reference sequences are used.}
  \item{ensemblNotation}{If set to TRUE, \dQuote{chr} is removed from the
reference sequences' names in the returned alignment. Default value is FALSE.}
}

\value{An object of class \code{AlignedRead} or a \code{AVASet} instance.}

\details{All reads are aligned against the reference and its reverse
complement. If the reads are not in 5' to 3' orientation, they should
be reversed before. Note that only exact and unique alignments are
reported. Use \code{matchPDict} directly for more flexibility.}


\seealso{\code{\link{matchPDict}}, \code{\link{DNAStringSet}}, 
\code{\link{AlignedRead}}, \code{\link{AVASet}}
}

\examples{
library("BSgenome.Scerevisiae.UCSC.sacCer2")
reads = DNAStringSet(c(
  "CCGTTCAAAGAGCCCTTGGCCCATAATCCACCGGTT",
  "ATCCTGCCACAGGAGTCCATGGAGGTTTCGCCA"))
alignShortReads(reads, Scerevisiae, seqNames="chrIII")
}

\author{Hans-Ulrich Klein}
\keyword{alignShortReads}

\name{baseQualityHist}

\docType{methods}

\alias{baseQualityHist}
\alias{baseQualityHist,QualityScaledDNAStringSet-method}
\alias{baseQualityHist,ShortReadQ-method}
\alias{baseQualityHist,SFFContainer-method}

\title{Plot A Histogram Of The Base Qualities.}

\description{
  Create a histogram based on the quality of every single base from all sequences.
}

\usage{
  baseQualityHist(object, xlab="Quality score", ylab="Number of bases", col="firebrick1", breaks=40,
     ...)
}

\arguments{
  \item{object}{An object of class \link{QualityScaledDNAStringSet}, \link{ShortReadQ} or 
    \link{SFFContainer}.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{col}{The plotting color.}
  \item{breaks}{The number of breaks in the histogram (see \sQuote{hist}).}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{setVariantFilter}

\alias{setVariantFilter}

\title{Filters output of variant information}

\description{
    This functions sets the filter to display only those variants, whose amplicon coverage (in percent) in forward and reverse direction in at 
    least one sample is higher than a given value. The coverage is defined as the percentual amount of reads that cover a variant.}

\usage{setVariantFilter(object, filter=0)}

\arguments{
  \item{object}{An instance of an \code{link{AVASet-class}} or \code{\link{MapperSet-class}}.}
  \item{filter}{A filter value between 0 and 1. If two values are given in a vector, the variants are filterd according to the forward (first value) 
	and reverse direction (second value) separately. In this case, a variant has to meet both requirements.}
}

\details{
    Setting the filter affects the assayData and the featureData of the variant slot. See also \code{\link{getVariantPercentages}} for further details.
}

\value{
    \code{setVariantFilter} returns the given \code{link{AVASet-class}}/\code{link{MapperSet-class}} instance with an updated filter value.
}

\seealso{
   \code{link{AVASet-class}},
   \code{link{MapperSet-class}},
   \code{\link{getVariantPercentages}}.
}

\examples{

# load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
data(avaSetExample)
avaSetExample

# use only those variants that are covered by at least 10% of all reads in one sample in both directions together (259 -> 4 variants)
avaSetExample = setVariantFilter(avaSetExample, filter=0.1)
avaSetExample

# use only those variants that are covered by at least 0.1% of all reads in one sample in forward direction
# and by at least 0% in reverse direction (259 -> 6 variants)
avaSetExample = setVariantFilter(avaSetExample, filter=c(0.1, 0))
avaSetExample

# reset filter values to zero
avaSetExample = setVariantFilter(avaSetExample, filter=0)
# or simply
avaSetExample = setVariantFilter(avaSetExample)

}

\author{Christoph Bartenhagen}

\name{readSFF}

\alias{readSFF}

\title{Function To Read In Roche's .sff Files}

\description{
    This function reads in files in Roche's Standard Flowgram Format (SFF) and store the contents in 
    an \code{\link{SFFContainer-class}} object.
}

\usage{readSFF(files)}

\arguments{
  \item{files}{The name of the .sff file to read in or a character vector of multiple file names or
    the name of a directory containing .sff files.}
}

\value{
  An object or a list of objects of class \code{SFFContainer} storing all the information from the 
  .sff file(s).
}

\seealso{
  \code{\link{writeSFF}}, \code{\linkS4class{SFFContainer}}.
}

\examples{
  file <- system.file("extdata", "SFF", "example.sff", package="R453Plus1Toolbox")
  sffContainer <- readSFF(file)
  sffContainer
}

\author{Christian Ruckert}

\name{convertCigar}

\alias{convertCigar}
\alias{extendedCIGARToList}
\alias{listToExtendedCIGAR}

\title{Basic functions for CIGAR strings}

\description{
    These are temporary methods, that are likely to be replaced by
    methods from the Rsamtools package in near future.
}

\usage{
    extendedCIGARToList(cigars)
    listToExtendedCIGAR(cigarList)
}

\arguments{
    \item{cigars}{A character vector with CIGAR strings.}
    \item{cigarList}{A list of converted CIGAR strings as produced by \code{\link{extendedCIGARToList}}}
}

\name{sequenceQualityHist}

\docType{methods}

\alias{sequenceQualityHist}
\alias{sequenceQualityHist,QualityScaledDNAStringSet-method}
\alias{sequenceQualityHist,ShortReadQ-method}
\alias{sequenceQualityHist,SFFContainer-method}

\title{A Histogram Of The Sequence Qualities}

\description{
  This function creates a histogram of the mean qualities of the sequences.
}

\usage{
  sequenceQualityHist(object, xlab="Mean of quality scores per sequence", 
    ylab="Number of sequences", col="firebrick1", ...)
}

\arguments{
  \item{object}{An object of class \link{QualityScaledDNAStringSet}, \link{ShortReadQ} or 
    \link{SFFContainer}.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{col}{The plotting color.}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{detectBreakpoints}

\alias{detectBreakpoints}
\alias{detectBreakpoints,list-method}

\title{Clustering and consensus breakpoint detection for chimeric reads}
\description{
  Given a set of chimeric reads, this methods computes all putative
  breakpoints. First, chimeric reads are clustered such that all reads
  spanning the same breakpoint form a cluster. Then, a consensus
  breakpoint sequence and breakpoint position is computed for each cluster.
}

\usage{
detectBreakpoints(chimericReads, bpDist=100, minClusterSize=4, removeSoftClips=TRUE, bsGenome)
}

\arguments{
  \item{chimericReads}{A list storing chimeric reads as returned by
    \code{\link{filterChimericReads}}. The list must have the format as
    defined by the \code{\link{scanBam}} method.}
  \item{bpDist}{The maximum distance in base pairs between the
    breakpoints of two chimeric reads at which the reads are merge
    to a cluster.}
  
  \item{minClusterSize}{Cluster whose size is below minClusterSize are
    be excluded from breakpoint detection.}
  \item{removeSoftClips}{If true, soft-clipped bases at the beginning or the
    end of a sequence are removed (see details below).}
  \item{bsGenome}{A \code{bsGenome} instance providing the reference
    sequences. If missing, the library
    \code{BSgenome.Hsapiens.UCSC.hg19} is used by default.}
}


\details{
  This method is usually invoked after calling
  \code{\link{filterChimericReads}} and before calling
  \code{\link{mergeBreakpoints}}. It first forms clusters of chimeric
  reads (reads with exactly two local alignments) that span the same
  breakpoint and than computes a consensus breakpoint sequence for each
  cluster.
  
  To carry out a hierarchical clustering, a measure for the distance
  between two chimeric reads must be defined. If reads span different
  chromosomes, their distance is set to infinity. The strand
  information of the local alignments may also indicate that two chimeric
  reads do not span the same breakpoint even if they span the same
  chromosomes. For example, the first reads has two local alignments on
  the positive strand whereas the second read has one local alignment on
  the positive strand and the other on the negative strand. In this case,
  the distance is set to inifinty, too. Finally, the distance measure
  distinguishes between the two breakpoints (sometimes
  called the pathogenic and the reciproce breakpoint) that originate from
  the same structual variant. The distance between a read from the
  pathogenic and a read from the reciproce breakpoint is infinity so that
  two different clusters will emerge. These two related breakpoints can be
  merge later using the \code{\link{mergeBreakpoints}} method. We observed
  that the breakpoints of these two cases often differ by a few ten or even
  a few hundred basepairs.
  
  If the chromosome and strand information between two reads \eqn{x} and
  \eqn{y} are coherent, the Euclidian distance is used:
  \deqn{d(x,y) = (bp(x, ChrA) - bp(y, ChrA))^2 + (bp(x, ChrB) - bp(y, ChrB))^2}
  where \eqn{bp} gives the coordinates of the breakpoint for the given
  read and chromosome. Hierarchical clustering is applied with complete
  linkage and the dendrogram is cutted at a height of \code{bpDist} to
  obtain the final clusters. The \code{bpDist} argument does usually not
  influence the result, because we observed that reads spanning the same
  breakpoint have very little variation (only a few base pairs) in their
  local alignments due to sequencing errors or due to ambiguity caused by
  same/similar sequence of both chromosome near the breakpoint.
  
  Although the given set of reads may belong to the same chimeric DNA,
  their individual breakpoints may differ in a few base pairs. 
  Furthermore, a single read may have more than one possible breakpoint
  if a (small) part of the read was aligned to both parts. \cr
  The following step determines a consensus breakpoint for each cluster.
  It uses the supplied \code{bsGenome} to construct a
  chimeric reference sequence for all possible breakpoints over all reads within
  each cluster. After the reads were realigned to the chimeric reference
  sequences, the one that yields the highest alignment score is taken to 
  represent best the chimeric DNA and its breakpoints. \cr

  As a preprocessing step, \code{detectBreakpoints} offers to remove soft
  clips occuring after the alignment: \cr
  Some reads may contain soft-clipped bases (e.g. linker sequences) at the beginning of the first part
  of the read or at the end of the second part. By default,
  \code{detectBreakpoints} removes these unaligned subsequences
  and adjusts the cigar string, the sequence, the sequence width
  (qwidth) and the local start/end coordinates.
  
}

\value{
  \code{detectBreakpoints} returns an object of class
  \code{breakpoints}, which is a list of breakpoint clusters, which gives
  access to all alignments and consensus breakpoints:  \cr
  \item{seqs}{This \code{\link{IRanges}} \code{DataFrame} is
    mainly a rearranged version of the alignment input in
    \code{chimericReads}. In addition, it shows the corresponding
    breakpoints and local alignment coordinates.}
  \item{commonBps}{A dataframe listing the breakpoints for both parts
    of the chimeric reference, the associated chromosome, strand and the reference
    sequence itself, including positions "localStart"/"localEnd"
    indicating which part of the reference belongs to which breakpoint.}
  \item{commonAlign}{An object of class
    \code{\link{PairwiseAlignmentsSingleSubject}} of the
    Biostrings package that contains the alignment to the
    (best) consensus reference sequence.}
  \item{alignedReads}{On the basis of \code{commonAlign} and \code{commonBps},
    \code{alignedReads} is an instance of class \code{\link{AlignedRead}}
    containing all aligned reads including their associated chromosomes, strands, and
    positions. Since the reference is a chimeric sequence each read has
    two chromosome and two strand entries.}
}

\author{Hans-Ulrich Klein, Christoph Bartenhagen}

\seealso{
  \code{\link{filterChimericReads}}
  \code{\link{mergeBreakpoints}}
  \code{\link{plotChimericReads}}
}

\examples{
# Construct a small example with three chimeric reads
# (=6 local alignments) in bam format as given by
# aligners such as BWA-SW.
# The first two reads originate from the same case but
# from different strands. The third read originate from
# the reciprocal breakpoint.
library("BSgenome.Scerevisiae.UCSC.sacCer2")
bamReads = list()
bamReads[[1]] = list(
    qname=c("seq1", "seq1", "seq2", "seq2", "seq3", "seq3"),
    flag = as.integer(c(0, 0, 16, 16, 0, 0)),
    rname = factor(c("II", "III", "III", "II", "III", "II")),
    strand = factor(c("+", "+", "-", "-", "+", "+")),
    pos = as.integer(c(99951, 200000, 200000, 99951, 199950, 100001)),
    qwidth = as.integer(c(100, 100, 100, 100, 100, 100)),
    cigar = c("50M50S","50S50M","50S50M","50M50S","50M50S", "50S50M"),
    seq = DNAStringSet(c(
        paste(substr(Scerevisiae$chrII, start=99951, stop=100000),
              substr(Scerevisiae$chrIII, start=200000, stop=200049),
              sep=""),
        paste(substr(Scerevisiae$chrII, start=99951, stop=100000),
              substr(Scerevisiae$chrIII, start=200000, stop=200049),
              sep=""),
        paste(substr(Scerevisiae$chrIII, start=200000, stop=200049),
              substr(Scerevisiae$chrII, start=99951, stop=100000),
              sep=""),
        paste(substr(Scerevisiae$chrIII, start=200000, stop=200049),
              substr(Scerevisiae$chrII, start=99951, stop=100000),
              sep=""),
        paste(substr(Scerevisiae$chrIII, start=199950, stop=199999),
              substr(Scerevisiae$chrII, start=100001, stop=100050),
              sep=""),
        paste(substr(Scerevisiae$chrIII, start=199950, stop=199999),
              substr(Scerevisiae$chrII, start=100001, stop=100050),
              sep="")))
)

bps = detectBreakpoints(bamReads, minClusterSize=1, bsGenome=Scerevisiae)
summary(bps)
table(bps)

mergeBreakpoints(bps)
}

\name{coverageOnTarget}

\alias{coverageOnTarget}
\alias{coverageOnTarget,list,GRanges-method}

\title{
Computes the coverage restricted to the target region.
}
\description{
This method computes the approximate coverage of each base in a given region.
}

\usage{
coverageOnTarget(alnReads, targetRegion)
}

\arguments{
  \item{alnReads}{
  A list as returned by \code{scanBam} storing aligned reads.
}
  \item{targetRegion}{
  The target region as a \code{GRanges} object. The chromosome names must
  fit to the chromosome names used in the alignment information of the
  given reads.
}
}

\details{
The detailed alignment information given by the CIGAR strings in .bam
files are ignored by the function. Instead, it is assumed that the whole
read alignes to the reference without indels. This is often not true for
longer read (e.g. generated with Roche 454 Sequencing), but saves
computation time.
}

\value{
A list of the same length as the \code{alnReads} argument. Each list
element is an integer vector of the same length as the target region (in
bases) and stores the coverage generated by the reads from the
corresponding list element of \code{alnReads}.
}

\author{
Hans-Ulrich Klein
}

\seealso{
\code{\link{scanBam}}
}

\examples{
library(Rsamtools)
bamFile = system.file("extdata", "SVDetection", "bam", "N01.bam", package="R453Plus1Toolbox")
bam = scanBam(bamFile)
region = GRanges(IRanges(start=118307205, end=118395936), seqnames=11)
cov = coverageOnTarget(bam, region)
}

\keyword{"coverage"}

\name{captureArray}
\alias{captureArray}
\docType{data}
\title{Custom capture array design}
\description{
Design of a custom Roche NimbleGen 385k capture array. The array captures
short segments corresponding to all exon regions of 92 distinct 
target genes (genome build hg19). In addition, contiguous genomic
regions were represented for three additional genes, i.e. CBFB, MLL, and RUNX1.
}
\usage{data(captureArray)}
\format{
Formal class 'CompressedIRangesList'
}
\source{
\sQuote{Targeted next-generation sequencing detects point mutations, 
insertions, deletions, and balanced chromosomal rearrangements as 
well as identifies novel leukemia-specific fusion genes in a single
procedure} (Leukemia, submitted)
}
\examples{
data(captureArray)
}
\keyword{datasets}

\name{mergeBreakpoints}
\alias{mergeBreakpoints}

\title{
Identify and merge related breakpoints caused by the same variant.
}
\description{
Structural variation like transversions or inversion cause two
breakpoints. In the context of fusion genes, these are called the
pathogenic breakpoint and the reciproce breakpoint. The method
\code{\link{detectBreakpoints}} processes each breakpoint individually
and does explicitly not put reads from the pathogenic and reciproce
breakpoint into the same cluster. Hence, it is usually sensible to call
this methods afterward to search for related pairs of breakpoints to
gain more confidence about the existence of a structural variation.
}

\usage{
mergeBreakpoints(breakpoints, maxDist, mergeBPs)
}

\arguments{
  \item{breakpoints}{
  An object of class \code{Breakpoints} storing the breakpoints that will
  (potentially) be merged.
}
  \item{maxDist}{
  The maximal distance in basepairs at which two breakpoints will be merged. Default value is 1000.
}
  \item{mergeBPs}{
  An optional list of vectors of length two giving the breakpoints that
  should be merged. If this argument is given, the method will not
  search for related breakpoints.
}
}

\details{
If the \code{maxDist} argument is given, the method compares each pair
of breakpoints and checks, whether the two breakpoints may belong to the
same structural variation. In addition to the spanned chromosomes, the
orientation and the strand information of the reads spanning the
breakpoints are also compared for this purpose. If chrosmosome,
orientation and strand information of two breakpoints go well together,
they will be merged, if  the absolute distance of the breakpoints on
chromosome A plus the absolute distance on chromosome B is smaller or
equal to \code{maxDist}. If one breakpoint has more than one potential
mate breakpoint for merging, it will be merged with the first candidate
and a warning message is printed. The default value of \code{maxDist}
is 1000.\\

If the \code{mergeBPs} argument is given, the method will not search for
related breakpoints but simply merge the given breakpoints.
\code{mergeBPs} must be a \code{list} with vectors of
length two that either contain the names of the indices of the
breakpoints that should be merged.\\

The arguments \code{maxDist} and \code{mergeBPs} cannot be given
together. The given \code{Breakpoints} object must not contain
breakpoints that have been merged before.
}

\value{
An object of class \code{Breakpoints} storing merged and unmerged
breakpoints. 
}

\author{
Hans-Ulrich Klein
}

\seealso{
\code{\link{detectBreakpoints}}, \code{\link{Breakpoints-class}},
\code{\link{plotChimericReads}}
}

\examples{
# Load bam file and filter chimeric reads
library(Rsamtools)
bamFile = system.file("extdata", "SVDetection", "bam", "N01.bam", package="R453Plus1Toolbox")
bam = scanBam(bamFile)
data(captureArray)
linker = sequenceCaptureLinkers("gSel3")[[1]]
filterReads = filterChimericReads(bam, targetRegion=captureArray, linkerSeq=linker)

# detect breakpoints of size >= 3
breakpoints = detectBreakpoints(filterReads, minClusterSize=3)
table(breakpoints)
summary(breakpoints)

# merge breakpoints
breakpoints = mergeBreakpoints(breakpoints)
summary(breakpoints)
}

\keyword{"reciproce breakpoint"}
\keyword{"pathogenic breakpoint"}

\name{annotateVariants}

\alias{annotateVariants}
\alias{annotateVariants,AVASet,missing-method}
\alias{annotateVariants,MapperSet,BSgenome-method}
\alias{annotateVariants,MapperSet,missing-method}
\alias{annotateVariants,data.frame,missing-method}

\title{Adds genomic information to variants}

\description{This method annotates given genomic variants (mutations).
Annotation includes affected genes, exons and codons. Resulting amino acid
changes are returned as well as dbSNP identifiers, if the mutation is already
known. All information is fetched from the Ensembl GRCh37 server via biomaRt
using the datasets \code{hsapiens_gene_ensembl} and \code{hsapiens_snp}.}

\usage{annotateVariants(object, bsGenome)}

\arguments{
    \item{object}{A data frame storing variants or an instance of AVASet/MapperSet or a data frame (see details).}
    \item{bsGenome}{An object of class \code{BSGenome} giving the genome to be used as reference sequence
    to calculate amino acid changes. This argument is only applicable when \code{object} is of type 
    \code{\link{MapperSet}}. Default is \sQuote{BSgenome.Hsapiens.UCSC.hg19}. Note that the genome
    should fit  to the Ensembl annotation.}
}

\details{If a data frame is given, the following columns must be
  present:
  \tabular{ll}{
    start \tab genomic start position in the current Ensembl genome\cr
    end \tab genomic end position in the current Ensembl genome\cr
    chromosome \tab chromosome in ensembl notation (i.e. "1", "2", ..., "Y")\cr
    strand \tab "+" or "-" relative to the nucleotide bases given below\cr
    seqRef \tab reference sequence\cr
    seqMut \tab sequence of the observed variant\cr
    seqSur \tab reference sequence extended for 3 bases in both directions
  }
  The rownames of the data frame are used as mutations' names (IDs). See
  examples for a properly defined data drame.
}

\value{An object of class \code{AnnotatedVariants}. Affected genes, transcripts
and exon as well as known SNPs are stored in a list-like structure. See the
documentation of class \code{\link{AnnotatedVariants-class}} for details.}

\seealso{\code{\link{AnnotatedVariants-class}}, \code{\link{AVASet-class}}, 
\code{\link{MapperSet-class}}, \code{\link{htmlReport}}}

\examples{
variants = data.frame(
    start=c(106157528, 106154991,106156184),
    end=c(106157528, 106154994,106156185),
    chromosome=c("4", "4", "4"),
    strand=c("+", "+", "+"),
    seqRef=c("A", "ATAG", "---"),
    seqMut=c("G", "----", "ATA"),
    seqSur=c("TACAGAA", "TTTATAGATA", "AGC---TCC"),
    stringsAsFactors=FALSE)
rownames(variants) = c("snp", "del", "ins")
\dontrun{annotateVariants(variants)}
}

\author{Hans-Ulrich Klein}
\keyword{annotateVariants}

\name{nucleotideCharts}

\docType{methods}

\alias{nucleotideCharts}
\alias{nucleotideCharts,DNAStringSet-method}
\alias{nucleotideCharts,ShortRead-method}
\alias{nucleotideCharts,SFFContainer-method}

\title{Nucleotide Charts}

\description{
  This function plots the relative frequency of the four bases for each position in the sequences.
}

\usage{
  nucleotideCharts(object, range=0.95, linetypes=c(A="l", C="l", G="l", T="l", N="l"), 
    linecols=c(A="black", C="red", G="blue", T="green", N="grey70"), xlab="Position", 
    ylab="Frequency", ...)
}

\arguments{
  \item{object}{An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.}
  \item{range}{An integer vector of length one or two. If length one only bases up to the percent 
    quantile of read lengths defined by the given value are shown. A vector of length two gives the 
    start and end base between which the nucleotide frequencies are plotted.}
  \item{linetypes}{The line types used for the four nucleotids + N, see (see \sQuote{par})}
  \item{linecols}{The colors in which the four nucleotids + N should be plotted.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{mapperSetExample}
\alias{mapperSetExample}
\docType{data}
\title{
    GS Reference Mapper data import
}

\description{
    This is an example of an \code{link{MapperSet-class}} object containing the output of Roche's GS Reference Mapper Software.
    It consists of 3 samples and 111 variants.
}

\usage{data(mapperSetExample)}

\format{
    Formal class 'MapperSet'
}

\source{
    \sQuote{Targeted next-generation sequencing detects point mutations,
    insertions, deletions, and balanced chromosomal rearrangements as
    well as identifies novel leukemia-specific fusion genes in a single
    procedure} (Leukemia, submitted)
}

\examples{
    data(mapperSetExample)
    mapperSetExample
}
\keyword{datasets}

\name{breakpoints}
\alias{breakpoints}
\docType{data}
\title{
    Putative breakpoints of chimeric reads
}
\description{
    This example holds two consensus (pathogenic and reciproce) breakpoints of 12 chimeric reads indicating an inversion on chromosome 16.
    The Breakpoints object gives access to the breakpoint locations as well as alignment information for each of the 12 reads.
}
\usage{data(breakpoints)}

\format{
    Formal class 'Breakpoints'
}

\source{
    \sQuote{Targeted next-generation sequencing detects point mutations,
    insertions, deletions, and balanced chromosomal rearrangements as
    well as identifies novel leukemia-specific fusion genes in a single
    procedure} (Leukemia, submitted)
}

\examples{
    data(breakpoints)
}
\keyword{datasets}

\name{complexity.dust}

\docType{methods}

\alias{complexity.dust}
\alias{complexity.dust,DNAStringSet-method}
\alias{complexity.dust,ShortRead-method}
\alias{complexity.dust,SFFContainer-method}

\title{Sequence Complexity Using The DUST Algorithm}

\description{
  This function evaluates the sequence complexity using the DUST algorithm.
}

\usage{
  complexity.dust(object, xlab="Complexity score (0=high, 100=low)", ylab="Number of sequences", 
    xlim=c(0, 100), col="firebrick1", breaks=100, ...)
}

\arguments{
  \item{object}{An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{xlim}{The limits of the X axis.}
  \item{col}{The plotting color.}
  \item{breaks}{The number of breaks in the histogram (see \sQuote{hist}).}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\details{
  The complexity score is based on how often different trinucleotides occur and is scaled between 0 
  and 100. A sequence of homopolymer repeats (e.g. TTTTTTTTTT) has a score of 100, of dinucleotide 
  repeats (e.g. TATATATATA) has a score around 49, and of trinucleotide repeats (e.g. TAGTAGTAG) has 
  a score around 32. Scores above seven can be considered low-complexity.
}

\value{
  A numeric vector containing the complexity score for each sequence.
}

\references{
  Schmieder R. (2011) Quality control and preprocessing of metagenomic datasets.
  \emph{Bioinformatics}, 2011 Mar 15;27(6):863-4. 
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{variants}
\alias{variants}
\docType{data}

\title{
        Example data for \code{\link{plotVariants}}
}
\description{
        This data.frame is part of the vignette example of the \code{\link{plotVariants}} function. It has the columns
  	"label", "pos" "mutation" and "color" specifying an annotation for each mutation, its position, the mutation type and an individual color.
	The position is given as amino acids / codons.
}

\usage{data(plotVariantsExample)}

\format{
	data.frame
}

\examples{
    	data(plotVariantsExample)
    	variants
}
\keyword{datasets}

\name{baseFrequency}

\docType{methods}

\alias{baseFrequency}
\alias{baseFrequency,DNAStringSet-method}
\alias{baseFrequency,ShortRead-method}
\alias{baseFrequency,SFFContainer-method}

\title{Absolute And Relative Frequency Of The Four Bases.}

\description{
  This function returns the absolute and the relative frequency of the four bases (A, C, G, T).
}

\usage{
  baseFrequency(object)
}

\arguments{
  \item{object}{
    An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.
  }
}

\details{
  This function makes use of the \link{alphabetFrequency} function from package 
  \code{Biostrings}.
}

\value{
  A \link{data.frame} with two columns containing the absolute and relative frequencies 
  respectively and six rows, one for each of the four bases (A, C, G, T), one for other symbols 
  contained in the reads and one summarizing the five aforementioned rows.
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{fDataAmp}

\alias{fDataAmp}

\title{Access the amplicon data of an AVASet.}

\description{
    Similar to \code{fData} of the Biobase ExpressionSet, this function returns the feature data of the amplicon slot of an instance of the 
    AVASet.
}

\usage{
    fDataAmp(object) 
}

\arguments{
  \item{object}{An \code{link{AVASet-class}} object.}
}

\value{
    The feature data of the amplicon slot contains the names, primers, start/end positions and reference sequences of all 
    amplicons (see\code{\link{AVASet-class}} for details). It returns a data frame.
}

\seealso{
    \code{\link{featureDataAmp}}, \code{\link{assayDataAmp}},\code{\link{AVASet-class}}
}

\examples{
    # load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
    data(avaSetExample)
    avaSetExample

    # show contents amplicon feature data
    fDataAmp(avaSetExample)
}

\author{Christoph Bartenhagen}

\name{featureDataAmp}

\alias{featureDataAmp}

\title{Access the amplicon data of an AVASet}

\description{
    Similar to \code{featureData} of the Biobase ExpressionSet, this function returns the feature data and feature meta of the amplicon slot of an
    instance of the AVASet.
}

\usage{
    featureDataAmp(object) 
}

\arguments{
  \item{object}{An \code{link{AVASet-class}} object.}
}

\value{
    The feature data of the amplicon slot contains the names, primers, start/end positions and reference sequences of all amplicons (see 
    \code{\link{AVASet-class}} for details). The returned object is of class \code{\link{AnnotatedDataFrame}}.
}

\seealso{
    \code{\link{fDataAmp}}, \code{\link{assayDataAmp}}, \code{\link{AVASet-class}}, 
}

\examples{
    # load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
    data(avaSetExample)
    avaSetExample

    # show contents amplicon feature data
    featureDataAmp(avaSetExample)

}

\author{Christoph Bartenhagen}

\name{getVariantPercentages}

\alias{getVariantPercentages}

\title{Variant coverage}

\description{
    This function computes the coverage for each variant (in forward and/or reverse direction) for all samples. The coverage is defined as the 
    percentual amount of reads that cover a variant.
}

\usage{getVariantPercentages(object, direction="both")}

\arguments{
  \item{object}{An instance of class \code{\link{AVASet-class}} or \code{\link{MapperSet-class}}.}
  \item{direction}{A character indicating the direction ("forward", "reverse" or "both").}
}

\details{
    If the direction was set to "both", the percentages are computed over the sum of both directions. Otherwise it is computed only over the 
    occurences in one direction (forward or reverse). The occurences can be accesses via \code{assayData}.}

\value{
    \code{getVariantPercentages} returns a data frame with all percentages/frequencies for all samples.
}

\seealso{
    \code{\link{setVariantFilter}}.
}

\examples{

# load a (filtered) AVA dataset containing 6 samples, 4 amplicons and 4 variants
data(avaSetFiltered)
avaSetFiltered

# both directions
getVariantPercentages(avaSetFiltered, direction="both")
# this is equivalent to
(assayData(avaSetFiltered)[[1]] + assayData(avaSetFiltered)[[3]]) / (assayData(avaSetFiltered)[[2]] + assayData(avaSetFiltered)[[4]])

# forward direction only
getVariantPercentages(avaSetFiltered, direction="forward")
# this is equivalent to
assayData(avaSetFiltered)[[1]] / assayData(avaSetFiltered)[[2]]

# reverse direction only
getVariantPercentages(avaSetFiltered, direction="reverse")
# this is equivalent to
assayData(avaSetFiltered)[[3]] / assayData(avaSetFiltered)[[4]]


}

\author{Christoph Bartenhagen}

\name{AnnotatedVariants-class}
\Rdversion{1.1}
\docType{class}
\alias{AnnotatedVariants-class}
\alias{annotatedVariants}
\alias{annotatedVariants,AnnotatedVariants-method}
\alias{annotatedVariants<-,AnnotatedVariants,list-method}
\alias{names,AnnotatedVariants-method}
\alias{names<-,AnnotatedVariants,character-method}

\title{Class "AnnotatedVariants"}
\description{A class for storing annotation about variants. An object of this class
is returned by the method \code{annotateVariants}. The class has not been designed
to be created by users directly. }

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("AnnotatedVariants")}. The
method \code{annotateVariants} returns AnnotatedVariants-objects.
}

\section{Slots}{
  \describe{
    \item{\code{annotatedVariants}:}{Object of class \code{"list"} with one entry for
      each variant.}
  }
}

\section{Methods}{
  \describe{
    \item{annotatedVariants}{\code{signature(object = "AnnotatedVariants")}: Get the 
list with variants.}
    \item{annotatedVariants<-}{\code{signature(object = "AnnotatedVariants", value = "list")}:
Set a new list with variants.}
    \item{names}{\code{signature(x = "AnnotatedVariants")}: Get the
names of the with variants.}
    \item{names<-}{\code{signature(x = "AnnotatedVariants", value = "character")}:
Set the names of the variants.}
  }
}

\details{The list encapsulated by this class has one element for each variant. Each
element is a nested list with the elements \code{genes}, \code{transcripts}, \code{exons}
and \code{snps}. All these elements are data frames listing genes, transcripts, exons or
snps respectively that were affected by the variant. Use the example below to explore
the data frames' contents.}

\author{Hans-Ulrich Klein}

\seealso{
\code{\link{annotateVariants}}, \code{\link{htmlReport}}
}

\examples{
variants = data.frame(
    start=c(106157528, 106154991,106156184),
    end=c(106157528, 106154994,106156185),
    chromosome=c("4", "4", "4"),
    strand=c("+", "+", "+"),
    seqRef=c("A", "ATAG", "---"),
    seqMut=c("G", "----", "ATA"),
    seqSur=c("TACAGAA", "TTTATAGATA", "AGC---TCC"),
    stringsAsFactors=FALSE)
rownames(variants) = c("snp", "del", "ins")
\dontrun{av = annotateVariants(variants)}
\dontrun{annotatedVariants(av)[["snp"]]}
}


\keyword{classes}

\name{avaSetExample}
\alias{avaSetExample}
\docType{data}
\title{
    Amplicon Variant Analyzer data import
}
\description{
    This is an example of an \code{link{AVASet-class}} object containing the output of Roche's Amplicon Variant Analyzer Software.
    It consists of 6 samples, 4 amplicons and 259 variants.
}

\usage{data(avaSetExample)}

\format{
    Formal class 'AVASet'
}

\source{
    \sQuote{Next-generation sequencing technology reveals a characteristic
    pattern of molecular mutations in 72.8% of chronic myelomonocytic
    leukemia by detecting frequent alterations in TET2, CBL, RAS, and RUNX1}
    (Kohlmann A et al., J Clin Oncol. 2010 Aug 20;28(24):3858-65. Epub 2010 Jul 19)
}

\examples{
    data(avaSetExample)
    avaSetExample
}
\keyword{datasets}

\name{readLengthStats}

\docType{methods}

\alias{readLengthStats}
\alias{readLengthStats,DNAStringSet-method}
\alias{readLengthStats,ShortRead-method}
\alias{readLengthStats,SFFContainer-method}

\title{Statistics For The Read Lengths}

\description{
  This function returns the mean, median, minimum, maximum and standard deviation of the read 
  lengths over a set of sequences.
}

\usage{
  readLengthStats(object)
}

\arguments{
  \item{object}{
    An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.
  }

}

\value{
  A \link{vector} with five entries: mean, median, min, max and sd.
}

\author{
Christian Ruckert
}

\keyword{methods}

\name{SFFContainer-class}
\Rdversion{1.1}
\docType{class}
\alias{SFFContainer-class}
\alias{SFFContainer}
\alias{addRead,SFFContainer,SFFRead-method}
\alias{addRead}
\alias{getRead,SFFContainer,character-method}
\alias{getRead}
\alias{clipAdapterLeft<-,SFFContainer,numeric-method}
\alias{clipAdapterLeft<-}
\alias{clipAdapterLeft,SFFContainer-method}
\alias{clipAdapterLeft}
\alias{clipAdapterRight<-,SFFContainer,numeric-method}
\alias{clipAdapterRight<-}
\alias{clipAdapterRight,SFFContainer-method}
\alias{clipAdapterRight}
\alias{clipQualityLeft<-,SFFContainer,numeric-method}
\alias{clipQualityLeft<-}
\alias{clipQualityLeft,SFFContainer-method}
\alias{clipQualityLeft}
\alias{clipQualityRight<-,SFFContainer,numeric-method}
\alias{clipQualityRight<-}
\alias{clipQualityRight,SFFContainer-method}
\alias{clipQualityRight}
\alias{name<-,SFFContainer,character-method}
\alias{name<-}
\alias{name,SFFContainer-method}
\alias{name}
\alias{flowChars<-,SFFContainer,character-method}
\alias{flowChars<-}
\alias{flowChars,SFFContainer-method}
\alias{flowChars}
\alias{flowgramFormat<-,SFFContainer,numeric-method}
\alias{flowgramFormat<-}
\alias{flowgramFormat,SFFContainer-method}
\alias{flowgramFormat}
\alias{flowgrams<-,SFFContainer,list-method}
\alias{flowgrams<-}
\alias{flowgrams,SFFContainer-method}
\alias{flowgrams}
\alias{flowIndexes<-,SFFContainer,list-method}
\alias{flowIndexes<-}
\alias{flowIndexes,SFFContainer-method}
\alias{flowIndexes}
\alias{keySequence<-,SFFContainer,character-method}
\alias{keySequence<-}
\alias{keySequence,SFFContainer-method}
\alias{keySequence}
\alias{reads<-,SFFContainer,QualityScaledDNAStringSet-method}
\alias{reads<-}
\alias{reads,SFFContainer-method}
\alias{reads}
\alias{[,SFFContainer,ANY,ANY-method}
\alias{[,SFFContainer,ANY,ANY,ANY-method}

\title{Class \code{"SFFContainer"}}
\description{
This class is a container for data from files in Roche's Standard Flowgram Format (SFF).
}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("SFFContainer", ...)}.
Usually, objects will be created by calling the \code{\link{readSFF}} method on a file in SFF 
format.
}

\section{Slots}{
  \describe{
    \item{\code{name}:}{Object of class \code{"character"} containing the name of the file this 
      SFFContainer was created from.}
    \item{\code{flowgramFormat}:}{Object of class \code{"numeric"} representing the format used to 
      encode each of the flowgram values for each read. Currently, only one flowgram format has been 
      adopted and is coded by the value 1.}
    \item{\code{flowChars}:}{Object of class \code{"character"} containing the array of nucleotide 
      bases ('A', 'C', 'G', 'T') that correspond to the nucleotides used for each flow of each 
      read.}
    \item{\code{keySequence}:}{Object of class \code{"character"} representing the nucleotide bases
      of the key sequence used for these reads.}
    \item{\code{clipQualityLeft}:}{Object of class \code{"numeric"} representing the position of the 
      first base after the clipping point for an attached quality sequence for each read. If only a 
      combined (quality+adapter) clipping position is computed it should be stored in 
      clipQualityLeft. If no clipping value is computed the field is set to 0. The position values 
      use 1-based indexing.}
    \item{\code{clipQualityRight}:}{Object of class \code{"numeric"} representing the position of 
      the last base before the clipping point for an attached quality sequence for each read. If 
      only a combined (quality+adapter) clipping position is computed it should be stored in 
      clipQualityRight. If no clipping value is computed the field is set to 0. The position values 
      use 1-based indexing.}
    \item{\code{clipAdapterLeft}:}{Object of class \code{"numeric"} representing the position of the 
      first base after the clipping point for an attached adapter sequence for each read. If only a 
      combined (quality+adapter) clipping position is computed it should be stored in 
      clipQualityLeft. If no clipping value is computed the field is set to 0. The position values 
      use 1-based indexing.}
    \item{\code{clipAdapterRight}:}{Object of class \code{"numeric"} representing the position of 
      the last base before the clipping point for an attached adapter sequence for each read. If 
      only a combined (quality+adapter) clipping position is computed it should be stored in 
      clipQualityRight. If no clipping value is computed the field is set to 0. The position values 
      use 1-based indexing.}
    \item{\code{flowgrams}:}{Object of class \code{"list"} containing the homopolymer stretch 
      estimates for each flow using one list item for each read.}
    \item{\code{flowIndexes}:}{Object of class \code{"list"} containing the flow positions for 
      each base in the called sequence, i.e. for each base, the position in the flowgram whose 
      estimate resulted in that base being called. Each read has its own list item.}
    \item{\code{reads}:}{Object of class \code{"QualityScaledDNAStringSet"} containing the 
      basecalled nucleotide sequences of each read together with the quality scores for each of the 
      bases in the sequence using the standard -log10 probability scale.}
  }
}

\section{Methods}{
  \describe{
    \item{addRead}{\code{signature(object = "SFFContainer", read = "SFFRead")}: Adds an object of 
      class \code{\linkS4class{SFFRead}} to the \code{\linkS4class{SFFContainer}}}
    \item{getRead}{\code{signature(object = "SFFContainer", readname = "character")}: Returns the 
      read with the given name as an object of class \code{\linkS4class{SFFRead}}.}
    \item{clipAdapterLeft<-}{\code{signature(object = "SFFContainer", value = "numeric")}: 
      Setter-method for the clipAdapterLeft slot.}
    \item{clipAdapterLeft}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the clipAdapterLeft slot.}
    \item{clipAdapterRight<-}{\code{signature(object = "SFFContainer", value = "numeric")}: 
      Setter-method for the clipAdapterRight slot.}
    \item{clipAdapterRight}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the clipAdapterRight slot.}
    \item{clipQualityLeft<-}{\code{signature(object = "SFFContainer", value = "numeric")}: 
      Setter-method for the clipQualityLeft slot.}
    \item{clipQualityLeft}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the clipQualityLeft slot.}
    \item{clipQualityRight<-}{\code{signature(object = "SFFContainer", value = "numeric")}: 
      Setter-method for the clipQualityRight slot.}
    \item{clipQualityRight}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the clipQualityRight slot.}
    \item{name<-}{\code{signature(object = "SFFContainer", value = "character")}: 
      Setter-method for the name slot.}
    \item{name}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the name slot.}
    \item{flowChars<-}{\code{signature(object = "SFFContainer", value = "character")}: 
      Setter-method for the flowChars slot.}
    \item{flowChars}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the flowChars slot.}
    \item{flowgramFormat<-}{\code{signature(object = "SFFContainer", value = "numeric")}: 
      Setter-method for the flowgramFormat slot.}
    \item{flowgramFormat}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the flowgramFormat slot.}
    \item{flowgrams<-}{\code{signature(object = "SFFContainer", value = "list")}: 
      Setter-method for the flowgrams slot.}
    \item{flowgrams}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the flowgrams slot.}
    \item{flowIndexes<-}{\code{signature(object = "SFFContainer", value = "list")}: 
      Setter-method for the flowIndexes slot.}
    \item{flowIndexes}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the flowIndexes slot.}
    \item{keySequence<-}{\code{signature(object = "SFFContainer", value = "character")}: 
      Setter-method for the keySequence slot.}
    \item{keySequence}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the keySequence slot.}
    \item{reads<-}{\code{signature(object = "SFFContainer", value = "QualityScaledDNAStringSet")}: 
      Setter-method for the reads slot.}
    \item{reads}{\code{signature(object = "SFFContainer")}: 
      Getter-method for the reads slot.}
    \item{[}{\code{signature(x = "SFFContainer", i = "ANY", j = "ANY")}:
      Subsetting a SFFContainer object.}
  }
}

\author{
  Christian Ruckert
}

\seealso{
  \code{\link{readSFF}}, \code{\linkS4class{SFFRead}}
}

\examples{
  showClass("SFFContainer")
}

\keyword{classes}

\name{removeLinker}

\alias{removeLinker}
\alias{removeLinker,XStringSet,DNAString,logical,numeric,numeric-method}
\alias{removeLinker,XStringSet,DNAString,missing,missing,missing-method}

\title{Remove linker sequences located at the start of short reads}

\description{If linkers are attached during sample preparation, it may
             be useful to remove the linkers' sequences after sequencing.
             This method finds and removes linker sequences that are
	     located at the start of the given reads.
}

\usage{
\S4method{removeLinker}{XStringSet,DNAString,logical,numeric,numeric}(reads, linker, removeReadsWithoutLinker, minOverlap, penalty)
}

\arguments{
  \item{reads}{A \code{DNAStringSet} instance that contains reads
  possibly having linkers at their start site}
  \item{linker}{A \code{DNAString} instance with the linker's sequence}
  \item{removeReadsWithoutLinker}{Whether reads without linkers should
  be removed. Default is FALSE}
  \item{minOverlap}{The minimal score that must be achived when aligning
  the linker. Default is length(linker)/2}
  \item{penalty}{The penalty for substitutions or indels. Default is 2}
}

\value{\code{removeLinker} returns a \code{DNAStringSet} with trimmed reads.}

\details{The best alignment of the linker within the start (length of
  linker + 5) of each given sequence is computed. The followong scoring
  schema is used: Each matching bases scores +1. Each substitution or
  indel scores the given penalty argument (default: penalty=2). There
  are no penalties for gaps and the end of the linker (overlap). An
  alignment is considered as match, if the scores is larger of equal to
  minOverlap (default: minOverlap=round(length(linker)/2)). In cases of
  a successful match, the subsequence from position 1 until the end of
  the linker's alignment is removed.  
}
  

\seealso{\code{\link{sequenceCaptureLinkers}}, \code{\link{DNAStringSet}}, 
  \code{\link{pairwiseAlignment}}
}

\examples{
    linker = sequenceCaptureLinkers()[[1]]
    reads = DNAStringSet(c(
        "CTCGAGAATTCTGGATCCTCAAA",
             "GAATTCTGGATCCTCAAA",
        "CTCGAGAAAAAAAAATCCTCAAA"))
    removeLinker(reads, linker)
}

\author{Hans-Ulrich Klein}
\keyword{removeLinker}

\name{getAlignedReads}
\alias{getAlignedReads}
\alias{getAlignedReads,AVASet-method}

\title{
  Import reads from an Amplicon Variant Analyzer project
}
\description{
  For a given AVASet, this function imports all aligned reads belonging to all (or some selected) amplicons of all samples.
}
\usage{
getAlignedReads(object, amplicons, dir)
}

\arguments{
  \item{object}{An instance of the \code{link{AVASet-class}}.}
  \item{amplicons}{An (optional) character vector of amplicon names as
    mentioned in the amplicon feature data (see \code{\link{fDataAmp}}).}
  \item{dir}{Usually, the method tries to retrieve the path to the AVA
    project from the given \code{link{AVASet}} object. However, if it
    fails to find the directory, \code{dir} can be used to set
    the root directory of the AVA project.}
}

\details{
  This function reports all reads for all samples together. If you want to get the reads for some samples individually, try subsetting your 
  AVASet as in the examples below.
}

\value{
  One \code{\link{DNAStringSet}} that contains all aligned reads for all samples (eventually restricted to some given amplicons).
}

\author{Christoph Bartenhagen}

\seealso{
  \code{\link{AVASet-class}},
  \code{\link{fDataAmp}}
}

\examples{

# load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
data(avaSetExample)

# import all reads for amplicon "TET2_E11.04" of the first sample
avaProjectDir = system.file("extdata", "AVASet", package = "R453Plus1Toolbox")
alnReads = getAlignedReads(avaSetExample[, 1], dir=avaProjectDir, amplicons="TET2_E11.04")
show(alnReads)

}

\name{plotVariants}

\alias{plotVariants}
\alias{plotVariants,AnnotatedVariants,character-method}
\alias{plotVariants,data.frame,character-method}

\title{Plots variant positions}

\description{This function illustrates the positions and types of mutations within a given gene and transcript. The plot shows only coding regions (thus, units are amino acids / codons). The coding 
region is further divided into exons labeled with their rank in the transcript. Transcript annotation is obtained from the ensembl GRCh37 server.}

\usage{plotVariants(data, gene, transcript, regions, mutationInfo, groupBy, horiz=FALSE, cex=1, title="", legend=TRUE)}

\arguments{
  \item{data}{This can be either a data.frame or an instance of class
  \code{annotatedVariants} you get by calling
  \code{\link{annotateVariants}}. A data frame requires the columns
  "label", "pos" "mutation" and "color" specifying an annotation for each
  mutation, its position (a single numeric value), the mutation type and (optionally) the color of
  the mutation (e.g. depicting a certain group identity independent from
  the mutation type). The position needs to be given as amino acids / codons.}
  \item{gene}{A string containing the Ensembl id of the gene of interest
  (required).}
  \item{transcript}{A string containing an Ensembl transcript id
  (optional, but recommended). If no Ensembl transcript-id is passed, it
  is chosen automatically and the function will return an appropriate data frame with all annotated
  transcripts for the given gene.}
  \item{regions}{A data frame having columns "name", "start", "end" and
  "color". The plot will highlight these regions with the given colors
  and print their name in the legend.}
  \item{mutationInfo}{A data frame with annotations for the different
  mutations occurring in the data (optional but recommended). It requires
  the columns "mutation", "legend" and "color". The first column must
  list the exact mutation names that occur in the data column
  "mutation". The column "legend" allows for a more detailed name of the
  mutation that will appear in the legend. The color of each mutation
  type is optional and can also be assigned automatically.}
  \item{groupBy}{By default, the mutations will be grouped by their
  position (i.e. the column "pos"). If necessary, one might give the name of an other column
  in the \code{\link{data}} here; for example the mutation labels. Please see
  the details below for more information.}
  \item{horiz}{In more comprehensive datasets, more than one
  mutation may be listed for a single position. If  \code{horiz=FALSE}, these
  overlapping mutations will be aligned vertically. If
  \code{horiz=TRUE} they will be aligned horizontally in groups allowing
  a label for every single mutation.}
  \item{cex}{A numeric value \code{> 0} giving the factor by which the
  labels are magnified relative to the default text size. If the width
  of the device is too small for all mutation labels, their size will be
  scaled automatically until it fits.}
  \item{title}{A title for the plot (optional).}
  \item{legend}{A logical value (TRUE/FALSE) whether to plot a legend
  for the mutations types (see \code{mutationInfo}) and the
  highlighted regions (if any).}
}

\details{The plot will show the coding part of the gene and its exons as
x-axis at the bottom. Mutations will be marked and ordered above according to
their genomic position. The axis units are amino acids / codons (hence, all given
genomic positions should be divided by three if necessary). \cr
Passing a data frame to this function allows a much more individual and detailed
annotation of the mutations like labels, colors and user defined
mutation types. \cr
Passing an instance of class \code{\link{annotatedVariants}} is useful for integration into the
R453Plus1Toolbox pipeline and for compatibility to older versions of the
plot. Then, it will only distinguish deletions and missense, nonsense
and silent mutations. \cr
By default, the plot will group mutations (horizontally or vertically) by their position. It is
possible to group by an other column in the data (see parameter
groupBy), but in the current version this makes
only sense if the mutations in one group are locally clustered, i.e. have
the same or a similar position. The parameter groupBy is
mainly useful to modify or even disable the automatic grouping of different mutations at the same position.
}

\note{Depending on the amount of mutations and the size of the gene, the plot
may not fit into the device or the text may become too small. It is
recommended to carefully select the right size of your device
before starting this function to ensure a well scaled and beautiful
plot. \cr
This function requires the package \code{TeachingDemos} to work, which
can be found at CRAN.}

\seealso{
\code{\link{annotateVariants}}.
}

\value{
  The function will return a data frame containing all Ensembl
  transcript information for the given gene ("ensembl_transcript_id", "rank", "cds_start", "cds_end"
  and "cds_length"). This data frame may prove useful for retrieving a
  Ensembl transcript id for future plots.
}
\examples{

# EXAMPLE 1: Working with intances of class annotatedVariants

# one missense, one nonsense point mutation and one deletion
variants = data.frame(
    row.names=c("missense", "deletion", "nonsense"), 
    start=c(106157528, 106157635, 106193892),
    end=c(106157528, 106157635, 106193892),
    chromosome=c("4", "4", "4"),
    strand=c("+", "+", "+"),
    seqRef=c("A", "G", "C"),
    seqMut=c("G", "-", "T"),
    seqSur=c("TACAGAA", "TAAGCAG", "CGGCGAA"),
    stringsAsFactors=FALSE)

# annotate variants with affected genes, exons and codons (may take a minute to finish)
\dontrun{varAnnot = annotateVariants(variants)}

# plot variants for gene TET2 having the Ensembl id "ENSG00000168769"
# when passing no transcript, the largest transcript annotated in the Ensembl database for this gene will be selected automatically
\dontrun{plotVariants(data=varAnnot, gene="ENSG00000168769", title="plotVariants Example", legend=TRUE)}

# EXAMPLE 2: Working with a data frame

# two missense at one position, one nonsense point mutation and one deletion
# it is possible to assign a color to every single mutation independently from its type
variants = data.frame(
    label=c("A>G","A>G(2)","delG","C>T"),
    pos=c(831,831,867,1437),
    mutation=c("M","M","D","N"),
    color=c("black", "black", "green", "red"),
    stringsAsFactors=FALSE
)

# more detailed names for mutation abbreviations can be passed as mutationInfo
# this is useful for the legend, but can also be generated automatically
mutationInfo = data.frame(
    mutation=c("M","D","S","N"),
    legend=c("Missense","Nonsense","Silent","Deletion"),
    stringsAsFactors=FALSE
)

# regions of interest can be highlighted using the regions parameter
regions = data.frame(
    name = c("region1", "region2"),
    start = c(700, 1400),
    end = c(1000, 1900),
    color = c("red", "blue")
)

# using the horiz parameter, multiple mutations occurring at the same place can be either aligned ...
# ... vertically
\dontrun{plotVariants(data=variants, gene="ENSG00000168769", transcript="ENST00000513237", regions=regions, mutationInfo=mutationInfo, horiz=FALSE, title="'plotVariants' Example", legend=TRUE)}

# ... or horizontally in groups
\dontrun{plotVariants(data=variants, gene="ENSG00000168769", transcript="ENST00000513237", regions=regions, mutationInfo=mutationInfo, horiz=TRUE, title="'plotVariants' Example", legend=TRUE)}

# group mutations by their label and not by their position (which is the default)
\dontrun{plotVariants(data=variants, gene="ENSG00000168769", transcript="ENST00000513237", regions=regions, mutationInfo=mutationInfo, groupBy="label", horiz=TRUE, title="'plotVariants' Example", legend=TRUE)}

}

\author{Christoph Bartenhagen}

\name{writeSFF}

\alias{writeSFF}

\title{Function To Write Files In Roche's .sff Format}

\description{
    This function takes an object of class \code{\link{SFFContainer-class}} and writes its contents into
    a file in Roche's Standard Flowgram Format (SFF) with the given filename.
}

\usage{writeSFF(sffContainer, filename)}

\arguments{
  \item{sffContainer}{An \code{\link{SFFContainer-class}} object.}
  \item{filename}{The name of the file to write into.}
}

\seealso{
  \code{\link{readSFF}}, \code{\linkS4class{SFFContainer}}.
}

\examples{
  file <- system.file("extdata", "SFF", "example.sff", package="R453Plus1Toolbox")
  sffContainer <- readSFF(file)
  sffContainer2 <- sffContainer[1:5] 
  \dontrun{writeSFF(sffContainer2, "output.sff")}
}

\author{Christian Ruckert}

\name{positionQualityBoxplot}

\docType{methods}

\alias{positionQualityBoxplot}
\alias{positionQualityBoxplot,QualityScaledDNAStringSet-method}
\alias{positionQualityBoxplot,ShortReadQ-method}
\alias{positionQualityBoxplot,SFFContainer-method}

\title{Boxplot Of The Quality For Each Position}

\description{
  Creates a boxplot of the quality scores over all sequences at each position.
}

\usage{
  positionQualityBoxplot(object, range, binsize=10, 
    xlab=paste("Read position in bp (Bin size: ", binsize, "bp)", sep=""), ylab="Quality score",
    col="firebrick1", ...)
}

\arguments{
  \item{object}{An object of class \link{QualityScaledDNAStringSet}, \link{ShortReadQ} or 
    \link{SFFContainer}.}
  \item{range}{A numeric vector of length one or two. If length one only bases from the first until 
    this position are plotted. If two all bases between these two positions are plotted.}
  \item{binsize}{Number of positions to summarize in one box in the plot.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{col}{The plotting color.}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{referenceSequences}

\alias{referenceSequences}

\title{Access the reference sequences of an AVASet}

\description{
    This function give access to a slot of an instance of the AVASet storing information about all reference sequences of the 
    amplicons.
}

\usage{referenceSequences(object)}

\arguments{
  \item{object}{An \code{link{AVASet-class}} object.}
}

\value{
    The data is stored in an object of class \code{AlignedRead} and thus gives information about all reference sequences and 
    their position on a chromosome (if \code{\link{alignShortReads}} has been called before).
}

\seealso{
    \code{\link{alignShortReads}}
}

\examples{

    # load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
    data(avaSetExample)

    referenceSequences(avaSetExample)

}

\author{Christoph Bartenhagen}

\name{getAminoAbbr}
\alias{getAminoAbbr}
\title{Get amino acid abbreviations}
\description{This function returns a table with amino acid names as first column and the
  according abbreviations as row names.}
\usage{getAminoAbbr()}

\name{avaSetFiltered}
\alias{avaSetFiltered}
\docType{data}
\title{
    Amplicon Variant Analyzer data import
}
\description{
    This is an example of an \code{link{AVASet-class}} object containing the output of Roche's Amplicon Variant Analyzer Software.
    It consists of 6 samples, 4 amplicons and 4 variants. The variants were previously filtered according to the amplicon coverage 
    (see \code{\link{setVariantFilter}} for details about filtering an AVASet object).
}
\usage{data(avaSetFiltered)}

\format{
    Formal class 'AVASet'
}
\source{
    \sQuote{Next-generation sequencing technology reveals a characteristic
    pattern of molecular mutations in 72.8% of chronic myelomonocytic 
    leukemia by detecting frequent alterations in TET2, CBL, RAS, and RUNX1} 
    (Kohlmann A et al., J Clin Oncol. 2010 Aug 20;28(24):3858-65. Epub 2010 Jul 19)
}

\examples{
    data(avaSetFiltered)
    avaSetFiltered
}
\keyword{datasets}

\name{Breakpoints-class}
\Rdversion{1.1}
\docType{class}
\alias{Breakpoints-class}
\alias{alignedReadsC1<-}
\alias{alignedReadsC1<-,Breakpoints,list-method}
\alias{alignedReadsC1}
\alias{alignedReadsC1,Breakpoints-method}
\alias{alignedReadsC2<-}
\alias{alignedReadsC2<-,Breakpoints,list-method}
\alias{alignedReadsC2}
\alias{alignedReadsC2,Breakpoints-method}
\alias{[,Breakpoints,ANY,ANY-method}
\alias{[,Breakpoints,ANY,ANY,ANY-method}
\alias{commonAlignC1<-}
\alias{commonAlignC1<-,Breakpoints,list-method}
\alias{commonAlignC1}
\alias{commonAlignC1,Breakpoints-method}
\alias{commonAlignC2<-}
\alias{commonAlignC2<-,Breakpoints,list-method}
\alias{commonAlignC2}
\alias{commonAlignC2,Breakpoints-method}
\alias{commonBpsC1<-}
\alias{commonBpsC1<-,Breakpoints,list-method}
\alias{commonBpsC1}
\alias{commonBpsC1,Breakpoints-method}
\alias{commonBpsC2<-}
\alias{commonBpsC2<-,Breakpoints,list-method}
\alias{commonBpsC2}
\alias{commonBpsC2,Breakpoints-method}
\alias{length,Breakpoints-method}
\alias{mergeBreakpoints,Breakpoints,missing,list-method}
\alias{mergeBreakpoints,Breakpoints,missing,missing-method}
\alias{mergeBreakpoints,Breakpoints,numeric,missing-method}
\alias{names<-,Breakpoints,ANY-method}
\alias{names,Breakpoints-method}
\alias{plotChimericReads,Breakpoints-method}
\alias{seqsC1<-}
\alias{seqsC1<-,Breakpoints,list-method}
\alias{seqsC1}
\alias{seqsC1,Breakpoints-method}
\alias{seqsC2<-}
\alias{seqsC2<-,Breakpoints,list-method}
\alias{seqsC2}
\alias{seqsC2,Breakpoints-method}

\title{Class "Breakpoints"}
\description{
Container to store chimeric reads that were clustered to putative
breakpoints indicating structural variants. Related information like
breakpoint position or alignment information about the chimeric reads
is stored as well.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("Breakpoints", ...)}.
Usually, objects will be created by calling the
\code{\link{detectBreakpoints}} method. It is not intended that users
create objects of this class manually.

All slots of this class can be found twice. One slot name ends with
\dQuote{C1} and the other \dQuote{C2}. The slots labeled with
\dQuote{C2} are empty until \code{\link{mergeBreakpoints}} has been
called and contain information about putativly associated breakpoints
detected by \code{\link{mergeBreakpoints}}.
}
\section{Slots}{
  \describe{
    \item{\code{seqsC1}:}{Object of class \code{"list"} with one data
      frame for each breakpoint. The data frame stores all chimeric reads
      covering the first breakpoint together with the alignment
      information.}
    \item{\code{seqsC2}:}{Object of class \code{"list"} with one data
      frame for each breakpoint. The data frame stores all chimeric reads
      covering the second breakpoint together with the alignment
      information.}
    \item{\code{commonBpsC1}:}{Object of class \code{"list"} with one
      data frame for each breakpoint. The data frame stores the
      consensus breakpoint sequence as well as the breakpoint
      coordinates of the first breakpoint.}
    \item{\code{commonBpsC2}:}{Object of class \code{"list"} with one
      data frame for each breakpoint. The data frame stores the
      consensus breakpoint sequence as well as the breakpoint
      coordinates of the second breakpoint.}
    \item{\code{commonAlignC1}:}{Object of class \code{"list"} with one
      object of class \code{\link{PairwiseAlignmentsSingleSubject-class}}
      for each breakpoint storing the alignments of the chimeric
      reads against the consensus breakpoint sequence for the
      first breakpoint.}
    \item{\code{commonAlignC2}:}{Object of class \code{"list"} with one
      object of class \code{\link{PairwiseAlignmentsSingleSubject-class}}
      for each breakpoint storing the alignments of the chimeric
      reads against the consensus breakpoint sequence for the
      second breakpoint.}
    \item{\code{alignedReadsC1}:}{Object of class \code{"list"} with one
      object of class \code{\link{AlignedRead-class}} storing all
      chimeric reads covering the first breakpoint and their alignments.}
    \item{\code{alignedReadsC2}:}{Object of class \code{"list"} with one
      object of class \code{\link{AlignedRead-class}} storing all
      chimeric reads covering the second breakpoint and their alignments.}
  }
}
\section{Methods}{
  \describe{
    \item{alignedReadsC1<-}{\code{signature(object = "Breakpoints",
	value = "list")}: 
      Setter-method for the alignedReadsC1 slot.}
    \item{alignedReadsC1}{\code{signature(object = "Breakpoints")}:
      Getter-method for the alignedReadsC1 slot.}
    \item{alignedReadsC2<-}{\code{signature(object = "Breakpoints",
	value = "list")}:
      Setter-method for the alignedReadsC2 slot.}
    \item{alignedReadsC2}{\code{signature(object = "Breakpoints")}:
      Getter-method for the alignedReadsC2 slot.}
    \item{commonAlignC1<-}{\code{signature(object = "Breakpoints",
	value =  "list")}: 
      Setter-method for the commonAlignC1 slot.}    
    \item{commonAlignC1}{\code{signature(object = "Breakpoints")}:
      Getter-method for the commonAlignC1 slot.}
    \item{commonAlignC2<-}{\code{signature(object = "Breakpoints",
	value = "list")}:
      Setter-method for the commonAlignC2 slot.}
    \item{commonAlignC2}{\code{signature(object = "Breakpoints")}:
      Getter-method for the commonAlignC2 slot.}
    \item{commonBpsC1<-}{\code{signature(object = "Breakpoints",
	value = "list")}:
      Setter-method for the commonBpsC1 slot.}
    \item{commonBpsC1}{\code{signature(object = "Breakpoints")}:
      Getter-method for the commonBpsC1 slot.}
    \item{commonBpsC2<-}{\code{signature(object = "Breakpoints",
	value = "list")}:
      Setter-method for the commonBpsC2 slot.}
    \item{commonBpsC2}{\code{signature(object = "Breakpoints")}:
      Getter-method for the commonBpsC2 slot.}
    \item{seqsC1<-}{\code{signature(object = "Breakpoints",
	value =	"list")}:
      Setter-method for the seqsC1 slot.}
    \item{seqsC1}{\code{signature(object = "Breakpoints")}:
      Getter-method for the seqsC1 slot.}
    \item{sqsC2<-}{\code{signature(object = "Breakpoints",
	value = "list")}:
      Setter-method for the seqsC2 slot.}
    \item{seqsC2}{\code{signature(object = "Breakpoints")}:
      Getter-method for the seqsC2 slot.}    
    \item{[}{\code{signature(x = "Breakpoints", i = "ANY", j = "ANY")}:
      Subsetting a Breakpoints object.}
    \item{length}{\code{signature(x = "Breakpoints")}:
      Returns the number of breakpoints stored.}
    \item{mergeBreakpoints}{\code{signature(breakpoints = "Breakpoints",
	maxDist = "missing", mergeBPs = "list")}:
      Merge presumably related breakpoints.}
    \item{mergeBreakpoints}{\code{signature(breakpoints = "Breakpoints",
	maxDist = "missing", mergeBPs = "missing")}:
      Merge presumably related breakpoints.}
    \item{mergeBreakpoints}{\code{signature(breakpoints = "Breakpoints",
	maxDist = "numeric", mergeBPs = "missing")}:
      Merge presumably related breakpoints.}
    \item{names<-}{\code{signature(x = "Breakpoints", value = "ANY")}:
      Set the names of the breakpoints.}
    \item{names}{\code{signature(x = "Breakpoints")}:
      Get the names of the breakpoints.}
    \item{plotChimericReads}{\code{signature(brpData = "Breakpoints")}:
      Plot the structural variant and the chimeric reads covering its
      breakpoints.}
    \item{summary}{\code{signature(object = "Breakpoints")}:
      Create a data frame summaring information about all breakpoints.}
    \item{table}{\code{signature(... = "Breakpoints")}:
      Create a frequency table of cluster sizes.}
  }
}

\author{
Hans-Ulrich Klein, Christoph Bartenhagen
}

\seealso{
\code{\link{filterChimericReads}}, \code{\link{detectBreakpoints}},
\code{\link{mergeBreakpoints}}, \code{\link{plotChimericReads}}

}

\keyword{classes}

\name{demultiplexReads}

\alias{demultiplexReads}
\alias{demultiplexReads,XStringSet,XStringSet,missing,logical-method}
\alias{demultiplexReads,XStringSet,XStringSet,missing,missing-method}
\alias{demultiplexReads,XStringSet,XStringSet,numeric,logical-method}
\alias{demultiplexReads,XStringSet,XStringSet,numeric,missing-method}


\title{Performs MID/Multiplex filtering}

\description{Roche's Genome Sequencer allows to load two or more samples
             on one region. To allocate sequences to samples, each sample has 
             a unique multiplex sequence. The multiplex sequence should
             be the prefix of all sequences from that sample. This method
             demultiplexes a given set of sequences according to the given
             multiplex sequences (MIDs).}

\usage{
\S4method{demultiplexReads}{XStringSet,XStringSet,numeric,logical}(reads, mids, numMismatches, trim)}

\arguments{
  \item{reads}{A \code{DNAStringSet} instance that contains reads starting 
               with MIDs}
  \item{mids}{A \code{DNAStringSet} instance that contains the MIDs}
  \item{numMismatches}{The maximal number of mismatches allowed, default 2.}
  \item{trim}{Whether the MIDs should be cutted-out, default TRUE}
}

\value{\code{demultiplexReads} returns a list with one \code{DNAStringSet} 
       instance for each MID.}

\details{All given MIDs must have the same length. The algorithm
  computes the number of mismachtes for each MID. The read is assigned
  to the MID with the lowest number of mismatches. If two or more MIDs
  have the same number of mismachtes, or if the number of mismachtes is
  greater than the given argument \code{numMismachtes}, the read is not
  assigned to any MID. The default number of allowed mismatches is 2.}

\seealso{\code{\link{genomeSequencerMIDs}}, \code{\link{DNAStringSet}}}

\examples{
	library(Biostrings)
    mids = genomeSequencerMIDs(c("MID1", "MID2", "MID3"))
    reads = DNAStringSet(c(
        paste(as.character(mids[["MID1"]]), "A", sep=""),
        paste(as.character(mids[["MID1"]]), "AA", sep=""),
        paste(as.character(mids[["MID2"]]), "AAA", sep="")))
    demultiplexReads(reads, mids)
}

\author{Hans-Ulrich Klein}
\keyword{demultiplex}

\name{filterChimericReads}

\alias{filterChimericReads}
\alias{filterChimericReads,list,IntegerRangesList,DNAString,missing,missing-method}
\alias{filterChimericReads,list,IntegerRangesList,DNAString,numeric,numeric-method}
\alias{filterChimericReads,list,IntegerRangesList,missing,missing,missing-method}
\alias{filterChimericReads,list,IntegerRangesList,missing,numeric,numeric-method}
\alias{filterChimericReads,list,missing,DNAString,missing,missing-method}
\alias{filterChimericReads,list,missing,DNAString,numeric,numeric-method}
\alias{filterChimericReads,list,missing,missing,missing,missing-method}
\alias{filterChimericReads,list,missing,missing,numeric,numeric-method}


\title{Extract chimeric reads and apply filtering steps to remove artificial
  chimeric reads.}

\description{Chimeric reads may be caused by sequencing a chromosomal
  aberration or by technical issues during sample preparation. This
  method implements several filter steps to remove false chimeric
  reads.}

\usage{
  \S4method{filterChimericReads}{list,IntegerRangesList,DNAString,numeric,numeric}(alnReads, targetRegion, linkerSeq, minDist, dupReadDist)
}

\arguments{
  \item{alnReads}{A list storing the aligned reads as produced by the function \link{scanBam}.}
  \item{targetRegion}{A object of class IRangesList containing the target region
  of e.g. a used capture array. The parameter may be omitted in case of
  a non targeted sequencing approach.}
  \item{linkerSeq}{A linker sequence that was used during sample
    preparation. It may be omitted.}
  \item{minDist}{The minimum distance between two local alignments (see details), default 1000}
  \item{dupReadDist}{The maximum distance between the 5 prime start
  position of two duplicated reads (see details), default 1.}
}

\value{A \code{list} containing only filtered chimeric reads. The \code{list}
  has the same structure like the given argument
  \code{alnReads}. Additionally, one element \dQuote{log} with logging
  information of each filtering step is added.}

\details{The following filter steps are performed:
  
  1. All chimeric reads with exactly two local alignments are
  extracted. Reads with more than two local alignments are discarded.
  
  2. If the targetRegion argument is given, chimeric reads must have
  one local alignment at least overlapping the the target region. If
  both local alignments are outside the target region, the read is
  discarded.

  3. If the linkerSeq argument is given, all chimeric reads that have
  the linker sequence between their local alignments are removed. When
  searching the linker sequence, 4 mismatches or indels are allowed and
  the linker sequence must not start or end within the first or last ten
  bases of the read. The function searches for the linkerSeq and for it's
  reverse complement.

  4. Two local alignment of a read must have minDist reads between the
  alignments (if both alignment are on the same chromosome). Otherwise,
  the read seems to span a deletion and not a chromosomal aberration and
  is discarded.

  5. Duplicated reads are removed. Two reads are duplicated, if the lie
  on the same strand and have the same 5 prime start position. Due to
  sequencing and alignment errors, the start position may vary for a
  maximum of dupReadDist bases. In case of duplicated reads, only the longest
  read is kept.

  Reads passing all filtering steps are returned in the \code{list}
  structure as given by the \code{alnReads} argument (as derived from
  the \code{scanBam} method). A data frame with information about the
  number of reads that passed each filter is added to the list.
}

\author{Hans-Ulrich Klein}

\seealso{\code{\link{detectBreakpoints}},
  \code{\link{mergeBreakpoints}}, 
  \code{\link{Breakpoints-class}},
  \code{\link{scanBam}}, \code{\link{sequenceCaptureLinkers}}}

\examples{
library(Rsamtools)
bamFile = system.file("extdata", "SVDetection", "bam", "N01.bam", package="R453Plus1Toolbox")
bam = scanBam(bamFile)
data(captureArray)
linker = sequenceCaptureLinkers("gSel3")[[1]]
filterReads = filterChimericReads(bam, targetRegion=captureArray, linkerSeq=linker)
}
    
\keyword{filterChimericReads}

\name{qualityReportSFF}

\alias{qualityReportSFF}

\title{Function To Create A Quality Report In PDF Format}

\description{
  This function takes a character vector consisting of filenames pointing to files in Roche's SFF format as input
  and creates a quality report in PDF format as output.
}

\usage{qualityReportSFF(sfffiles, outfile)}

\arguments{
  \item{sfffiles}{A character vector of the SFF files to read in.}
  \item{outfile}{The name of the PDF report file created. Defaults to \sQuote{qcreport.pdf} in the current directory.}
}

\details{
  The function uses the qualityReport.Rnw file from the extdata directory of the package and \code{Sweave}
  to create a .tex file which is afterwards converted to .pdf format. In the .Rnw file the following quality
  control functions are used: \code{\link{readLengthStats}}, \code{\link{readLengthHist}}, 
  \code{\link{baseQualityStats}}, \code{\link{baseQualityHist}}, \code{\link{sequenceQualityHist}}, 
  \code{\link{positionQualityBoxplot}}, \code{\link{baseFrequency}}, \code{\link{nucleotideCharts}}, 
  \code{\link{gcContent}}, \code{\link{gcPerPosition}}, \code{\link{gcContentHist}}, 
  \code{\link{complexity.dust}}, \code{\link{complexity.entropy}}, \code{\link{dinucleotideOddsRatio}}.
}

\examples{
  \dontrun{
    file <- system.file("extdata", "SFF", "example.sff", package="R453Plus1Toolbox")
    qualityReportSFF(file, "QualityReport.pdf")
  }
}

\author{Christian Ruckert}

\name{MapperSet-class}
\Rdversion{1.1}
\docType{class}
\alias{MapperSet-class}
\alias{annotateVariants,MapperSet-method}
\alias{htmlReport,MapperSet-method}
\alias{setVariantFilter,MapperSet-method}
\alias{getVariantPercentages,MapperSet-method}
\alias{getReadStatus}
\alias{getReadStatus,MapperSet-method}

\title{Class to Contain GS Reference Mapper Output}
\description{	 
    Container to store data imported from a project of Roche's GS Reference Mapper Software.
    It stores all information into a Biobase ExpressionSet.
}
\section{Objects from the Class}{
    Objects can be created by calls of the form \code{MapperSet(filename)}. 
    While filename is a vector containing all sample directories (i.e. directories that contain the files "mapping/454HCDiffs.txt" and
    "mapping/454NewblerMetrics.txt").
}
\section{Slots}{
  \describe{
    \item{\code{assayData}:}{Object of class \code{AssayData}. Contains the number of reads with the respective difference and the total coverage 
	for every variant in forward and reverse direction.}
    \item{\code{featureData}:}{Object of class \code{AnnotatedDataFrame}. Contains information about the type, location and reference of each
        variant. If available, it shows further Ensembl variant-ids for known SNPs.}
    \item{\code{phenoData}:}{Object of class \code{AnnotatedDataFrame}. By default, the phenoData contains the accession number of every sample.}
    \item{\code{variantFilterPerc}:}{Object of class \code{numeric}. Contains a threshold to display only those variants, whose
        coverage (in percent) in forward and reverse direction in at least one sample is higher than this filter value. See
        \code{\link{setVariantFilter}} for details about setting this value.}
    \item{\code{variantFilter}:}{Object of class \code{character}. Contains a vector of variant names whose
        coverage (in percent) in forward and reverse direction in at least one sample is higher than the filter value(s) in
        \code{variantFilterPerc}.}
    \item{\code{dirs}:}{Object of class \code{character}. Based on a directory given at instantiation of the object, it contains a vector of several 
        directories containing all relevant GS Mapper project files.}
    \item{\code{experimentData}:}{Object of class \code{MIAME}. Contains details of the experiment.}
    \item{\code{annotation}:}{Object of class \code{character} Label associated with the annotation package used in the experiment.}
    \item{\code{protocolData}:}{Object of class \code{AnnotatedDataFrame}. Contains additional information about the samples.}
    \item{\code{.__classVersion__}:}{Object of class \code{Versions}. Remembers the R and R453Toolbox version numbers used to created the
        MapperSet instance.}
  }
}
\section{Extends}{
    Class \code{\linkS4class{eSet}}, directly.
    Class \code{\linkS4class{VersionedBiobase}}, by class "eSet", distance 2.
    Class \code{\linkS4class{Versioned}}, by class "eSet", distance 3.
}
\section{Methods}{
  \describe{
    \item{setVariantFilter(object):}{Sets the filter to display only those variants, whose coverage (in percent) in forward and reverse
        direction in at least one sample is higher than the given value.}
    \item{getVariantPercentages(object)}{Computes the coverage for every variant over all reads (forward and/or reverse) and for each
        sample.}
    \item{annotateVariants(object):}{Annotates given genomic variants. See \code{\link{annotateVariants}} for details.}
    \item{htmlReport(object):}{Exports all (filtered) variant data into a html report. See \code{\link{htmlReport}} for details}
    \item{getReadStatus(object):}{Reads the file "454ReadStatus.txt" in the GSM project directory which contains information about the 
        alignment of each read (chr, pos, strand, etc.)and returns its content in a dataframe.}
	 }
}
\author{Christoph Bartenhagen}

\seealso{
    \code{\link{AVASet}},
    \code{\link{annotateVariants}},
    \code{\link{htmlReport}},
    \code{\link{setVariantFilter}},
    \code{\link{getVariantPercentages}}

}

\examples{

# sum up class structure
showClass("MapperSet")

# load a GS Mapper dataset containing 3 samples and 111 variants
data(mapperSetExample)
mapperSetExample

# show contents of assay, feature and pheno data
assayData(mapperSetExample)
fData(mapperSetExample)
pData(mapperSetExample)
}

\keyword{classes}

\name{dinucleotideOddsRatio}

\docType{methods}

\alias{dinucleotideOddsRatio}
\alias{dinucleotideOddsRatio,DNAStringSet-method}
\alias{dinucleotideOddsRatio,ShortRead-method}
\alias{dinucleotideOddsRatio,SFFContainer-method}

\title{Dinucleotide Odds Ratio}

\description{
  This function calculates the dinucleotide odds ratio for each of the sixtheen possible 
  dinucleotides.
}

\usage{
  dinucleotideOddsRatio(object, xlab="Under-/over-representation of dinucleotides", 
    col="firebrick1", ...)
}

\arguments{
  \item{object}{An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.}
  \item{xlab}{The X axis label.}
  \item{col}{The plotting color.}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\details{
  The dinucleotide odds ratio assigns a value between 0 and 2 to each of the sixtheen possible 
  dinucleotides (AA, AC, AG, AT, CA, CC, CG, CT, GA, GC, GG, GT, TA, TC, TG, TT). For values below 1
  the dinucleotide is under-represented compared to the randomly expected frequency of this 
  dinucleotide in a sequence of the given length and with the given frequencies of the four 
  nucleotides (A, C, G, T). For values above 1 this dinucleotide is over-represented.
}

\value{
  A matrix with sixtheen columns, one for each dinucleotide, containing the dinucleotide odds ratio 
  values for each sequence in a seperate row.
}

\references{
  Schmieder R. (2011) Quality control and preprocessing of metagenomic datasets.
  \emph{Bioinformatics}, 2011 Mar 15;27(6):863-4. 
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{sff2fastq}

\docType{methods}

\alias{sff2fastq}
\alias{sff2fastq,SFFContainer-method}

\title{Write A SFFContainer Object To A FASTQ File}

\description{
  This function takes a \link{SFFContainer} object and writes it to a file in FASTQ format.
}

\usage{
  sff2fastq(x, outdir, fname)
}

\arguments{
  \item{x}{An object of class \link{SFFContainer}.}
  \item{outdir}{The directory where the file should be stored, defaults to the current working 
    directory.}
  \item{fname}{The name of the file to write. Defaults to the filename slot of the 
    \link{SFFContainer}, with .sff substituted with .fastq.}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{plotAmpliconCoverage}

\alias{plotAmpliconCoverage}
\alias{plotAmpliconCoverage,AVASet,character,logical-method}
\alias{plotAmpliconCoverage,AVASet,character,missing-method}
\alias{plotAmpliconCoverage,AVASet,missing,logical-method}
\alias{plotAmpliconCoverage,AVASet,missing,missing-method}

\title{Creates a plot visualizing the number of reads per amplicon}

\description{A function for visualizing the number of reads per amplicon
or per MID / pico titer plate.}


\usage{
\S4method{plotAmpliconCoverage}{AVASet,character,logical}(avaSet, type="amplicon", bothDirections=TRUE, cex.names=0.8, cex.axis=0.8, las=3, col=c(rgb(217/255, 214/255, 209/255), rgb(173/255, 38/255, 36/255)), ...)}

\arguments{
  \item{avaSet}{An instance of AVASet.}
  \item{type}{A character vector specifying the type of plot.}
  \item{bothDirections}{A logical value determining whether the plot
  sums forward and reverse reads or shows them separately.}
  \item{cex.names}{Font size of the amplicon name labels.}
  \item{cex.axis}{Font size of axes' labels.}
  \item{las}{Orientation of amplicon name labels.}
  \item{col}{Colors used in the plot.}
  \item{\dots}{Arguments to be passed to methods, such as graphical
  parameters (see \sQuote{par}).}
}

\details{
If the argumnet type is \dQuote{amplicon}, the number of reads for each
amplicon are visualized. In case of a AVASet with one sample, a
barplot with one bar for each amplicon is created. In case of more than
one sample, a boxplot with one box for each amplicon is plotted. If type
is \dQuote{mid}, a boxplot with one box for each MID is created. If type 
is \dQuote{ptp}, a boxplot with one box for each pico titer plate is 
created.}

\seealso{\code{\link{AVASet}}}

\examples{
\dontrun{data(avaSetExample)
plotAmpliconCoverage(avaSetExample)
plotAmpliconCoverage(avaSetExample[,1])}
}

\author{Hans-Ulrich Klein}

\keyword{amplicon coverage}

\name{gcContentHist}

\docType{methods}

\alias{gcContentHist}
\alias{gcContentHist,DNAStringSet-method}
\alias{gcContentHist,ShortRead-method}
\alias{gcContentHist,SFFContainer-method}

\title{GC-Content Histogram}

\description{
  This function creates a histogram of the relative GC-content per sequence.
}

\usage{
  gcContentHist(object, xlab="GC content", ylab="Number of sequences", col="firebrick1", 
    breaks=50, ...)
}

\arguments{
  \item{object}{An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{col}{The plotting color.}
  \item{breaks}{The number of breaks in the histogram (see \sQuote{hist}).}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{flowgramBarplot}

\docType{methods}

\alias{flowgramBarplot}
\alias{flowgramBarplot,SFFRead-method}

\title{Create A Barplot Of The Flow Intensities}

\description{
  This function creates a barplot of the flow intensities with one bar
  for each nucleotide in the flow. With the height giving the measured intensity.
}

\usage{
  flowgramBarplot(x, range=c(0, length(flowgram(x))), xlab="Flow sequence", 
    ylab="Flow intensity", col=c(A="black", C="red", G="blue", T="green"), ...)
}

\arguments{
  \item{x}{An object of class \code{\link{SFFRead}}.}
  \item{range}{Two positions between which the flows should be plotted.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{col}{The colors in which the four nucleotids should be plotted.}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{avaSetFiltered_annot}
\alias{avaSetFiltered_annot}
\docType{data}
\title{
    AVASet variant annotations
}
\description{
    These are example annotations for 4 variants of an AVASet (try \code{data(avaSetFiltered)} to retrieve the corresponding \code{link{AVASet-class}} object). 
    The annotations include affected genes, exons and codons as well as resulting amino acid changes and dbSNP identifiers (if the mutation is already known).
}
\usage{data(avaSetFiltered_annot)}

\format{
    Formal class 'AnnotatedVariants'
}
\source{
    \sQuote{Next-generation sequencing technology reveals a characteristic
    pattern of molecular mutations in 72.8\% of chronic myelomonocytic 
    leukemia by detecting frequent alterations in TET2, CBL, RAS, and RUNX1} 
    (Kohlmann A et al., J Clin Oncol. 2010 Aug 20;28(24):3858-65. Epub 2010 Jul 19)
}

\examples{
    data(avaSetFiltered_annot)
}
\keyword{datasets}

\name{plotChimericReads}

\alias{plotChimericReads}

\title{Plots chimeric reads}

\description{This function plots a given set of aligned chimeric reads along a reference sequence. It plots the breakpoints of translations or 
 	inversions and marks deletions, insertions and mismatches. Optionally, it displays all base pairs in a given region around the breakpoint.}

\usage{plotChimericReads(brpData, geneSymbols=FALSE, plotMut=TRUE, plotBasePairs=FALSE, maxBasePairs=50, legend=FALSE, title="", 
    col=c("red", "green", "black", "orange"))}

\arguments{
  \item{brpData}{A \code{Breakpoints} object containing the consensus breakpoint of all reads and the consensus reference sequence as returned by the methods 
	\code{\link{detectBreakpoints}} and \code{\link{mergeBreakpoints}}. Since only one plot is made, the
	function will only work for objects of class \code{Breakpoints} having length one.}
  \item{geneSymbols}{Boolean value whether to automatically load and plot the gene symbols from the Ensembl database. Additionally,
	\code{geneSymbols} can be a vector of two strings for an own annotation.}
  \item{plotMut}{Boolean value whether to mark deletions, insertions and mismatches.}
  \item{plotBasePairs}{Optionally, \code{plotChimericReads} displays all base pairs in a given region around the breakpoint (see maxBasePairs).}  
  \item{maxBasePairs}{The maximum number of base pairs to be plotted. Only used in conjunction with \code{plotBasePairs=TRUE}.}
  \item{legend}{A logical value (TRUE/FALSE) whether to plot a legend that explains the colouration of the insertions, deletions, mismatches and 
	breakpoints.}
  \item{title}{A title for the plot.}
  \item{col}{A vector of four colours to draw insertions, deletions, mismatches and breakpoints. In this order, the default colours are "red", 
	"green", "black" and "orange" (use \code{colours()} to see a list of possible values).}
}

\details{
  This method is intended to be run after the pipeline for structural
  variant detection. Therefore, see the methods
  \code{\link{filterChimericReads}}, \code{\link{detectBreakpoints}} and
  \code{\link{mergeBreakpoints}} to correctly preprocess your alignment
  before running \code{plotChimericReads}.
}

\note{
    It is recommended to first create and resize the output device (e.g. the plotting window or a pdf file) before plotting. For example, on Unix 
    systems you may try \code{X11(width=w, height=h)} or \code{pdf(file="plotChimericReads.pdf", width=w, height=h)} for some window width w (e.g. 
    w=12) and window height h (e.g. h=6).
}

\seealso{
  \code{\link{Breakpoints-class}},  
  \code{\link{detectBreakpoints}},
  \code{\link{mergeBreakpoints}}
}

\examples{
# load breakpoint data containing twelve chimeric reads describing an inversion in chromosome 16
data("breakpoints")
breakpoints

# standard plot 
# (only arrangement of reads plotted; breakpoints in orange, deletions
# in red, insertions in green and mismatches in black by default)
plotChimericReads(breakpoints)

# plot base pairs in the breakpoint region (+/- 32bp) 
\dontrun{plotChimericReads(breakpoints, plotBasePairs=TRUE, maxBasePairs=32)}

# use custom colours and display a legend:
# deletions="brown", insertions="blue", mismatches="yellow", breakpoints="gray"
plotChimericReads(breakpoints, col=c("brown", "blue", "yellow", "gray"), legend=TRUE)

}

\author{Christoph Bartenhagen}
\keyword{plotChimericReads, detectBreakpoints, mergeBreakpoints, Breakpoints}

\name{regions}
\alias{regions}
\docType{data}

\title{
        Example data for \code{\link{plotVariants}}
}
\description{
        This data.frame is part of the vignette example of the \code{\link{plotVariants}} function. It has the columns "name", "start", "end" and
  	"color". The plot will highlight these regions with the given colors and print their name in the legend.
}

\usage{data(plotVariantsExample)}

\format{
	data.frame
}

\examples{
   	data(plotVariantsExample)
	regions
}
\keyword{datasets}

\name{baseQualityStats}

\docType{methods}

\alias{baseQualityStats}
\alias{baseQualityStats,QualityScaledDNAStringSet-method}
\alias{baseQualityStats,ShortReadQ-method}
\alias{baseQualityStats,SFFContainer-method}

\title{Statistics Of Base Quality}

\description{
  This function returns mean, minimum, maximum and standard deviation of the base quality scores 
  over all sequences.
}

\usage{
  baseQualityStats(object)
}

\arguments{
  \item{object}{An object of class \link{QualityScaledDNAStringSet}, \link{ShortReadQ} or 
    \link{SFFContainer}.}
}

\value{
  A numeric vector with four slots: mean, min, max, sd.
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{gcContent}

\docType{methods}

\alias{gcContent}
\alias{gcContent,DNAStringSet-method}
\alias{gcContent,ShortRead-method}
\alias{gcContent,SFFContainer-method}

\title{Calculate The Overall GC-Content}

\description{
  This function calculates the GC-content summarized over all sequences.
}

\usage{
  gcContent(object)
}

\arguments{
  \item{object}{
    An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.
  }
}

\details{
  The GC-content is calculated as follows: \cr
    (#G + #C / #G + #C + #A + #T) * 100 \cr
  Where #G is the number of base G in all sequences.
}

\value{
  A numeric vector of length one containing the overall GC-content.
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{htmlReport}

\alias{htmlReport}

\title{HTML-Report Builder for the AVASet and MapperSet}

\description{
    This function creates a HTML variant and quality report for a given AVASet or MapperSet instance.
}

\usage{htmlReport(object, annot, blocks=c(), transcripts=c(), sampleCols, minMut=3, dir="HTMLReport", title="Summary")}

\arguments{
  \item{object}{An \code{\link{AVASet-class}} or \code{\link{MapperSet-class}} instance.}
  \item{annot}{An instance of class \code{AnnotatedVariants} you get
    by calling \code{\link{annotateVariants}}. 
    If no such argument is supplied the data will be read from the
    Ensembl database automatically for all variants.}
  \item{blocks}{Character vector of block names for each variant. The
    variants will then be structured into several blocks. If no such list is 
    supplied, the report consists of just one (big) table for all variants.}
  \item{transcripts}{Character vector containing Ensembl transcript-IDs
    that order the according entries on the transcript
    pages. Transcripts given in this argument will appear on top of the
    transcript page.}
  \item{sampleCols}{Character vector of column names of the sample data
    (phenoData) of the AVASet/MapperSet object to filter the sample output 
    on the transcript pages. All columns will be listed if no such argument is given.}
  \item{minMut}{If the value of minMut is greater than zero, the report
    lists only variants, whose coverage for at least one sample is
    higher than minMut (percentage between 0 and 100).}
  \item{dir}{Character with the desired output directory. By defaultm
    the directory "HTMLReport" will be created in the current directory.}
  \item{title}{Heading for the first page with the variant information.}
}

\details{
  The report is structured into two (MapperSet) or three (AVASet)
  parts containing variant and quality information:
  \itemize{
    \item The main page sums up given variant information like the name,
      type, reference gene, position (see \code{\link{fData}},
      \code{\link{annotateVariants}}). \cr
      Using the argument \code{blocks}, the main page can be individually 
      structured by assigning a block name to each variant. \cr
      The main page can be further structured by samples. For a
      given AVASet object, every sample links to another short
      quality report showing only the amplicon coverage for this sample.
    \item Every variant on the main page links to a
      page with further details about the affected genes and transcripts
      (e.g. Ensembl gene-IDs, transcript-IDs, codon sequences, changes of
      amino acids (if coding)). 
    \item Only in case of AVASet object: A quality report shows
      the coverage of every amplicon in forward and/or reverse
      direction. Further plots display the coverage by MID and PTP
      (if this information is given in the pheno data of the object).
  }
}

\seealso{
    \code{\link{annotateVariants}}.
}

\examples{
# note: all examples save the report to the directory "htmlReportExample" in your current R working directory

# load a filtered AVA dataset containing 6 samples, 4 amplicons and 4 variants
# and its variant annotations
data("avaSetFiltered")
data("avaSetFiltered_annot")

# create a full report showing all (unfiltered) information
htmlReport(avaSetFiltered, avaSetFiltered_annot, dir="htmlReportExample", title="htmlReport Example", minMut=0)
# create a report that emphasizes on samples with variants covered by at least 50% of the reads 
htmlReport(avaSetFiltered, avaSetFiltered_annot, dir="htmlReportExample", title="htmlReport Example", minMut=50)

# create a report that is structured by the reference genes
library("ShortRead")
refs = sapply(fData(avaSetFiltered)$referenceSeq, function(x) 
    subset(pData(alignData(referenceSequences(avaSetFiltered))), pData(alignData(referenceSequences(avaSetFiltered)))$refSeqID == x)$name)
htmlReport(avaSetFiltered, avaSetFiltered_annot, dir="htmlReportExample", title="htmlReport Example", minMut=0, blocks=refs)

# create a report whose sample information only lists the sample ids
pData(avaSetFiltered)
sampleCols = "SampleID"
htmlReport(avaSetFiltered, avaSetFiltered_annot, dir="htmlReportExample", title="htmlReport Example", minMut=0, sampleCols=sampleCols)
}

\author{Christoph Bartenhagen, Hans-Ulrich Klein, Christian Ruckert}

\name{complexity.entropy}

\docType{methods}

\alias{complexity.entropy}
\alias{complexity.entropy,DNAStringSet-method}
\alias{complexity.entropy,ShortRead-method}
\alias{complexity.entropy,SFFContainer-method}

\title{Sequence Complexity Using The Shannon-Wiener Algorithm}

\description{
  This function evaluates the sequence complexity using the Shannon-Wiener Algorithm.
}

\usage{
  complexity.entropy(object, xlab="Complexity score (0=low, 100=high)", ylab="Number of sequences", 
    xlim=c(0, 100), col="firebrick1", breaks=100, ...)
}

\arguments{
  \item{object}{An object of class \link{DNAStringSet}, \link{ShortRead} or \link{SFFContainer}.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{xlim}{The limits of the X axis.}
  \item{col}{The plotting color.}
  \item{breaks}{The number of breaks in the histogram (see \sQuote{hist}).}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\details{
  The entropy approach evaluates the entropy of trinucleotides in a sequence. The entropy values are 
  scaled from 0 to 100 and lower entropy values imply lower complexity. A sequence of homopolymer 
  repeats (e.g. TTTTTTTTTT) has an entropy value of 0, of dinucleotide repeats (e.g. TATATATATA) has 
  an entropy value around 16, and of trinucleotide repeats (e.g. TAGTAGTAG) has an entropy value 
  around 26. Scores below 70 can be considered low-complexity.
}

\value{
  A numeric vector containing the complexity score for each sequence.
}

\references{
  Schmieder R. (2011) Quality control and preprocessing of metagenomic datasets.
  \emph{Bioinformatics}, 2011 Mar 15;27(6):863-4. 
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{readsOnTarget}
\alias{readsOnTarget}
\alias{readsOnTarget,list,GRanges-method}

\title{
Check for each read whether it aligns within the given region.
}

\description{
This methods checks (approximately) whether the given reads align within
a given region. 
}

\usage{
  readsOnTarget(alnReads, targetRegion) 
}

\arguments{
  \item{alnReads}{
  A list as returned by \code{scanBam} storing aligned reads.
}
  \item{targetRegion}{
  The target region as a \code{GRanges} object. The chromosome names must
  fit to the chromosome names used in the alignment information of the
  given reads.
}
}

\details{
The detailed alignment information given by the CIGAR strings in .bam
files are ignored by the function. Instead, it is assumed that the whole
read alignes to the reference without indels. This is often not true for
longer read (e.g. generated with Roche 454 Sequencing), but saves
computation time. Hence, this method is useful to approximate the number
of reads that align in the target region of a targeted sequencing experiment.
}


\value{A list with one logical vector for each list entry in alnReads. 
The logical vector indicates for each read whether it overlaps with at least
one base from any target region or not.}

\author{Hans-Ulrich Klein}

\seealso{
\code{\link{scanBam}}
}

\examples{
library(Rsamtools)
bamFile = system.file("extdata", "SVDetection", "bam", "N01.bam", package="R453Plus1Toolbox")
bam = scanBam(bamFile)
region = GRanges(IRanges(start=118307205, end=118395936), seqnames=11)
targetReads = readsOnTarget(bam, region)
sum(targetReads[[1]])
}


\keyword{readsOnTarget}

\name{AVASet-class}
\Rdversion{1.1}
\docType{class}
\alias{AVASet-class}
\alias{[,AVASet,ANY,ANY-method}
\alias{[,AVASet,ANY,ANY,ANY-method}
\alias{annotateVariants,AVASet-method}
\alias{assayDataAmp,AVASet-method}
\alias{assayDataAmp<-}
\alias{assayDataAmp<-,AVASet,AssayData-method}
\alias{fDataAmp,AVASet-method}
\alias{featureDataAmp,AVASet-method}
\alias{featureDataAmp<-}
\alias{featureDataAmp<-,AVASet,AnnotatedDataFrame-method}
\alias{htmlReport,AVASet-method}
\alias{alignShortReads,AVASet,DNAStringSet,character-method}
\alias{referenceSequences,AVASet-method}
\alias{referenceSequences<-}
\alias{referenceSequences<-,AVASet,AlignedRead-method}
\alias{setVariantFilter,AVASet-method}
\alias{getVariantPercentages,AVASet-method}

\title{Class to contain Amplicon Variant Analyzer Output}
\description{
    Container to store data imported from a project of Roche's Amplicon Variant Analyzer Software.
    It stores all information into an extended version of the Biobase ExpressionSet.
}
\section{Objects from the Class}{
    Objects can be created by calls of the form \code{AVASet(dirname, avaBin)}.
    dirname is a character giving the proejct directory and avaBin is a
    character giving the path to the AVA software installation (i.e. the
    directory containing the doAmplicon binary). The constructor will
    start the AVA software command line and import all necessary data.
    
    If the AVA software is not installed on the same machine that runs
    R, all data must be exported manually using the AVA Command
    Line Interface (AVA-CLI). After having exported all text files, the constructor
    \code{AVASet(dirname, avaBin, file_sample, file_amp, file_reference, file_variant, file_variantHits)}
    can be used to import them. See the example below.
    
    Finally, old project folders generated by AVA software < 2.6 can be
    imported using \code{AVASet(dirname)}. Where dirname is the path
    to the project folder (i.e. a directory that contains the files
    and subdirectories "Amplicons/ProjectDef/ampliconsProject.txt",
    "Amplicons/Results/Variants/currentVariantDefs.txt",
    "Amplicons/Results/Variants", "Amplicons/Results/Align").
}
\section{Slots}{
  \describe{
    \item{\code{assayData}:}{Object of class \code{AssayData}. Contains the number of reads and the total read depth for every variant and each 
	sample in forward and reverse direction. Its column number equals \code{nrow(phenoData)}.}
    \item{\code{featureData}:}{Object of class \code{AnnotatedDataFrame}. Contains information about the type, position and reference of each 
	variant.}
    \item{\code{phenoData}:}{Object of class \code{AnnotatedDataFrame}. Contains the sample-IDs and name, annotation and group of the read data 
	for all samples. If available, the lane, pico titer plate(s) (PTP) or MID(s) of each sample are shown as well.}
    \item{\code{assayDataAmp}:}{Object of class \code{AssayData}. Contains the number of reads for every amplicon and each sample in forward/reverse 
	direction. Its column number equals \code{nrow(featureDataAmp)}.}
    \item{\code{featureDataAmp}:}{Object of class \code{AnnotatedDataFrame}. Contains the primer sequences, reference sequences and the coordinates 
	of the target regions for every amplicon.}
    \item{\code{referenceSequences}:}{Object of class
      \code{AlignedRead}. If additional alignment information were computed via 
	\code{alignShortReads}, this slot knows about the chromosome, position and the strand of each reference sequence.}
    \item{\code{variantFilterPerc}:}{Object of class \code{numeric}. Contains a threshold to display only those variants, whose 
	coverage (in percent) in forward and reverse direction in at least one sample is higher than this filter value. See 
	\code{\link{setVariantFilter}} for details about setting this value.}
    \item{\code{variantFilter}:}{Object of class \code{character}. Contains a vector of variant names whose
        coverage (in percent) in forward and reverse direction in at least one sample is higher than the filter value in 
	\code{variantFilterPerc}.}
    \item{\code{dirs}:}{Object of class \code{character}. Based on a directory given at instantiation of the object, it contains a vector of several 
	directories containing all relevant AVA-project files.}
    \item{\code{experimentData}:}{Object of class \code{MIAME}. Contains details of the experiment.}
    \item{\code{annotation}:}{Object of class \code{character}. Label associated with the annotation package used in the experiment.}
    \item{\code{protocolData}:}{Object of class \code{annotatedDataFrame}. Contains additional information about the samples.}
    \item{\code{.__classVersion__}:}{Object of class \code{Versions}. Remembers the R and R453Toolbox version numbers used to created the 
	AVASet instance.}
  }
}
\section{Extends}{
    Class \code{\linkS4class{eSet}}, directly.
    Class \code{\linkS4class{VersionedBiobase}}, by class "eSet", distance 2. 
    Class \code{\linkS4class{Versioned}}, by class "eSet", distance 3.
}
\section{Methods}{
  \describe{
    \item{object[i,j]:}{Allows subsetting an AVASet object by features (i) and samples (j).}
    \item{assayDataAmp(object), assayDataAmp(object)<-value:}{Similar to \code{assayData} of the Biobase ExpressionSet, this function 
	returns/replaces the amplicon assay data. }
    \item{fDataAmp(object):}{Similar to \code{fData} of the Biobase ExpressionSet, this function returns the amplicon feature data
    	as a data frame.}
    \item{featureDataAmp(object), featureDataAmp(object)<-value:}{Similar to \code{featureData} of the Biobase ExpressionSet, this function 
	returns/replaces the amplicon feature data and feature meta.}
    \item{referenceSequences(object), referenceSequences(object)<-value:}{Returns/replaces the reference sequence slot.}
    \item{alignShortReads(object, bsGenome):}{Retrieve the chromosomal positions of the amplicon 
	sequences.}
    \item{setVariantFilter(object):}{Sets the filter to display only those variants, whose coverage (in percent) in forward and reverse 
	direction in at least one sample is higher than the given value.}
    \item{getVariantPercentages(object)}{Computes the coverage for every variant over all reads (forward and/or reverse) and for each 
	sample.}
    \item{annotateVariants(object):}{Annotates given genomic variants. See \code{\link{annotateVariants}} for details.}
    \item{htmlReport(object):}{Exports all (filtered) variant data into a html report. See \code{\link{htmlReport}} for details}
	 }
}

\author{Christoph Bartenhagen}

\seealso{
    \code{\link{MapperSet-class}},
    \code{\link{annotateVariants}},
    \code{\link{alignShortReads}},
    \code{\link{htmlReport}},
    \code{\link{setVariantFilter}},
    \code{\link{getVariantPercentages}}
}

\examples{

    # sum up class structure
    showClass("AVASet")

    # load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
    data(avaSetExample)
    avaSetExample

    # show contents of assay, feature and pheno data
    head(assayData(avaSetExample)$variantForwCount)
    head(assayData(avaSetExample)$totalForwCount)
    head(assayData(avaSetExample)$variantRevCount)
    head(assayData(avaSetExample)$totalRevCount)
    head(fData(avaSetExample))
    pData(avaSetExample)
    assayDataAmp(avaSetExample)
    fDataAmp(avaSetExample)
    referenceSequences(avaSetExample)

    # Use these commands to export a project from within the AVA-CLI (doAmplicon):
    # > list sample -outputFile sample.csv
    # > list amplicon -outputFile amp.csv
    # > list reference -outputFile reference.csv
    # > list variant -outputFile variant.csv
    # > report variantHits -outputFile variantHits.csv

    # Load an AVA dataset containing 6 samples, 4 amplicons and 222 variants
    # by specifying five files, that were exported with the AVA-CLI:
    projectDir = system.file("extdata", "AVASet_doAmplicon", package="R453Plus1Toolbox")
    avaSetExample = AVASet(dirname=projectDir, file_sample="sample.csv", file_amp="amp.csv", file_reference="reference.csv", file_variant="variant.csv", file_variantHits="variantHits.csv")
}

\keyword{classes}

\name{homopolymerHist}

\docType{methods}

\alias{homopolymerHist}
\alias{homopolymerHist,SFFRead-method}

\title{Create A Histogram Of The Homopolymer Stretches}

\description{
  This function creates a histogram for the different lengths of the homopolymer stretches with one bar
  for each nucleotide in the flow. With the height giving the number of this homopolymer stretch in the flowgram.
}

\usage{
  homopolymerHist(x, range=c(0, length(flowgram(x))), xlab="Homopolymer length", 
    ylab="Number of homopolymers", col=c(A="black", C="red", G="blue", T="green"), ...)
}

\arguments{
  \item{x}{An object of class \code{\link{SFFRead}}.}
  \item{range}{Two positions between which the flows should be plotted.}
  \item{xlab}{The X axis label.}
  \item{ylab}{The Y axis label.}
  \item{col}{The colors in which the four nucleotids should be plotted.}
  \item{\dots}{Arguments to be passed to methods, such as graphical parameters (see \sQuote{par}).}
}

\author{
  Christian Ruckert
}

\keyword{methods}

\name{mutationInfo}
\alias{mutationInfo}
\docType{data}
\title{
	Example data for \code{\link{plotVariants}}
}
\description{
 	This data.frame is part of the vignette example of the \code{\link{plotVariants}} function. It contains annotations for the different
  	mutations occurring in the example data. It has columns "mutation", "legend" and "color".
}

\usage{data(plotVariantsExample)}

\format{
	data.frame
}

\examples{
    data(plotVariantsExample)
    mutationInfo
}
\keyword{datasets}

\name{assayDataAmp}

\alias{assayDataAmp}

\title{Access the amplicon data of an AVASet.}

\description{
    Similar to \code{assayData} of the Biobase ExpressionSet, this function returns the assay data of the amplicon slot of an instance of 
    the AVASet.
}

\usage{
    assayDataAmp(object) 
}

\arguments{
  \item{object}{An \code{link{AVASet-class}} object.}
}

\value{
    The assay data of the amplicon slot consists of a list of two data frames with the number of forward and reverse reads of all amplicons for each 
    sample (see \code{\link{AVASet-class}} for details).
}

\seealso{
\code{\link{fDataAmp}}, \code{\link{featureDataAmp}}, \code{\link{AVASet-class}}
}

\examples{
    # load an AVA dataset containing 6 samples, 4 amplicons and 259 variants
    data(avaSetExample)

    # show contents of amplicon assay data
    assayDataAmp(avaSetExample)
}
\author{Christoph Bartenhagen}

\name{genomeSequencerMIDs}

\alias{genomeSequencerMIDs}
\alias{genomeSequencerMIDs,character-method}
\alias{genomeSequencerMIDs,missing-method}

\title{Retrieve GS multiplex sequences}

\description{This method returns the standard multiplex sequences
             used by the Genome Sequence MID library kits.}

\usage{\S4method{genomeSequencerMIDs}{missing}()
       \S4method{genomeSequencerMIDs}{character}(mid)}

\arguments{\item{mid}{Character vector with multiplex sequences' IDs (MIDs)}}

\value{\code{genomeSequencerMIDs} returns a \code{DNAStringSet} with the
       requested multiplex sequences.}
     
\details{If the argument \code{mid} is omitted, all 14 available
         multiplex sequences are returned.}
     
\seealso{\code{\link{demultiplexReads}}}

\examples{
    genomeSequencerMIDs()
    genomeSequencerMIDs(c("MID1", "MID3"))
}

\author{Hans-Ulrich Klein}
\keyword{demultiplex}

\name{plotVariationFrequency}

\alias{plotVariationFrequency}
\alias{plotVariationFrequency,character,numeric-method}

\title{Create an AVA style variation frequency plot}

\description{
This method creates a plot similar to the variation frequency plot in
Roche's GS Amplicon Variant Analyzer. The plot shows the reference
sequence along the x-axis and indicates variants as bars at the
appropriate positions. The height of the bars corresponds to the
percentage of reads carrying the variant. A second y-axis indicates the
absolute number of reads covering the variant.}

\usage{plotVariationFrequency(object, plotRange, ...)}

\arguments{
  \item{object}{A character pointing to an Amplicon Variant Analyser Global Alignmnet export file.}
  \item{plotRange}{A two dimensional numeric vector giving the start and end base of the reference sequence that should be plotted.}
  \item{\dots}{Arguments passed to other plotting methods. Especially, argument \code{col}: Optional character vector of length 7 specifying the bars' colors indicating 
  different base substitutions or deletions. See details. And argument \code{sequenceCex}: Optional numeric value specifying the size of the reference sequence's bases.}
}

\details{
The text file used as imput must have the format generated by the AVA
export function. Such a file can be generated using the export button in
the Global Alignment view of the AVA software.
The \code{col} argument specifies the colours used for different bases
and deletions. The following listing gives the meaning of the i-th position
of the col vector (default values in braces): 
\enumerate{
  \item A (green)
  \item C (blue)
  \item G (black)
  \item T (red)
  \item N (purple)
  \item deletion (gray)
}
}

\examples{
\dontrun{
file = system.file("extdata", "AVAVarFreqExport", "AVAVarFreqExport.xls", package="R453Plus1Toolbox")
plotVariationFrequency(file, plotRange=c(50, 150))}
}

\author{Hans-Ulrich Klein}

\keyword{Variation Frequency Coverage}

\name{sequenceCaptureLinkers}

\alias{sequenceCaptureLinkers}
\alias{sequenceCaptureLinkers,character-method}
\alias{sequenceCaptureLinkers,missing-method}

\title{Retrieve NimbleGen's sequence capture linkers}

\description{This method returns the NimbleGen's linker sequences 
             used with their sequence capture arrays. See pp.29-30
             in the NimbleGen Arrays User's Guide.}

\usage{\S4method{sequenceCaptureLinkers}{character}(name)
       \S4method{sequenceCaptureLinkers}{missing}()}

\arguments{\item{name}{Character vector with linker sequences' names}}

\value{\code{sequenceCaptureLinkers} returns a \code{DNAStringSet} with the
       requested linker sequences.}
     
\details{If the argument \code{name} is omitted, both linker
         sequences are returned.}
     
\seealso{\code{\link{removeLinker}}}

\examples{
    sequenceCaptureLinkers()}

\author{Hans-Ulrich Klein}
\keyword{linker}
